# SlothPy
# Copyright (C) 2023 Mikolaj Tadeusz Zychowicz (MTZ)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from numpy import ndarray, array


def lebedev_laikov_grid(grid: int) -> ndarray:
    """
    Returns Lebedev-Laikov-Grids [1] over a hemisphere.

    Parameters
    ----------
    grid : int
        An integer from 0 to 11 controling the number of points in the grid
        over the hemisphere.

        0 - 17 points,
        1 - 29 points,
        2 - 93 points,
        3 - 161 points,
        4 - 185 points,
        5 - 229 points,
        6 - 309 points,
        7 - 401 points,
        8 - 505 points,
        9 - 889 points,
        10 - 1381 points,
        11 - 2949 points

    Returns
    -------
    ndarray
        Grid array in the form [[direction_x, direction_y, direction_z,
        weight], ...].

    Raises
    ------
    ValueError
        If grid is not an integer from 0 to 11.

    Notes
    -----
    Grids 0-2 can be considered as insufficient in most cases (only should be
    used for experimental purposes), 3-5 minimal to standard, 6-8 very precise,
    but costly, 9-11 very dense - only for particular use-cases (extremely
    costly - especially 10-11).

    References
    ----------
    .. [1] V.I. Lebedev, and D.N. Laikov
           "A quadrature formula for the sphere of the 131st
           algebraic order of accuracy"
           Doklady Mathematics, Vol. 59, No. 3, 1999, pp. 477-481.
    """
    if grid == 0:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.047619047619],
                [-1.0, 0.0, 0.0, 0.047619047619],
                [0.0, 1.0, 0.0, 0.047619047619],
                [0.0, -1.0, 0.0, 0.047619047619],
                [0.0, 0.0, 1.0, 0.095238095238],
                [0.0, 0.707106781187, 0.707106781187, 0.07619047619],
                [0.0, -0.707106781187, 0.707106781187, 0.07619047619],
                [0.707106781187, 0.0, 0.707106781187, 0.07619047619],
                [-0.707106781187, 0.0, 0.707106781187, 0.07619047619],
                [0.707106781187, 0.707106781187, 0.0, 0.038095238095],
                [-0.707106781187, 0.707106781187, 0.0, 0.038095238095],
                [0.707106781187, -0.707106781187, 0.0, 0.038095238095],
                [-0.707106781187, -0.707106781187, 0.0, 0.038095238095],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.064285714286],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.064285714286],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.064285714286],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.064285714286,
                ],
            ]
        )
        return grid_array

    if grid == 1:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.012698412698],
                [-1.0, 0.0, 0.0, 0.012698412698],
                [0.0, 1.0, 0.0, 0.012698412698],
                [0.0, -1.0, 0.0, 0.012698412698],
                [0.0, 0.0, 1.0, 0.025396825397],
                [0.0, 0.707106781187, 0.707106781187, 0.045149911817],
                [0.0, -0.707106781187, 0.707106781187, 0.045149911817],
                [0.707106781187, 0.0, 0.707106781187, 0.045149911817],
                [-0.707106781187, 0.0, 0.707106781187, 0.045149911817],
                [0.707106781187, 0.707106781187, 0.0, 0.022574955908],
                [-0.707106781187, 0.707106781187, 0.0, 0.022574955908],
                [0.707106781187, -0.707106781187, 0.0, 0.022574955908],
                [-0.707106781187, -0.707106781187, 0.0, 0.022574955908],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.0421875],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.0421875],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.0421875],
                [-0.57735026919, -0.57735026919, 0.57735026919, 0.0421875],
                [
                    0.301511344578,
                    0.301511344578,
                    0.904534033733,
                    0.040346671076,
                ],
                [
                    -0.301511344578,
                    0.301511344578,
                    0.904534033733,
                    0.040346671076,
                ],
                [
                    0.301511344578,
                    -0.301511344578,
                    0.904534033733,
                    0.040346671076,
                ],
                [
                    -0.301511344578,
                    -0.301511344578,
                    0.904534033733,
                    0.040346671076,
                ],
                [
                    0.301511344578,
                    0.904534033733,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    -0.301511344578,
                    0.904534033733,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    0.301511344578,
                    -0.904534033733,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    -0.301511344578,
                    -0.904534033733,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    0.904534033733,
                    0.301511344578,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    -0.904534033733,
                    0.301511344578,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    0.904534033733,
                    -0.301511344578,
                    0.301511344578,
                    0.040346671076,
                ],
                [
                    -0.904534033733,
                    -0.301511344578,
                    0.301511344578,
                    0.040346671076,
                ],
            ]
        )
        return grid_array

    if grid == 2:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.005544842902],
                [-1.0, 0.0, 0.0, 0.005544842902],
                [0.0, 1.0, 0.0, 0.005544842902],
                [0.0, -1.0, 0.0, 0.005544842902],
                [0.0, 0.0, 1.0, 0.011089685804],
                [0.0, 0.707106781187, 0.707106781187, 0.012142665541],
                [0.0, -0.707106781187, 0.707106781187, 0.012142665541],
                [0.707106781187, 0.0, 0.707106781187, 0.012142665541],
                [-0.707106781187, 0.0, 0.707106781187, 0.012142665541],
                [0.707106781187, 0.707106781187, 0.0, 0.006071332771],
                [-0.707106781187, 0.707106781187, 0.0, 0.006071332771],
                [0.707106781187, -0.707106781187, 0.0, 0.006071332771],
                [-0.707106781187, -0.707106781187, 0.0, 0.006071332771],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.012767349547],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.012767349547],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.012767349547],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.012767349547,
                ],
                [
                    0.255125262111,
                    0.255125262111,
                    0.932642590313,
                    0.010366775175,
                ],
                [
                    -0.255125262111,
                    0.255125262111,
                    0.932642590313,
                    0.010366775175,
                ],
                [
                    0.255125262111,
                    -0.255125262111,
                    0.932642590313,
                    0.010366775175,
                ],
                [
                    -0.255125262111,
                    -0.255125262111,
                    0.932642590313,
                    0.010366775175,
                ],
                [
                    0.255125262111,
                    0.932642590313,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    -0.255125262111,
                    0.932642590313,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    0.255125262111,
                    -0.932642590313,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    -0.255125262111,
                    -0.932642590313,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    0.932642590313,
                    0.255125262111,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    -0.932642590313,
                    0.255125262111,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    0.932642590313,
                    -0.255125262111,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    -0.932642590313,
                    -0.255125262111,
                    0.255125262111,
                    0.010366775175,
                ],
                [
                    0.674360146036,
                    0.674360146036,
                    0.300793595138,
                    0.01263585802,
                ],
                [
                    -0.674360146036,
                    0.674360146036,
                    0.300793595138,
                    0.01263585802,
                ],
                [
                    0.674360146036,
                    -0.674360146036,
                    0.300793595138,
                    0.01263585802,
                ],
                [
                    -0.674360146036,
                    -0.674360146036,
                    0.300793595138,
                    0.01263585802,
                ],
                [
                    0.674360146036,
                    0.300793595138,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    -0.674360146036,
                    0.300793595138,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    0.674360146036,
                    -0.300793595138,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    -0.674360146036,
                    -0.300793595138,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    0.300793595138,
                    0.674360146036,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    -0.300793595138,
                    0.674360146036,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    0.300793595138,
                    -0.674360146036,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    -0.300793595138,
                    -0.674360146036,
                    0.674360146036,
                    0.01263585802,
                ],
                [
                    0.431891069672,
                    0.431891069672,
                    0.791795559393,
                    0.012403340013,
                ],
                [
                    -0.431891069672,
                    0.431891069672,
                    0.791795559393,
                    0.012403340013,
                ],
                [
                    0.431891069672,
                    -0.431891069672,
                    0.791795559393,
                    0.012403340013,
                ],
                [
                    -0.431891069672,
                    -0.431891069672,
                    0.791795559393,
                    0.012403340013,
                ],
                [
                    0.431891069672,
                    0.791795559393,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    -0.431891069672,
                    0.791795559393,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    0.431891069672,
                    -0.791795559393,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    -0.431891069672,
                    -0.791795559393,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    0.791795559393,
                    0.431891069672,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    -0.791795559393,
                    0.431891069672,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    0.791795559393,
                    -0.431891069672,
                    0.431891069672,
                    0.012403340013,
                ],
                [
                    -0.791795559393,
                    -0.431891069672,
                    0.431891069672,
                    0.012403340013,
                ],
                [0.261393136034, 0.965232421976, 0.0, 0.005477143385],
                [-0.261393136034, 0.965232421976, 0.0, 0.005477143385],
                [0.261393136034, -0.965232421976, 0.0, 0.005477143385],
                [-0.261393136034, -0.965232421976, 0.0, 0.005477143385],
                [0.965232421976, 0.261393136034, 0.0, 0.005477143385],
                [-0.965232421976, 0.261393136034, 0.0, 0.005477143385],
                [0.965232421976, -0.261393136034, 0.0, 0.005477143385],
                [-0.965232421976, -0.261393136034, 0.0, 0.005477143385],
                [0.261393136034, 0.0, 0.965232421976, 0.01095428677],
                [-0.261393136034, 0.0, 0.965232421976, 0.01095428677],
                [0.965232421976, 0.0, 0.261393136034, 0.01095428677],
                [-0.965232421976, 0.0, 0.261393136034, 0.01095428677],
                [0.0, 0.261393136034, 0.965232421976, 0.01095428677],
                [0.0, -0.261393136034, 0.965232421976, 0.01095428677],
                [0.0, 0.965232421976, 0.261393136034, 0.01095428677],
                [0.0, -0.965232421976, 0.261393136034, 0.01095428677],
                [
                    0.49904531618,
                    0.144663074433,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    -0.49904531618,
                    0.144663074433,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    0.49904531618,
                    -0.144663074433,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    -0.49904531618,
                    -0.144663074433,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    0.49904531618,
                    0.854415804685,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    -0.49904531618,
                    0.854415804685,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    0.49904531618,
                    -0.854415804685,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    -0.49904531618,
                    -0.854415804685,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    0.144663074433,
                    0.49904531618,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    -0.144663074433,
                    0.49904531618,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    0.144663074433,
                    -0.49904531618,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    -0.144663074433,
                    -0.49904531618,
                    0.854415804685,
                    0.011936767975,
                ],
                [
                    0.144663074433,
                    0.854415804685,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    -0.144663074433,
                    0.854415804685,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    0.144663074433,
                    -0.854415804685,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    -0.144663074433,
                    -0.854415804685,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    0.854415804685,
                    0.49904531618,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    -0.854415804685,
                    0.49904531618,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    0.854415804685,
                    -0.49904531618,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    -0.854415804685,
                    -0.49904531618,
                    0.144663074433,
                    0.011936767975,
                ],
                [
                    0.854415804685,
                    0.144663074433,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    -0.854415804685,
                    0.144663074433,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    0.854415804685,
                    -0.144663074433,
                    0.49904531618,
                    0.011936767975,
                ],
                [
                    -0.854415804685,
                    -0.144663074433,
                    0.49904531618,
                    0.011936767975,
                ],
            ]
        )
        return grid_array

    if grid == 3:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.000854591173],
                [-1.0, 0.0, 0.0, 0.000854591173],
                [0.0, 1.0, 0.0, 0.000854591173],
                [0.0, -1.0, 0.0, 0.000854591173],
                [0.0, 0.0, 1.0, 0.001709182345],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.00719823857],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.00719823857],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.00719823857],
                [-0.57735026919, -0.57735026919, 0.57735026919, 0.00719823857],
                [
                    0.351564034557,
                    0.351564034557,
                    0.867643624544,
                    0.006899576849,
                ],
                [
                    -0.351564034557,
                    0.351564034557,
                    0.867643624544,
                    0.006899576849,
                ],
                [
                    0.351564034557,
                    -0.351564034557,
                    0.867643624544,
                    0.006899576849,
                ],
                [
                    -0.351564034557,
                    -0.351564034557,
                    0.867643624544,
                    0.006899576849,
                ],
                [
                    0.351564034557,
                    0.867643624544,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    -0.351564034557,
                    0.867643624544,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    0.351564034557,
                    -0.867643624544,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    -0.351564034557,
                    -0.867643624544,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    0.867643624544,
                    0.351564034557,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    -0.867643624544,
                    0.351564034557,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    0.867643624544,
                    -0.351564034557,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    -0.867643624544,
                    -0.351564034557,
                    0.351564034557,
                    0.006899576849,
                ],
                [
                    0.656632941022,
                    0.656632941022,
                    0.371034178385,
                    0.007209645203,
                ],
                [
                    -0.656632941022,
                    0.656632941022,
                    0.371034178385,
                    0.007209645203,
                ],
                [
                    0.656632941022,
                    -0.656632941022,
                    0.371034178385,
                    0.007209645203,
                ],
                [
                    -0.656632941022,
                    -0.656632941022,
                    0.371034178385,
                    0.007209645203,
                ],
                [
                    0.656632941022,
                    0.371034178385,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    -0.656632941022,
                    0.371034178385,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    0.656632941022,
                    -0.371034178385,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    -0.656632941022,
                    -0.371034178385,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    0.371034178385,
                    0.656632941022,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    -0.371034178385,
                    0.656632941022,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    0.371034178385,
                    -0.656632941022,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    -0.371034178385,
                    -0.656632941022,
                    0.656632941022,
                    0.007209645203,
                ],
                [
                    0.472905413258,
                    0.472905413258,
                    0.743452042988,
                    0.007153459323,
                ],
                [
                    -0.472905413258,
                    0.472905413258,
                    0.743452042988,
                    0.007153459323,
                ],
                [
                    0.472905413258,
                    -0.472905413258,
                    0.743452042988,
                    0.007153459323,
                ],
                [
                    -0.472905413258,
                    -0.472905413258,
                    0.743452042988,
                    0.007153459323,
                ],
                [
                    0.472905413258,
                    0.743452042988,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    -0.472905413258,
                    0.743452042988,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    0.472905413258,
                    -0.743452042988,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    -0.472905413258,
                    -0.743452042988,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    0.743452042988,
                    0.472905413258,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    -0.743452042988,
                    0.472905413258,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    0.743452042988,
                    -0.472905413258,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    -0.743452042988,
                    -0.472905413258,
                    0.472905413258,
                    0.007153459323,
                ],
                [
                    0.096183085226,
                    0.096183085226,
                    0.990705621379,
                    0.004704202827,
                ],
                [
                    -0.096183085226,
                    0.096183085226,
                    0.990705621379,
                    0.004704202827,
                ],
                [
                    0.096183085226,
                    -0.096183085226,
                    0.990705621379,
                    0.004704202827,
                ],
                [
                    -0.096183085226,
                    -0.096183085226,
                    0.990705621379,
                    0.004704202827,
                ],
                [
                    0.096183085226,
                    0.990705621379,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    -0.096183085226,
                    0.990705621379,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    0.096183085226,
                    -0.990705621379,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    -0.096183085226,
                    -0.990705621379,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    0.990705621379,
                    0.096183085226,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    -0.990705621379,
                    0.096183085226,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    0.990705621379,
                    -0.096183085226,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    -0.990705621379,
                    -0.096183085226,
                    0.096183085226,
                    0.004704202827,
                ],
                [
                    0.221964523629,
                    0.221964523629,
                    0.949454317226,
                    0.006217906245,
                ],
                [
                    -0.221964523629,
                    0.221964523629,
                    0.949454317226,
                    0.006217906245,
                ],
                [
                    0.221964523629,
                    -0.221964523629,
                    0.949454317226,
                    0.006217906245,
                ],
                [
                    -0.221964523629,
                    -0.221964523629,
                    0.949454317226,
                    0.006217906245,
                ],
                [
                    0.221964523629,
                    0.949454317226,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    -0.221964523629,
                    0.949454317226,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    0.221964523629,
                    -0.949454317226,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    -0.221964523629,
                    -0.949454317226,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    0.949454317226,
                    0.221964523629,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    -0.949454317226,
                    0.221964523629,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    0.949454317226,
                    -0.221964523629,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    -0.949454317226,
                    -0.221964523629,
                    0.221964523629,
                    0.006217906245,
                ],
                [
                    0.701176641609,
                    0.701176641609,
                    0.129238672711,
                    0.007300091615,
                ],
                [
                    -0.701176641609,
                    0.701176641609,
                    0.129238672711,
                    0.007300091615,
                ],
                [
                    0.701176641609,
                    -0.701176641609,
                    0.129238672711,
                    0.007300091615,
                ],
                [
                    -0.701176641609,
                    -0.701176641609,
                    0.129238672711,
                    0.007300091615,
                ],
                [
                    0.701176641609,
                    0.129238672711,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    -0.701176641609,
                    0.129238672711,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    0.701176641609,
                    -0.129238672711,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    -0.701176641609,
                    -0.129238672711,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    0.129238672711,
                    0.701176641609,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    -0.129238672711,
                    0.701176641609,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    0.129238672711,
                    -0.701176641609,
                    0.701176641609,
                    0.007300091615,
                ],
                [
                    -0.129238672711,
                    -0.701176641609,
                    0.701176641609,
                    0.007300091615,
                ],
                [0.264415288706, 0.964408914879, 0.0, 0.002982344963],
                [-0.264415288706, 0.964408914879, 0.0, 0.002982344963],
                [0.264415288706, -0.964408914879, 0.0, 0.002982344963],
                [-0.264415288706, -0.964408914879, 0.0, 0.002982344963],
                [0.964408914879, 0.264415288706, 0.0, 0.002982344963],
                [-0.964408914879, 0.264415288706, 0.0, 0.002982344963],
                [0.964408914879, -0.264415288706, 0.0, 0.002982344963],
                [-0.964408914879, -0.264415288706, 0.0, 0.002982344963],
                [0.264415288706, 0.0, 0.964408914879, 0.005964689926],
                [-0.264415288706, 0.0, 0.964408914879, 0.005964689926],
                [0.964408914879, 0.0, 0.264415288706, 0.005964689926],
                [-0.964408914879, 0.0, 0.264415288706, 0.005964689926],
                [0.0, 0.264415288706, 0.964408914879, 0.005964689926],
                [0.0, -0.264415288706, 0.964408914879, 0.005964689926],
                [0.0, 0.964408914879, 0.264415288706, 0.005964689926],
                [0.0, -0.964408914879, 0.264415288706, 0.005964689926],
                [0.571895589188, 0.820326419828, 0.0, 0.003600820932],
                [-0.571895589188, 0.820326419828, 0.0, 0.003600820932],
                [0.571895589188, -0.820326419828, 0.0, 0.003600820932],
                [-0.571895589188, -0.820326419828, 0.0, 0.003600820932],
                [0.820326419828, 0.571895589188, 0.0, 0.003600820932],
                [-0.820326419828, 0.571895589188, 0.0, 0.003600820932],
                [0.820326419828, -0.571895589188, 0.0, 0.003600820932],
                [-0.820326419828, -0.571895589188, 0.0, 0.003600820932],
                [0.571895589188, 0.0, 0.820326419828, 0.007201641864],
                [-0.571895589188, 0.0, 0.820326419828, 0.007201641864],
                [0.820326419828, 0.0, 0.571895589188, 0.007201641864],
                [-0.820326419828, 0.0, 0.571895589188, 0.007201641864],
                [0.0, 0.571895589188, 0.820326419828, 0.007201641864],
                [0.0, -0.571895589188, 0.820326419828, 0.007201641864],
                [0.0, 0.820326419828, 0.571895589188, 0.007201641864],
                [0.0, -0.820326419828, 0.571895589188, 0.007201641864],
                [
                    0.251003475177,
                    0.800072749407,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    -0.251003475177,
                    0.800072749407,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    0.251003475177,
                    -0.800072749407,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    -0.251003475177,
                    -0.800072749407,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    0.251003475177,
                    0.544867737258,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    -0.251003475177,
                    0.544867737258,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    0.251003475177,
                    -0.544867737258,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    -0.251003475177,
                    -0.544867737258,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    0.800072749407,
                    0.251003475177,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    -0.800072749407,
                    0.251003475177,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    0.800072749407,
                    -0.251003475177,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    -0.800072749407,
                    -0.251003475177,
                    0.544867737258,
                    0.007143081109,
                ],
                [
                    0.800072749407,
                    0.544867737258,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    -0.800072749407,
                    0.544867737258,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    0.800072749407,
                    -0.544867737258,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    -0.800072749407,
                    -0.544867737258,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    0.544867737258,
                    0.251003475177,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    -0.544867737258,
                    0.251003475177,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    0.544867737258,
                    -0.251003475177,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    -0.544867737258,
                    -0.251003475177,
                    0.800072749407,
                    0.007143081109,
                ],
                [
                    0.544867737258,
                    0.800072749407,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    -0.544867737258,
                    0.800072749407,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    0.544867737258,
                    -0.800072749407,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    -0.544867737258,
                    -0.800072749407,
                    0.251003475177,
                    0.007143081109,
                ],
                [
                    0.123354853258,
                    0.412772408317,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    -0.123354853258,
                    0.412772408317,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    0.123354853258,
                    -0.412772408317,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    -0.123354853258,
                    -0.412772408317,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    0.123354853258,
                    0.902442529533,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    -0.123354853258,
                    0.902442529533,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    0.123354853258,
                    -0.902442529533,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    -0.123354853258,
                    -0.902442529533,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    0.412772408317,
                    0.123354853258,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    -0.412772408317,
                    0.123354853258,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    0.412772408317,
                    -0.123354853258,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    -0.412772408317,
                    -0.123354853258,
                    0.902442529533,
                    0.00678462441,
                ],
                [
                    0.412772408317,
                    0.902442529533,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    -0.412772408317,
                    0.902442529533,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    0.412772408317,
                    -0.902442529533,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    -0.412772408317,
                    -0.902442529533,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    0.902442529533,
                    0.123354853258,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    -0.902442529533,
                    0.123354853258,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    0.902442529533,
                    -0.123354853258,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    -0.902442529533,
                    -0.123354853258,
                    0.412772408317,
                    0.00678462441,
                ],
                [
                    0.902442529533,
                    0.412772408317,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    -0.902442529533,
                    0.412772408317,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    0.902442529533,
                    -0.412772408317,
                    0.123354853258,
                    0.00678462441,
                ],
                [
                    -0.902442529533,
                    -0.412772408317,
                    0.123354853258,
                    0.00678462441,
                ],
            ]
        )
        return grid_array

    if grid == 4:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.003006796749],
                [-1.0, 0.0, 0.0, 0.003006796749],
                [0.0, 1.0, 0.0, 0.003006796749],
                [0.0, -1.0, 0.0, 0.003006796749],
                [0.0, 0.0, 1.0, 0.006013593499],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.006101255491],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.006101255491],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.006101255491],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.006101255491,
                ],
                [
                    0.706896546391,
                    0.706896546391,
                    0.024383301669,
                    0.003242209201,
                ],
                [
                    -0.706896546391,
                    0.706896546391,
                    0.024383301669,
                    0.003242209201,
                ],
                [
                    0.706896546391,
                    -0.706896546391,
                    0.024383301669,
                    0.003242209201,
                ],
                [
                    -0.706896546391,
                    -0.706896546391,
                    0.024383301669,
                    0.003242209201,
                ],
                [
                    0.706896546391,
                    0.024383301669,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    -0.706896546391,
                    0.024383301669,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    0.706896546391,
                    -0.024383301669,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    -0.706896546391,
                    -0.024383301669,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    0.024383301669,
                    0.706896546391,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    -0.024383301669,
                    0.706896546391,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    0.024383301669,
                    -0.706896546391,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    -0.024383301669,
                    -0.706896546391,
                    0.706896546391,
                    0.003242209201,
                ],
                [
                    0.479468262571,
                    0.479468262571,
                    0.734996850588,
                    0.00601140297,
                ],
                [
                    -0.479468262571,
                    0.479468262571,
                    0.734996850588,
                    0.00601140297,
                ],
                [
                    0.479468262571,
                    -0.479468262571,
                    0.734996850588,
                    0.00601140297,
                ],
                [
                    -0.479468262571,
                    -0.479468262571,
                    0.734996850588,
                    0.00601140297,
                ],
                [
                    0.479468262571,
                    0.734996850588,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    -0.479468262571,
                    0.734996850588,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    0.479468262571,
                    -0.734996850588,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    -0.479468262571,
                    -0.734996850588,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    0.734996850588,
                    0.479468262571,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    -0.734996850588,
                    0.479468262571,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    0.734996850588,
                    -0.479468262571,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    -0.734996850588,
                    -0.479468262571,
                    0.479468262571,
                    0.00601140297,
                ],
                [
                    0.192753315488,
                    0.192753315488,
                    0.962129055136,
                    0.005981985059,
                ],
                [
                    -0.192753315488,
                    0.192753315488,
                    0.962129055136,
                    0.005981985059,
                ],
                [
                    0.192753315488,
                    -0.192753315488,
                    0.962129055136,
                    0.005981985059,
                ],
                [
                    -0.192753315488,
                    -0.192753315488,
                    0.962129055136,
                    0.005981985059,
                ],
                [
                    0.192753315488,
                    0.962129055136,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    -0.192753315488,
                    0.962129055136,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    0.192753315488,
                    -0.962129055136,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    -0.192753315488,
                    -0.962129055136,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    0.962129055136,
                    0.192753315488,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    -0.962129055136,
                    0.192753315488,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    0.962129055136,
                    -0.192753315488,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    -0.962129055136,
                    -0.192753315488,
                    0.192753315488,
                    0.005981985059,
                ],
                [
                    0.693035796133,
                    0.693035796133,
                    0.198501311223,
                    0.005964341288,
                ],
                [
                    -0.693035796133,
                    0.693035796133,
                    0.198501311223,
                    0.005964341288,
                ],
                [
                    0.693035796133,
                    -0.693035796133,
                    0.198501311223,
                    0.005964341288,
                ],
                [
                    -0.693035796133,
                    -0.693035796133,
                    0.198501311223,
                    0.005964341288,
                ],
                [
                    0.693035796133,
                    0.198501311223,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    -0.693035796133,
                    0.198501311223,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    0.693035796133,
                    -0.198501311223,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    -0.693035796133,
                    -0.198501311223,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    0.198501311223,
                    0.693035796133,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    -0.198501311223,
                    0.693035796133,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    0.198501311223,
                    -0.693035796133,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    -0.198501311223,
                    -0.693035796133,
                    0.693035796133,
                    0.005964341288,
                ],
                [
                    0.360830211552,
                    0.360830211552,
                    0.860001812128,
                    0.005443128475,
                ],
                [
                    -0.360830211552,
                    0.360830211552,
                    0.860001812128,
                    0.005443128475,
                ],
                [
                    0.360830211552,
                    -0.360830211552,
                    0.860001812128,
                    0.005443128475,
                ],
                [
                    -0.360830211552,
                    -0.360830211552,
                    0.860001812128,
                    0.005443128475,
                ],
                [
                    0.360830211552,
                    0.860001812128,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    -0.360830211552,
                    0.860001812128,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    0.360830211552,
                    -0.860001812128,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    -0.360830211552,
                    -0.860001812128,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    0.860001812128,
                    0.360830211552,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    -0.860001812128,
                    0.360830211552,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    0.860001812128,
                    -0.360830211552,
                    0.360830211552,
                    0.005443128475,
                ],
                [
                    -0.860001812128,
                    -0.360830211552,
                    0.360830211552,
                    0.005443128475,
                ],
                [0.64984861615, 0.64984861615, 0.394199888606, 0.006067027592],
                [
                    -0.64984861615,
                    0.64984861615,
                    0.394199888606,
                    0.006067027592,
                ],
                [
                    0.64984861615,
                    -0.64984861615,
                    0.394199888606,
                    0.006067027592,
                ],
                [
                    -0.64984861615,
                    -0.64984861615,
                    0.394199888606,
                    0.006067027592,
                ],
                [0.64984861615, 0.394199888606, 0.64984861615, 0.006067027592],
                [
                    -0.64984861615,
                    0.394199888606,
                    0.64984861615,
                    0.006067027592,
                ],
                [
                    0.64984861615,
                    -0.394199888606,
                    0.64984861615,
                    0.006067027592,
                ],
                [
                    -0.64984861615,
                    -0.394199888606,
                    0.64984861615,
                    0.006067027592,
                ],
                [0.394199888606, 0.64984861615, 0.64984861615, 0.006067027592],
                [
                    -0.394199888606,
                    0.64984861615,
                    0.64984861615,
                    0.006067027592,
                ],
                [
                    0.394199888606,
                    -0.64984861615,
                    0.64984861615,
                    0.006067027592,
                ],
                [
                    -0.394199888606,
                    -0.64984861615,
                    0.64984861615,
                    0.006067027592,
                ],
                [0.193294501323, 0.981140782843, 0.0, 0.003007949555],
                [-0.193294501323, 0.981140782843, 0.0, 0.003007949555],
                [0.193294501323, -0.981140782843, 0.0, 0.003007949555],
                [-0.193294501323, -0.981140782843, 0.0, 0.003007949555],
                [0.981140782843, 0.193294501323, 0.0, 0.003007949555],
                [-0.981140782843, 0.193294501323, 0.0, 0.003007949555],
                [0.981140782843, -0.193294501323, 0.0, 0.003007949555],
                [-0.981140782843, -0.193294501323, 0.0, 0.003007949555],
                [0.193294501323, 0.0, 0.981140782843, 0.00601589911],
                [-0.193294501323, 0.0, 0.981140782843, 0.00601589911],
                [0.981140782843, 0.0, 0.193294501323, 0.00601589911],
                [-0.981140782843, 0.0, 0.193294501323, 0.00601589911],
                [0.0, 0.193294501323, 0.981140782843, 0.00601589911],
                [0.0, -0.193294501323, 0.981140782843, 0.00601589911],
                [0.0, 0.981140782843, 0.193294501323, 0.00601589911],
                [0.0, -0.981140782843, 0.193294501323, 0.00601589911],
                [0.38004949199, 0.924966152699, 0.0, 0.002881964603],
                [-0.38004949199, 0.924966152699, 0.0, 0.002881964603],
                [0.38004949199, -0.924966152699, 0.0, 0.002881964603],
                [-0.38004949199, -0.924966152699, 0.0, 0.002881964603],
                [0.924966152699, 0.38004949199, 0.0, 0.002881964603],
                [-0.924966152699, 0.38004949199, 0.0, 0.002881964603],
                [0.924966152699, -0.38004949199, 0.0, 0.002881964603],
                [-0.924966152699, -0.38004949199, 0.0, 0.002881964603],
                [0.38004949199, 0.0, 0.924966152699, 0.005763929206],
                [-0.38004949199, 0.0, 0.924966152699, 0.005763929206],
                [0.924966152699, 0.0, 0.38004949199, 0.005763929206],
                [-0.924966152699, 0.0, 0.38004949199, 0.005763929206],
                [0.0, 0.38004949199, 0.924966152699, 0.005763929206],
                [0.0, -0.38004949199, 0.924966152699, 0.005763929206],
                [0.0, 0.924966152699, 0.38004949199, 0.005763929206],
                [0.0, -0.924966152699, 0.38004949199, 0.005763929206],
                [
                    0.28995588255,
                    0.793453785658,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    -0.28995588255,
                    0.793453785658,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    0.28995588255,
                    -0.793453785658,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    -0.28995588255,
                    -0.793453785658,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    0.28995588255,
                    0.535123047718,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    -0.28995588255,
                    0.535123047718,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    0.28995588255,
                    -0.535123047718,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    -0.28995588255,
                    -0.535123047718,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    0.793453785658,
                    0.28995588255,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    -0.793453785658,
                    0.28995588255,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    0.793453785658,
                    -0.28995588255,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    -0.793453785658,
                    -0.28995588255,
                    0.535123047718,
                    0.005916715253,
                ],
                [
                    0.793453785658,
                    0.535123047718,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    -0.793453785658,
                    0.535123047718,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    0.793453785658,
                    -0.535123047718,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    -0.793453785658,
                    -0.535123047718,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    0.535123047718,
                    0.28995588255,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    -0.535123047718,
                    0.28995588255,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    0.535123047718,
                    -0.28995588255,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    -0.535123047718,
                    -0.28995588255,
                    0.793453785658,
                    0.005916715253,
                ],
                [
                    0.535123047718,
                    0.793453785658,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    -0.535123047718,
                    0.793453785658,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    0.535123047718,
                    -0.793453785658,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    -0.535123047718,
                    -0.793453785658,
                    0.28995588255,
                    0.005916715253,
                ],
                [
                    0.096841214551,
                    0.828080150669,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    -0.096841214551,
                    0.828080150669,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    0.096841214551,
                    -0.828080150669,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    -0.096841214551,
                    -0.828080150669,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    0.096841214551,
                    0.552182074349,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    -0.096841214551,
                    0.552182074349,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    0.096841214551,
                    -0.552182074349,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    -0.096841214551,
                    -0.552182074349,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    0.828080150669,
                    0.096841214551,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    -0.828080150669,
                    0.096841214551,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    0.828080150669,
                    -0.096841214551,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    -0.828080150669,
                    -0.096841214551,
                    0.552182074349,
                    0.006072040053,
                ],
                [
                    0.828080150669,
                    0.552182074349,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    -0.828080150669,
                    0.552182074349,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    0.828080150669,
                    -0.552182074349,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    -0.828080150669,
                    -0.552182074349,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    0.552182074349,
                    0.096841214551,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    -0.552182074349,
                    0.096841214551,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    0.552182074349,
                    -0.096841214551,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    -0.552182074349,
                    -0.096841214551,
                    0.828080150669,
                    0.006072040053,
                ],
                [
                    0.552182074349,
                    0.828080150669,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    -0.552182074349,
                    0.828080150669,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    0.552182074349,
                    -0.828080150669,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    -0.552182074349,
                    -0.828080150669,
                    0.096841214551,
                    0.006072040053,
                ],
                [
                    0.183343464704,
                    0.907465826531,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    -0.183343464704,
                    0.907465826531,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    0.183343464704,
                    -0.907465826531,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    -0.183343464704,
                    -0.907465826531,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    0.183343464704,
                    0.378009189874,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    -0.183343464704,
                    0.378009189874,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    0.183343464704,
                    -0.378009189874,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    -0.183343464704,
                    -0.378009189874,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    0.907465826531,
                    0.183343464704,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    -0.907465826531,
                    0.183343464704,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    0.907465826531,
                    -0.183343464704,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    -0.907465826531,
                    -0.183343464704,
                    0.378009189874,
                    0.005664374808,
                ],
                [
                    0.907465826531,
                    0.378009189874,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    -0.907465826531,
                    0.378009189874,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    0.907465826531,
                    -0.378009189874,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    -0.907465826531,
                    -0.378009189874,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    0.378009189874,
                    0.183343464704,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    -0.378009189874,
                    0.183343464704,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    0.378009189874,
                    -0.183343464704,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    -0.378009189874,
                    -0.183343464704,
                    0.907465826531,
                    0.005664374808,
                ],
                [
                    0.378009189874,
                    0.907465826531,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    -0.378009189874,
                    0.907465826531,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    0.378009189874,
                    -0.907465826531,
                    0.183343464704,
                    0.005664374808,
                ],
                [
                    -0.378009189874,
                    -0.907465826531,
                    0.183343464704,
                    0.005664374808,
                ],
            ]
        )
        return grid_array

    if grid == 5:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.000526589797],
                [-1.0, 0.0, 0.0, 0.000526589797],
                [0.0, 1.0, 0.0, 0.000526589797],
                [0.0, -1.0, 0.0, 0.000526589797],
                [0.0, 0.0, 1.0, 0.001053179594],
                [0.0, 0.707106781187, 0.707106781187, 0.005096439944],
                [0.0, -0.707106781187, 0.707106781187, 0.005096439944],
                [0.707106781187, 0.0, 0.707106781187, 0.005096439944],
                [-0.707106781187, 0.0, 0.707106781187, 0.005096439944],
                [0.707106781187, 0.707106781187, 0.0, 0.002548219972],
                [-0.707106781187, 0.707106781187, 0.0, 0.002548219972],
                [0.707106781187, -0.707106781187, 0.0, 0.002548219972],
                [-0.707106781187, -0.707106781187, 0.0, 0.002548219972],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.005024634838],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.005024634838],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.005024634838],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.005024634838,
                ],
                [
                    0.690934630751,
                    0.690934630751,
                    0.212646824708,
                    0.005060807602,
                ],
                [
                    -0.690934630751,
                    0.690934630751,
                    0.212646824708,
                    0.005060807602,
                ],
                [
                    0.690934630751,
                    -0.690934630751,
                    0.212646824708,
                    0.005060807602,
                ],
                [
                    -0.690934630751,
                    -0.690934630751,
                    0.212646824708,
                    0.005060807602,
                ],
                [
                    0.690934630751,
                    0.212646824708,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    -0.690934630751,
                    0.212646824708,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    0.690934630751,
                    -0.212646824708,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    -0.690934630751,
                    -0.212646824708,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    0.212646824708,
                    0.690934630751,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    -0.212646824708,
                    0.690934630751,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    0.212646824708,
                    -0.690934630751,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    -0.212646824708,
                    -0.690934630751,
                    0.690934630751,
                    0.005060807602,
                ],
                [
                    0.177483605461,
                    0.177483605461,
                    0.967987158791,
                    0.004028558042,
                ],
                [
                    -0.177483605461,
                    0.177483605461,
                    0.967987158791,
                    0.004028558042,
                ],
                [
                    0.177483605461,
                    -0.177483605461,
                    0.967987158791,
                    0.004028558042,
                ],
                [
                    -0.177483605461,
                    -0.177483605461,
                    0.967987158791,
                    0.004028558042,
                ],
                [
                    0.177483605461,
                    0.967987158791,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    -0.177483605461,
                    0.967987158791,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    0.177483605461,
                    -0.967987158791,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    -0.177483605461,
                    -0.967987158791,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    0.967987158791,
                    0.177483605461,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    -0.967987158791,
                    0.177483605461,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    0.967987158791,
                    -0.177483605461,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    -0.967987158791,
                    -0.177483605461,
                    0.177483605461,
                    0.004028558042,
                ],
                [
                    0.491434263778,
                    0.491434263778,
                    0.719016501041,
                    0.005003450337,
                ],
                [
                    -0.491434263778,
                    0.491434263778,
                    0.719016501041,
                    0.005003450337,
                ],
                [
                    0.491434263778,
                    -0.491434263778,
                    0.719016501041,
                    0.005003450337,
                ],
                [
                    -0.491434263778,
                    -0.491434263778,
                    0.719016501041,
                    0.005003450337,
                ],
                [
                    0.491434263778,
                    0.719016501041,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    -0.491434263778,
                    0.719016501041,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    0.491434263778,
                    -0.719016501041,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    -0.491434263778,
                    -0.719016501041,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    0.719016501041,
                    0.491434263778,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    -0.719016501041,
                    0.491434263778,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    0.719016501041,
                    -0.491434263778,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    -0.719016501041,
                    -0.491434263778,
                    0.491434263778,
                    0.005003450337,
                ],
                [
                    0.645666470742,
                    0.645666470742,
                    0.407712664898,
                    0.005026534349,
                ],
                [
                    -0.645666470742,
                    0.645666470742,
                    0.407712664898,
                    0.005026534349,
                ],
                [
                    0.645666470742,
                    -0.645666470742,
                    0.407712664898,
                    0.005026534349,
                ],
                [
                    -0.645666470742,
                    -0.645666470742,
                    0.407712664898,
                    0.005026534349,
                ],
                [
                    0.645666470742,
                    0.407712664898,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    -0.645666470742,
                    0.407712664898,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    0.645666470742,
                    -0.407712664898,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    -0.645666470742,
                    -0.407712664898,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    0.407712664898,
                    0.645666470742,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    -0.407712664898,
                    0.645666470742,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    0.407712664898,
                    -0.645666470742,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    -0.407712664898,
                    -0.645666470742,
                    0.645666470742,
                    0.005026534349,
                ],
                [
                    0.286128901031,
                    0.286128901031,
                    0.914472801121,
                    0.004605389564,
                ],
                [
                    -0.286128901031,
                    0.286128901031,
                    0.914472801121,
                    0.004605389564,
                ],
                [
                    0.286128901031,
                    -0.286128901031,
                    0.914472801121,
                    0.004605389564,
                ],
                [
                    -0.286128901031,
                    -0.286128901031,
                    0.914472801121,
                    0.004605389564,
                ],
                [
                    0.286128901031,
                    0.914472801121,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    -0.286128901031,
                    0.914472801121,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    0.286128901031,
                    -0.914472801121,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    -0.286128901031,
                    -0.914472801121,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    0.914472801121,
                    0.286128901031,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    -0.914472801121,
                    0.286128901031,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    0.914472801121,
                    -0.286128901031,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    -0.914472801121,
                    -0.286128901031,
                    0.286128901031,
                    0.004605389564,
                ],
                [
                    0.075680843672,
                    0.075680843672,
                    0.994255912631,
                    0.002924991243,
                ],
                [
                    -0.075680843672,
                    0.075680843672,
                    0.994255912631,
                    0.002924991243,
                ],
                [
                    0.075680843672,
                    -0.075680843672,
                    0.994255912631,
                    0.002924991243,
                ],
                [
                    -0.075680843672,
                    -0.075680843672,
                    0.994255912631,
                    0.002924991243,
                ],
                [
                    0.075680843672,
                    0.994255912631,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    -0.075680843672,
                    0.994255912631,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    0.075680843672,
                    -0.994255912631,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    -0.075680843672,
                    -0.994255912631,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    0.994255912631,
                    0.075680843672,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    -0.994255912631,
                    0.075680843672,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    0.994255912631,
                    -0.075680843672,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    -0.994255912631,
                    -0.075680843672,
                    0.075680843672,
                    0.002924991243,
                ],
                [
                    0.392725976337,
                    0.392725976337,
                    0.831584400419,
                    0.004890746875,
                ],
                [
                    -0.392725976337,
                    0.392725976337,
                    0.831584400419,
                    0.004890746875,
                ],
                [
                    0.392725976337,
                    -0.392725976337,
                    0.831584400419,
                    0.004890746875,
                ],
                [
                    -0.392725976337,
                    -0.392725976337,
                    0.831584400419,
                    0.004890746875,
                ],
                [
                    0.392725976337,
                    0.831584400419,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    -0.392725976337,
                    0.831584400419,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    0.392725976337,
                    -0.831584400419,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    -0.392725976337,
                    -0.831584400419,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    0.831584400419,
                    0.392725976337,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    -0.831584400419,
                    0.392725976337,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    0.831584400419,
                    -0.392725976337,
                    0.392725976337,
                    0.004890746875,
                ],
                [
                    -0.831584400419,
                    -0.392725976337,
                    0.392725976337,
                    0.004890746875,
                ],
                [0.881813287779, 0.471598691151, 0.0, 0.002417442376],
                [-0.881813287779, 0.471598691151, 0.0, 0.002417442376],
                [0.881813287779, -0.471598691151, 0.0, 0.002417442376],
                [-0.881813287779, -0.471598691151, 0.0, 0.002417442376],
                [0.471598691151, 0.881813287779, 0.0, 0.002417442376],
                [-0.471598691151, 0.881813287779, 0.0, 0.002417442376],
                [0.471598691151, -0.881813287779, 0.0, 0.002417442376],
                [-0.471598691151, -0.881813287779, 0.0, 0.002417442376],
                [0.881813287779, 0.0, 0.471598691151, 0.004834884751],
                [-0.881813287779, 0.0, 0.471598691151, 0.004834884751],
                [0.471598691151, 0.0, 0.881813287779, 0.004834884751],
                [-0.471598691151, 0.0, 0.881813287779, 0.004834884751],
                [0.0, 0.881813287779, 0.471598691151, 0.004834884751],
                [0.0, -0.881813287779, 0.471598691151, 0.004834884751],
                [0.0, 0.471598691151, 0.881813287779, 0.004834884751],
                [0.0, -0.471598691151, 0.881813287779, 0.004834884751],
                [0.977642811118, 0.210272522857, 0.0, 0.001910951282],
                [-0.977642811118, 0.210272522857, 0.0, 0.001910951282],
                [0.977642811118, -0.210272522857, 0.0, 0.001910951282],
                [-0.977642811118, -0.210272522857, 0.0, 0.001910951282],
                [0.210272522857, 0.977642811118, 0.0, 0.001910951282],
                [-0.210272522857, 0.977642811118, 0.0, 0.001910951282],
                [0.210272522857, -0.977642811118, 0.0, 0.001910951282],
                [-0.210272522857, -0.977642811118, 0.0, 0.001910951282],
                [0.977642811118, 0.0, 0.210272522857, 0.003821902564],
                [-0.977642811118, 0.0, 0.210272522857, 0.003821902564],
                [0.210272522857, 0.0, 0.977642811118, 0.003821902564],
                [-0.210272522857, 0.0, 0.977642811118, 0.003821902564],
                [0.0, 0.977642811118, 0.210272522857, 0.003821902564],
                [0.0, -0.977642811118, 0.210272522857, 0.003821902564],
                [0.0, 0.210272522857, 0.977642811118, 0.003821902564],
                [0.0, -0.210272522857, 0.977642811118, 0.003821902564],
                [
                    0.20548236964,
                    0.868946032287,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    -0.20548236964,
                    0.868946032287,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    0.20548236964,
                    -0.868946032287,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    -0.20548236964,
                    -0.868946032287,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    0.20548236964,
                    0.450233038258,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    -0.20548236964,
                    0.450233038258,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    0.20548236964,
                    -0.450233038258,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    -0.20548236964,
                    -0.450233038258,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    0.868946032287,
                    0.20548236964,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    -0.868946032287,
                    0.20548236964,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    0.868946032287,
                    -0.20548236964,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    -0.868946032287,
                    -0.20548236964,
                    0.450233038258,
                    0.004833860089,
                ],
                [
                    0.868946032287,
                    0.450233038258,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    -0.868946032287,
                    0.450233038258,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    0.868946032287,
                    -0.450233038258,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    -0.868946032287,
                    -0.450233038258,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    0.450233038258,
                    0.20548236964,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    -0.450233038258,
                    0.20548236964,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    0.450233038258,
                    -0.20548236964,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    -0.450233038258,
                    -0.20548236964,
                    0.868946032287,
                    0.004833860089,
                ],
                [
                    0.450233038258,
                    0.868946032287,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    -0.450233038258,
                    0.868946032287,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    0.450233038258,
                    -0.868946032287,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    -0.450233038258,
                    -0.868946032287,
                    0.20548236964,
                    0.004833860089,
                ],
                [
                    0.590515704893,
                    0.799927854386,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    -0.590515704893,
                    0.799927854386,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    0.590515704893,
                    -0.799927854386,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    -0.590515704893,
                    -0.799927854386,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    0.590515704893,
                    0.106801826076,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    -0.590515704893,
                    0.106801826076,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    0.590515704893,
                    -0.106801826076,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    -0.590515704893,
                    -0.106801826076,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    0.799927854386,
                    0.590515704893,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    -0.799927854386,
                    0.590515704893,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    0.799927854386,
                    -0.590515704893,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    -0.799927854386,
                    -0.590515704893,
                    0.106801826076,
                    0.005024473709,
                ],
                [
                    0.799927854386,
                    0.106801826076,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    -0.799927854386,
                    0.106801826076,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    0.799927854386,
                    -0.106801826076,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    -0.799927854386,
                    -0.106801826076,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    0.106801826076,
                    0.590515704893,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    -0.106801826076,
                    0.590515704893,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    0.106801826076,
                    -0.590515704893,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    -0.106801826076,
                    -0.590515704893,
                    0.799927854386,
                    0.005024473709,
                ],
                [
                    0.106801826076,
                    0.799927854386,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    -0.106801826076,
                    0.799927854386,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    0.106801826076,
                    -0.799927854386,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    -0.106801826076,
                    -0.799927854386,
                    0.590515704893,
                    0.005024473709,
                ],
                [
                    0.555015236108,
                    0.771746262692,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    -0.555015236108,
                    0.771746262692,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    0.555015236108,
                    -0.771746262692,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    -0.555015236108,
                    -0.771746262692,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    0.555015236108,
                    0.310428403517,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    -0.555015236108,
                    0.310428403517,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    0.555015236108,
                    -0.310428403517,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    -0.555015236108,
                    -0.310428403517,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    0.771746262692,
                    0.555015236108,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    -0.771746262692,
                    0.555015236108,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    0.771746262692,
                    -0.555015236108,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    -0.771746262692,
                    -0.555015236108,
                    0.310428403517,
                    0.004993288109,
                ],
                [
                    0.771746262692,
                    0.310428403517,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    -0.771746262692,
                    0.310428403517,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    0.771746262692,
                    -0.310428403517,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    -0.771746262692,
                    -0.310428403517,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    0.310428403517,
                    0.555015236108,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    -0.310428403517,
                    0.555015236108,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    0.310428403517,
                    -0.555015236108,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    -0.310428403517,
                    -0.555015236108,
                    0.771746262692,
                    0.004993288109,
                ],
                [
                    0.310428403517,
                    0.771746262692,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    -0.310428403517,
                    0.771746262692,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    0.310428403517,
                    -0.771746262692,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    -0.310428403517,
                    -0.771746262692,
                    0.555015236108,
                    0.004993288109,
                ],
                [
                    0.937180985855,
                    0.334436314534,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    -0.937180985855,
                    0.334436314534,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    0.937180985855,
                    -0.334436314534,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    -0.937180985855,
                    -0.334436314534,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    0.937180985855,
                    0.099217696364,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    -0.937180985855,
                    0.099217696364,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    0.937180985855,
                    -0.099217696364,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    -0.937180985855,
                    -0.099217696364,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    0.334436314534,
                    0.937180985855,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    -0.334436314534,
                    0.937180985855,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    0.334436314534,
                    -0.937180985855,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    -0.334436314534,
                    -0.937180985855,
                    0.099217696364,
                    0.004473215521,
                ],
                [
                    0.334436314534,
                    0.099217696364,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    -0.334436314534,
                    0.099217696364,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    0.334436314534,
                    -0.099217696364,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    -0.334436314534,
                    -0.099217696364,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    0.099217696364,
                    0.937180985855,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    -0.099217696364,
                    0.937180985855,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    0.099217696364,
                    -0.937180985855,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    -0.099217696364,
                    -0.937180985855,
                    0.334436314534,
                    0.004473215521,
                ],
                [
                    0.099217696364,
                    0.334436314534,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    -0.099217696364,
                    0.334436314534,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    0.099217696364,
                    -0.334436314534,
                    0.937180985855,
                    0.004473215521,
                ],
                [
                    -0.099217696364,
                    -0.334436314534,
                    0.937180985855,
                    0.004473215521,
                ],
            ]
        )
        return grid_array

    if grid == 6:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.00030951213],
                [-1.0, 0.0, 0.0, 0.00030951213],
                [0.0, 1.0, 0.0, 0.00030951213],
                [0.0, -1.0, 0.0, 0.00030951213],
                [0.0, 0.0, 1.0, 0.000619024259],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.003704759397],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.003704759397],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.003704759397],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.003704759397,
                ],
                [
                    0.704095493823,
                    0.704095493823,
                    0.092190407077,
                    0.003743581279,
                ],
                [
                    -0.704095493823,
                    0.704095493823,
                    0.092190407077,
                    0.003743581279,
                ],
                [
                    0.704095493823,
                    -0.704095493823,
                    0.092190407077,
                    0.003743581279,
                ],
                [
                    -0.704095493823,
                    -0.704095493823,
                    0.092190407077,
                    0.003743581279,
                ],
                [
                    0.704095493823,
                    0.092190407077,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    -0.704095493823,
                    0.092190407077,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    0.704095493823,
                    -0.092190407077,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    -0.704095493823,
                    -0.092190407077,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    0.092190407077,
                    0.704095493823,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    -0.092190407077,
                    0.704095493823,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    0.092190407077,
                    -0.704095493823,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    -0.092190407077,
                    -0.704095493823,
                    0.704095493823,
                    0.003743581279,
                ],
                [
                    0.680774406646,
                    0.680774406646,
                    0.270356088359,
                    0.003717625171,
                ],
                [
                    -0.680774406646,
                    0.680774406646,
                    0.270356088359,
                    0.003717625171,
                ],
                [
                    0.680774406646,
                    -0.680774406646,
                    0.270356088359,
                    0.003717625171,
                ],
                [
                    -0.680774406646,
                    -0.680774406646,
                    0.270356088359,
                    0.003717625171,
                ],
                [
                    0.680774406646,
                    0.270356088359,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    -0.680774406646,
                    0.270356088359,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    0.680774406646,
                    -0.270356088359,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    -0.680774406646,
                    -0.270356088359,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    0.270356088359,
                    0.680774406646,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    -0.270356088359,
                    0.680774406646,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    0.270356088359,
                    -0.680774406646,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    -0.270356088359,
                    -0.680774406646,
                    0.680774406646,
                    0.003717625171,
                ],
                [
                    0.637254693926,
                    0.637254693926,
                    0.433373868777,
                    0.003704057657,
                ],
                [
                    -0.637254693926,
                    0.637254693926,
                    0.433373868777,
                    0.003704057657,
                ],
                [
                    0.637254693926,
                    -0.637254693926,
                    0.433373868777,
                    0.003704057657,
                ],
                [
                    -0.637254693926,
                    -0.637254693926,
                    0.433373868777,
                    0.003704057657,
                ],
                [
                    0.637254693926,
                    0.433373868777,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    -0.637254693926,
                    0.433373868777,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    0.637254693926,
                    -0.433373868777,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    -0.637254693926,
                    -0.433373868777,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    0.433373868777,
                    0.637254693926,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    -0.433373868777,
                    0.637254693926,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    0.433373868777,
                    -0.637254693926,
                    0.637254693926,
                    0.003704057657,
                ],
                [
                    -0.433373868777,
                    -0.637254693926,
                    0.637254693926,
                    0.003704057657,
                ],
                [0.50444197078, 0.50444197078, 0.700768575374, 0.003693431912],
                [
                    -0.50444197078,
                    0.50444197078,
                    0.700768575374,
                    0.003693431912,
                ],
                [
                    0.50444197078,
                    -0.50444197078,
                    0.700768575374,
                    0.003693431912,
                ],
                [
                    -0.50444197078,
                    -0.50444197078,
                    0.700768575374,
                    0.003693431912,
                ],
                [0.50444197078, 0.700768575374, 0.50444197078, 0.003693431912],
                [
                    -0.50444197078,
                    0.700768575374,
                    0.50444197078,
                    0.003693431912,
                ],
                [
                    0.50444197078,
                    -0.700768575374,
                    0.50444197078,
                    0.003693431912,
                ],
                [
                    -0.50444197078,
                    -0.700768575374,
                    0.50444197078,
                    0.003693431912,
                ],
                [0.700768575374, 0.50444197078, 0.50444197078, 0.003693431912],
                [
                    -0.700768575374,
                    0.50444197078,
                    0.50444197078,
                    0.003693431912,
                ],
                [
                    0.700768575374,
                    -0.50444197078,
                    0.50444197078,
                    0.003693431912,
                ],
                [
                    -0.700768575374,
                    -0.50444197078,
                    0.50444197078,
                    0.003693431912,
                ],
                [
                    0.421576178401,
                    0.421576178401,
                    0.802836877335,
                    0.003636943556,
                ],
                [
                    -0.421576178401,
                    0.421576178401,
                    0.802836877335,
                    0.003636943556,
                ],
                [
                    0.421576178401,
                    -0.421576178401,
                    0.802836877335,
                    0.003636943556,
                ],
                [
                    -0.421576178401,
                    -0.421576178401,
                    0.802836877335,
                    0.003636943556,
                ],
                [
                    0.421576178401,
                    0.802836877335,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    -0.421576178401,
                    0.802836877335,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    0.421576178401,
                    -0.802836877335,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    -0.421576178401,
                    -0.802836877335,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    0.802836877335,
                    0.421576178401,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    -0.802836877335,
                    0.421576178401,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    0.802836877335,
                    -0.421576178401,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    -0.802836877335,
                    -0.421576178401,
                    0.421576178401,
                    0.003636943556,
                ],
                [
                    0.331792073647,
                    0.331792073647,
                    0.883078727934,
                    0.003499129315,
                ],
                [
                    -0.331792073647,
                    0.331792073647,
                    0.883078727934,
                    0.003499129315,
                ],
                [
                    0.331792073647,
                    -0.331792073647,
                    0.883078727934,
                    0.003499129315,
                ],
                [
                    -0.331792073647,
                    -0.331792073647,
                    0.883078727934,
                    0.003499129315,
                ],
                [
                    0.331792073647,
                    0.883078727934,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    -0.331792073647,
                    0.883078727934,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    0.331792073647,
                    -0.883078727934,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    -0.331792073647,
                    -0.883078727934,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    0.883078727934,
                    0.331792073647,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    -0.883078727934,
                    0.331792073647,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    0.883078727934,
                    -0.331792073647,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    -0.883078727934,
                    -0.331792073647,
                    0.331792073647,
                    0.003499129315,
                ],
                [
                    0.238473670142,
                    0.238473670142,
                    0.94141415822,
                    0.003234421295,
                ],
                [
                    -0.238473670142,
                    0.238473670142,
                    0.94141415822,
                    0.003234421295,
                ],
                [
                    0.238473670142,
                    -0.238473670142,
                    0.94141415822,
                    0.003234421295,
                ],
                [
                    -0.238473670142,
                    -0.238473670142,
                    0.94141415822,
                    0.003234421295,
                ],
                [
                    0.238473670142,
                    0.94141415822,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    -0.238473670142,
                    0.94141415822,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    0.238473670142,
                    -0.94141415822,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    -0.238473670142,
                    -0.94141415822,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    0.94141415822,
                    0.238473670142,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    -0.94141415822,
                    0.238473670142,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    0.94141415822,
                    -0.238473670142,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    -0.94141415822,
                    -0.238473670142,
                    0.238473670142,
                    0.003234421295,
                ],
                [
                    0.145903644916,
                    0.145903644916,
                    0.978480583763,
                    0.00276947447,
                ],
                [
                    -0.145903644916,
                    0.145903644916,
                    0.978480583763,
                    0.00276947447,
                ],
                [
                    0.145903644916,
                    -0.145903644916,
                    0.978480583763,
                    0.00276947447,
                ],
                [
                    -0.145903644916,
                    -0.145903644916,
                    0.978480583763,
                    0.00276947447,
                ],
                [
                    0.145903644916,
                    0.978480583763,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    -0.145903644916,
                    0.978480583763,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    0.145903644916,
                    -0.978480583763,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    -0.145903644916,
                    -0.978480583763,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    0.978480583763,
                    0.145903644916,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    -0.978480583763,
                    0.145903644916,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    0.978480583763,
                    -0.145903644916,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    -0.978480583763,
                    -0.145903644916,
                    0.145903644916,
                    0.00276947447,
                ],
                [
                    0.060950341155,
                    0.060950341155,
                    0.996278129754,
                    0.001952866233,
                ],
                [
                    -0.060950341155,
                    0.060950341155,
                    0.996278129754,
                    0.001952866233,
                ],
                [
                    0.060950341155,
                    -0.060950341155,
                    0.996278129754,
                    0.001952866233,
                ],
                [
                    -0.060950341155,
                    -0.060950341155,
                    0.996278129754,
                    0.001952866233,
                ],
                [
                    0.060950341155,
                    0.996278129754,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    -0.060950341155,
                    0.996278129754,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    0.060950341155,
                    -0.996278129754,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    -0.060950341155,
                    -0.996278129754,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    0.996278129754,
                    0.060950341155,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    -0.996278129754,
                    0.060950341155,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    0.996278129754,
                    -0.060950341155,
                    0.060950341155,
                    0.001952866233,
                ],
                [
                    -0.996278129754,
                    -0.060950341155,
                    0.060950341155,
                    0.001952866233,
                ],
                [0.611684344201, 0.791101929627, 0.0, 0.001857161197],
                [-0.611684344201, 0.791101929627, 0.0, 0.001857161197],
                [0.611684344201, -0.791101929627, 0.0, 0.001857161197],
                [-0.611684344201, -0.791101929627, 0.0, 0.001857161197],
                [0.791101929627, 0.611684344201, 0.0, 0.001857161197],
                [-0.791101929627, 0.611684344201, 0.0, 0.001857161197],
                [0.791101929627, -0.611684344201, 0.0, 0.001857161197],
                [-0.791101929627, -0.611684344201, 0.0, 0.001857161197],
                [0.611684344201, 0.0, 0.791101929627, 0.003714322394],
                [-0.611684344201, 0.0, 0.791101929627, 0.003714322394],
                [0.791101929627, 0.0, 0.611684344201, 0.003714322394],
                [-0.791101929627, 0.0, 0.611684344201, 0.003714322394],
                [0.0, 0.611684344201, 0.791101929627, 0.003714322394],
                [0.0, -0.611684344201, 0.791101929627, 0.003714322394],
                [0.0, 0.791101929627, 0.611684344201, 0.003714322394],
                [0.0, -0.791101929627, 0.611684344201, 0.003714322394],
                [0.39647553482, 0.918045287711, 0.0, 0.001705153996],
                [-0.39647553482, 0.918045287711, 0.0, 0.001705153996],
                [0.39647553482, -0.918045287711, 0.0, 0.001705153996],
                [-0.39647553482, -0.918045287711, 0.0, 0.001705153996],
                [0.918045287711, 0.39647553482, 0.0, 0.001705153996],
                [-0.918045287711, 0.39647553482, 0.0, 0.001705153996],
                [0.918045287711, -0.39647553482, 0.0, 0.001705153996],
                [-0.918045287711, -0.39647553482, 0.0, 0.001705153996],
                [0.39647553482, 0.0, 0.918045287711, 0.003410307993],
                [-0.39647553482, 0.0, 0.918045287711, 0.003410307993],
                [0.918045287711, 0.0, 0.39647553482, 0.003410307993],
                [-0.918045287711, 0.0, 0.39647553482, 0.003410307993],
                [0.0, 0.39647553482, 0.918045287711, 0.003410307993],
                [0.0, -0.39647553482, 0.918045287711, 0.003410307993],
                [0.0, 0.918045287711, 0.39647553482, 0.003410307993],
                [0.0, -0.918045287711, 0.39647553482, 0.003410307993],
                [0.172478200991, 0.985013335028, 0.0, 0.001300321686],
                [-0.172478200991, 0.985013335028, 0.0, 0.001300321686],
                [0.172478200991, -0.985013335028, 0.0, 0.001300321686],
                [-0.172478200991, -0.985013335028, 0.0, 0.001300321686],
                [0.985013335028, 0.172478200991, 0.0, 0.001300321686],
                [-0.985013335028, 0.172478200991, 0.0, 0.001300321686],
                [0.985013335028, -0.172478200991, 0.0, 0.001300321686],
                [-0.985013335028, -0.172478200991, 0.0, 0.001300321686],
                [0.172478200991, 0.0, 0.985013335028, 0.002600643372],
                [-0.172478200991, 0.0, 0.985013335028, 0.002600643372],
                [0.985013335028, 0.0, 0.172478200991, 0.002600643372],
                [-0.985013335028, 0.0, 0.172478200991, 0.002600643372],
                [0.0, 0.172478200991, 0.985013335028, 0.002600643372],
                [0.0, -0.172478200991, 0.985013335028, 0.002600643372],
                [0.0, 0.985013335028, 0.172478200991, 0.002600643372],
                [0.0, -0.985013335028, 0.172478200991, 0.002600643372],
                [
                    0.561026380862,
                    0.351828092773,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    -0.561026380862,
                    0.351828092773,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    0.561026380862,
                    -0.351828092773,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    -0.561026380862,
                    -0.351828092773,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    0.561026380862,
                    0.749310611904,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    -0.561026380862,
                    0.749310611904,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    0.561026380862,
                    -0.749310611904,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    -0.561026380862,
                    -0.749310611904,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    0.351828092773,
                    0.561026380862,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    -0.351828092773,
                    0.561026380862,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    0.351828092773,
                    -0.561026380862,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    -0.351828092773,
                    -0.561026380862,
                    0.749310611904,
                    0.003685732946,
                ],
                [
                    0.351828092773,
                    0.749310611904,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    -0.351828092773,
                    0.749310611904,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    0.351828092773,
                    -0.749310611904,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    -0.351828092773,
                    -0.749310611904,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    0.749310611904,
                    0.561026380862,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    -0.749310611904,
                    0.561026380862,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    0.749310611904,
                    -0.561026380862,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    -0.749310611904,
                    -0.561026380862,
                    0.351828092773,
                    0.003685732946,
                ],
                [
                    0.749310611904,
                    0.351828092773,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    -0.749310611904,
                    0.351828092773,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    0.749310611904,
                    -0.351828092773,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    -0.749310611904,
                    -0.351828092773,
                    0.561026380862,
                    0.003685732946,
                ],
                [
                    0.474239284255,
                    0.263471665594,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    -0.474239284255,
                    0.263471665594,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    0.474239284255,
                    -0.263471665594,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    -0.474239284255,
                    -0.263471665594,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    0.474239284255,
                    0.840047488359,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    -0.474239284255,
                    0.840047488359,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    0.474239284255,
                    -0.840047488359,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    -0.474239284255,
                    -0.840047488359,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    0.263471665594,
                    0.474239284255,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    -0.263471665594,
                    0.474239284255,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    0.263471665594,
                    -0.474239284255,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    -0.263471665594,
                    -0.474239284255,
                    0.840047488359,
                    0.003605317869,
                ],
                [
                    0.263471665594,
                    0.840047488359,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    -0.263471665594,
                    0.840047488359,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    0.263471665594,
                    -0.840047488359,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    -0.263471665594,
                    -0.840047488359,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    0.840047488359,
                    0.474239284255,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    -0.840047488359,
                    0.474239284255,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    0.840047488359,
                    -0.474239284255,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    -0.840047488359,
                    -0.474239284255,
                    0.263471665594,
                    0.003605317869,
                ],
                [
                    0.840047488359,
                    0.263471665594,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    -0.840047488359,
                    0.263471665594,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    0.840047488359,
                    -0.263471665594,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    -0.840047488359,
                    -0.263471665594,
                    0.474239284255,
                    0.003605317869,
                ],
                [
                    0.598412649789,
                    0.181664084036,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    -0.598412649789,
                    0.181664084036,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    0.598412649789,
                    -0.181664084036,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    -0.598412649789,
                    -0.181664084036,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    0.598412649789,
                    0.78032074248,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    -0.598412649789,
                    0.78032074248,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    0.598412649789,
                    -0.78032074248,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    -0.598412649789,
                    -0.78032074248,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    0.181664084036,
                    0.598412649789,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    -0.181664084036,
                    0.598412649789,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    0.181664084036,
                    -0.598412649789,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    -0.181664084036,
                    -0.598412649789,
                    0.78032074248,
                    0.003699661121,
                ],
                [
                    0.181664084036,
                    0.78032074248,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    -0.181664084036,
                    0.78032074248,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    0.181664084036,
                    -0.78032074248,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    -0.181664084036,
                    -0.78032074248,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    0.78032074248,
                    0.598412649789,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    -0.78032074248,
                    0.598412649789,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    0.78032074248,
                    -0.598412649789,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    -0.78032074248,
                    -0.598412649789,
                    0.181664084036,
                    0.003699661121,
                ],
                [
                    0.78032074248,
                    0.181664084036,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    -0.78032074248,
                    0.181664084036,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    0.78032074248,
                    -0.181664084036,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    -0.78032074248,
                    -0.181664084036,
                    0.598412649789,
                    0.003699661121,
                ],
                [
                    0.37910354077,
                    0.172079522566,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    -0.37910354077,
                    0.172079522566,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    0.37910354077,
                    -0.172079522566,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    -0.37910354077,
                    -0.172079522566,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    0.37910354077,
                    0.909213475092,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    -0.37910354077,
                    0.909213475092,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    0.37910354077,
                    -0.909213475092,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    -0.37910354077,
                    -0.909213475092,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    0.172079522566,
                    0.37910354077,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    -0.172079522566,
                    0.37910354077,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    0.172079522566,
                    -0.37910354077,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    -0.172079522566,
                    -0.37910354077,
                    0.909213475092,
                    0.003427809014,
                ],
                [
                    0.172079522566,
                    0.909213475092,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    -0.172079522566,
                    0.909213475092,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    0.172079522566,
                    -0.909213475092,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    -0.172079522566,
                    -0.909213475092,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    0.909213475092,
                    0.37910354077,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    -0.909213475092,
                    0.37910354077,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    0.909213475092,
                    -0.37910354077,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    -0.909213475092,
                    -0.37910354077,
                    0.172079522566,
                    0.003427809014,
                ],
                [
                    0.909213475092,
                    0.172079522566,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    -0.909213475092,
                    0.172079522566,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    0.909213475092,
                    -0.172079522566,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    -0.909213475092,
                    -0.172079522566,
                    0.37910354077,
                    0.003427809014,
                ],
                [
                    0.277867319059,
                    0.082130215819,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    -0.277867319059,
                    0.082130215819,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    0.277867319059,
                    -0.082130215819,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    -0.277867319059,
                    -0.082130215819,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    0.277867319059,
                    0.95710207431,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    -0.277867319059,
                    0.95710207431,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    0.277867319059,
                    -0.95710207431,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    -0.277867319059,
                    -0.95710207431,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    0.082130215819,
                    0.277867319059,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    -0.082130215819,
                    0.277867319059,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    0.082130215819,
                    -0.277867319059,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    -0.082130215819,
                    -0.277867319059,
                    0.95710207431,
                    0.003110427207,
                ],
                [
                    0.082130215819,
                    0.95710207431,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    -0.082130215819,
                    0.95710207431,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    0.082130215819,
                    -0.95710207431,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    -0.082130215819,
                    -0.95710207431,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    0.95710207431,
                    0.277867319059,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    -0.95710207431,
                    0.277867319059,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    0.95710207431,
                    -0.277867319059,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    -0.95710207431,
                    -0.277867319059,
                    0.082130215819,
                    0.003110427207,
                ],
                [
                    0.95710207431,
                    0.082130215819,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    -0.95710207431,
                    0.082130215819,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    0.95710207431,
                    -0.082130215819,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    -0.95710207431,
                    -0.082130215819,
                    0.277867319059,
                    0.003110427207,
                ],
                [
                    0.503356427108,
                    0.089992058421,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    -0.503356427108,
                    0.089992058421,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    0.503356427108,
                    -0.089992058421,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    -0.503356427108,
                    -0.089992058421,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    0.503356427108,
                    0.859379855891,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    -0.503356427108,
                    0.859379855891,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    0.503356427108,
                    -0.859379855891,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    -0.503356427108,
                    -0.859379855891,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    0.089992058421,
                    0.503356427108,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    -0.089992058421,
                    0.503356427108,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    0.089992058421,
                    -0.503356427108,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    -0.089992058421,
                    -0.503356427108,
                    0.859379855891,
                    0.003604478256,
                ],
                [
                    0.089992058421,
                    0.859379855891,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    -0.089992058421,
                    0.859379855891,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    0.089992058421,
                    -0.859379855891,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    -0.089992058421,
                    -0.859379855891,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    0.859379855891,
                    0.503356427108,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    -0.859379855891,
                    0.503356427108,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    0.859379855891,
                    -0.503356427108,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    -0.859379855891,
                    -0.503356427108,
                    0.089992058421,
                    0.003604478256,
                ],
                [
                    0.859379855891,
                    0.089992058421,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    -0.859379855891,
                    0.089992058421,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    0.859379855891,
                    -0.089992058421,
                    0.503356427108,
                    0.003604478256,
                ],
                [
                    -0.859379855891,
                    -0.089992058421,
                    0.503356427108,
                    0.003604478256,
                ],
            ]
        )
        return grid_array

    if grid == 7:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.000219294209],
                [-1.0, 0.0, 0.0, 0.000219294209],
                [0.0, 1.0, 0.0, 0.000219294209],
                [0.0, -1.0, 0.0, 0.000219294209],
                [0.0, 0.0, 1.0, 0.000438588418],
                [0.0, 0.707106781187, 0.707106781187, 0.002872867235],
                [0.0, -0.707106781187, 0.707106781187, 0.002872867235],
                [0.707106781187, 0.0, 0.707106781187, 0.002872867235],
                [-0.707106781187, 0.0, 0.707106781187, 0.002872867235],
                [0.707106781187, 0.707106781187, 0.0, 0.001436433617],
                [-0.707106781187, 0.707106781187, 0.0, 0.001436433617],
                [0.707106781187, -0.707106781187, 0.0, 0.001436433617],
                [-0.707106781187, -0.707106781187, 0.0, 0.001436433617],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.002843880689],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.002843880689],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.002843880689],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.002843880689,
                ],
                [
                    0.050872044105,
                    0.050872044105,
                    0.997408677653,
                    0.001359624702,
                ],
                [
                    -0.050872044105,
                    0.050872044105,
                    0.997408677653,
                    0.001359624702,
                ],
                [
                    0.050872044105,
                    -0.050872044105,
                    0.997408677653,
                    0.001359624702,
                ],
                [
                    -0.050872044105,
                    -0.050872044105,
                    0.997408677653,
                    0.001359624702,
                ],
                [
                    0.050872044105,
                    0.997408677653,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    -0.050872044105,
                    0.997408677653,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    0.050872044105,
                    -0.997408677653,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    -0.050872044105,
                    -0.997408677653,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    0.997408677653,
                    0.050872044105,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    -0.997408677653,
                    0.050872044105,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    0.997408677653,
                    -0.050872044105,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    -0.997408677653,
                    -0.050872044105,
                    0.050872044105,
                    0.001359624702,
                ],
                [
                    0.122819879018,
                    0.122819879018,
                    0.984799753572,
                    0.001982636847,
                ],
                [
                    -0.122819879018,
                    0.122819879018,
                    0.984799753572,
                    0.001982636847,
                ],
                [
                    0.122819879018,
                    -0.122819879018,
                    0.984799753572,
                    0.001982636847,
                ],
                [
                    -0.122819879018,
                    -0.122819879018,
                    0.984799753572,
                    0.001982636847,
                ],
                [
                    0.122819879018,
                    0.984799753572,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    -0.122819879018,
                    0.984799753572,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    0.122819879018,
                    -0.984799753572,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    -0.122819879018,
                    -0.984799753572,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    0.984799753572,
                    0.122819879018,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    -0.984799753572,
                    0.122819879018,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    0.984799753572,
                    -0.122819879018,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    -0.984799753572,
                    -0.122819879018,
                    0.122819879018,
                    0.001982636847,
                ],
                [
                    0.202689081441,
                    0.202689081441,
                    0.958036675983,
                    0.002360415666,
                ],
                [
                    -0.202689081441,
                    0.202689081441,
                    0.958036675983,
                    0.002360415666,
                ],
                [
                    0.202689081441,
                    -0.202689081441,
                    0.958036675983,
                    0.002360415666,
                ],
                [
                    -0.202689081441,
                    -0.202689081441,
                    0.958036675983,
                    0.002360415666,
                ],
                [
                    0.202689081441,
                    0.958036675983,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    -0.202689081441,
                    0.958036675983,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    0.202689081441,
                    -0.958036675983,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    -0.202689081441,
                    -0.958036675983,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    0.958036675983,
                    0.202689081441,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    -0.958036675983,
                    0.202689081441,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    0.958036675983,
                    -0.202689081441,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    -0.958036675983,
                    -0.202689081441,
                    0.202689081441,
                    0.002360415666,
                ],
                [
                    0.284774515646,
                    0.284774515646,
                    0.915317950483,
                    0.002593199204,
                ],
                [
                    -0.284774515646,
                    0.284774515646,
                    0.915317950483,
                    0.002593199204,
                ],
                [
                    0.284774515646,
                    -0.284774515646,
                    0.915317950483,
                    0.002593199204,
                ],
                [
                    -0.284774515646,
                    -0.284774515646,
                    0.915317950483,
                    0.002593199204,
                ],
                [
                    0.284774515646,
                    0.915317950483,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    -0.284774515646,
                    0.915317950483,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    0.284774515646,
                    -0.915317950483,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    -0.284774515646,
                    -0.915317950483,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    0.915317950483,
                    0.284774515646,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    -0.915317950483,
                    0.284774515646,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    0.915317950483,
                    -0.284774515646,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    -0.915317950483,
                    -0.284774515646,
                    0.284774515646,
                    0.002593199204,
                ],
                [
                    0.365671907898,
                    0.365671907898,
                    0.855901928698,
                    0.002731742855,
                ],
                [
                    -0.365671907898,
                    0.365671907898,
                    0.855901928698,
                    0.002731742855,
                ],
                [
                    0.365671907898,
                    -0.365671907898,
                    0.855901928698,
                    0.002731742855,
                ],
                [
                    -0.365671907898,
                    -0.365671907898,
                    0.855901928698,
                    0.002731742855,
                ],
                [
                    0.365671907898,
                    0.855901928698,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    -0.365671907898,
                    0.855901928698,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    0.365671907898,
                    -0.855901928698,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    -0.365671907898,
                    -0.855901928698,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    0.855901928698,
                    0.365671907898,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    -0.855901928698,
                    0.365671907898,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    0.855901928698,
                    -0.365671907898,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    -0.855901928698,
                    -0.365671907898,
                    0.365671907898,
                    0.002731742855,
                ],
                [
                    0.442826488671,
                    0.442826488671,
                    0.779621319528,
                    0.00280597721,
                ],
                [
                    -0.442826488671,
                    0.442826488671,
                    0.779621319528,
                    0.00280597721,
                ],
                [
                    0.442826488671,
                    -0.442826488671,
                    0.779621319528,
                    0.00280597721,
                ],
                [
                    -0.442826488671,
                    -0.442826488671,
                    0.779621319528,
                    0.00280597721,
                ],
                [
                    0.442826488671,
                    0.779621319528,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    -0.442826488671,
                    0.779621319528,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    0.442826488671,
                    -0.779621319528,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    -0.442826488671,
                    -0.779621319528,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    0.779621319528,
                    0.442826488671,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    -0.779621319528,
                    0.442826488671,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    0.779621319528,
                    -0.442826488671,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    -0.779621319528,
                    -0.442826488671,
                    0.442826488671,
                    0.00280597721,
                ],
                [
                    0.514061962725,
                    0.514061962725,
                    0.686644447264,
                    0.002837291127,
                ],
                [
                    -0.514061962725,
                    0.514061962725,
                    0.686644447264,
                    0.002837291127,
                ],
                [
                    0.514061962725,
                    -0.514061962725,
                    0.686644447264,
                    0.002837291127,
                ],
                [
                    -0.514061962725,
                    -0.514061962725,
                    0.686644447264,
                    0.002837291127,
                ],
                [
                    0.514061962725,
                    0.686644447264,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    -0.514061962725,
                    0.686644447264,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    0.514061962725,
                    -0.686644447264,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    -0.514061962725,
                    -0.686644447264,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    0.686644447264,
                    0.514061962725,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    -0.686644447264,
                    0.514061962725,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    0.686644447264,
                    -0.514061962725,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    -0.686644447264,
                    -0.514061962725,
                    0.514061962725,
                    0.002837291127,
                ],
                [
                    0.630640121917,
                    0.630640121917,
                    0.452311920314,
                    0.002842753484,
                ],
                [
                    -0.630640121917,
                    0.630640121917,
                    0.452311920314,
                    0.002842753484,
                ],
                [
                    0.630640121917,
                    -0.630640121917,
                    0.452311920314,
                    0.002842753484,
                ],
                [
                    -0.630640121917,
                    -0.630640121917,
                    0.452311920314,
                    0.002842753484,
                ],
                [
                    0.630640121917,
                    0.452311920314,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    -0.630640121917,
                    0.452311920314,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    0.630640121917,
                    -0.452311920314,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    -0.630640121917,
                    -0.452311920314,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    0.452311920314,
                    0.630640121917,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    -0.452311920314,
                    0.630640121917,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    0.452311920314,
                    -0.630640121917,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    -0.452311920314,
                    -0.630640121917,
                    0.630640121917,
                    0.002842753484,
                ],
                [
                    0.671688333202,
                    0.671688333202,
                    0.312521305002,
                    0.002847992951,
                ],
                [
                    -0.671688333202,
                    0.671688333202,
                    0.312521305002,
                    0.002847992951,
                ],
                [
                    0.671688333202,
                    -0.671688333202,
                    0.312521305002,
                    0.002847992951,
                ],
                [
                    -0.671688333202,
                    -0.671688333202,
                    0.312521305002,
                    0.002847992951,
                ],
                [
                    0.671688333202,
                    0.312521305002,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    -0.671688333202,
                    0.312521305002,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    0.671688333202,
                    -0.312521305002,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    -0.671688333202,
                    -0.312521305002,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    0.312521305002,
                    0.671688333202,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    -0.312521305002,
                    0.671688333202,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    0.312521305002,
                    -0.671688333202,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    -0.312521305002,
                    -0.671688333202,
                    0.671688333202,
                    0.002847992951,
                ],
                [
                    0.697979268534,
                    0.697979268534,
                    0.160155803499,
                    0.002863108084,
                ],
                [
                    -0.697979268534,
                    0.697979268534,
                    0.160155803499,
                    0.002863108084,
                ],
                [
                    0.697979268534,
                    -0.697979268534,
                    0.160155803499,
                    0.002863108084,
                ],
                [
                    -0.697979268534,
                    -0.697979268534,
                    0.160155803499,
                    0.002863108084,
                ],
                [
                    0.697979268534,
                    0.160155803499,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    -0.697979268534,
                    0.160155803499,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    0.697979268534,
                    -0.160155803499,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    -0.697979268534,
                    -0.160155803499,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    0.160155803499,
                    0.697979268534,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    -0.160155803499,
                    0.697979268534,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    0.160155803499,
                    -0.697979268534,
                    0.697979268534,
                    0.002863108084,
                ],
                [
                    -0.160155803499,
                    -0.697979268534,
                    0.697979268534,
                    0.002863108084,
                ],
                [0.14468656742, 0.989477537496, 0.0, 0.00092544015],
                [-0.14468656742, 0.989477537496, 0.0, 0.00092544015],
                [0.14468656742, -0.989477537496, 0.0, 0.00092544015],
                [-0.14468656742, -0.989477537496, 0.0, 0.00092544015],
                [0.989477537496, 0.14468656742, 0.0, 0.00092544015],
                [-0.989477537496, 0.14468656742, 0.0, 0.00092544015],
                [0.989477537496, -0.14468656742, 0.0, 0.00092544015],
                [-0.989477537496, -0.14468656742, 0.0, 0.00092544015],
                [0.14468656742, 0.0, 0.989477537496, 0.0018508803],
                [-0.14468656742, 0.0, 0.989477537496, 0.0018508803],
                [0.989477537496, 0.0, 0.14468656742, 0.0018508803],
                [-0.989477537496, 0.0, 0.14468656742, 0.0018508803],
                [0.0, 0.14468656742, 0.989477537496, 0.0018508803],
                [0.0, -0.14468656742, 0.989477537496, 0.0018508803],
                [0.0, 0.989477537496, 0.14468656742, 0.0018508803],
                [0.0, -0.989477537496, 0.14468656742, 0.0018508803],
                [0.339026347541, 0.940776878794, 0.0, 0.001250239995],
                [-0.339026347541, 0.940776878794, 0.0, 0.001250239995],
                [0.339026347541, -0.940776878794, 0.0, 0.001250239995],
                [-0.339026347541, -0.940776878794, 0.0, 0.001250239995],
                [0.940776878794, 0.339026347541, 0.0, 0.001250239995],
                [-0.940776878794, 0.339026347541, 0.0, 0.001250239995],
                [0.940776878794, -0.339026347541, 0.0, 0.001250239995],
                [-0.940776878794, -0.339026347541, 0.0, 0.001250239995],
                [0.339026347541, 0.0, 0.940776878794, 0.00250047999],
                [-0.339026347541, 0.0, 0.940776878794, 0.00250047999],
                [0.940776878794, 0.0, 0.339026347541, 0.00250047999],
                [-0.940776878794, 0.0, 0.339026347541, 0.00250047999],
                [0.0, 0.339026347541, 0.940776878794, 0.00250047999],
                [0.0, -0.339026347541, 0.940776878794, 0.00250047999],
                [0.0, 0.940776878794, 0.339026347541, 0.00250047999],
                [0.0, -0.940776878794, 0.339026347541, 0.00250047999],
                [0.533580465126, 0.845749305194, 0.0, 0.001394365843],
                [-0.533580465126, 0.845749305194, 0.0, 0.001394365843],
                [0.533580465126, -0.845749305194, 0.0, 0.001394365843],
                [-0.533580465126, -0.845749305194, 0.0, 0.001394365843],
                [0.845749305194, 0.533580465126, 0.0, 0.001394365843],
                [-0.845749305194, 0.533580465126, 0.0, 0.001394365843],
                [0.845749305194, -0.533580465126, 0.0, 0.001394365843],
                [-0.845749305194, -0.533580465126, 0.0, 0.001394365843],
                [0.533580465126, 0.0, 0.845749305194, 0.002788731687],
                [-0.533580465126, 0.0, 0.845749305194, 0.002788731687],
                [0.845749305194, 0.0, 0.533580465126, 0.002788731687],
                [-0.845749305194, 0.0, 0.533580465126, 0.002788731687],
                [0.0, 0.533580465126, 0.845749305194, 0.002788731687],
                [0.0, -0.533580465126, 0.845749305194, 0.002788731687],
                [0.0, 0.845749305194, 0.533580465126, 0.002788731687],
                [0.0, -0.845749305194, 0.533580465126, 0.002788731687],
                [
                    0.069440243933,
                    0.235518789424,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    -0.069440243933,
                    0.235518789424,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    0.069440243933,
                    -0.235518789424,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    -0.069440243933,
                    -0.235518789424,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    0.069440243933,
                    0.969385863498,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    -0.069440243933,
                    0.969385863498,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    0.069440243933,
                    -0.969385863498,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    -0.069440243933,
                    -0.969385863498,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    0.235518789424,
                    0.069440243933,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    -0.235518789424,
                    0.069440243933,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    0.235518789424,
                    -0.069440243933,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    -0.235518789424,
                    -0.069440243933,
                    0.969385863498,
                    0.002254178189,
                ],
                [
                    0.235518789424,
                    0.969385863498,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    -0.235518789424,
                    0.969385863498,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    0.235518789424,
                    -0.969385863498,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    -0.235518789424,
                    -0.969385863498,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    0.969385863498,
                    0.069440243933,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    -0.969385863498,
                    0.069440243933,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    0.969385863498,
                    -0.069440243933,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    -0.969385863498,
                    -0.069440243933,
                    0.235518789424,
                    0.002254178189,
                ],
                [
                    0.969385863498,
                    0.235518789424,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    -0.969385863498,
                    0.235518789424,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    0.969385863498,
                    -0.235518789424,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    -0.969385863498,
                    -0.235518789424,
                    0.069440243933,
                    0.002254178189,
                ],
                [
                    0.226900410953,
                    0.410218247405,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    -0.226900410953,
                    0.410218247405,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    0.226900410953,
                    -0.410218247405,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    -0.226900410953,
                    -0.410218247405,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    0.226900410953,
                    0.883310360522,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    -0.226900410953,
                    0.883310360522,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    0.226900410953,
                    -0.883310360522,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    -0.226900410953,
                    -0.883310360522,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    0.410218247405,
                    0.226900410953,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    -0.410218247405,
                    0.226900410953,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    0.410218247405,
                    -0.226900410953,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    -0.410218247405,
                    -0.226900410953,
                    0.883310360522,
                    0.002691507522,
                ],
                [
                    0.410218247405,
                    0.883310360522,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    -0.410218247405,
                    0.883310360522,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    0.410218247405,
                    -0.883310360522,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    -0.410218247405,
                    -0.883310360522,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    0.883310360522,
                    0.226900410953,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    -0.883310360522,
                    0.226900410953,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    0.883310360522,
                    -0.226900410953,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    -0.883310360522,
                    -0.226900410953,
                    0.410218247405,
                    0.002691507522,
                ],
                [
                    0.883310360522,
                    0.410218247405,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    -0.883310360522,
                    0.410218247405,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    0.883310360522,
                    -0.410218247405,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    -0.883310360522,
                    -0.410218247405,
                    0.226900410953,
                    0.002691507522,
                ],
                [
                    0.080255746078,
                    0.621430241748,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    -0.080255746078,
                    0.621430241748,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    0.080255746078,
                    -0.621430241748,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    -0.080255746078,
                    -0.621430241748,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    0.080255746078,
                    0.779348105703,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    -0.080255746078,
                    0.779348105703,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    0.080255746078,
                    -0.779348105703,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    -0.080255746078,
                    -0.779348105703,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    0.621430241748,
                    0.080255746078,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    -0.621430241748,
                    0.080255746078,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    0.621430241748,
                    -0.080255746078,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    -0.621430241748,
                    -0.080255746078,
                    0.779348105703,
                    0.002849914567,
                ],
                [
                    0.621430241748,
                    0.779348105703,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    -0.621430241748,
                    0.779348105703,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    0.621430241748,
                    -0.779348105703,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    -0.621430241748,
                    -0.779348105703,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    0.779348105703,
                    0.080255746078,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    -0.779348105703,
                    0.080255746078,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    0.779348105703,
                    -0.080255746078,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    -0.779348105703,
                    -0.080255746078,
                    0.621430241748,
                    0.002849914567,
                ],
                [
                    0.779348105703,
                    0.621430241748,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    -0.779348105703,
                    0.621430241748,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    0.779348105703,
                    -0.621430241748,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    -0.779348105703,
                    -0.621430241748,
                    0.080255746078,
                    0.002849914567,
                ],
                [
                    0.14679995279,
                    0.324528434572,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    -0.14679995279,
                    0.324528434572,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    0.14679995279,
                    -0.324528434572,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    -0.14679995279,
                    -0.324528434572,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    0.14679995279,
                    0.934414827052,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    -0.14679995279,
                    0.934414827052,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    0.14679995279,
                    -0.934414827052,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    -0.14679995279,
                    -0.934414827052,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    0.324528434572,
                    0.14679995279,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    -0.324528434572,
                    0.14679995279,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    0.324528434572,
                    -0.14679995279,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    -0.324528434572,
                    -0.14679995279,
                    0.934414827052,
                    0.002523046682,
                ],
                [
                    0.324528434572,
                    0.934414827052,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    -0.324528434572,
                    0.934414827052,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    0.324528434572,
                    -0.934414827052,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    -0.324528434572,
                    -0.934414827052,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    0.934414827052,
                    0.14679995279,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    -0.934414827052,
                    0.14679995279,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    0.934414827052,
                    -0.14679995279,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    -0.934414827052,
                    -0.14679995279,
                    0.324528434572,
                    0.002523046682,
                ],
                [
                    0.934414827052,
                    0.324528434572,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    -0.934414827052,
                    0.324528434572,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    0.934414827052,
                    -0.324528434572,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    -0.934414827052,
                    -0.324528434572,
                    0.14679995279,
                    0.002523046682,
                ],
                [
                    0.157150776982,
                    0.52244821897,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    -0.157150776982,
                    0.52244821897,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    0.157150776982,
                    -0.52244821897,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    -0.157150776982,
                    -0.52244821897,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    0.157150776982,
                    0.838064133458,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    -0.157150776982,
                    0.838064133458,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    0.157150776982,
                    -0.838064133458,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    -0.157150776982,
                    -0.838064133458,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    0.52244821897,
                    0.157150776982,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    -0.52244821897,
                    0.157150776982,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    0.52244821897,
                    -0.157150776982,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    -0.52244821897,
                    -0.157150776982,
                    0.838064133458,
                    0.002785094212,
                ],
                [
                    0.52244821897,
                    0.838064133458,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    -0.52244821897,
                    0.838064133458,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    0.52244821897,
                    -0.838064133458,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    -0.52244821897,
                    -0.838064133458,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    0.838064133458,
                    0.157150776982,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    -0.838064133458,
                    0.157150776982,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    0.838064133458,
                    -0.157150776982,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    -0.838064133458,
                    -0.157150776982,
                    0.52244821897,
                    0.002785094212,
                ],
                [
                    0.838064133458,
                    0.52244821897,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    -0.838064133458,
                    0.52244821897,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    0.838064133458,
                    -0.52244821897,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    -0.838064133458,
                    -0.52244821897,
                    0.157150776982,
                    0.002785094212,
                ],
                [
                    0.236570299316,
                    0.601754663409,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    -0.236570299316,
                    0.601754663409,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    0.236570299316,
                    -0.601754663409,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    -0.236570299316,
                    -0.601754663409,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    0.236570299316,
                    0.762840624605,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    -0.236570299316,
                    0.762840624605,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    0.236570299316,
                    -0.762840624605,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    -0.236570299316,
                    -0.762840624605,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    0.601754663409,
                    0.236570299316,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    -0.601754663409,
                    0.236570299316,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    0.601754663409,
                    -0.236570299316,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    -0.601754663409,
                    -0.236570299316,
                    0.762840624605,
                    0.002837523356,
                ],
                [
                    0.601754663409,
                    0.762840624605,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    -0.601754663409,
                    0.762840624605,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    0.601754663409,
                    -0.762840624605,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    -0.601754663409,
                    -0.762840624605,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    0.762840624605,
                    0.236570299316,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    -0.762840624605,
                    0.236570299316,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    0.762840624605,
                    -0.236570299316,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    -0.762840624605,
                    -0.236570299316,
                    0.601754663409,
                    0.002837523356,
                ],
                [
                    0.762840624605,
                    0.601754663409,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    -0.762840624605,
                    0.601754663409,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    0.762840624605,
                    -0.601754663409,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    -0.762840624605,
                    -0.601754663409,
                    0.236570299316,
                    0.002837523356,
                ],
                [
                    0.077148158668,
                    0.434657551614,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    -0.077148158668,
                    0.434657551614,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    0.077148158668,
                    -0.434657551614,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    -0.077148158668,
                    -0.434657551614,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    0.077148158668,
                    0.897285336133,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    -0.077148158668,
                    0.897285336133,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    0.077148158668,
                    -0.897285336133,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    -0.077148158668,
                    -0.897285336133,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    0.434657551614,
                    0.077148158668,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    -0.434657551614,
                    0.077148158668,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    0.434657551614,
                    -0.077148158668,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    -0.434657551614,
                    -0.077148158668,
                    0.897285336133,
                    0.002676733369,
                ],
                [
                    0.434657551614,
                    0.897285336133,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    -0.434657551614,
                    0.897285336133,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    0.434657551614,
                    -0.897285336133,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    -0.434657551614,
                    -0.897285336133,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    0.897285336133,
                    0.077148158668,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    -0.897285336133,
                    0.077148158668,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    0.897285336133,
                    -0.077148158668,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    -0.897285336133,
                    -0.077148158668,
                    0.434657551614,
                    0.002676733369,
                ],
                [
                    0.897285336133,
                    0.434657551614,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    -0.897285336133,
                    0.434657551614,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    0.897285336133,
                    -0.434657551614,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    -0.897285336133,
                    -0.434657551614,
                    0.077148158668,
                    0.002676733369,
                ],
                [
                    0.306293666621,
                    0.490882658904,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    -0.306293666621,
                    0.490882658904,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    0.306293666621,
                    -0.490882658904,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    -0.306293666621,
                    -0.490882658904,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    0.306293666621,
                    0.815609223204,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    -0.306293666621,
                    0.815609223204,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    0.306293666621,
                    -0.815609223204,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    -0.306293666621,
                    -0.815609223204,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    0.490882658904,
                    0.306293666621,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    -0.490882658904,
                    0.306293666621,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    0.490882658904,
                    -0.306293666621,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    -0.490882658904,
                    -0.306293666621,
                    0.815609223204,
                    0.002787401725,
                ],
                [
                    0.490882658904,
                    0.815609223204,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    -0.490882658904,
                    0.815609223204,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    0.490882658904,
                    -0.815609223204,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    -0.490882658904,
                    -0.815609223204,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    0.815609223204,
                    0.306293666621,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    -0.815609223204,
                    0.306293666621,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    0.815609223204,
                    -0.306293666621,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    -0.815609223204,
                    -0.306293666621,
                    0.490882658904,
                    0.002787401725,
                ],
                [
                    0.815609223204,
                    0.490882658904,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    -0.815609223204,
                    0.490882658904,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    0.815609223204,
                    -0.490882658904,
                    0.306293666621,
                    0.002787401725,
                ],
                [
                    -0.815609223204,
                    -0.490882658904,
                    0.306293666621,
                    0.002787401725,
                ],
                [0.382247737952, 0.56487681491, 0.73130079366, 0.002831829515],
                [
                    -0.382247737952,
                    0.56487681491,
                    0.73130079366,
                    0.002831829515,
                ],
                [
                    0.382247737952,
                    -0.56487681491,
                    0.73130079366,
                    0.002831829515,
                ],
                [
                    -0.382247737952,
                    -0.56487681491,
                    0.73130079366,
                    0.002831829515,
                ],
                [0.382247737952, 0.73130079366, 0.56487681491, 0.002831829515],
                [
                    -0.382247737952,
                    0.73130079366,
                    0.56487681491,
                    0.002831829515,
                ],
                [
                    0.382247737952,
                    -0.73130079366,
                    0.56487681491,
                    0.002831829515,
                ],
                [
                    -0.382247737952,
                    -0.73130079366,
                    0.56487681491,
                    0.002831829515,
                ],
                [0.56487681491, 0.382247737952, 0.73130079366, 0.002831829515],
                [
                    -0.56487681491,
                    0.382247737952,
                    0.73130079366,
                    0.002831829515,
                ],
                [
                    0.56487681491,
                    -0.382247737952,
                    0.73130079366,
                    0.002831829515,
                ],
                [
                    -0.56487681491,
                    -0.382247737952,
                    0.73130079366,
                    0.002831829515,
                ],
                [0.56487681491, 0.73130079366, 0.382247737952, 0.002831829515],
                [
                    -0.56487681491,
                    0.73130079366,
                    0.382247737952,
                    0.002831829515,
                ],
                [
                    0.56487681491,
                    -0.73130079366,
                    0.382247737952,
                    0.002831829515,
                ],
                [
                    -0.56487681491,
                    -0.73130079366,
                    0.382247737952,
                    0.002831829515,
                ],
                [0.73130079366, 0.382247737952, 0.56487681491, 0.002831829515],
                [
                    -0.73130079366,
                    0.382247737952,
                    0.56487681491,
                    0.002831829515,
                ],
                [
                    0.73130079366,
                    -0.382247737952,
                    0.56487681491,
                    0.002831829515,
                ],
                [
                    -0.73130079366,
                    -0.382247737952,
                    0.56487681491,
                    0.002831829515,
                ],
                [0.73130079366, 0.56487681491, 0.382247737952, 0.002831829515],
                [
                    -0.73130079366,
                    0.56487681491,
                    0.382247737952,
                    0.002831829515,
                ],
                [
                    0.73130079366,
                    -0.56487681491,
                    0.382247737952,
                    0.002831829515,
                ],
                [
                    -0.73130079366,
                    -0.56487681491,
                    0.382247737952,
                    0.002831829515,
                ],
            ]
        )
        return grid_array

    if grid == 8:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 0.000143829419],
                [-1.0, 0.0, 0.0, 0.000143829419],
                [0.0, 1.0, 0.0, 0.000143829419],
                [0.0, -1.0, 0.0, 0.000143829419],
                [0.0, 0.0, 1.0, 0.000287658838],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.002251544577],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.002251544577],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.002251544577],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.002251544577,
                ],
                [
                    0.042929635453,
                    0.042929635453,
                    0.998155345024,
                    0.000989605868,
                ],
                [
                    -0.042929635453,
                    0.042929635453,
                    0.998155345024,
                    0.000989605868,
                ],
                [
                    0.042929635453,
                    -0.042929635453,
                    0.998155345024,
                    0.000989605868,
                ],
                [
                    -0.042929635453,
                    -0.042929635453,
                    0.998155345024,
                    0.000989605868,
                ],
                [
                    0.042929635453,
                    0.998155345024,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    -0.042929635453,
                    0.998155345024,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    0.042929635453,
                    -0.998155345024,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    -0.042929635453,
                    -0.998155345024,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    0.998155345024,
                    0.042929635453,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    -0.998155345024,
                    0.042929635453,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    0.998155345024,
                    -0.042929635453,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    -0.998155345024,
                    -0.042929635453,
                    0.042929635453,
                    0.000989605868,
                ],
                [
                    0.105142685409,
                    0.105142685409,
                    0.988883224355,
                    0.001471598022,
                ],
                [
                    -0.105142685409,
                    0.105142685409,
                    0.988883224355,
                    0.001471598022,
                ],
                [
                    0.105142685409,
                    -0.105142685409,
                    0.988883224355,
                    0.001471598022,
                ],
                [
                    -0.105142685409,
                    -0.105142685409,
                    0.988883224355,
                    0.001471598022,
                ],
                [
                    0.105142685409,
                    0.988883224355,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    -0.105142685409,
                    0.988883224355,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    0.105142685409,
                    -0.988883224355,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    -0.105142685409,
                    -0.988883224355,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    0.988883224355,
                    0.105142685409,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    -0.988883224355,
                    0.105142685409,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    0.988883224355,
                    -0.105142685409,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    -0.988883224355,
                    -0.105142685409,
                    0.105142685409,
                    0.001471598022,
                ],
                [
                    0.175002486762,
                    0.175002486762,
                    0.968890220435,
                    0.001777826554,
                ],
                [
                    -0.175002486762,
                    0.175002486762,
                    0.968890220435,
                    0.001777826554,
                ],
                [
                    0.175002486762,
                    -0.175002486762,
                    0.968890220435,
                    0.001777826554,
                ],
                [
                    -0.175002486762,
                    -0.175002486762,
                    0.968890220435,
                    0.001777826554,
                ],
                [
                    0.175002486762,
                    0.968890220435,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    -0.175002486762,
                    0.968890220435,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    0.175002486762,
                    -0.968890220435,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    -0.175002486762,
                    -0.968890220435,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    0.968890220435,
                    0.175002486762,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    -0.968890220435,
                    0.175002486762,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    0.968890220435,
                    -0.175002486762,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    -0.968890220435,
                    -0.175002486762,
                    0.175002486762,
                    0.001777826554,
                ],
                [
                    0.247765337965,
                    0.247765337965,
                    0.936602730407,
                    0.001977669568,
                ],
                [
                    -0.247765337965,
                    0.247765337965,
                    0.936602730407,
                    0.001977669568,
                ],
                [
                    0.247765337965,
                    -0.247765337965,
                    0.936602730407,
                    0.001977669568,
                ],
                [
                    -0.247765337965,
                    -0.247765337965,
                    0.936602730407,
                    0.001977669568,
                ],
                [
                    0.247765337965,
                    0.936602730407,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    -0.247765337965,
                    0.936602730407,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    0.247765337965,
                    -0.936602730407,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    -0.247765337965,
                    -0.936602730407,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    0.936602730407,
                    0.247765337965,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    -0.936602730407,
                    0.247765337965,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    0.936602730407,
                    -0.247765337965,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    -0.936602730407,
                    -0.247765337965,
                    0.247765337965,
                    0.001977669568,
                ],
                [
                    0.320656712396,
                    0.320656712396,
                    0.891267942648,
                    0.002106599363,
                ],
                [
                    -0.320656712396,
                    0.320656712396,
                    0.891267942648,
                    0.002106599363,
                ],
                [
                    0.320656712396,
                    -0.320656712396,
                    0.891267942648,
                    0.002106599363,
                ],
                [
                    -0.320656712396,
                    -0.320656712396,
                    0.891267942648,
                    0.002106599363,
                ],
                [
                    0.320656712396,
                    0.891267942648,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    -0.320656712396,
                    0.891267942648,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    0.320656712396,
                    -0.891267942648,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    -0.320656712396,
                    -0.891267942648,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    0.891267942648,
                    0.320656712396,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    -0.891267942648,
                    0.320656712396,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    0.891267942648,
                    -0.320656712396,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    -0.891267942648,
                    -0.320656712396,
                    0.320656712396,
                    0.002106599363,
                ],
                [
                    0.391652074985,
                    0.391652074985,
                    0.832596723702,
                    0.002185557614,
                ],
                [
                    -0.391652074985,
                    0.391652074985,
                    0.832596723702,
                    0.002185557614,
                ],
                [
                    0.391652074985,
                    -0.391652074985,
                    0.832596723702,
                    0.002185557614,
                ],
                [
                    -0.391652074985,
                    -0.391652074985,
                    0.832596723702,
                    0.002185557614,
                ],
                [
                    0.391652074985,
                    0.832596723702,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    -0.391652074985,
                    0.832596723702,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    0.391652074985,
                    -0.832596723702,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    -0.391652074985,
                    -0.832596723702,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    0.832596723702,
                    0.391652074985,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    -0.832596723702,
                    0.391652074985,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    0.832596723702,
                    -0.391652074985,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    -0.832596723702,
                    -0.391652074985,
                    0.391652074985,
                    0.002185557614,
                ],
                [
                    0.459082587419,
                    0.459082587419,
                    0.760582905315,
                    0.002228778788,
                ],
                [
                    -0.459082587419,
                    0.459082587419,
                    0.760582905315,
                    0.002228778788,
                ],
                [
                    0.459082587419,
                    -0.459082587419,
                    0.760582905315,
                    0.002228778788,
                ],
                [
                    -0.459082587419,
                    -0.459082587419,
                    0.760582905315,
                    0.002228778788,
                ],
                [
                    0.459082587419,
                    0.760582905315,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    -0.459082587419,
                    0.760582905315,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    0.459082587419,
                    -0.760582905315,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    -0.459082587419,
                    -0.760582905315,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    0.760582905315,
                    0.459082587419,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    -0.760582905315,
                    0.459082587419,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    0.760582905315,
                    -0.459082587419,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    -0.760582905315,
                    -0.459082587419,
                    0.459082587419,
                    0.002228778788,
                ],
                [
                    0.521456388842,
                    0.521456388842,
                    0.675400969108,
                    0.002247449576,
                ],
                [
                    -0.521456388842,
                    0.521456388842,
                    0.675400969108,
                    0.002247449576,
                ],
                [
                    0.521456388842,
                    -0.521456388842,
                    0.675400969108,
                    0.002247449576,
                ],
                [
                    -0.521456388842,
                    -0.521456388842,
                    0.675400969108,
                    0.002247449576,
                ],
                [
                    0.521456388842,
                    0.675400969108,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    -0.521456388842,
                    0.675400969108,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    0.521456388842,
                    -0.675400969108,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    -0.521456388842,
                    -0.675400969108,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    0.675400969108,
                    0.521456388842,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    -0.675400969108,
                    0.521456388842,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    0.675400969108,
                    -0.521456388842,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    -0.675400969108,
                    -0.521456388842,
                    0.521456388842,
                    0.002247449576,
                ],
                [
                    0.625317024465,
                    0.625317024465,
                    0.466858905696,
                    0.00225047865,
                ],
                [
                    -0.625317024465,
                    0.625317024465,
                    0.466858905696,
                    0.00225047865,
                ],
                [
                    0.625317024465,
                    -0.625317024465,
                    0.466858905696,
                    0.00225047865,
                ],
                [
                    -0.625317024465,
                    -0.625317024465,
                    0.466858905696,
                    0.00225047865,
                ],
                [
                    0.625317024465,
                    0.466858905696,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    -0.625317024465,
                    0.466858905696,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    0.625317024465,
                    -0.466858905696,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    -0.625317024465,
                    -0.466858905696,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    0.466858905696,
                    0.625317024465,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    -0.466858905696,
                    0.625317024465,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    0.466858905696,
                    -0.625317024465,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    -0.466858905696,
                    -0.625317024465,
                    0.625317024465,
                    0.00225047865,
                ],
                [
                    0.663792674452,
                    0.663792674452,
                    0.344613654237,
                    0.002252306544,
                ],
                [
                    -0.663792674452,
                    0.663792674452,
                    0.344613654237,
                    0.002252306544,
                ],
                [
                    0.663792674452,
                    -0.663792674452,
                    0.344613654237,
                    0.002252306544,
                ],
                [
                    -0.663792674452,
                    -0.663792674452,
                    0.344613654237,
                    0.002252306544,
                ],
                [
                    0.663792674452,
                    0.344613654237,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    -0.663792674452,
                    0.344613654237,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    0.663792674452,
                    -0.344613654237,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    -0.663792674452,
                    -0.344613654237,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    0.344613654237,
                    0.663792674452,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    -0.344613654237,
                    0.663792674452,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    0.344613654237,
                    -0.663792674452,
                    0.663792674452,
                    0.002252306544,
                ],
                [
                    -0.344613654237,
                    -0.663792674452,
                    0.663792674452,
                    0.002252306544,
                ],
                [0.69104103985, 0.69104103985, 0.21195415185, 0.002260573862],
                [-0.69104103985, 0.69104103985, 0.21195415185, 0.002260573862],
                [0.69104103985, -0.69104103985, 0.21195415185, 0.002260573862],
                [
                    -0.69104103985,
                    -0.69104103985,
                    0.21195415185,
                    0.002260573862,
                ],
                [0.69104103985, 0.21195415185, 0.69104103985, 0.002260573862],
                [-0.69104103985, 0.21195415185, 0.69104103985, 0.002260573862],
                [0.69104103985, -0.21195415185, 0.69104103985, 0.002260573862],
                [
                    -0.69104103985,
                    -0.21195415185,
                    0.69104103985,
                    0.002260573862,
                ],
                [0.21195415185, 0.69104103985, 0.69104103985, 0.002260573862],
                [-0.21195415185, 0.69104103985, 0.69104103985, 0.002260573862],
                [0.21195415185, -0.69104103985, 0.69104103985, 0.002260573862],
                [
                    -0.21195415185,
                    -0.69104103985,
                    0.69104103985,
                    0.002260573862,
                ],
                [
                    0.705290700746,
                    0.705290700746,
                    0.07162440145,
                    0.002269973069,
                ],
                [
                    -0.705290700746,
                    0.705290700746,
                    0.07162440145,
                    0.002269973069,
                ],
                [
                    0.705290700746,
                    -0.705290700746,
                    0.07162440145,
                    0.002269973069,
                ],
                [
                    -0.705290700746,
                    -0.705290700746,
                    0.07162440145,
                    0.002269973069,
                ],
                [
                    0.705290700746,
                    0.07162440145,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    -0.705290700746,
                    0.07162440145,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    0.705290700746,
                    -0.07162440145,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    -0.705290700746,
                    -0.07162440145,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    0.07162440145,
                    0.705290700746,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    -0.07162440145,
                    0.705290700746,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    0.07162440145,
                    -0.705290700746,
                    0.705290700746,
                    0.002269973069,
                ],
                [
                    -0.07162440145,
                    -0.705290700746,
                    0.705290700746,
                    0.002269973069,
                ],
                [0.123668676266, 0.992323565431, 0.0, 0.000682336793],
                [-0.123668676266, 0.992323565431, 0.0, 0.000682336793],
                [0.123668676266, -0.992323565431, 0.0, 0.000682336793],
                [-0.123668676266, -0.992323565431, 0.0, 0.000682336793],
                [0.992323565431, 0.123668676266, 0.0, 0.000682336793],
                [-0.992323565431, 0.123668676266, 0.0, 0.000682336793],
                [0.992323565431, -0.123668676266, 0.0, 0.000682336793],
                [-0.992323565431, -0.123668676266, 0.0, 0.000682336793],
                [0.123668676266, 0.0, 0.992323565431, 0.001364673585],
                [-0.123668676266, 0.0, 0.992323565431, 0.001364673585],
                [0.992323565431, 0.0, 0.123668676266, 0.001364673585],
                [-0.992323565431, 0.0, 0.123668676266, 0.001364673585],
                [0.0, 0.123668676266, 0.992323565431, 0.001364673585],
                [0.0, -0.123668676266, 0.992323565431, 0.001364673585],
                [0.0, 0.992323565431, 0.123668676266, 0.001364673585],
                [0.0, -0.992323565431, 0.123668676266, 0.001364673585],
                [0.294077711447, 0.955781512497, 0.0, 0.000945415816],
                [-0.294077711447, 0.955781512497, 0.0, 0.000945415816],
                [0.294077711447, -0.955781512497, 0.0, 0.000945415816],
                [-0.294077711447, -0.955781512497, 0.0, 0.000945415816],
                [0.955781512497, 0.294077711447, 0.0, 0.000945415816],
                [-0.955781512497, 0.294077711447, 0.0, 0.000945415816],
                [0.955781512497, -0.294077711447, 0.0, 0.000945415816],
                [-0.955781512497, -0.294077711447, 0.0, 0.000945415816],
                [0.294077711447, 0.0, 0.955781512497, 0.001890831632],
                [-0.294077711447, 0.0, 0.955781512497, 0.001890831632],
                [0.955781512497, 0.0, 0.294077711447, 0.001890831632],
                [-0.955781512497, 0.0, 0.294077711447, 0.001890831632],
                [0.0, 0.294077711447, 0.955781512497, 0.001890831632],
                [0.0, -0.294077711447, 0.955781512497, 0.001890831632],
                [0.0, 0.955781512497, 0.294077711447, 0.001890831632],
                [0.0, -0.955781512497, 0.294077711447, 0.001890831632],
                [0.469775384921, 0.882785980701, 0.0, 0.001074429975],
                [-0.469775384921, 0.882785980701, 0.0, 0.001074429975],
                [0.469775384921, -0.882785980701, 0.0, 0.001074429975],
                [-0.469775384921, -0.882785980701, 0.0, 0.001074429975],
                [0.882785980701, 0.469775384921, 0.0, 0.001074429975],
                [-0.882785980701, 0.469775384921, 0.0, 0.001074429975],
                [0.882785980701, -0.469775384921, 0.0, 0.001074429975],
                [-0.882785980701, -0.469775384921, 0.0, 0.001074429975],
                [0.469775384921, 0.0, 0.882785980701, 0.002148859951],
                [-0.469775384921, 0.0, 0.882785980701, 0.002148859951],
                [0.882785980701, 0.0, 0.469775384921, 0.002148859951],
                [-0.882785980701, 0.0, 0.469775384921, 0.002148859951],
                [0.0, 0.469775384921, 0.882785980701, 0.002148859951],
                [0.0, -0.469775384921, 0.882785980701, 0.002148859951],
                [0.0, 0.882785980701, 0.469775384921, 0.002148859951],
                [0.0, -0.882785980701, 0.469775384921, 0.002148859951],
                [0.633456324114, 0.773778447257, 0.0, 0.001129300087],
                [-0.633456324114, 0.773778447257, 0.0, 0.001129300087],
                [0.633456324114, -0.773778447257, 0.0, 0.001129300087],
                [-0.633456324114, -0.773778447257, 0.0, 0.001129300087],
                [0.773778447257, 0.633456324114, 0.0, 0.001129300087],
                [-0.773778447257, 0.633456324114, 0.0, 0.001129300087],
                [0.773778447257, -0.633456324114, 0.0, 0.001129300087],
                [-0.773778447257, -0.633456324114, 0.0, 0.001129300087],
                [0.633456324114, 0.0, 0.773778447257, 0.002258600173],
                [-0.633456324114, 0.0, 0.773778447257, 0.002258600173],
                [0.773778447257, 0.0, 0.633456324114, 0.002258600173],
                [-0.773778447257, 0.0, 0.633456324114, 0.002258600173],
                [0.0, 0.633456324114, 0.773778447257, 0.002258600173],
                [0.0, -0.633456324114, 0.773778447257, 0.002258600173],
                [0.0, 0.773778447257, 0.633456324114, 0.002258600173],
                [0.0, -0.773778447257, 0.633456324114, 0.002258600173],
                [0.059740486142, 0.202912875278, 0.977372722845, 0.0016873769],
                [
                    -0.059740486142,
                    0.202912875278,
                    0.977372722845,
                    0.0016873769,
                ],
                [
                    0.059740486142,
                    -0.202912875278,
                    0.977372722845,
                    0.0016873769,
                ],
                [
                    -0.059740486142,
                    -0.202912875278,
                    0.977372722845,
                    0.0016873769,
                ],
                [0.059740486142, 0.977372722845, 0.202912875278, 0.0016873769],
                [
                    -0.059740486142,
                    0.977372722845,
                    0.202912875278,
                    0.0016873769,
                ],
                [
                    0.059740486142,
                    -0.977372722845,
                    0.202912875278,
                    0.0016873769,
                ],
                [
                    -0.059740486142,
                    -0.977372722845,
                    0.202912875278,
                    0.0016873769,
                ],
                [0.202912875278, 0.059740486142, 0.977372722845, 0.0016873769],
                [
                    -0.202912875278,
                    0.059740486142,
                    0.977372722845,
                    0.0016873769,
                ],
                [
                    0.202912875278,
                    -0.059740486142,
                    0.977372722845,
                    0.0016873769,
                ],
                [
                    -0.202912875278,
                    -0.059740486142,
                    0.977372722845,
                    0.0016873769,
                ],
                [0.202912875278, 0.977372722845, 0.059740486142, 0.0016873769],
                [
                    -0.202912875278,
                    0.977372722845,
                    0.059740486142,
                    0.0016873769,
                ],
                [
                    0.202912875278,
                    -0.977372722845,
                    0.059740486142,
                    0.0016873769,
                ],
                [
                    -0.202912875278,
                    -0.977372722845,
                    0.059740486142,
                    0.0016873769,
                ],
                [0.977372722845, 0.059740486142, 0.202912875278, 0.0016873769],
                [
                    -0.977372722845,
                    0.059740486142,
                    0.202912875278,
                    0.0016873769,
                ],
                [
                    0.977372722845,
                    -0.059740486142,
                    0.202912875278,
                    0.0016873769,
                ],
                [
                    -0.977372722845,
                    -0.059740486142,
                    0.202912875278,
                    0.0016873769,
                ],
                [0.977372722845, 0.202912875278, 0.059740486142, 0.0016873769],
                [
                    -0.977372722845,
                    0.202912875278,
                    0.059740486142,
                    0.0016873769,
                ],
                [
                    0.977372722845,
                    -0.202912875278,
                    0.059740486142,
                    0.0016873769,
                ],
                [
                    -0.977372722845,
                    -0.202912875278,
                    0.059740486142,
                    0.0016873769,
                ],
                [
                    0.137576040847,
                    0.460262194248,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    -0.137576040847,
                    0.460262194248,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    0.137576040847,
                    -0.460262194248,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    -0.137576040847,
                    -0.460262194248,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    0.137576040847,
                    0.877058461866,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    -0.137576040847,
                    0.877058461866,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    0.137576040847,
                    -0.877058461866,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    -0.137576040847,
                    -0.877058461866,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    0.460262194248,
                    0.137576040847,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    -0.460262194248,
                    0.137576040847,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    0.460262194248,
                    -0.137576040847,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    -0.460262194248,
                    -0.137576040847,
                    0.877058461866,
                    0.002150511441,
                ],
                [
                    0.460262194248,
                    0.877058461866,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    -0.460262194248,
                    0.877058461866,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    0.460262194248,
                    -0.877058461866,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    -0.460262194248,
                    -0.877058461866,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    0.877058461866,
                    0.137576040847,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    -0.877058461866,
                    0.137576040847,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    0.877058461866,
                    -0.137576040847,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    -0.877058461866,
                    -0.137576040847,
                    0.460262194248,
                    0.002150511441,
                ],
                [
                    0.877058461866,
                    0.460262194248,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    -0.877058461866,
                    0.460262194248,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    0.877058461866,
                    -0.460262194248,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    -0.877058461866,
                    -0.460262194248,
                    0.137576040847,
                    0.002150511441,
                ],
                [
                    0.339101652634,
                    0.503067399966,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    -0.339101652634,
                    0.503067399966,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    0.339101652634,
                    -0.503067399966,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    -0.339101652634,
                    -0.503067399966,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    0.339101652634,
                    0.794942299964,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    -0.339101652634,
                    0.794942299964,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    0.339101652634,
                    -0.794942299964,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    -0.339101652634,
                    -0.794942299964,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    0.503067399966,
                    0.339101652634,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    -0.503067399966,
                    0.339101652634,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    0.503067399966,
                    -0.339101652634,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    -0.503067399966,
                    -0.339101652634,
                    0.794942299964,
                    0.002217154474,
                ],
                [
                    0.503067399966,
                    0.794942299964,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    -0.503067399966,
                    0.794942299964,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    0.503067399966,
                    -0.794942299964,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    -0.503067399966,
                    -0.794942299964,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    0.794942299964,
                    0.339101652634,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    -0.794942299964,
                    0.339101652634,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    0.794942299964,
                    -0.339101652634,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    -0.794942299964,
                    -0.339101652634,
                    0.503067399966,
                    0.002217154474,
                ],
                [
                    0.794942299964,
                    0.503067399966,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    -0.794942299964,
                    0.503067399966,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    0.794942299964,
                    -0.503067399966,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    -0.794942299964,
                    -0.503067399966,
                    0.339101652634,
                    0.002217154474,
                ],
                [
                    0.127167519144,
                    0.281760642244,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    -0.127167519144,
                    0.281760642244,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    0.127167519144,
                    -0.281760642244,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    -0.127167519144,
                    -0.281760642244,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    0.127167519144,
                    0.951020169374,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    -0.127167519144,
                    0.951020169374,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    0.127167519144,
                    -0.951020169374,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    -0.127167519144,
                    -0.951020169374,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    0.281760642244,
                    0.127167519144,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    -0.281760642244,
                    0.127167519144,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    0.281760642244,
                    -0.127167519144,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    -0.281760642244,
                    -0.127167519144,
                    0.951020169374,
                    0.001913295065,
                ],
                [
                    0.281760642244,
                    0.951020169374,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    -0.281760642244,
                    0.951020169374,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    0.281760642244,
                    -0.951020169374,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    -0.281760642244,
                    -0.951020169374,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    0.951020169374,
                    0.127167519144,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    -0.951020169374,
                    0.127167519144,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    0.951020169374,
                    -0.127167519144,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    -0.951020169374,
                    -0.127167519144,
                    0.281760642244,
                    0.001913295065,
                ],
                [
                    0.951020169374,
                    0.281760642244,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    -0.951020169374,
                    0.281760642244,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    0.951020169374,
                    -0.281760642244,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    -0.951020169374,
                    -0.281760642244,
                    0.127167519144,
                    0.001913295065,
                ],
                [
                    0.269312074041,
                    0.433156129172,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    -0.269312074041,
                    0.433156129172,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    0.269312074041,
                    -0.433156129172,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    -0.269312074041,
                    -0.433156129172,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    0.269312074041,
                    0.860143461602,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    -0.269312074041,
                    0.860143461602,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    0.269312074041,
                    -0.860143461602,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    -0.269312074041,
                    -0.860143461602,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    0.433156129172,
                    0.269312074041,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    -0.433156129172,
                    0.269312074041,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    0.433156129172,
                    -0.269312074041,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    -0.433156129172,
                    -0.269312074041,
                    0.860143461602,
                    0.002161326501,
                ],
                [
                    0.433156129172,
                    0.860143461602,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    -0.433156129172,
                    0.860143461602,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    0.433156129172,
                    -0.860143461602,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    -0.433156129172,
                    -0.860143461602,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    0.860143461602,
                    0.269312074041,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    -0.860143461602,
                    0.269312074041,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    0.860143461602,
                    -0.269312074041,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    -0.860143461602,
                    -0.269312074041,
                    0.433156129172,
                    0.002161326501,
                ],
                [
                    0.860143461602,
                    0.433156129172,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    -0.860143461602,
                    0.433156129172,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    0.860143461602,
                    -0.433156129172,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    -0.860143461602,
                    -0.433156129172,
                    0.269312074041,
                    0.002161326501,
                ],
                [
                    0.14197864526,
                    0.625616735858,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    -0.14197864526,
                    0.625616735858,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    0.14197864526,
                    -0.625616735858,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    -0.14197864526,
                    -0.625616735858,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    0.14197864526,
                    0.767102186221,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    -0.14197864526,
                    0.767102186221,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    0.14197864526,
                    -0.767102186221,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    -0.14197864526,
                    -0.767102186221,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    0.625616735858,
                    0.14197864526,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    -0.625616735858,
                    0.14197864526,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    0.625616735858,
                    -0.14197864526,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    -0.625616735858,
                    -0.14197864526,
                    0.767102186221,
                    0.002253594262,
                ],
                [
                    0.625616735858,
                    0.767102186221,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    -0.625616735858,
                    0.767102186221,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    0.625616735858,
                    -0.767102186221,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    -0.625616735858,
                    -0.767102186221,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    0.767102186221,
                    0.14197864526,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    -0.767102186221,
                    0.14197864526,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    0.767102186221,
                    -0.14197864526,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    -0.767102186221,
                    -0.14197864526,
                    0.625616735858,
                    0.002253594262,
                ],
                [
                    0.767102186221,
                    0.625616735858,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    -0.767102186221,
                    0.625616735858,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    0.767102186221,
                    -0.625616735858,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    -0.767102186221,
                    -0.625616735858,
                    0.14197864526,
                    0.002253594262,
                ],
                [
                    0.067092846007,
                    0.379839521686,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    -0.067092846007,
                    0.379839521686,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    0.067092846007,
                    -0.379839521686,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    -0.067092846007,
                    -0.379839521686,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    0.067092846007,
                    0.922616110731,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    -0.067092846007,
                    0.922616110731,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    0.067092846007,
                    -0.922616110731,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    -0.067092846007,
                    -0.922616110731,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    0.379839521686,
                    0.067092846007,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    -0.379839521686,
                    0.067092846007,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    0.379839521686,
                    -0.067092846007,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    -0.379839521686,
                    -0.067092846007,
                    0.922616110731,
                    0.002045137431,
                ],
                [
                    0.379839521686,
                    0.922616110731,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    -0.379839521686,
                    0.922616110731,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    0.379839521686,
                    -0.922616110731,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    -0.379839521686,
                    -0.922616110731,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    0.922616110731,
                    0.067092846007,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    -0.922616110731,
                    0.067092846007,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    0.922616110731,
                    -0.067092846007,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    -0.922616110731,
                    -0.067092846007,
                    0.379839521686,
                    0.002045137431,
                ],
                [
                    0.922616110731,
                    0.379839521686,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    -0.922616110731,
                    0.379839521686,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    0.922616110731,
                    -0.379839521686,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    -0.922616110731,
                    -0.379839521686,
                    0.067092846007,
                    0.002045137431,
                ],
                [
                    0.070577381833,
                    0.551750542142,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    -0.070577381833,
                    0.551750542142,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    0.070577381833,
                    -0.551750542142,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    -0.070577381833,
                    -0.551750542142,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    0.070577381833,
                    0.831017552413,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    -0.070577381833,
                    0.831017552413,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    0.070577381833,
                    -0.831017552413,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    -0.070577381833,
                    -0.831017552413,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    0.551750542142,
                    0.070577381833,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    -0.551750542142,
                    0.070577381833,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    0.551750542142,
                    -0.070577381833,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    -0.551750542142,
                    -0.070577381833,
                    0.831017552413,
                    0.002217920535,
                ],
                [
                    0.551750542142,
                    0.831017552413,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    -0.551750542142,
                    0.831017552413,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    0.551750542142,
                    -0.831017552413,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    -0.551750542142,
                    -0.831017552413,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    0.831017552413,
                    0.070577381833,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    -0.831017552413,
                    0.070577381833,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    0.831017552413,
                    -0.070577381833,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    -0.831017552413,
                    -0.070577381833,
                    0.551750542142,
                    0.002217920535,
                ],
                [
                    0.831017552413,
                    0.551750542142,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    -0.831017552413,
                    0.551750542142,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    0.831017552413,
                    -0.551750542142,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    -0.831017552413,
                    -0.551750542142,
                    0.070577381833,
                    0.002217920535,
                ],
                [
                    0.278388847788,
                    0.602961915616,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    -0.278388847788,
                    0.602961915616,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    0.278388847788,
                    -0.602961915616,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    -0.278388847788,
                    -0.602961915616,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    0.278388847788,
                    0.747620610834,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    -0.278388847788,
                    0.747620610834,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    0.278388847788,
                    -0.747620610834,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    -0.278388847788,
                    -0.747620610834,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    0.602961915616,
                    0.278388847788,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    -0.602961915616,
                    0.278388847788,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    0.602961915616,
                    -0.278388847788,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    -0.602961915616,
                    -0.278388847788,
                    0.747620610834,
                    0.002245581307,
                ],
                [
                    0.602961915616,
                    0.747620610834,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    -0.602961915616,
                    0.747620610834,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    0.602961915616,
                    -0.747620610834,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    -0.602961915616,
                    -0.747620610834,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    0.747620610834,
                    0.278388847788,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    -0.747620610834,
                    0.278388847788,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    0.747620610834,
                    -0.278388847788,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    -0.747620610834,
                    -0.278388847788,
                    0.602961915616,
                    0.002245581307,
                ],
                [
                    0.747620610834,
                    0.602961915616,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    -0.747620610834,
                    0.602961915616,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    0.747620610834,
                    -0.602961915616,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    -0.747620610834,
                    -0.602961915616,
                    0.278388847788,
                    0.002245581307,
                ],
                [
                    0.197957893892,
                    0.358960632959,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    -0.197957893892,
                    0.358960632959,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    0.197957893892,
                    -0.358960632959,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    -0.197957893892,
                    -0.358960632959,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    0.197957893892,
                    0.912118378409,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    -0.197957893892,
                    0.912118378409,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    0.197957893892,
                    -0.912118378409,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    -0.197957893892,
                    -0.912118378409,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    0.358960632959,
                    0.197957893892,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    -0.358960632959,
                    0.197957893892,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    0.358960632959,
                    -0.197957893892,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    -0.358960632959,
                    -0.197957893892,
                    0.912118378409,
                    0.002064803694,
                ],
                [
                    0.358960632959,
                    0.912118378409,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    -0.358960632959,
                    0.912118378409,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    0.358960632959,
                    -0.912118378409,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    -0.358960632959,
                    -0.912118378409,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    0.912118378409,
                    0.197957893892,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    -0.912118378409,
                    0.197957893892,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    0.912118378409,
                    -0.197957893892,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    -0.912118378409,
                    -0.197957893892,
                    0.358960632959,
                    0.002064803694,
                ],
                [
                    0.912118378409,
                    0.358960632959,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    -0.912118378409,
                    0.358960632959,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    0.912118378409,
                    -0.358960632959,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    -0.912118378409,
                    -0.358960632959,
                    0.197957893892,
                    0.002064803694,
                ],
                [
                    0.20873070611,
                    0.534866643814,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    -0.20873070611,
                    0.534866643814,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    0.20873070611,
                    -0.534866643814,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    -0.20873070611,
                    -0.534866643814,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    0.20873070611,
                    0.818748536281,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    -0.20873070611,
                    0.818748536281,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    0.20873070611,
                    -0.818748536281,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    -0.20873070611,
                    -0.818748536281,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    0.534866643814,
                    0.20873070611,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    -0.534866643814,
                    0.20873070611,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    0.534866643814,
                    -0.20873070611,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    -0.534866643814,
                    -0.20873070611,
                    0.818748536281,
                    0.002214498765,
                ],
                [
                    0.534866643814,
                    0.818748536281,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    -0.534866643814,
                    0.818748536281,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    0.534866643814,
                    -0.818748536281,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    -0.534866643814,
                    -0.818748536281,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    0.818748536281,
                    0.20873070611,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    -0.818748536281,
                    0.20873070611,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    0.818748536281,
                    -0.20873070611,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    -0.818748536281,
                    -0.20873070611,
                    0.534866643814,
                    0.002214498765,
                ],
                [
                    0.818748536281,
                    0.534866643814,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    -0.818748536281,
                    0.534866643814,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    0.818748536281,
                    -0.534866643814,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    -0.818748536281,
                    -0.534866643814,
                    0.20873070611,
                    0.002214498765,
                ],
                [
                    0.405512213787,
                    0.567499754607,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    -0.405512213787,
                    0.567499754607,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    0.405512213787,
                    -0.567499754607,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    -0.405512213787,
                    -0.567499754607,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    0.405512213787,
                    0.716591845467,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    -0.405512213787,
                    0.716591845467,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    0.405512213787,
                    -0.716591845467,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    -0.405512213787,
                    -0.716591845467,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    0.567499754607,
                    0.405512213787,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    -0.567499754607,
                    0.405512213787,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    0.567499754607,
                    -0.405512213787,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    -0.567499754607,
                    -0.405512213787,
                    0.716591845467,
                    0.002243560097,
                ],
                [
                    0.567499754607,
                    0.716591845467,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    -0.567499754607,
                    0.716591845467,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    0.567499754607,
                    -0.716591845467,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    -0.567499754607,
                    -0.716591845467,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    0.716591845467,
                    0.405512213787,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    -0.716591845467,
                    0.405512213787,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    0.716591845467,
                    -0.405512213787,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    -0.716591845467,
                    -0.405512213787,
                    0.567499754607,
                    0.002243560097,
                ],
                [
                    0.716591845467,
                    0.567499754607,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    -0.716591845467,
                    0.567499754607,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    0.716591845467,
                    -0.567499754607,
                    0.405512213787,
                    0.002243560097,
                ],
                [
                    -0.716591845467,
                    -0.567499754607,
                    0.405512213787,
                    0.002243560097,
                ],
            ]
        )
        return grid_array

    if grid == 9:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 6.3090494e-05],
                [-1.0, 0.0, 0.0, 6.3090494e-05],
                [0.0, 1.0, 0.0, 6.3090494e-05],
                [0.0, -1.0, 0.0, 6.3090494e-05],
                [0.0, 0.0, 1.0, 0.000126180989],
                [0.0, 0.707106781187, 0.707106781187, 0.001279657541],
                [0.0, -0.707106781187, 0.707106781187, 0.001279657541],
                [0.707106781187, 0.0, 0.707106781187, 0.001279657541],
                [-0.707106781187, 0.0, 0.707106781187, 0.001279657541],
                [0.707106781187, 0.707106781187, 0.0, 0.000639828771],
                [-0.707106781187, 0.707106781187, 0.0, 0.000639828771],
                [0.707106781187, -0.707106781187, 0.0, 0.000639828771],
                [-0.707106781187, -0.707106781187, 0.0, 0.000639828771],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.001271437015],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.001271437015],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.001271437015],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.001271437015,
                ],
                [
                    0.028609231262,
                    0.028609231262,
                    0.999181176651,
                    0.000444241432,
                ],
                [
                    -0.028609231262,
                    0.028609231262,
                    0.999181176651,
                    0.000444241432,
                ],
                [
                    0.028609231262,
                    -0.028609231262,
                    0.999181176651,
                    0.000444241432,
                ],
                [
                    -0.028609231262,
                    -0.028609231262,
                    0.999181176651,
                    0.000444241432,
                ],
                [
                    0.028609231262,
                    0.999181176651,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    -0.028609231262,
                    0.999181176651,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    0.028609231262,
                    -0.999181176651,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    -0.028609231262,
                    -0.999181176651,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    0.999181176651,
                    0.028609231262,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    -0.999181176651,
                    0.028609231262,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    0.999181176651,
                    -0.028609231262,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    -0.999181176651,
                    -0.028609231262,
                    0.028609231262,
                    0.000444241432,
                ],
                [
                    0.071425567677,
                    0.071425567677,
                    0.994885308246,
                    0.000695156804,
                ],
                [
                    -0.071425567677,
                    0.071425567677,
                    0.994885308246,
                    0.000695156804,
                ],
                [
                    0.071425567677,
                    -0.071425567677,
                    0.994885308246,
                    0.000695156804,
                ],
                [
                    -0.071425567677,
                    -0.071425567677,
                    0.994885308246,
                    0.000695156804,
                ],
                [
                    0.071425567677,
                    0.994885308246,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    -0.071425567677,
                    0.994885308246,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    0.071425567677,
                    -0.994885308246,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    -0.071425567677,
                    -0.994885308246,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    0.994885308246,
                    0.071425567677,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    -0.994885308246,
                    0.071425567677,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    0.994885308246,
                    -0.071425567677,
                    0.071425567677,
                    0.000695156804,
                ],
                [
                    -0.994885308246,
                    -0.071425567677,
                    0.071425567677,
                    0.000695156804,
                ],
                [0.1209199541, 0.1209199541, 0.98526987643, 0.000870148489],
                [-0.1209199541, 0.1209199541, 0.98526987643, 0.000870148489],
                [0.1209199541, -0.1209199541, 0.98526987643, 0.000870148489],
                [-0.1209199541, -0.1209199541, 0.98526987643, 0.000870148489],
                [0.1209199541, 0.98526987643, 0.1209199541, 0.000870148489],
                [-0.1209199541, 0.98526987643, 0.1209199541, 0.000870148489],
                [0.1209199541, -0.98526987643, 0.1209199541, 0.000870148489],
                [-0.1209199541, -0.98526987643, 0.1209199541, 0.000870148489],
                [0.98526987643, 0.1209199541, 0.1209199541, 0.000870148489],
                [-0.98526987643, 0.1209199541, 0.1209199541, 0.000870148489],
                [0.98526987643, -0.1209199541, 0.1209199541, 0.000870148489],
                [-0.98526987643, -0.1209199541, 0.1209199541, 0.000870148489],
                [
                    0.173867310659,
                    0.173867310659,
                    0.969298878865,
                    0.000995713827,
                ],
                [
                    -0.173867310659,
                    0.173867310659,
                    0.969298878865,
                    0.000995713827,
                ],
                [
                    0.173867310659,
                    -0.173867310659,
                    0.969298878865,
                    0.000995713827,
                ],
                [
                    -0.173867310659,
                    -0.173867310659,
                    0.969298878865,
                    0.000995713827,
                ],
                [
                    0.173867310659,
                    0.969298878865,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    -0.173867310659,
                    0.969298878865,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    0.173867310659,
                    -0.969298878865,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    -0.173867310659,
                    -0.969298878865,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    0.969298878865,
                    0.173867310659,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    -0.969298878865,
                    0.173867310659,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    0.969298878865,
                    -0.173867310659,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    -0.969298878865,
                    -0.173867310659,
                    0.173867310659,
                    0.000995713827,
                ],
                [
                    0.228464543847,
                    0.228464543847,
                    0.946365629347,
                    0.001087007244,
                ],
                [
                    -0.228464543847,
                    0.228464543847,
                    0.946365629347,
                    0.001087007244,
                ],
                [
                    0.228464543847,
                    -0.228464543847,
                    0.946365629347,
                    0.001087007244,
                ],
                [
                    -0.228464543847,
                    -0.228464543847,
                    0.946365629347,
                    0.001087007244,
                ],
                [
                    0.228464543847,
                    0.946365629347,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    -0.228464543847,
                    0.946365629347,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    0.228464543847,
                    -0.946365629347,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    -0.228464543847,
                    -0.946365629347,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    0.946365629347,
                    0.228464543847,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    -0.946365629347,
                    0.228464543847,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    0.946365629347,
                    -0.228464543847,
                    0.228464543847,
                    0.001087007244,
                ],
                [
                    -0.946365629347,
                    -0.228464543847,
                    0.228464543847,
                    0.001087007244,
                ],
                [0.28348076717, 0.28348076717, 0.916120794049, 0.001153182678],
                [
                    -0.28348076717,
                    0.28348076717,
                    0.916120794049,
                    0.001153182678,
                ],
                [
                    0.28348076717,
                    -0.28348076717,
                    0.916120794049,
                    0.001153182678,
                ],
                [
                    -0.28348076717,
                    -0.28348076717,
                    0.916120794049,
                    0.001153182678,
                ],
                [0.28348076717, 0.916120794049, 0.28348076717, 0.001153182678],
                [
                    -0.28348076717,
                    0.916120794049,
                    0.28348076717,
                    0.001153182678,
                ],
                [
                    0.28348076717,
                    -0.916120794049,
                    0.28348076717,
                    0.001153182678,
                ],
                [
                    -0.28348076717,
                    -0.916120794049,
                    0.28348076717,
                    0.001153182678,
                ],
                [0.916120794049, 0.28348076717, 0.28348076717, 0.001153182678],
                [
                    -0.916120794049,
                    0.28348076717,
                    0.28348076717,
                    0.001153182678,
                ],
                [
                    0.916120794049,
                    -0.28348076717,
                    0.28348076717,
                    0.001153182678,
                ],
                [
                    -0.916120794049,
                    -0.28348076717,
                    0.28348076717,
                    0.001153182678,
                ],
                [
                    0.337968014547,
                    0.337968014547,
                    0.878382173252,
                    0.001200240072,
                ],
                [
                    -0.337968014547,
                    0.337968014547,
                    0.878382173252,
                    0.001200240072,
                ],
                [
                    0.337968014547,
                    -0.337968014547,
                    0.878382173252,
                    0.001200240072,
                ],
                [
                    -0.337968014547,
                    -0.337968014547,
                    0.878382173252,
                    0.001200240072,
                ],
                [
                    0.337968014547,
                    0.878382173252,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    -0.337968014547,
                    0.878382173252,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    0.337968014547,
                    -0.878382173252,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    -0.337968014547,
                    -0.878382173252,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    0.878382173252,
                    0.337968014547,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    -0.878382173252,
                    0.337968014547,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    0.878382173252,
                    -0.337968014547,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    -0.878382173252,
                    -0.337968014547,
                    0.337968014547,
                    0.001200240072,
                ],
                [
                    0.391135545482,
                    0.391135545482,
                    0.833082210902,
                    0.001232435635,
                ],
                [
                    -0.391135545482,
                    0.391135545482,
                    0.833082210902,
                    0.001232435635,
                ],
                [
                    0.391135545482,
                    -0.391135545482,
                    0.833082210902,
                    0.001232435635,
                ],
                [
                    -0.391135545482,
                    -0.391135545482,
                    0.833082210902,
                    0.001232435635,
                ],
                [
                    0.391135545482,
                    0.833082210902,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    -0.391135545482,
                    0.833082210902,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    0.391135545482,
                    -0.833082210902,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    -0.391135545482,
                    -0.833082210902,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    0.833082210902,
                    0.391135545482,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    -0.833082210902,
                    0.391135545482,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    0.833082210902,
                    -0.391135545482,
                    0.391135545482,
                    0.001232435635,
                ],
                [
                    -0.833082210902,
                    -0.391135545482,
                    0.391135545482,
                    0.001232435635,
                ],
                [0.4422860353, 0.4422860353, 0.780234660828, 0.00125304363],
                [-0.4422860353, 0.4422860353, 0.780234660828, 0.00125304363],
                [0.4422860353, -0.4422860353, 0.780234660828, 0.00125304363],
                [-0.4422860353, -0.4422860353, 0.780234660828, 0.00125304363],
                [0.4422860353, 0.780234660828, 0.4422860353, 0.00125304363],
                [-0.4422860353, 0.780234660828, 0.4422860353, 0.00125304363],
                [0.4422860353, -0.780234660828, 0.4422860353, 0.00125304363],
                [-0.4422860353, -0.780234660828, 0.4422860353, 0.00125304363],
                [0.780234660828, 0.4422860353, 0.4422860353, 0.00125304363],
                [-0.780234660828, 0.4422860353, 0.4422860353, 0.00125304363],
                [0.780234660828, -0.4422860353, 0.4422860353, 0.00125304363],
                [-0.780234660828, -0.4422860353, 0.4422860353, 0.00125304363],
                [
                    0.490778156873,
                    0.490778156873,
                    0.719912217894,
                    0.001264797432,
                ],
                [
                    -0.490778156873,
                    0.490778156873,
                    0.719912217894,
                    0.001264797432,
                ],
                [
                    0.490778156873,
                    -0.490778156873,
                    0.719912217894,
                    0.001264797432,
                ],
                [
                    -0.490778156873,
                    -0.490778156873,
                    0.719912217894,
                    0.001264797432,
                ],
                [
                    0.490778156873,
                    0.719912217894,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    -0.490778156873,
                    0.719912217894,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    0.490778156873,
                    -0.719912217894,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    -0.490778156873,
                    -0.719912217894,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    0.719912217894,
                    0.490778156873,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    -0.719912217894,
                    0.490778156873,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    0.719912217894,
                    -0.490778156873,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    -0.719912217894,
                    -0.490778156873,
                    0.490778156873,
                    0.001264797432,
                ],
                [
                    0.536000615321,
                    0.536000615321,
                    0.652232075837,
                    0.00127015357,
                ],
                [
                    -0.536000615321,
                    0.536000615321,
                    0.652232075837,
                    0.00127015357,
                ],
                [
                    0.536000615321,
                    -0.536000615321,
                    0.652232075837,
                    0.00127015357,
                ],
                [
                    -0.536000615321,
                    -0.536000615321,
                    0.652232075837,
                    0.00127015357,
                ],
                [
                    0.536000615321,
                    0.652232075837,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    -0.536000615321,
                    0.652232075837,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    0.536000615321,
                    -0.652232075837,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    -0.536000615321,
                    -0.652232075837,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    0.652232075837,
                    0.536000615321,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    -0.652232075837,
                    0.536000615321,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    0.652232075837,
                    -0.536000615321,
                    0.536000615321,
                    0.00127015357,
                ],
                [
                    -0.652232075837,
                    -0.536000615321,
                    0.536000615321,
                    0.00127015357,
                ],
                [0.61421059736, 0.61421059736, 0.495470164775, 0.001270872555],
                [
                    -0.61421059736,
                    0.61421059736,
                    0.495470164775,
                    0.001270872555,
                ],
                [
                    0.61421059736,
                    -0.61421059736,
                    0.495470164775,
                    0.001270872555,
                ],
                [
                    -0.61421059736,
                    -0.61421059736,
                    0.495470164775,
                    0.001270872555,
                ],
                [0.61421059736, 0.495470164775, 0.61421059736, 0.001270872555],
                [
                    -0.61421059736,
                    0.495470164775,
                    0.61421059736,
                    0.001270872555,
                ],
                [
                    0.61421059736,
                    -0.495470164775,
                    0.61421059736,
                    0.001270872555,
                ],
                [
                    -0.61421059736,
                    -0.495470164775,
                    0.61421059736,
                    0.001270872555,
                ],
                [0.495470164775, 0.61421059736, 0.61421059736, 0.001270872555],
                [
                    -0.495470164775,
                    0.61421059736,
                    0.61421059736,
                    0.001270872555,
                ],
                [
                    0.495470164775,
                    -0.61421059736,
                    0.61421059736,
                    0.001270872555,
                ],
                [
                    -0.495470164775,
                    -0.61421059736,
                    0.61421059736,
                    0.001270872555,
                ],
                [
                    0.645930038798,
                    0.645930038798,
                    0.406876848638,
                    0.001270460493,
                ],
                [
                    -0.645930038798,
                    0.645930038798,
                    0.406876848638,
                    0.001270460493,
                ],
                [
                    0.645930038798,
                    -0.645930038798,
                    0.406876848638,
                    0.001270460493,
                ],
                [
                    -0.645930038798,
                    -0.645930038798,
                    0.406876848638,
                    0.001270460493,
                ],
                [
                    0.645930038798,
                    0.406876848638,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    -0.645930038798,
                    0.406876848638,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    0.645930038798,
                    -0.406876848638,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    -0.645930038798,
                    -0.406876848638,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    0.406876848638,
                    0.645930038798,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    -0.406876848638,
                    0.645930038798,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    0.406876848638,
                    -0.645930038798,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    -0.406876848638,
                    -0.645930038798,
                    0.645930038798,
                    0.001270460493,
                ],
                [
                    0.671805612509,
                    0.671805612509,
                    0.312016727121,
                    0.001271623576,
                ],
                [
                    -0.671805612509,
                    0.671805612509,
                    0.312016727121,
                    0.001271623576,
                ],
                [
                    0.671805612509,
                    -0.671805612509,
                    0.312016727121,
                    0.001271623576,
                ],
                [
                    -0.671805612509,
                    -0.671805612509,
                    0.312016727121,
                    0.001271623576,
                ],
                [
                    0.671805612509,
                    0.312016727121,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    -0.671805612509,
                    0.312016727121,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    0.671805612509,
                    -0.312016727121,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    -0.671805612509,
                    -0.312016727121,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    0.312016727121,
                    0.671805612509,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    -0.312016727121,
                    0.671805612509,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    0.312016727121,
                    -0.671805612509,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    -0.312016727121,
                    -0.671805612509,
                    0.671805612509,
                    0.001271623576,
                ],
                [
                    0.691088853319,
                    0.691088853319,
                    0.21164213578,
                    0.001274620318,
                ],
                [
                    -0.691088853319,
                    0.691088853319,
                    0.21164213578,
                    0.001274620318,
                ],
                [
                    0.691088853319,
                    -0.691088853319,
                    0.21164213578,
                    0.001274620318,
                ],
                [
                    -0.691088853319,
                    -0.691088853319,
                    0.21164213578,
                    0.001274620318,
                ],
                [
                    0.691088853319,
                    0.21164213578,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    -0.691088853319,
                    0.21164213578,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    0.691088853319,
                    -0.21164213578,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    -0.691088853319,
                    -0.21164213578,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    0.21164213578,
                    0.691088853319,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    -0.21164213578,
                    0.691088853319,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    0.21164213578,
                    -0.691088853319,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    -0.21164213578,
                    -0.691088853319,
                    0.691088853319,
                    0.001274620318,
                ],
                [
                    0.703046741682,
                    0.703046741682,
                    0.107007280218,
                    0.001278085792,
                ],
                [
                    -0.703046741682,
                    0.703046741682,
                    0.107007280218,
                    0.001278085792,
                ],
                [
                    0.703046741682,
                    -0.703046741682,
                    0.107007280218,
                    0.001278085792,
                ],
                [
                    -0.703046741682,
                    -0.703046741682,
                    0.107007280218,
                    0.001278085792,
                ],
                [
                    0.703046741682,
                    0.107007280218,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    -0.703046741682,
                    0.107007280218,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    0.703046741682,
                    -0.107007280218,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    -0.703046741682,
                    -0.107007280218,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    0.107007280218,
                    0.703046741682,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    -0.107007280218,
                    0.703046741682,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    0.107007280218,
                    -0.703046741682,
                    0.703046741682,
                    0.001278085792,
                ],
                [
                    -0.107007280218,
                    -0.703046741682,
                    0.703046741682,
                    0.001278085792,
                ],
                [0.083549511664, 0.996503627239, 0.0, 0.000318691345],
                [-0.083549511664, 0.996503627239, 0.0, 0.000318691345],
                [0.083549511664, -0.996503627239, 0.0, 0.000318691345],
                [-0.083549511664, -0.996503627239, 0.0, 0.000318691345],
                [0.996503627239, 0.083549511664, 0.0, 0.000318691345],
                [-0.996503627239, 0.083549511664, 0.0, 0.000318691345],
                [0.996503627239, -0.083549511664, 0.0, 0.000318691345],
                [-0.996503627239, -0.083549511664, 0.0, 0.000318691345],
                [0.083549511664, 0.0, 0.996503627239, 0.00063738269],
                [-0.083549511664, 0.0, 0.996503627239, 0.00063738269],
                [0.996503627239, 0.0, 0.083549511664, 0.00063738269],
                [-0.996503627239, 0.0, 0.083549511664, 0.00063738269],
                [0.0, 0.083549511664, 0.996503627239, 0.00063738269],
                [0.0, -0.083549511664, 0.996503627239, 0.00063738269],
                [0.0, 0.996503627239, 0.083549511664, 0.00063738269],
                [0.0, -0.996503627239, 0.083549511664, 0.00063738269],
                [0.20501430091, 0.978758977697, 0.0, 0.000467802856],
                [-0.20501430091, 0.978758977697, 0.0, 0.000467802856],
                [0.20501430091, -0.978758977697, 0.0, 0.000467802856],
                [-0.20501430091, -0.978758977697, 0.0, 0.000467802856],
                [0.978758977697, 0.20501430091, 0.0, 0.000467802856],
                [-0.978758977697, 0.20501430091, 0.0, 0.000467802856],
                [0.978758977697, -0.20501430091, 0.0, 0.000467802856],
                [-0.978758977697, -0.20501430091, 0.0, 0.000467802856],
                [0.20501430091, 0.0, 0.978758977697, 0.000935605712],
                [-0.20501430091, 0.0, 0.978758977697, 0.000935605712],
                [0.978758977697, 0.0, 0.20501430091, 0.000935605712],
                [-0.978758977697, 0.0, 0.20501430091, 0.000935605712],
                [0.0, 0.20501430091, 0.978758977697, 0.000935605712],
                [0.0, -0.20501430091, 0.978758977697, 0.000935605712],
                [0.0, 0.978758977697, 0.20501430091, 0.000935605712],
                [0.0, -0.978758977697, 0.20501430091, 0.000935605712],
                [0.337020829071, 0.941497191059, 0.0, 0.00055388297],
                [-0.337020829071, 0.941497191059, 0.0, 0.00055388297],
                [0.337020829071, -0.941497191059, 0.0, 0.00055388297],
                [-0.337020829071, -0.941497191059, 0.0, 0.00055388297],
                [0.941497191059, 0.337020829071, 0.0, 0.00055388297],
                [-0.941497191059, 0.337020829071, 0.0, 0.00055388297],
                [0.941497191059, -0.337020829071, 0.0, 0.00055388297],
                [-0.941497191059, -0.337020829071, 0.0, 0.00055388297],
                [0.337020829071, 0.0, 0.941497191059, 0.00110776594],
                [-0.337020829071, 0.0, 0.941497191059, 0.00110776594],
                [0.941497191059, 0.0, 0.337020829071, 0.00110776594],
                [-0.941497191059, 0.0, 0.337020829071, 0.00110776594],
                [0.0, 0.337020829071, 0.941497191059, 0.00110776594],
                [0.0, -0.337020829071, 0.941497191059, 0.00110776594],
                [0.0, 0.941497191059, 0.337020829071, 0.00110776594],
                [0.0, -0.941497191059, 0.337020829071, 0.00110776594],
                [0.468905148423, 0.883248527755, 0.0, 0.000604447591],
                [-0.468905148423, 0.883248527755, 0.0, 0.000604447591],
                [0.468905148423, -0.883248527755, 0.0, 0.000604447591],
                [-0.468905148423, -0.883248527755, 0.0, 0.000604447591],
                [0.883248527755, 0.468905148423, 0.0, 0.000604447591],
                [-0.883248527755, 0.468905148423, 0.0, 0.000604447591],
                [0.883248527755, -0.468905148423, 0.0, 0.000604447591],
                [-0.883248527755, -0.468905148423, 0.0, 0.000604447591],
                [0.468905148423, 0.0, 0.883248527755, 0.001208895181],
                [-0.468905148423, 0.0, 0.883248527755, 0.001208895181],
                [0.883248527755, 0.0, 0.468905148423, 0.001208895181],
                [-0.883248527755, 0.0, 0.468905148423, 0.001208895181],
                [0.0, 0.468905148423, 0.883248527755, 0.001208895181],
                [0.0, -0.468905148423, 0.883248527755, 0.001208895181],
                [0.0, 0.883248527755, 0.468905148423, 0.001208895181],
                [0.0, -0.883248527755, 0.468905148423, 0.001208895181],
                [0.593940042456, 0.804509307571, 0.0, 0.00063135751],
                [-0.593940042456, 0.804509307571, 0.0, 0.00063135751],
                [0.593940042456, -0.804509307571, 0.0, 0.00063135751],
                [-0.593940042456, -0.804509307571, 0.0, 0.00063135751],
                [0.804509307571, 0.593940042456, 0.0, 0.00063135751],
                [-0.804509307571, 0.593940042456, 0.0, 0.00063135751],
                [0.804509307571, -0.593940042456, 0.0, 0.00063135751],
                [-0.804509307571, -0.593940042456, 0.0, 0.00063135751],
                [0.593940042456, 0.0, 0.804509307571, 0.001262715021],
                [-0.593940042456, 0.0, 0.804509307571, 0.001262715021],
                [0.804509307571, 0.0, 0.593940042456, 0.001262715021],
                [-0.804509307571, 0.0, 0.593940042456, 0.001262715021],
                [0.0, 0.593940042456, 0.804509307571, 0.001262715021],
                [0.0, -0.593940042456, 0.804509307571, 0.001262715021],
                [0.0, 0.804509307571, 0.593940042456, 0.001262715021],
                [0.0, -0.804509307571, 0.593940042456, 0.001262715021],
                [
                    0.139498331183,
                    0.040975811621,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    -0.139498331183,
                    0.040975811621,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    0.139498331183,
                    -0.040975811621,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    -0.139498331183,
                    -0.040975811621,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    0.139498331183,
                    0.989374144831,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    -0.139498331183,
                    0.989374144831,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    0.139498331183,
                    -0.989374144831,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    -0.139498331183,
                    -0.989374144831,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    0.040975811621,
                    0.139498331183,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    -0.040975811621,
                    0.139498331183,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    0.040975811621,
                    -0.139498331183,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    -0.040975811621,
                    -0.139498331183,
                    0.989374144831,
                    0.000815725286,
                ],
                [
                    0.040975811621,
                    0.989374144831,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    -0.040975811621,
                    0.989374144831,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    0.040975811621,
                    -0.989374144831,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    -0.040975811621,
                    -0.989374144831,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    0.989374144831,
                    0.139498331183,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    -0.989374144831,
                    0.139498331183,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    0.989374144831,
                    -0.139498331183,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    -0.989374144831,
                    -0.139498331183,
                    0.040975811621,
                    0.000815725286,
                ],
                [
                    0.989374144831,
                    0.040975811621,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    -0.989374144831,
                    0.040975811621,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    0.989374144831,
                    -0.040975811621,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    -0.989374144831,
                    -0.040975811621,
                    0.139498331183,
                    0.000815725286,
                ],
                [
                    0.196799918049,
                    0.088519873913,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    -0.196799918049,
                    0.088519873913,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    0.196799918049,
                    -0.088519873913,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    -0.196799918049,
                    -0.088519873913,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    0.196799918049,
                    0.976439462629,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    -0.196799918049,
                    0.976439462629,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    0.196799918049,
                    -0.976439462629,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    -0.196799918049,
                    -0.976439462629,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    0.088519873913,
                    0.196799918049,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    -0.088519873913,
                    0.196799918049,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    0.088519873913,
                    -0.196799918049,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    -0.088519873913,
                    -0.196799918049,
                    0.976439462629,
                    0.000951986612,
                ],
                [
                    0.088519873913,
                    0.976439462629,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    -0.088519873913,
                    0.976439462629,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    0.088519873913,
                    -0.976439462629,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    -0.088519873913,
                    -0.976439462629,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    0.976439462629,
                    0.196799918049,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    -0.976439462629,
                    0.196799918049,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    0.976439462629,
                    -0.196799918049,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    -0.976439462629,
                    -0.196799918049,
                    0.088519873913,
                    0.000951986612,
                ],
                [
                    0.976439462629,
                    0.088519873913,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    -0.976439462629,
                    0.088519873913,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    0.976439462629,
                    -0.088519873913,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    -0.976439462629,
                    -0.088519873913,
                    0.196799918049,
                    0.000951986612,
                ],
                [
                    0.254618373255,
                    0.139768018297,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    -0.254618373255,
                    0.139768018297,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    0.254618373255,
                    -0.139768018297,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    -0.254618373255,
                    -0.139768018297,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    0.254618373255,
                    0.956887864414,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    -0.254618373255,
                    0.956887864414,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    0.254618373255,
                    -0.956887864414,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    -0.254618373255,
                    -0.956887864414,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    0.139768018297,
                    0.254618373255,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    -0.139768018297,
                    0.254618373255,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    0.139768018297,
                    -0.254618373255,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    -0.139768018297,
                    -0.254618373255,
                    0.956887864414,
                    0.001053630237,
                ],
                [
                    0.139768018297,
                    0.956887864414,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    -0.139768018297,
                    0.956887864414,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    0.139768018297,
                    -0.956887864414,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    -0.139768018297,
                    -0.956887864414,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    0.956887864414,
                    0.254618373255,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    -0.956887864414,
                    0.254618373255,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    0.956887864414,
                    -0.254618373255,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    -0.956887864414,
                    -0.254618373255,
                    0.139768018297,
                    0.001053630237,
                ],
                [
                    0.956887864414,
                    0.139768018297,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    -0.956887864414,
                    0.139768018297,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    0.956887864414,
                    -0.139768018297,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    -0.956887864414,
                    -0.139768018297,
                    0.254618373255,
                    0.001053630237,
                ],
                [
                    0.312128107471,
                    0.192945254223,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    -0.312128107471,
                    0.192945254223,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    0.312128107471,
                    -0.192945254223,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    -0.312128107471,
                    -0.192945254223,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    0.312128107471,
                    0.930240922234,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    -0.312128107471,
                    0.930240922234,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    0.312128107471,
                    -0.930240922234,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    -0.312128107471,
                    -0.930240922234,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    0.192945254223,
                    0.312128107471,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    -0.192945254223,
                    0.312128107471,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    0.192945254223,
                    -0.312128107471,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    -0.192945254223,
                    -0.312128107471,
                    0.930240922234,
                    0.001128609712,
                ],
                [
                    0.192945254223,
                    0.930240922234,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    -0.192945254223,
                    0.930240922234,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    0.192945254223,
                    -0.930240922234,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    -0.192945254223,
                    -0.930240922234,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    0.930240922234,
                    0.312128107471,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    -0.930240922234,
                    0.312128107471,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    0.930240922234,
                    -0.312128107471,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    -0.930240922234,
                    -0.312128107471,
                    0.192945254223,
                    0.001128609712,
                ],
                [
                    0.930240922234,
                    0.192945254223,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    -0.930240922234,
                    0.192945254223,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    0.930240922234,
                    -0.192945254223,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    -0.930240922234,
                    -0.192945254223,
                    0.312128107471,
                    0.001128609712,
                ],
                [
                    0.36859810785,
                    0.246789833706,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    -0.36859810785,
                    0.246789833706,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    0.36859810785,
                    -0.246789833706,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    -0.36859810785,
                    -0.246789833706,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    0.36859810785,
                    0.896231115767,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    -0.36859810785,
                    0.896231115767,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    0.36859810785,
                    -0.896231115767,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    -0.36859810785,
                    -0.896231115767,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    0.246789833706,
                    0.36859810785,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    -0.246789833706,
                    0.36859810785,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    0.246789833706,
                    -0.36859810785,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    -0.246789833706,
                    -0.36859810785,
                    0.896231115767,
                    0.001182900215,
                ],
                [
                    0.246789833706,
                    0.896231115767,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    -0.246789833706,
                    0.896231115767,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    0.246789833706,
                    -0.896231115767,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    -0.246789833706,
                    -0.896231115767,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    0.896231115767,
                    0.36859810785,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    -0.896231115767,
                    0.36859810785,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    0.896231115767,
                    -0.36859810785,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    -0.896231115767,
                    -0.36859810785,
                    0.246789833706,
                    0.001182900215,
                ],
                [
                    0.896231115767,
                    0.246789833706,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    -0.896231115767,
                    0.246789833706,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    0.896231115767,
                    -0.246789833706,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    -0.896231115767,
                    -0.246789833706,
                    0.36859810785,
                    0.001182900215,
                ],
                [
                    0.423376032155,
                    0.300310412479,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    -0.423376032155,
                    0.300310412479,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    0.423376032155,
                    -0.300310412479,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    -0.423376032155,
                    -0.300310412479,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    0.423376032155,
                    0.85473176585,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    -0.423376032155,
                    0.85473176585,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    0.423376032155,
                    -0.85473176585,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    -0.423376032155,
                    -0.85473176585,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    0.300310412479,
                    0.423376032155,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    -0.300310412479,
                    0.423376032155,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    0.300310412479,
                    -0.423376032155,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    -0.300310412479,
                    -0.423376032155,
                    0.85473176585,
                    0.001220912252,
                ],
                [
                    0.300310412479,
                    0.85473176585,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    -0.300310412479,
                    0.85473176585,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    0.300310412479,
                    -0.85473176585,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    -0.300310412479,
                    -0.85473176585,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    0.85473176585,
                    0.423376032155,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    -0.85473176585,
                    0.423376032155,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    0.85473176585,
                    -0.423376032155,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    -0.85473176585,
                    -0.423376032155,
                    0.300310412479,
                    0.001220912252,
                ],
                [
                    0.85473176585,
                    0.300310412479,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    -0.85473176585,
                    0.300310412479,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    0.85473176585,
                    -0.300310412479,
                    0.423376032155,
                    0.001220912252,
                ],
                [
                    -0.85473176585,
                    -0.300310412479,
                    0.423376032155,
                    0.001220912252,
                ],
                [0.475867123606, 0.352668432818, 0.8057142528, 0.001246050572],
                [
                    -0.475867123606,
                    0.352668432818,
                    0.8057142528,
                    0.001246050572,
                ],
                [
                    0.475867123606,
                    -0.352668432818,
                    0.8057142528,
                    0.001246050572,
                ],
                [
                    -0.475867123606,
                    -0.352668432818,
                    0.8057142528,
                    0.001246050572,
                ],
                [0.475867123606, 0.8057142528, 0.352668432818, 0.001246050572],
                [
                    -0.475867123606,
                    0.8057142528,
                    0.352668432818,
                    0.001246050572,
                ],
                [
                    0.475867123606,
                    -0.8057142528,
                    0.352668432818,
                    0.001246050572,
                ],
                [
                    -0.475867123606,
                    -0.8057142528,
                    0.352668432818,
                    0.001246050572,
                ],
                [0.352668432818, 0.475867123606, 0.8057142528, 0.001246050572],
                [
                    -0.352668432818,
                    0.475867123606,
                    0.8057142528,
                    0.001246050572,
                ],
                [
                    0.352668432818,
                    -0.475867123606,
                    0.8057142528,
                    0.001246050572,
                ],
                [
                    -0.352668432818,
                    -0.475867123606,
                    0.8057142528,
                    0.001246050572,
                ],
                [0.352668432818, 0.8057142528, 0.475867123606, 0.001246050572],
                [
                    -0.352668432818,
                    0.8057142528,
                    0.475867123606,
                    0.001246050572,
                ],
                [
                    0.352668432818,
                    -0.8057142528,
                    0.475867123606,
                    0.001246050572,
                ],
                [
                    -0.352668432818,
                    -0.8057142528,
                    0.475867123606,
                    0.001246050572,
                ],
                [0.8057142528, 0.475867123606, 0.352668432818, 0.001246050572],
                [
                    -0.8057142528,
                    0.475867123606,
                    0.352668432818,
                    0.001246050572,
                ],
                [
                    0.8057142528,
                    -0.475867123606,
                    0.352668432818,
                    0.001246050572,
                ],
                [
                    -0.8057142528,
                    -0.475867123606,
                    0.352668432818,
                    0.001246050572,
                ],
                [0.8057142528, 0.352668432818, 0.475867123606, 0.001246050572],
                [
                    -0.8057142528,
                    0.352668432818,
                    0.475867123606,
                    0.001246050572,
                ],
                [
                    0.8057142528,
                    -0.352668432818,
                    0.475867123606,
                    0.001246050572,
                ],
                [
                    -0.8057142528,
                    -0.352668432818,
                    0.475867123606,
                    0.001246050572,
                ],
                [
                    0.52551785798,
                    0.403113486115,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    -0.52551785798,
                    0.403113486115,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    0.52551785798,
                    -0.403113486115,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    -0.52551785798,
                    -0.403113486115,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    0.52551785798,
                    0.749219926495,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    -0.52551785798,
                    0.749219926495,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    0.52551785798,
                    -0.749219926495,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    -0.52551785798,
                    -0.749219926495,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    0.403113486115,
                    0.52551785798,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    -0.403113486115,
                    0.52551785798,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    0.403113486115,
                    -0.52551785798,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    -0.403113486115,
                    -0.52551785798,
                    0.749219926495,
                    0.001261123752,
                ],
                [
                    0.403113486115,
                    0.749219926495,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    -0.403113486115,
                    0.749219926495,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    0.403113486115,
                    -0.749219926495,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    -0.403113486115,
                    -0.749219926495,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    0.749219926495,
                    0.52551785798,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    -0.749219926495,
                    0.52551785798,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    0.749219926495,
                    -0.52551785798,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    -0.749219926495,
                    -0.52551785798,
                    0.403113486115,
                    0.001261123752,
                ],
                [
                    0.749219926495,
                    0.403113486115,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    -0.749219926495,
                    0.403113486115,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    0.749219926495,
                    -0.403113486115,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    -0.749219926495,
                    -0.403113486115,
                    0.52551785798,
                    0.001261123752,
                ],
                [
                    0.571802563373,
                    0.450942644834,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    -0.571802563373,
                    0.450942644834,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    0.571802563373,
                    -0.450942644834,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    -0.571802563373,
                    -0.450942644834,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    0.571802563373,
                    0.685341199396,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    -0.571802563373,
                    0.685341199396,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    0.571802563373,
                    -0.685341199396,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    -0.571802563373,
                    -0.685341199396,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    0.450942644834,
                    0.571802563373,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    -0.450942644834,
                    0.571802563373,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    0.450942644834,
                    -0.571802563373,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    -0.450942644834,
                    -0.571802563373,
                    0.685341199396,
                    0.001268618554,
                ],
                [
                    0.450942644834,
                    0.685341199396,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    -0.450942644834,
                    0.685341199396,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    0.450942644834,
                    -0.685341199396,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    -0.450942644834,
                    -0.685341199396,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    0.685341199396,
                    0.571802563373,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    -0.685341199396,
                    0.571802563373,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    0.685341199396,
                    -0.571802563373,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    -0.685341199396,
                    -0.571802563373,
                    0.450942644834,
                    0.001268618554,
                ],
                [
                    0.685341199396,
                    0.450942644834,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    -0.685341199396,
                    0.450942644834,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    0.685341199396,
                    -0.450942644834,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    -0.685341199396,
                    -0.450942644834,
                    0.571802563373,
                    0.001268618554,
                ],
                [
                    0.268692777272,
                    0.047113225024,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    -0.268692777272,
                    0.047113225024,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    0.268692777272,
                    -0.047113225024,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    -0.268692777272,
                    -0.047113225024,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    0.268692777272,
                    0.96207304061,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    -0.268692777272,
                    0.96207304061,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    0.268692777272,
                    -0.96207304061,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    -0.268692777272,
                    -0.96207304061,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    0.047113225024,
                    0.268692777272,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    -0.047113225024,
                    0.268692777272,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    0.047113225024,
                    -0.268692777272,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    -0.047113225024,
                    -0.268692777272,
                    0.96207304061,
                    0.001035253789,
                ],
                [
                    0.047113225024,
                    0.96207304061,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    -0.047113225024,
                    0.96207304061,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    0.047113225024,
                    -0.96207304061,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    -0.047113225024,
                    -0.96207304061,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    0.96207304061,
                    0.268692777272,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    -0.96207304061,
                    0.268692777272,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    0.96207304061,
                    -0.268692777272,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    -0.96207304061,
                    -0.268692777272,
                    0.047113225024,
                    0.001035253789,
                ],
                [
                    0.96207304061,
                    0.047113225024,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    -0.96207304061,
                    0.047113225024,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    0.96207304061,
                    -0.047113225024,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    -0.96207304061,
                    -0.047113225024,
                    0.268692777272,
                    0.001035253789,
                ],
                [
                    0.33060068199,
                    0.097844873039,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    -0.33060068199,
                    0.097844873039,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    0.33060068199,
                    -0.097844873039,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    -0.33060068199,
                    -0.097844873039,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    0.33060068199,
                    0.938685021659,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    -0.33060068199,
                    0.938685021659,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    0.33060068199,
                    -0.938685021659,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    -0.33060068199,
                    -0.938685021659,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    0.097844873039,
                    0.33060068199,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    -0.097844873039,
                    0.33060068199,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    0.097844873039,
                    -0.33060068199,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    -0.097844873039,
                    -0.33060068199,
                    0.938685021659,
                    0.001112968063,
                ],
                [
                    0.097844873039,
                    0.938685021659,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    -0.097844873039,
                    0.938685021659,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    0.097844873039,
                    -0.938685021659,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    -0.097844873039,
                    -0.938685021659,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    0.938685021659,
                    0.33060068199,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    -0.938685021659,
                    0.33060068199,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    0.938685021659,
                    -0.33060068199,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    -0.938685021659,
                    -0.33060068199,
                    0.097844873039,
                    0.001112968063,
                ],
                [
                    0.938685021659,
                    0.097844873039,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    -0.938685021659,
                    0.097844873039,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    0.938685021659,
                    -0.097844873039,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    -0.938685021659,
                    -0.097844873039,
                    0.33060068199,
                    0.001112968063,
                ],
                [
                    0.390490685059,
                    0.150539581003,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    -0.390490685059,
                    0.150539581003,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    0.390490685059,
                    -0.150539581003,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    -0.390490685059,
                    -0.150539581003,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    0.390490685059,
                    0.908215205463,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    -0.390490685059,
                    0.908215205463,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    0.390490685059,
                    -0.908215205463,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    -0.390490685059,
                    -0.908215205463,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    0.150539581003,
                    0.390490685059,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    -0.150539581003,
                    0.390490685059,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    0.150539581003,
                    -0.390490685059,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    -0.150539581003,
                    -0.390490685059,
                    0.908215205463,
                    0.001171285334,
                ],
                [
                    0.150539581003,
                    0.908215205463,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    -0.150539581003,
                    0.908215205463,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    0.150539581003,
                    -0.908215205463,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    -0.150539581003,
                    -0.908215205463,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    0.908215205463,
                    0.390490685059,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    -0.908215205463,
                    0.390490685059,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    0.908215205463,
                    -0.390490685059,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    -0.908215205463,
                    -0.390490685059,
                    0.150539581003,
                    0.001171285334,
                ],
                [
                    0.908215205463,
                    0.150539581003,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    -0.908215205463,
                    0.150539581003,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    0.908215205463,
                    -0.150539581003,
                    0.390490685059,
                    0.001171285334,
                ],
                [
                    -0.908215205463,
                    -0.150539581003,
                    0.390490685059,
                    0.001171285334,
                ],
                [0.44799579519, 0.20397281563, 0.870456695061, 0.001213277385],
                [
                    -0.44799579519,
                    0.20397281563,
                    0.870456695061,
                    0.001213277385,
                ],
                [
                    0.44799579519,
                    -0.20397281563,
                    0.870456695061,
                    0.001213277385,
                ],
                [
                    -0.44799579519,
                    -0.20397281563,
                    0.870456695061,
                    0.001213277385,
                ],
                [0.44799579519, 0.870456695061, 0.20397281563, 0.001213277385],
                [
                    -0.44799579519,
                    0.870456695061,
                    0.20397281563,
                    0.001213277385,
                ],
                [
                    0.44799579519,
                    -0.870456695061,
                    0.20397281563,
                    0.001213277385,
                ],
                [
                    -0.44799579519,
                    -0.870456695061,
                    0.20397281563,
                    0.001213277385,
                ],
                [0.20397281563, 0.44799579519, 0.870456695061, 0.001213277385],
                [
                    -0.20397281563,
                    0.44799579519,
                    0.870456695061,
                    0.001213277385,
                ],
                [
                    0.20397281563,
                    -0.44799579519,
                    0.870456695061,
                    0.001213277385,
                ],
                [
                    -0.20397281563,
                    -0.44799579519,
                    0.870456695061,
                    0.001213277385,
                ],
                [0.20397281563, 0.870456695061, 0.44799579519, 0.001213277385],
                [
                    -0.20397281563,
                    0.870456695061,
                    0.44799579519,
                    0.001213277385,
                ],
                [
                    0.20397281563,
                    -0.870456695061,
                    0.44799579519,
                    0.001213277385,
                ],
                [
                    -0.20397281563,
                    -0.870456695061,
                    0.44799579519,
                    0.001213277385,
                ],
                [0.870456695061, 0.44799579519, 0.20397281563, 0.001213277385],
                [
                    -0.870456695061,
                    0.44799579519,
                    0.20397281563,
                    0.001213277385,
                ],
                [
                    0.870456695061,
                    -0.44799579519,
                    0.20397281563,
                    0.001213277385,
                ],
                [
                    -0.870456695061,
                    -0.44799579519,
                    0.20397281563,
                    0.001213277385,
                ],
                [0.870456695061, 0.20397281563, 0.44799579519, 0.001213277385],
                [
                    -0.870456695061,
                    0.20397281563,
                    0.44799579519,
                    0.001213277385,
                ],
                [
                    0.870456695061,
                    -0.20397281563,
                    0.44799579519,
                    0.001213277385,
                ],
                [
                    -0.870456695061,
                    -0.20397281563,
                    0.44799579519,
                    0.001213277385,
                ],
                [
                    0.502707684892,
                    0.257152994112,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    -0.502707684892,
                    0.257152994112,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    0.502707684892,
                    -0.257152994112,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    -0.502707684892,
                    -0.257152994112,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    0.502707684892,
                    0.825322555835,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    -0.502707684892,
                    0.825322555835,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    0.502707684892,
                    -0.825322555835,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    -0.502707684892,
                    -0.825322555835,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    0.257152994112,
                    0.502707684892,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    -0.257152994112,
                    0.502707684892,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    0.257152994112,
                    -0.502707684892,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    -0.257152994112,
                    -0.502707684892,
                    0.825322555835,
                    0.001241764992,
                ],
                [
                    0.257152994112,
                    0.825322555835,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    -0.257152994112,
                    0.825322555835,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    0.257152994112,
                    -0.825322555835,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    -0.257152994112,
                    -0.825322555835,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    0.825322555835,
                    0.502707684892,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    -0.825322555835,
                    0.502707684892,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    0.825322555835,
                    -0.502707684892,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    -0.825322555835,
                    -0.502707684892,
                    0.257152994112,
                    0.001241764992,
                ],
                [
                    0.825322555835,
                    0.257152994112,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    -0.825322555835,
                    0.257152994112,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    0.825322555835,
                    -0.257152994112,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    -0.825322555835,
                    -0.257152994112,
                    0.502707684892,
                    0.001241764992,
                ],
                [
                    0.554208739226,
                    0.309219137582,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    -0.554208739226,
                    0.309219137582,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    0.554208739226,
                    -0.309219137582,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    -0.554208739226,
                    -0.309219137582,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    0.554208739226,
                    0.772810583726,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    -0.554208739226,
                    0.772810583726,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    0.554208739226,
                    -0.772810583726,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    -0.554208739226,
                    -0.772810583726,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    0.309219137582,
                    0.554208739226,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    -0.309219137582,
                    0.554208739226,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    0.309219137582,
                    -0.554208739226,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    -0.309219137582,
                    -0.554208739226,
                    0.772810583726,
                    0.00125926286,
                ],
                [
                    0.309219137582,
                    0.772810583726,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    -0.309219137582,
                    0.772810583726,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    0.309219137582,
                    -0.772810583726,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    -0.309219137582,
                    -0.772810583726,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    0.772810583726,
                    0.554208739226,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    -0.772810583726,
                    0.554208739226,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    0.772810583726,
                    -0.554208739226,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    -0.772810583726,
                    -0.554208739226,
                    0.309219137582,
                    0.00125926286,
                ],
                [
                    0.772810583726,
                    0.309219137582,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    -0.772810583726,
                    0.309219137582,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    0.772810583726,
                    -0.309219137582,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    -0.772810583726,
                    -0.309219137582,
                    0.554208739226,
                    0.00125926286,
                ],
                [
                    0.602085088738,
                    0.359380750613,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    -0.602085088738,
                    0.359380750613,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    0.602085088738,
                    -0.359380750613,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    -0.602085088738,
                    -0.359380750613,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    0.602085088738,
                    0.712978977256,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    -0.602085088738,
                    0.712978977256,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    0.602085088738,
                    -0.712978977256,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    -0.602085088738,
                    -0.712978977256,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    0.359380750613,
                    0.602085088738,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    -0.359380750613,
                    0.602085088738,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    0.359380750613,
                    -0.602085088738,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    -0.359380750613,
                    -0.602085088738,
                    0.712978977256,
                    0.001268084751,
                ],
                [
                    0.359380750613,
                    0.712978977256,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    -0.359380750613,
                    0.712978977256,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    0.359380750613,
                    -0.712978977256,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    -0.359380750613,
                    -0.712978977256,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    0.712978977256,
                    0.602085088738,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    -0.712978977256,
                    0.602085088738,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    0.712978977256,
                    -0.602085088738,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    -0.712978977256,
                    -0.602085088738,
                    0.359380750613,
                    0.001268084751,
                ],
                [
                    0.712978977256,
                    0.359380750613,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    -0.712978977256,
                    0.359380750613,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    0.712978977256,
                    -0.359380750613,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    -0.712978977256,
                    -0.359380750613,
                    0.602085088738,
                    0.001268084751,
                ],
                [
                    0.401985140918,
                    0.050633899344,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    -0.401985140918,
                    0.050633899344,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    0.401985140918,
                    -0.050633899344,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    -0.401985140918,
                    -0.050633899344,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    0.401985140918,
                    0.914245128354,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    -0.401985140918,
                    0.914245128354,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    0.401985140918,
                    -0.914245128354,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    -0.401985140918,
                    -0.914245128354,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    0.050633899344,
                    0.401985140918,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    -0.050633899344,
                    0.401985140918,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    0.050633899344,
                    -0.401985140918,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    -0.050633899344,
                    -0.401985140918,
                    0.914245128354,
                    0.001165925535,
                ],
                [
                    0.050633899344,
                    0.914245128354,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    -0.050633899344,
                    0.914245128354,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    0.050633899344,
                    -0.914245128354,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    -0.050633899344,
                    -0.914245128354,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    0.914245128354,
                    0.401985140918,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    -0.914245128354,
                    0.401985140918,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    0.914245128354,
                    -0.401985140918,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    -0.914245128354,
                    -0.401985140918,
                    0.050633899344,
                    0.001165925535,
                ],
                [
                    0.914245128354,
                    0.050633899344,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    -0.914245128354,
                    0.050633899344,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    0.914245128354,
                    -0.050633899344,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    -0.914245128354,
                    -0.050633899344,
                    0.401985140918,
                    0.001165925535,
                ],
                [
                    0.463561456745,
                    0.103242226916,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    -0.463561456745,
                    0.103242226916,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    0.463561456745,
                    -0.103242226916,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    -0.463561456745,
                    -0.103242226916,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    0.463561456745,
                    0.880029441781,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    -0.463561456745,
                    0.880029441781,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    0.463561456745,
                    -0.880029441781,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    -0.463561456745,
                    -0.880029441781,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    0.103242226916,
                    0.463561456745,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    -0.103242226916,
                    0.463561456745,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    0.103242226916,
                    -0.463561456745,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    -0.103242226916,
                    -0.463561456745,
                    0.880029441781,
                    0.001209738675,
                ],
                [
                    0.103242226916,
                    0.880029441781,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    -0.103242226916,
                    0.880029441781,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    0.103242226916,
                    -0.880029441781,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    -0.103242226916,
                    -0.880029441781,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    0.880029441781,
                    0.463561456745,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    -0.880029441781,
                    0.463561456745,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    0.880029441781,
                    -0.463561456745,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    -0.880029441781,
                    -0.463561456745,
                    0.103242226916,
                    0.001209738675,
                ],
                [
                    0.880029441781,
                    0.103242226916,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    -0.880029441781,
                    0.103242226916,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    0.880029441781,
                    -0.103242226916,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    -0.880029441781,
                    -0.103242226916,
                    0.463561456745,
                    0.001209738675,
                ],
                [
                    0.521586093159,
                    0.156632209401,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    -0.521586093159,
                    0.156632209401,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    0.521586093159,
                    -0.156632209401,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    -0.521586093159,
                    -0.156632209401,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    0.521586093159,
                    0.83869797806,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    -0.521586093159,
                    0.83869797806,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    0.521586093159,
                    -0.83869797806,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    -0.521586093159,
                    -0.83869797806,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    0.156632209401,
                    0.521586093159,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    -0.156632209401,
                    0.521586093159,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    0.156632209401,
                    -0.521586093159,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    -0.156632209401,
                    -0.521586093159,
                    0.83869797806,
                    0.001240472464,
                ],
                [
                    0.156632209401,
                    0.83869797806,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    -0.156632209401,
                    0.83869797806,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    0.156632209401,
                    -0.83869797806,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    -0.156632209401,
                    -0.83869797806,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    0.83869797806,
                    0.521586093159,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    -0.83869797806,
                    0.521586093159,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    0.83869797806,
                    -0.521586093159,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    -0.83869797806,
                    -0.521586093159,
                    0.156632209401,
                    0.001240472464,
                ],
                [
                    0.83869797806,
                    0.156632209401,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    -0.83869797806,
                    0.156632209401,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    0.83869797806,
                    -0.156632209401,
                    0.521586093159,
                    0.001240472464,
                ],
                [
                    -0.83869797806,
                    -0.156632209401,
                    0.521586093159,
                    0.001240472464,
                ],
                [0.57582024991, 0.209808282749, 0.79019714267, 0.001259801066],
                [
                    -0.57582024991,
                    0.209808282749,
                    0.79019714267,
                    0.001259801066,
                ],
                [
                    0.57582024991,
                    -0.209808282749,
                    0.79019714267,
                    0.001259801066,
                ],
                [
                    -0.57582024991,
                    -0.209808282749,
                    0.79019714267,
                    0.001259801066,
                ],
                [0.57582024991, 0.79019714267, 0.209808282749, 0.001259801066],
                [
                    -0.57582024991,
                    0.79019714267,
                    0.209808282749,
                    0.001259801066,
                ],
                [
                    0.57582024991,
                    -0.79019714267,
                    0.209808282749,
                    0.001259801066,
                ],
                [
                    -0.57582024991,
                    -0.79019714267,
                    0.209808282749,
                    0.001259801066,
                ],
                [0.209808282749, 0.57582024991, 0.79019714267, 0.001259801066],
                [
                    -0.209808282749,
                    0.57582024991,
                    0.79019714267,
                    0.001259801066,
                ],
                [
                    0.209808282749,
                    -0.57582024991,
                    0.79019714267,
                    0.001259801066,
                ],
                [
                    -0.209808282749,
                    -0.57582024991,
                    0.79019714267,
                    0.001259801066,
                ],
                [0.209808282749, 0.79019714267, 0.57582024991, 0.001259801066],
                [
                    -0.209808282749,
                    0.79019714267,
                    0.57582024991,
                    0.001259801066,
                ],
                [
                    0.209808282749,
                    -0.79019714267,
                    0.57582024991,
                    0.001259801066,
                ],
                [
                    -0.209808282749,
                    -0.79019714267,
                    0.57582024991,
                    0.001259801066,
                ],
                [0.79019714267, 0.57582024991, 0.209808282749, 0.001259801066],
                [
                    -0.79019714267,
                    0.57582024991,
                    0.209808282749,
                    0.001259801066,
                ],
                [
                    0.79019714267,
                    -0.57582024991,
                    0.209808282749,
                    0.001259801066,
                ],
                [
                    -0.79019714267,
                    -0.57582024991,
                    0.209808282749,
                    0.001259801066,
                ],
                [0.79019714267, 0.209808282749, 0.57582024991, 0.001259801066],
                [
                    -0.79019714267,
                    0.209808282749,
                    0.57582024991,
                    0.001259801066,
                ],
                [
                    0.79019714267,
                    -0.209808282749,
                    0.57582024991,
                    0.001259801066,
                ],
                [
                    -0.79019714267,
                    -0.209808282749,
                    0.57582024991,
                    0.001259801066,
                ],
                [
                    0.625989368388,
                    0.261882411455,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    -0.625989368388,
                    0.261882411455,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    0.625989368388,
                    -0.261882411455,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    -0.625989368388,
                    -0.261882411455,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    0.625989368388,
                    0.734544017222,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    -0.625989368388,
                    0.734544017222,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    0.625989368388,
                    -0.734544017222,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    -0.625989368388,
                    -0.734544017222,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    0.261882411455,
                    0.625989368388,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    -0.261882411455,
                    0.625989368388,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    0.261882411455,
                    -0.625989368388,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    -0.261882411455,
                    -0.625989368388,
                    0.734544017222,
                    0.001269544478,
                ],
                [
                    0.261882411455,
                    0.734544017222,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    -0.261882411455,
                    0.734544017222,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    0.261882411455,
                    -0.734544017222,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    -0.261882411455,
                    -0.734544017222,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    0.734544017222,
                    0.625989368388,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    -0.734544017222,
                    0.625989368388,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    0.734544017222,
                    -0.625989368388,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    -0.734544017222,
                    -0.625989368388,
                    0.261882411455,
                    0.001269544478,
                ],
                [
                    0.734544017222,
                    0.261882411455,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    -0.734544017222,
                    0.261882411455,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    0.734544017222,
                    -0.261882411455,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    -0.734544017222,
                    -0.261882411455,
                    0.625989368388,
                    0.001269544478,
                ],
                [
                    0.531379512481,
                    0.052632450193,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    -0.531379512481,
                    0.052632450193,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    0.531379512481,
                    -0.052632450193,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    -0.531379512481,
                    -0.052632450193,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    0.531379512481,
                    0.845497273149,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    -0.531379512481,
                    0.845497273149,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    0.531379512481,
                    -0.845497273149,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    -0.531379512481,
                    -0.845497273149,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    0.052632450193,
                    0.531379512481,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    -0.052632450193,
                    0.531379512481,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    0.052632450193,
                    -0.531379512481,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    -0.052632450193,
                    -0.531379512481,
                    0.845497273149,
                    0.001240755796,
                ],
                [
                    0.052632450193,
                    0.845497273149,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    -0.052632450193,
                    0.845497273149,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    0.052632450193,
                    -0.845497273149,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    -0.052632450193,
                    -0.845497273149,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    0.845497273149,
                    0.531379512481,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    -0.845497273149,
                    0.531379512481,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    0.845497273149,
                    -0.531379512481,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    -0.845497273149,
                    -0.531379512481,
                    0.052632450193,
                    0.001240755796,
                ],
                [
                    0.845497273149,
                    0.052632450193,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    -0.845497273149,
                    0.052632450193,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    0.845497273149,
                    -0.052632450193,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    -0.845497273149,
                    -0.052632450193,
                    0.531379512481,
                    0.001240755796,
                ],
                [
                    0.589331795593,
                    0.106105973098,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    -0.589331795593,
                    0.106105973098,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    0.589331795593,
                    -0.106105973098,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    -0.589331795593,
                    -0.106105973098,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    0.589331795593,
                    0.800892974857,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    -0.589331795593,
                    0.800892974857,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    0.589331795593,
                    -0.800892974857,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    -0.589331795593,
                    -0.800892974857,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    0.106105973098,
                    0.589331795593,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    -0.106105973098,
                    0.589331795593,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    0.106105973098,
                    -0.589331795593,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    -0.106105973098,
                    -0.589331795593,
                    0.800892974857,
                    0.001261682934,
                ],
                [
                    0.106105973098,
                    0.800892974857,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    -0.106105973098,
                    0.800892974857,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    0.106105973098,
                    -0.800892974857,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    -0.106105973098,
                    -0.800892974857,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    0.800892974857,
                    0.589331795593,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    -0.800892974857,
                    0.589331795593,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    0.800892974857,
                    -0.589331795593,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    -0.800892974857,
                    -0.589331795593,
                    0.106105973098,
                    0.001261682934,
                ],
                [
                    0.800892974857,
                    0.106105973098,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    -0.800892974857,
                    0.106105973098,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    0.800892974857,
                    -0.106105973098,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    -0.800892974857,
                    -0.106105973098,
                    0.589331795593,
                    0.001261682934,
                ],
                [
                    0.642624632122,
                    0.159417156403,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    -0.642624632122,
                    0.159417156403,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    0.642624632122,
                    -0.159417156403,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    -0.642624632122,
                    -0.159417156403,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    0.642624632122,
                    0.749412938529,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    -0.642624632122,
                    0.749412938529,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    0.642624632122,
                    -0.749412938529,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    -0.642624632122,
                    -0.749412938529,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    0.159417156403,
                    0.642624632122,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    -0.159417156403,
                    0.642624632122,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    0.159417156403,
                    -0.642624632122,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    -0.159417156403,
                    -0.642624632122,
                    0.749412938529,
                    0.001272541293,
                ],
                [
                    0.159417156403,
                    0.749412938529,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    -0.159417156403,
                    0.749412938529,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    0.159417156403,
                    -0.749412938529,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    -0.159417156403,
                    -0.749412938529,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    0.749412938529,
                    0.642624632122,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    -0.749412938529,
                    0.642624632122,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    0.749412938529,
                    -0.642624632122,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    -0.749412938529,
                    -0.642624632122,
                    0.159417156403,
                    0.001272541293,
                ],
                [
                    0.749412938529,
                    0.159417156403,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    -0.749412938529,
                    0.159417156403,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    0.749412938529,
                    -0.159417156403,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    -0.749412938529,
                    -0.159417156403,
                    0.642624632122,
                    0.001272541293,
                ],
                [
                    0.651190436738,
                    0.053547895366,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    -0.651190436738,
                    0.053547895366,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    0.651190436738,
                    -0.053547895366,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    -0.651190436738,
                    -0.053547895366,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    0.651190436738,
                    0.757022878124,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    -0.651190436738,
                    0.757022878124,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    0.651190436738,
                    -0.757022878124,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    -0.651190436738,
                    -0.757022878124,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    0.053547895366,
                    0.651190436738,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    -0.053547895366,
                    0.651190436738,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    0.053547895366,
                    -0.651190436738,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    -0.053547895366,
                    -0.651190436738,
                    0.757022878124,
                    0.001275082834,
                ],
                [
                    0.053547895366,
                    0.757022878124,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    -0.053547895366,
                    0.757022878124,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    0.053547895366,
                    -0.757022878124,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    -0.053547895366,
                    -0.757022878124,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    0.757022878124,
                    0.651190436738,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    -0.757022878124,
                    0.651190436738,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    0.757022878124,
                    -0.651190436738,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    -0.757022878124,
                    -0.651190436738,
                    0.053547895366,
                    0.001275082834,
                ],
                [
                    0.757022878124,
                    0.053547895366,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    -0.757022878124,
                    0.053547895366,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    0.757022878124,
                    -0.053547895366,
                    0.651190436738,
                    0.001275082834,
                ],
                [
                    -0.757022878124,
                    -0.053547895366,
                    0.651190436738,
                    0.001275082834,
                ],
            ]
        )
        return grid_array

    if grid == 10:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 2.9986751e-05],
                [-1.0, 0.0, 0.0, 2.9986751e-05],
                [0.0, 1.0, 0.0, 2.9986751e-05],
                [0.0, -1.0, 0.0, 2.9986751e-05],
                [0.0, 0.0, 1.0, 5.9973503e-05],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.000815572106],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.000815572106],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.000815572106],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.000815572106,
                ],
                [
                    0.020655625388,
                    0.020655625388,
                    0.999573254084,
                    0.000237069839,
                ],
                [
                    -0.020655625388,
                    0.020655625388,
                    0.999573254084,
                    0.000237069839,
                ],
                [
                    0.020655625388,
                    -0.020655625388,
                    0.999573254084,
                    0.000237069839,
                ],
                [
                    -0.020655625388,
                    -0.020655625388,
                    0.999573254084,
                    0.000237069839,
                ],
                [
                    0.020655625388,
                    0.999573254084,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    -0.020655625388,
                    0.999573254084,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    0.020655625388,
                    -0.999573254084,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    -0.020655625388,
                    -0.999573254084,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    0.999573254084,
                    0.020655625388,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    -0.999573254084,
                    0.020655625388,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    0.999573254084,
                    -0.020655625388,
                    0.020655625388,
                    0.000237069839,
                ],
                [
                    -0.999573254084,
                    -0.020655625388,
                    0.020655625388,
                    0.000237069839,
                ],
                [0.05250918173, 0.05250918173, 0.997238974202, 0.000382681729],
                [
                    -0.05250918173,
                    0.05250918173,
                    0.997238974202,
                    0.000382681729,
                ],
                [
                    0.05250918173,
                    -0.05250918173,
                    0.997238974202,
                    0.000382681729,
                ],
                [
                    -0.05250918173,
                    -0.05250918173,
                    0.997238974202,
                    0.000382681729,
                ],
                [0.05250918173, 0.997238974202, 0.05250918173, 0.000382681729],
                [
                    -0.05250918173,
                    0.997238974202,
                    0.05250918173,
                    0.000382681729,
                ],
                [
                    0.05250918173,
                    -0.997238974202,
                    0.05250918173,
                    0.000382681729,
                ],
                [
                    -0.05250918173,
                    -0.997238974202,
                    0.05250918173,
                    0.000382681729,
                ],
                [0.997238974202, 0.05250918173, 0.05250918173, 0.000382681729],
                [
                    -0.997238974202,
                    0.05250918173,
                    0.05250918173,
                    0.000382681729,
                ],
                [
                    0.997238974202,
                    -0.05250918173,
                    0.05250918173,
                    0.000382681729,
                ],
                [
                    -0.997238974202,
                    -0.05250918173,
                    0.05250918173,
                    0.000382681729,
                ],
                [0.08993480082, 0.08993480082, 0.991878754285, 0.000490577315],
                [
                    -0.08993480082,
                    0.08993480082,
                    0.991878754285,
                    0.000490577315,
                ],
                [
                    0.08993480082,
                    -0.08993480082,
                    0.991878754285,
                    0.000490577315,
                ],
                [
                    -0.08993480082,
                    -0.08993480082,
                    0.991878754285,
                    0.000490577315,
                ],
                [0.08993480082, 0.991878754285, 0.08993480082, 0.000490577315],
                [
                    -0.08993480082,
                    0.991878754285,
                    0.08993480082,
                    0.000490577315,
                ],
                [
                    0.08993480082,
                    -0.991878754285,
                    0.08993480082,
                    0.000490577315,
                ],
                [
                    -0.08993480082,
                    -0.991878754285,
                    0.08993480082,
                    0.000490577315,
                ],
                [0.991878754285, 0.08993480082, 0.08993480082, 0.000490577315],
                [
                    -0.991878754285,
                    0.08993480082,
                    0.08993480082,
                    0.000490577315,
                ],
                [
                    0.991878754285,
                    -0.08993480082,
                    0.08993480082,
                    0.000490577315,
                ],
                [
                    -0.991878754285,
                    -0.08993480082,
                    0.08993480082,
                    0.000490577315,
                ],
                [
                    0.130602392444,
                    0.130602392444,
                    0.982795009234,
                    0.000572481637,
                ],
                [
                    -0.130602392444,
                    0.130602392444,
                    0.982795009234,
                    0.000572481637,
                ],
                [
                    0.130602392444,
                    -0.130602392444,
                    0.982795009234,
                    0.000572481637,
                ],
                [
                    -0.130602392444,
                    -0.130602392444,
                    0.982795009234,
                    0.000572481637,
                ],
                [
                    0.130602392444,
                    0.982795009234,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    -0.130602392444,
                    0.982795009234,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    0.130602392444,
                    -0.982795009234,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    -0.130602392444,
                    -0.982795009234,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    0.982795009234,
                    0.130602392444,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    -0.982795009234,
                    0.130602392444,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    0.982795009234,
                    -0.130602392444,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    -0.982795009234,
                    -0.130602392444,
                    0.130602392444,
                    0.000572481637,
                ],
                [
                    0.173206038853,
                    0.173206038853,
                    0.969535629159,
                    0.000635606452,
                ],
                [
                    -0.173206038853,
                    0.173206038853,
                    0.969535629159,
                    0.000635606452,
                ],
                [
                    0.173206038853,
                    -0.173206038853,
                    0.969535629159,
                    0.000635606452,
                ],
                [
                    -0.173206038853,
                    -0.173206038853,
                    0.969535629159,
                    0.000635606452,
                ],
                [
                    0.173206038853,
                    0.969535629159,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    -0.173206038853,
                    0.969535629159,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    0.173206038853,
                    -0.969535629159,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    -0.173206038853,
                    -0.969535629159,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    0.969535629159,
                    0.173206038853,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    -0.969535629159,
                    0.173206038853,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    0.969535629159,
                    -0.173206038853,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    -0.969535629159,
                    -0.173206038853,
                    0.173206038853,
                    0.000635606452,
                ],
                [
                    0.216872708482,
                    0.216872708482,
                    0.951804841673,
                    0.000684589134,
                ],
                [
                    -0.216872708482,
                    0.216872708482,
                    0.951804841673,
                    0.000684589134,
                ],
                [
                    0.216872708482,
                    -0.216872708482,
                    0.951804841673,
                    0.000684589134,
                ],
                [
                    -0.216872708482,
                    -0.216872708482,
                    0.951804841673,
                    0.000684589134,
                ],
                [
                    0.216872708482,
                    0.951804841673,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    -0.216872708482,
                    0.951804841673,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    0.216872708482,
                    -0.951804841673,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    -0.216872708482,
                    -0.951804841673,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    0.951804841673,
                    0.216872708482,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    -0.951804841673,
                    0.216872708482,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    0.951804841673,
                    -0.216872708482,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    -0.951804841673,
                    -0.216872708482,
                    0.216872708482,
                    0.000684589134,
                ],
                [
                    0.260952830917,
                    0.260952830917,
                    0.929412308974,
                    0.000722558104,
                ],
                [
                    -0.260952830917,
                    0.260952830917,
                    0.929412308974,
                    0.000722558104,
                ],
                [
                    0.260952830917,
                    -0.260952830917,
                    0.929412308974,
                    0.000722558104,
                ],
                [
                    -0.260952830917,
                    -0.260952830917,
                    0.929412308974,
                    0.000722558104,
                ],
                [
                    0.260952830917,
                    0.929412308974,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    -0.260952830917,
                    0.929412308974,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    0.260952830917,
                    -0.929412308974,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    -0.260952830917,
                    -0.929412308974,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    0.929412308974,
                    0.260952830917,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    -0.929412308974,
                    0.260952830917,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    0.929412308974,
                    -0.260952830917,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    -0.929412308974,
                    -0.260952830917,
                    0.260952830917,
                    0.000722558104,
                ],
                [
                    0.304925292794,
                    0.304925292794,
                    0.902242279894,
                    0.000751727646,
                ],
                [
                    -0.304925292794,
                    0.304925292794,
                    0.902242279894,
                    0.000751727646,
                ],
                [
                    0.304925292794,
                    -0.304925292794,
                    0.902242279894,
                    0.000751727646,
                ],
                [
                    -0.304925292794,
                    -0.304925292794,
                    0.902242279894,
                    0.000751727646,
                ],
                [
                    0.304925292794,
                    0.902242279894,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    -0.304925292794,
                    0.902242279894,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    0.304925292794,
                    -0.902242279894,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    -0.304925292794,
                    -0.902242279894,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    0.902242279894,
                    0.304925292794,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    -0.902242279894,
                    0.304925292794,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    0.902242279894,
                    -0.304925292794,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    -0.902242279894,
                    -0.304925292794,
                    0.304925292794,
                    0.000751727646,
                ],
                [
                    0.348348413808,
                    0.348348413808,
                    0.870233741701,
                    0.00077374236,
                ],
                [
                    -0.348348413808,
                    0.348348413808,
                    0.870233741701,
                    0.00077374236,
                ],
                [
                    0.348348413808,
                    -0.348348413808,
                    0.870233741701,
                    0.00077374236,
                ],
                [
                    -0.348348413808,
                    -0.348348413808,
                    0.870233741701,
                    0.00077374236,
                ],
                [
                    0.348348413808,
                    0.870233741701,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    -0.348348413808,
                    0.870233741701,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    0.348348413808,
                    -0.870233741701,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    -0.348348413808,
                    -0.870233741701,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    0.870233741701,
                    0.348348413808,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    -0.870233741701,
                    0.348348413808,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    0.870233741701,
                    -0.348348413808,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    -0.870233741701,
                    -0.348348413808,
                    0.348348413808,
                    0.00077374236,
                ],
                [
                    0.390832154911,
                    0.390832154911,
                    0.833366938015,
                    0.000789885987,
                ],
                [
                    -0.390832154911,
                    0.390832154911,
                    0.833366938015,
                    0.000789885987,
                ],
                [
                    0.390832154911,
                    -0.390832154911,
                    0.833366938015,
                    0.000789885987,
                ],
                [
                    -0.390832154911,
                    -0.390832154911,
                    0.833366938015,
                    0.000789885987,
                ],
                [
                    0.390832154911,
                    0.833366938015,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    -0.390832154911,
                    0.833366938015,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    0.390832154911,
                    -0.833366938015,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    -0.390832154911,
                    -0.833366938015,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    0.833366938015,
                    0.390832154911,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    -0.833366938015,
                    0.390832154911,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    0.833366938015,
                    -0.390832154911,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    -0.833366938015,
                    -0.390832154911,
                    0.390832154911,
                    0.000789885987,
                ],
                [
                    0.432021007189,
                    0.432021007189,
                    0.791653774509,
                    0.000801213622,
                ],
                [
                    -0.432021007189,
                    0.432021007189,
                    0.791653774509,
                    0.000801213622,
                ],
                [
                    0.432021007189,
                    -0.432021007189,
                    0.791653774509,
                    0.000801213622,
                ],
                [
                    -0.432021007189,
                    -0.432021007189,
                    0.791653774509,
                    0.000801213622,
                ],
                [
                    0.432021007189,
                    0.791653774509,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    -0.432021007189,
                    0.791653774509,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    0.432021007189,
                    -0.791653774509,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    -0.432021007189,
                    -0.791653774509,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    0.791653774509,
                    0.432021007189,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    -0.791653774509,
                    0.432021007189,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    0.791653774509,
                    -0.432021007189,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    -0.791653774509,
                    -0.432021007189,
                    0.432021007189,
                    0.000801213622,
                ],
                [
                    0.471582479589,
                    0.471582479589,
                    0.745130813944,
                    0.00080863843,
                ],
                [
                    -0.471582479589,
                    0.471582479589,
                    0.745130813944,
                    0.00080863843,
                ],
                [
                    0.471582479589,
                    -0.471582479589,
                    0.745130813944,
                    0.00080863843,
                ],
                [
                    -0.471582479589,
                    -0.471582479589,
                    0.745130813944,
                    0.00080863843,
                ],
                [
                    0.471582479589,
                    0.745130813944,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    -0.471582479589,
                    0.745130813944,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    0.471582479589,
                    -0.745130813944,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    -0.471582479589,
                    -0.745130813944,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    0.745130813944,
                    0.471582479589,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    -0.745130813944,
                    0.471582479589,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    0.745130813944,
                    -0.471582479589,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    -0.745130813944,
                    -0.471582479589,
                    0.471582479589,
                    0.00080863843,
                ],
                [
                    0.509198479408,
                    0.509198479408,
                    0.693854319823,
                    0.000812989499,
                ],
                [
                    -0.509198479408,
                    0.509198479408,
                    0.693854319823,
                    0.000812989499,
                ],
                [
                    0.509198479408,
                    -0.509198479408,
                    0.693854319823,
                    0.000812989499,
                ],
                [
                    -0.509198479408,
                    -0.509198479408,
                    0.693854319823,
                    0.000812989499,
                ],
                [
                    0.509198479408,
                    0.693854319823,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    -0.509198479408,
                    0.693854319823,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    0.509198479408,
                    -0.693854319823,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    -0.509198479408,
                    -0.693854319823,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    0.693854319823,
                    0.509198479408,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    -0.693854319823,
                    0.509198479408,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    0.693854319823,
                    -0.509198479408,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    -0.693854319823,
                    -0.509198479408,
                    0.509198479408,
                    0.000812989499,
                ],
                [
                    0.544558014565,
                    0.544558014565,
                    0.637897434974,
                    0.000815049124,
                ],
                [
                    -0.544558014565,
                    0.544558014565,
                    0.637897434974,
                    0.000815049124,
                ],
                [
                    0.544558014565,
                    -0.544558014565,
                    0.637897434974,
                    0.000815049124,
                ],
                [
                    -0.544558014565,
                    -0.544558014565,
                    0.637897434974,
                    0.000815049124,
                ],
                [
                    0.544558014565,
                    0.637897434974,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    -0.544558014565,
                    0.637897434974,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    0.544558014565,
                    -0.637897434974,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    -0.544558014565,
                    -0.637897434974,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    0.637897434974,
                    0.544558014565,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    -0.637897434974,
                    0.544558014565,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    0.637897434974,
                    -0.544558014565,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    -0.637897434974,
                    -0.544558014565,
                    0.544558014565,
                    0.000815049124,
                ],
                [
                    0.607257579684,
                    0.607257579684,
                    0.512324568835,
                    0.000815284708,
                ],
                [
                    -0.607257579684,
                    0.607257579684,
                    0.512324568835,
                    0.000815284708,
                ],
                [
                    0.607257579684,
                    -0.607257579684,
                    0.512324568835,
                    0.000815284708,
                ],
                [
                    -0.607257579684,
                    -0.607257579684,
                    0.512324568835,
                    0.000815284708,
                ],
                [
                    0.607257579684,
                    0.512324568835,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    -0.607257579684,
                    0.512324568835,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    0.607257579684,
                    -0.512324568835,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    -0.607257579684,
                    -0.512324568835,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    0.512324568835,
                    0.607257579684,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    -0.512324568835,
                    0.607257579684,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    0.512324568835,
                    -0.607257579684,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    -0.512324568835,
                    -0.607257579684,
                    0.607257579684,
                    0.000815284708,
                ],
                [
                    0.633948450576,
                    0.633948450576,
                    0.442965827153,
                    0.000814856172,
                ],
                [
                    -0.633948450576,
                    0.633948450576,
                    0.442965827153,
                    0.000814856172,
                ],
                [
                    0.633948450576,
                    -0.633948450576,
                    0.442965827153,
                    0.000814856172,
                ],
                [
                    -0.633948450576,
                    -0.633948450576,
                    0.442965827153,
                    0.000814856172,
                ],
                [
                    0.633948450576,
                    0.442965827153,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    -0.633948450576,
                    0.442965827153,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    0.633948450576,
                    -0.442965827153,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    -0.633948450576,
                    -0.442965827153,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    0.442965827153,
                    0.633948450576,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    -0.442965827153,
                    0.633948450576,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    0.442965827153,
                    -0.633948450576,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    -0.442965827153,
                    -0.633948450576,
                    0.633948450576,
                    0.000814856172,
                ],
                [
                    0.657071825749,
                    0.657071825749,
                    0.369476970344,
                    0.000814832751,
                ],
                [
                    -0.657071825749,
                    0.657071825749,
                    0.369476970344,
                    0.000814832751,
                ],
                [
                    0.657071825749,
                    -0.657071825749,
                    0.369476970344,
                    0.000814832751,
                ],
                [
                    -0.657071825749,
                    -0.657071825749,
                    0.369476970344,
                    0.000814832751,
                ],
                [
                    0.657071825749,
                    0.369476970344,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    -0.657071825749,
                    0.369476970344,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    0.657071825749,
                    -0.369476970344,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    -0.657071825749,
                    -0.369476970344,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    0.369476970344,
                    0.657071825749,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    -0.369476970344,
                    0.657071825749,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    0.369476970344,
                    -0.657071825749,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    -0.369476970344,
                    -0.657071825749,
                    0.657071825749,
                    0.000814832751,
                ],
                [
                    0.676255733009,
                    0.676255733009,
                    0.292158120107,
                    0.000815529559,
                ],
                [
                    -0.676255733009,
                    0.676255733009,
                    0.292158120107,
                    0.000815529559,
                ],
                [
                    0.676255733009,
                    -0.676255733009,
                    0.292158120107,
                    0.000815529559,
                ],
                [
                    -0.676255733009,
                    -0.676255733009,
                    0.292158120107,
                    0.000815529559,
                ],
                [
                    0.676255733009,
                    0.292158120107,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    -0.676255733009,
                    0.292158120107,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    0.676255733009,
                    -0.292158120107,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    -0.676255733009,
                    -0.292158120107,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    0.292158120107,
                    0.676255733009,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    -0.292158120107,
                    0.676255733009,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    0.292158120107,
                    -0.676255733009,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    -0.292158120107,
                    -0.676255733009,
                    0.676255733009,
                    0.000815529559,
                ],
                [
                    0.691116169692,
                    0.691116169692,
                    0.211463661132,
                    0.000816903511,
                ],
                [
                    -0.691116169692,
                    0.691116169692,
                    0.211463661132,
                    0.000816903511,
                ],
                [
                    0.691116169692,
                    -0.691116169692,
                    0.211463661132,
                    0.000816903511,
                ],
                [
                    -0.691116169692,
                    -0.691116169692,
                    0.211463661132,
                    0.000816903511,
                ],
                [
                    0.691116169692,
                    0.211463661132,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    -0.691116169692,
                    0.211463661132,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    0.691116169692,
                    -0.211463661132,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    -0.691116169692,
                    -0.211463661132,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    0.211463661132,
                    0.691116169692,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    -0.211463661132,
                    0.691116169692,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    0.211463661132,
                    -0.691116169692,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    -0.211463661132,
                    -0.691116169692,
                    0.691116169692,
                    0.000816903511,
                ],
                [
                    0.701284191166,
                    0.701284191166,
                    0.128066258012,
                    0.000818493692,
                ],
                [
                    -0.701284191166,
                    0.701284191166,
                    0.128066258012,
                    0.000818493692,
                ],
                [
                    0.701284191166,
                    -0.701284191166,
                    0.128066258012,
                    0.000818493692,
                ],
                [
                    -0.701284191166,
                    -0.701284191166,
                    0.128066258012,
                    0.000818493692,
                ],
                [
                    0.701284191166,
                    0.128066258012,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    -0.701284191166,
                    0.128066258012,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    0.701284191166,
                    -0.128066258012,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    -0.701284191166,
                    -0.128066258012,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    0.128066258012,
                    0.701284191166,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    -0.128066258012,
                    0.701284191166,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    0.128066258012,
                    -0.701284191166,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    -0.128066258012,
                    -0.701284191166,
                    0.701284191166,
                    0.000818493692,
                ],
                [
                    0.706455927241,
                    0.706455927241,
                    0.042895754243,
                    0.000819574537,
                ],
                [
                    -0.706455927241,
                    0.706455927241,
                    0.042895754243,
                    0.000819574537,
                ],
                [
                    0.706455927241,
                    -0.706455927241,
                    0.042895754243,
                    0.000819574537,
                ],
                [
                    -0.706455927241,
                    -0.706455927241,
                    0.042895754243,
                    0.000819574537,
                ],
                [
                    0.706455927241,
                    0.042895754243,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    -0.706455927241,
                    0.042895754243,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    0.706455927241,
                    -0.042895754243,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    -0.706455927241,
                    -0.042895754243,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    0.042895754243,
                    0.706455927241,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    -0.042895754243,
                    0.706455927241,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    0.042895754243,
                    -0.706455927241,
                    0.706455927241,
                    0.000819574537,
                ],
                [
                    -0.042895754243,
                    -0.706455927241,
                    0.706455927241,
                    0.000819574537,
                ],
                [0.061235549899, 0.998123342793, 0.0, 0.000173898681],
                [-0.061235549899, 0.998123342793, 0.0, 0.000173898681],
                [0.061235549899, -0.998123342793, 0.0, 0.000173898681],
                [-0.061235549899, -0.998123342793, 0.0, 0.000173898681],
                [0.998123342793, 0.061235549899, 0.0, 0.000173898681],
                [-0.998123342793, 0.061235549899, 0.0, 0.000173898681],
                [0.998123342793, -0.061235549899, 0.0, 0.000173898681],
                [-0.998123342793, -0.061235549899, 0.0, 0.000173898681],
                [0.061235549899, 0.0, 0.998123342793, 0.000347797362],
                [-0.061235549899, 0.0, 0.998123342793, 0.000347797362],
                [0.998123342793, 0.0, 0.061235549899, 0.000347797362],
                [-0.998123342793, 0.0, 0.061235549899, 0.000347797362],
                [0.0, 0.061235549899, 0.998123342793, 0.000347797362],
                [0.0, -0.061235549899, 0.998123342793, 0.000347797362],
                [0.0, 0.998123342793, 0.061235549899, 0.000347797362],
                [0.0, -0.998123342793, 0.061235549899, 0.000347797362],
                [0.153307034831, 0.988178603832, 0.0, 0.000265961605],
                [-0.153307034831, 0.988178603832, 0.0, 0.000265961605],
                [0.153307034831, -0.988178603832, 0.0, 0.000265961605],
                [-0.153307034831, -0.988178603832, 0.0, 0.000265961605],
                [0.988178603832, 0.153307034831, 0.0, 0.000265961605],
                [-0.988178603832, 0.153307034831, 0.0, 0.000265961605],
                [0.988178603832, -0.153307034831, 0.0, 0.000265961605],
                [-0.988178603832, -0.153307034831, 0.0, 0.000265961605],
                [0.153307034831, 0.0, 0.988178603832, 0.000531923209],
                [-0.153307034831, 0.0, 0.988178603832, 0.000531923209],
                [0.988178603832, 0.0, 0.153307034831, 0.000531923209],
                [-0.988178603832, 0.0, 0.153307034831, 0.000531923209],
                [0.0, 0.153307034831, 0.988178603832, 0.000531923209],
                [0.0, -0.153307034831, 0.988178603832, 0.000531923209],
                [0.0, 0.988178603832, 0.153307034831, 0.000531923209],
                [0.0, -0.988178603832, 0.153307034831, 0.000531923209],
                [0.256390260524, 0.966573346574, 0.0, 0.000324059601],
                [-0.256390260524, 0.966573346574, 0.0, 0.000324059601],
                [0.256390260524, -0.966573346574, 0.0, 0.000324059601],
                [-0.256390260524, -0.966573346574, 0.0, 0.000324059601],
                [0.966573346574, 0.256390260524, 0.0, 0.000324059601],
                [-0.966573346574, 0.256390260524, 0.0, 0.000324059601],
                [0.966573346574, -0.256390260524, 0.0, 0.000324059601],
                [-0.966573346574, -0.256390260524, 0.0, 0.000324059601],
                [0.256390260524, 0.0, 0.966573346574, 0.000648119202],
                [-0.256390260524, 0.0, 0.966573346574, 0.000648119202],
                [0.966573346574, 0.0, 0.256390260524, 0.000648119202],
                [-0.966573346574, 0.0, 0.256390260524, 0.000648119202],
                [0.0, 0.256390260524, 0.966573346574, 0.000648119202],
                [0.0, -0.256390260524, 0.966573346574, 0.000648119202],
                [0.0, 0.966573346574, 0.256390260524, 0.000648119202],
                [0.0, -0.966573346574, 0.256390260524, 0.000648119202],
                [0.362934699166, 0.931814576051, 0.0, 0.000362119596],
                [-0.362934699166, 0.931814576051, 0.0, 0.000362119596],
                [0.362934699166, -0.931814576051, 0.0, 0.000362119596],
                [-0.362934699166, -0.931814576051, 0.0, 0.000362119596],
                [0.931814576051, 0.362934699166, 0.0, 0.000362119596],
                [-0.931814576051, 0.362934699166, 0.0, 0.000362119596],
                [0.931814576051, -0.362934699166, 0.0, 0.000362119596],
                [-0.931814576051, -0.362934699166, 0.0, 0.000362119596],
                [0.362934699166, 0.0, 0.931814576051, 0.000724239193],
                [-0.362934699166, 0.0, 0.931814576051, 0.000724239193],
                [0.931814576051, 0.0, 0.362934699166, 0.000724239193],
                [-0.931814576051, 0.0, 0.362934699166, 0.000724239193],
                [0.0, 0.362934699166, 0.931814576051, 0.000724239193],
                [0.0, -0.362934699166, 0.931814576051, 0.000724239193],
                [0.0, 0.931814576051, 0.362934699166, 0.000724239193],
                [0.0, -0.931814576051, 0.362934699166, 0.000724239193],
                [0.468394996899, 0.883519171767, 0.0, 0.000386883833],
                [-0.468394996899, 0.883519171767, 0.0, 0.000386883833],
                [0.468394996899, -0.883519171767, 0.0, 0.000386883833],
                [-0.468394996899, -0.883519171767, 0.0, 0.000386883833],
                [0.883519171767, 0.468394996899, 0.0, 0.000386883833],
                [-0.883519171767, 0.468394996899, 0.0, 0.000386883833],
                [0.883519171767, -0.468394996899, 0.0, 0.000386883833],
                [-0.883519171767, -0.468394996899, 0.0, 0.000386883833],
                [0.468394996899, 0.0, 0.883519171767, 0.000773767666],
                [-0.468394996899, 0.0, 0.883519171767, 0.000773767666],
                [0.883519171767, 0.0, 0.468394996899, 0.000773767666],
                [-0.883519171767, 0.0, 0.468394996899, 0.000773767666],
                [0.0, 0.468394996899, 0.883519171767, 0.000773767666],
                [0.0, -0.468394996899, 0.883519171767, 0.000773767666],
                [0.0, 0.883519171767, 0.468394996899, 0.000773767666],
                [0.0, -0.883519171767, 0.468394996899, 0.000773767666],
                [0.569447924066, 0.822027409383, 0.0, 0.000401891153],
                [-0.569447924066, 0.822027409383, 0.0, 0.000401891153],
                [0.569447924066, -0.822027409383, 0.0, 0.000401891153],
                [-0.569447924066, -0.822027409383, 0.0, 0.000401891153],
                [0.822027409383, 0.569447924066, 0.0, 0.000401891153],
                [-0.822027409383, 0.569447924066, 0.0, 0.000401891153],
                [0.822027409383, -0.569447924066, 0.0, 0.000401891153],
                [-0.822027409383, -0.569447924066, 0.0, 0.000401891153],
                [0.569447924066, 0.0, 0.822027409383, 0.000803782307],
                [-0.569447924066, 0.0, 0.822027409383, 0.000803782307],
                [0.822027409383, 0.0, 0.569447924066, 0.000803782307],
                [-0.822027409383, 0.0, 0.569447924066, 0.000803782307],
                [0.0, 0.569447924066, 0.822027409383, 0.000803782307],
                [0.0, -0.569447924066, 0.822027409383, 0.000803782307],
                [0.0, 0.822027409383, 0.569447924066, 0.000803782307],
                [0.0, -0.822027409383, 0.569447924066, 0.000803782307],
                [0.663446543099, 0.748223686106, 0.0, 0.000408992943],
                [-0.663446543099, 0.748223686106, 0.0, 0.000408992943],
                [0.663446543099, -0.748223686106, 0.0, 0.000408992943],
                [-0.663446543099, -0.748223686106, 0.0, 0.000408992943],
                [0.748223686106, 0.663446543099, 0.0, 0.000408992943],
                [-0.748223686106, 0.663446543099, 0.0, 0.000408992943],
                [0.748223686106, -0.663446543099, 0.0, 0.000408992943],
                [-0.748223686106, -0.663446543099, 0.0, 0.000408992943],
                [0.663446543099, 0.0, 0.748223686106, 0.000817985887],
                [-0.663446543099, 0.0, 0.748223686106, 0.000817985887],
                [0.748223686106, 0.0, 0.663446543099, 0.000817985887],
                [-0.748223686106, 0.0, 0.663446543099, 0.000817985887],
                [0.0, 0.663446543099, 0.748223686106, 0.000817985887],
                [0.0, -0.663446543099, 0.748223686106, 0.000817985887],
                [0.0, 0.748223686106, 0.663446543099, 0.000817985887],
                [0.0, -0.748223686106, 0.663446543099, 0.000817985887],
                [
                    0.103395857355,
                    0.030345440091,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    -0.103395857355,
                    0.030345440091,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    0.103395857355,
                    -0.030345440091,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    -0.103395857355,
                    -0.030345440091,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    0.103395857355,
                    0.994177273401,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    -0.103395857355,
                    0.994177273401,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    0.103395857355,
                    -0.994177273401,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    -0.103395857355,
                    -0.994177273401,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    0.030345440091,
                    0.103395857355,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    -0.030345440091,
                    0.103395857355,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    0.030345440091,
                    -0.103395857355,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    -0.030345440091,
                    -0.103395857355,
                    0.994177273401,
                    0.000455981506,
                ],
                [
                    0.030345440091,
                    0.994177273401,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    -0.030345440091,
                    0.994177273401,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    0.030345440091,
                    -0.994177273401,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    -0.030345440091,
                    -0.994177273401,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    0.994177273401,
                    0.103395857355,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    -0.994177273401,
                    0.103395857355,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    0.994177273401,
                    -0.103395857355,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    -0.994177273401,
                    -0.103395857355,
                    0.030345440091,
                    0.000455981506,
                ],
                [
                    0.994177273401,
                    0.030345440091,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    -0.994177273401,
                    0.030345440091,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    0.994177273401,
                    -0.030345440091,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    -0.994177273401,
                    -0.030345440091,
                    0.103395857355,
                    0.000455981506,
                ],
                [
                    0.147352141241,
                    0.066188030442,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    -0.147352141241,
                    0.066188030442,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    0.147352141241,
                    -0.066188030442,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    -0.147352141241,
                    -0.066188030442,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    0.147352141241,
                    0.986867007807,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    -0.147352141241,
                    0.986867007807,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    0.147352141241,
                    -0.986867007807,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    -0.147352141241,
                    -0.986867007807,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    0.066188030442,
                    0.147352141241,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    -0.066188030442,
                    0.147352141241,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    0.066188030442,
                    -0.147352141241,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    -0.066188030442,
                    -0.147352141241,
                    0.986867007807,
                    0.000543041098,
                ],
                [
                    0.066188030442,
                    0.986867007807,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    -0.066188030442,
                    0.986867007807,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    0.066188030442,
                    -0.986867007807,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    -0.066188030442,
                    -0.986867007807,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    0.986867007807,
                    0.147352141241,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    -0.986867007807,
                    0.147352141241,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    0.986867007807,
                    -0.147352141241,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    -0.986867007807,
                    -0.147352141241,
                    0.066188030442,
                    0.000543041098,
                ],
                [
                    0.986867007807,
                    0.066188030442,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    -0.986867007807,
                    0.066188030442,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    0.986867007807,
                    -0.066188030442,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    -0.986867007807,
                    -0.066188030442,
                    0.147352141241,
                    0.000543041098,
                ],
                [
                    0.192455215871,
                    0.105443112899,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    -0.192455215871,
                    0.105443112899,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    0.192455215871,
                    -0.105443112899,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    -0.192455215871,
                    -0.105443112899,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    0.192455215871,
                    0.975624282102,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    -0.192455215871,
                    0.975624282102,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    0.192455215871,
                    -0.975624282102,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    -0.192455215871,
                    -0.975624282102,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    0.105443112899,
                    0.192455215871,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    -0.105443112899,
                    0.192455215871,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    0.105443112899,
                    -0.192455215871,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    -0.105443112899,
                    -0.192455215871,
                    0.975624282102,
                    0.000611583579,
                ],
                [
                    0.105443112899,
                    0.975624282102,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    -0.105443112899,
                    0.975624282102,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    0.105443112899,
                    -0.975624282102,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    -0.105443112899,
                    -0.975624282102,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    0.975624282102,
                    0.192455215871,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    -0.975624282102,
                    0.192455215871,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    0.975624282102,
                    -0.192455215871,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    -0.975624282102,
                    -0.192455215871,
                    0.105443112899,
                    0.000611583579,
                ],
                [
                    0.975624282102,
                    0.105443112899,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    -0.975624282102,
                    0.105443112899,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    0.975624282102,
                    -0.105443112899,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    -0.975624282102,
                    -0.105443112899,
                    0.192455215871,
                    0.000611583579,
                ],
                [
                    0.238109436289,
                    0.146826355124,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    -0.238109436289,
                    0.146826355124,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    0.238109436289,
                    -0.146826355124,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    -0.238109436289,
                    -0.146826355124,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    0.238109436289,
                    0.960075995842,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    -0.238109436289,
                    0.960075995842,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    0.238109436289,
                    -0.960075995842,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    -0.238109436289,
                    -0.960075995842,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    0.146826355124,
                    0.238109436289,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    -0.146826355124,
                    0.238109436289,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    0.146826355124,
                    -0.238109436289,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    -0.146826355124,
                    -0.238109436289,
                    0.960075995842,
                    0.00066538261,
                ],
                [
                    0.146826355124,
                    0.960075995842,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    -0.146826355124,
                    0.960075995842,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    0.146826355124,
                    -0.960075995842,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    -0.146826355124,
                    -0.960075995842,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    0.960075995842,
                    0.238109436289,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    -0.960075995842,
                    0.238109436289,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    0.960075995842,
                    -0.238109436289,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    -0.960075995842,
                    -0.238109436289,
                    0.146826355124,
                    0.00066538261,
                ],
                [
                    0.960075995842,
                    0.146826355124,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    -0.960075995842,
                    0.146826355124,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    0.960075995842,
                    -0.146826355124,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    -0.960075995842,
                    -0.146826355124,
                    0.238109436289,
                    0.00066538261,
                ],
                [
                    0.283812170794,
                    0.189448610819,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    -0.283812170794,
                    0.189448610819,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    0.283812170794,
                    -0.189448610819,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    -0.283812170794,
                    -0.189448610819,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    0.283812170794,
                    0.939978656975,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    -0.283812170794,
                    0.939978656975,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    0.283812170794,
                    -0.939978656975,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    -0.283812170794,
                    -0.939978656975,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    0.189448610819,
                    0.283812170794,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    -0.189448610819,
                    0.283812170794,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    0.189448610819,
                    -0.283812170794,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    -0.189448610819,
                    -0.283812170794,
                    0.939978656975,
                    0.000707466942,
                ],
                [
                    0.189448610819,
                    0.939978656975,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    -0.189448610819,
                    0.939978656975,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    0.189448610819,
                    -0.939978656975,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    -0.189448610819,
                    -0.939978656975,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    0.939978656975,
                    0.283812170794,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    -0.939978656975,
                    0.283812170794,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    0.939978656975,
                    -0.283812170794,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    -0.939978656975,
                    -0.283812170794,
                    0.189448610819,
                    0.000707466942,
                ],
                [
                    0.939978656975,
                    0.189448610819,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    -0.939978656975,
                    0.189448610819,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    0.939978656975,
                    -0.189448610819,
                    0.283812170794,
                    0.000707466942,
                ],
                [
                    -0.939978656975,
                    -0.189448610819,
                    0.283812170794,
                    0.000707466942,
                ],
                [0.329132313337, 0.232637423876, 0.915178534128, 0.0007401135],
                [
                    -0.329132313337,
                    0.232637423876,
                    0.915178534128,
                    0.0007401135,
                ],
                [
                    0.329132313337,
                    -0.232637423876,
                    0.915178534128,
                    0.0007401135,
                ],
                [
                    -0.329132313337,
                    -0.232637423876,
                    0.915178534128,
                    0.0007401135,
                ],
                [0.329132313337, 0.915178534128, 0.232637423876, 0.0007401135],
                [
                    -0.329132313337,
                    0.915178534128,
                    0.232637423876,
                    0.0007401135,
                ],
                [
                    0.329132313337,
                    -0.915178534128,
                    0.232637423876,
                    0.0007401135,
                ],
                [
                    -0.329132313337,
                    -0.915178534128,
                    0.232637423876,
                    0.0007401135,
                ],
                [0.232637423876, 0.329132313337, 0.915178534128, 0.0007401135],
                [
                    -0.232637423876,
                    0.329132313337,
                    0.915178534128,
                    0.0007401135,
                ],
                [
                    0.232637423876,
                    -0.329132313337,
                    0.915178534128,
                    0.0007401135,
                ],
                [
                    -0.232637423876,
                    -0.329132313337,
                    0.915178534128,
                    0.0007401135,
                ],
                [0.232637423876, 0.915178534128, 0.329132313337, 0.0007401135],
                [
                    -0.232637423876,
                    0.915178534128,
                    0.329132313337,
                    0.0007401135,
                ],
                [
                    0.232637423876,
                    -0.915178534128,
                    0.329132313337,
                    0.0007401135,
                ],
                [
                    -0.232637423876,
                    -0.915178534128,
                    0.329132313337,
                    0.0007401135,
                ],
                [0.915178534128, 0.329132313337, 0.232637423876, 0.0007401135],
                [
                    -0.915178534128,
                    0.329132313337,
                    0.232637423876,
                    0.0007401135,
                ],
                [
                    0.915178534128,
                    -0.329132313337,
                    0.232637423876,
                    0.0007401135,
                ],
                [
                    -0.915178534128,
                    -0.329132313337,
                    0.232637423876,
                    0.0007401135,
                ],
                [0.915178534128, 0.232637423876, 0.329132313337, 0.0007401135],
                [
                    -0.915178534128,
                    0.232637423876,
                    0.329132313337,
                    0.0007401135,
                ],
                [
                    0.915178534128,
                    -0.232637423876,
                    0.329132313337,
                    0.0007401135,
                ],
                [
                    -0.915178534128,
                    -0.232637423876,
                    0.329132313337,
                    0.0007401135,
                ],
                [
                    0.373689697874,
                    0.275848580849,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    -0.373689697874,
                    0.275848580849,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    0.373689697874,
                    -0.275848580849,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    -0.373689697874,
                    -0.275848580849,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    0.373689697874,
                    0.885586568409,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    -0.373689697874,
                    0.885586568409,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    0.373689697874,
                    -0.885586568409,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    -0.373689697874,
                    -0.885586568409,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    0.275848580849,
                    0.373689697874,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    -0.275848580849,
                    0.373689697874,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    0.275848580849,
                    -0.373689697874,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    -0.275848580849,
                    -0.373689697874,
                    0.885586568409,
                    0.000765049075,
                ],
                [
                    0.275848580849,
                    0.885586568409,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    -0.275848580849,
                    0.885586568409,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    0.275848580849,
                    -0.885586568409,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    -0.275848580849,
                    -0.885586568409,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    0.885586568409,
                    0.373689697874,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    -0.885586568409,
                    0.373689697874,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    0.885586568409,
                    -0.373689697874,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    -0.885586568409,
                    -0.373689697874,
                    0.275848580849,
                    0.000765049075,
                ],
                [
                    0.885586568409,
                    0.275848580849,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    -0.885586568409,
                    0.275848580849,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    0.885586568409,
                    -0.275848580849,
                    0.373689697874,
                    0.000765049075,
                ],
                [
                    -0.885586568409,
                    -0.275848580849,
                    0.373689697874,
                    0.000765049075,
                ],
                [0.417140604076, 0.3186179332, 0.851161752592, 0.000783625034],
                [
                    -0.417140604076,
                    0.3186179332,
                    0.851161752592,
                    0.000783625034,
                ],
                [
                    0.417140604076,
                    -0.3186179332,
                    0.851161752592,
                    0.000783625034,
                ],
                [
                    -0.417140604076,
                    -0.3186179332,
                    0.851161752592,
                    0.000783625034,
                ],
                [0.417140604076, 0.851161752592, 0.3186179332, 0.000783625034],
                [
                    -0.417140604076,
                    0.851161752592,
                    0.3186179332,
                    0.000783625034,
                ],
                [
                    0.417140604076,
                    -0.851161752592,
                    0.3186179332,
                    0.000783625034,
                ],
                [
                    -0.417140604076,
                    -0.851161752592,
                    0.3186179332,
                    0.000783625034,
                ],
                [0.3186179332, 0.417140604076, 0.851161752592, 0.000783625034],
                [
                    -0.3186179332,
                    0.417140604076,
                    0.851161752592,
                    0.000783625034,
                ],
                [
                    0.3186179332,
                    -0.417140604076,
                    0.851161752592,
                    0.000783625034,
                ],
                [
                    -0.3186179332,
                    -0.417140604076,
                    0.851161752592,
                    0.000783625034,
                ],
                [0.3186179332, 0.851161752592, 0.417140604076, 0.000783625034],
                [
                    -0.3186179332,
                    0.851161752592,
                    0.417140604076,
                    0.000783625034,
                ],
                [
                    0.3186179332,
                    -0.851161752592,
                    0.417140604076,
                    0.000783625034,
                ],
                [
                    -0.3186179332,
                    -0.851161752592,
                    0.417140604076,
                    0.000783625034,
                ],
                [0.851161752592, 0.417140604076, 0.3186179332, 0.000783625034],
                [
                    -0.851161752592,
                    0.417140604076,
                    0.3186179332,
                    0.000783625034,
                ],
                [
                    0.851161752592,
                    -0.417140604076,
                    0.3186179332,
                    0.000783625034,
                ],
                [
                    -0.851161752592,
                    -0.417140604076,
                    0.3186179332,
                    0.000783625034,
                ],
                [0.851161752592, 0.3186179332, 0.417140604076, 0.000783625034],
                [
                    -0.851161752592,
                    0.3186179332,
                    0.417140604076,
                    0.000783625034,
                ],
                [
                    0.851161752592,
                    -0.3186179332,
                    0.417140604076,
                    0.000783625034,
                ],
                [
                    -0.851161752592,
                    -0.3186179332,
                    0.417140604076,
                    0.000783625034,
                ],
                [
                    0.459167798526,
                    0.36053297963,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    -0.459167798526,
                    0.36053297963,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    0.459167798526,
                    -0.36053297963,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    -0.459167798526,
                    -0.36053297963,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    0.459167798526,
                    0.811899564845,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    -0.459167798526,
                    0.811899564845,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    0.459167798526,
                    -0.811899564845,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    -0.459167798526,
                    -0.811899564845,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    0.36053297963,
                    0.459167798526,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    -0.36053297963,
                    0.459167798526,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    0.36053297963,
                    -0.459167798526,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    -0.36053297963,
                    -0.459167798526,
                    0.811899564845,
                    0.000796944084,
                ],
                [
                    0.36053297963,
                    0.811899564845,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    -0.36053297963,
                    0.811899564845,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    0.36053297963,
                    -0.811899564845,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    -0.36053297963,
                    -0.811899564845,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    0.811899564845,
                    0.459167798526,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    -0.811899564845,
                    0.459167798526,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    0.811899564845,
                    -0.459167798526,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    -0.811899564845,
                    -0.459167798526,
                    0.36053297963,
                    0.000796944084,
                ],
                [
                    0.811899564845,
                    0.36053297963,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    -0.811899564845,
                    0.36053297963,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    0.811899564845,
                    -0.36053297963,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    -0.811899564845,
                    -0.36053297963,
                    0.459167798526,
                    0.000796944084,
                ],
                [
                    0.499473383172,
                    0.401214725359,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    -0.499473383172,
                    0.401214725359,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    0.499473383172,
                    -0.401214725359,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    -0.499473383172,
                    -0.401214725359,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    0.499473383172,
                    0.767823601915,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    -0.499473383172,
                    0.767823601915,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    0.499473383172,
                    -0.767823601915,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    -0.499473383172,
                    -0.767823601915,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    0.401214725359,
                    0.499473383172,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    -0.401214725359,
                    0.499473383172,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    0.401214725359,
                    -0.499473383172,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    -0.401214725359,
                    -0.499473383172,
                    0.767823601915,
                    0.000805949201,
                ],
                [
                    0.401214725359,
                    0.767823601915,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    -0.401214725359,
                    0.767823601915,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    0.401214725359,
                    -0.767823601915,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    -0.401214725359,
                    -0.767823601915,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    0.767823601915,
                    0.499473383172,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    -0.767823601915,
                    0.499473383172,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    0.767823601915,
                    -0.499473383172,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    -0.767823601915,
                    -0.499473383172,
                    0.401214725359,
                    0.000805949201,
                ],
                [
                    0.767823601915,
                    0.401214725359,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    -0.767823601915,
                    0.401214725359,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    0.767823601915,
                    -0.401214725359,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    -0.767823601915,
                    -0.401214725359,
                    0.499473383172,
                    0.000805949201,
                ],
                [
                    0.537773183045,
                    0.440305002557,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    -0.537773183045,
                    0.440305002557,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    0.537773183045,
                    -0.440305002557,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    -0.537773183045,
                    -0.440305002557,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    0.537773183045,
                    0.718979490891,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    -0.537773183045,
                    0.718979490891,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    0.537773183045,
                    -0.718979490891,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    -0.537773183045,
                    -0.718979490891,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    0.440305002557,
                    0.537773183045,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    -0.440305002557,
                    0.537773183045,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    0.440305002557,
                    -0.537773183045,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    -0.440305002557,
                    -0.537773183045,
                    0.718979490891,
                    0.000811485726,
                ],
                [
                    0.440305002557,
                    0.718979490891,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    -0.440305002557,
                    0.718979490891,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    0.440305002557,
                    -0.718979490891,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    -0.440305002557,
                    -0.718979490891,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    0.718979490891,
                    0.537773183045,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    -0.718979490891,
                    0.537773183045,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    0.718979490891,
                    -0.537773183045,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    -0.718979490891,
                    -0.537773183045,
                    0.440305002557,
                    0.000811485726,
                ],
                [
                    0.718979490891,
                    0.440305002557,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    -0.718979490891,
                    0.440305002557,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    0.718979490891,
                    -0.440305002557,
                    0.537773183045,
                    0.000811485726,
                ],
                [
                    -0.718979490891,
                    -0.440305002557,
                    0.537773183045,
                    0.000811485726,
                ],
                [0.573791783, 0.477456590428, 0.665430833384, 0.000814343855],
                [-0.573791783, 0.477456590428, 0.665430833384, 0.000814343855],
                [0.573791783, -0.477456590428, 0.665430833384, 0.000814343855],
                [
                    -0.573791783,
                    -0.477456590428,
                    0.665430833384,
                    0.000814343855,
                ],
                [0.573791783, 0.665430833384, 0.477456590428, 0.000814343855],
                [-0.573791783, 0.665430833384, 0.477456590428, 0.000814343855],
                [0.573791783, -0.665430833384, 0.477456590428, 0.000814343855],
                [
                    -0.573791783,
                    -0.665430833384,
                    0.477456590428,
                    0.000814343855,
                ],
                [0.477456590428, 0.573791783, 0.665430833384, 0.000814343855],
                [-0.477456590428, 0.573791783, 0.665430833384, 0.000814343855],
                [0.477456590428, -0.573791783, 0.665430833384, 0.000814343855],
                [
                    -0.477456590428,
                    -0.573791783,
                    0.665430833384,
                    0.000814343855,
                ],
                [0.477456590428, 0.665430833384, 0.573791783, 0.000814343855],
                [-0.477456590428, 0.665430833384, 0.573791783, 0.000814343855],
                [0.477456590428, -0.665430833384, 0.573791783, 0.000814343855],
                [
                    -0.477456590428,
                    -0.665430833384,
                    0.573791783,
                    0.000814343855,
                ],
                [0.665430833384, 0.573791783, 0.477456590428, 0.000814343855],
                [-0.665430833384, 0.573791783, 0.477456590428, 0.000814343855],
                [0.665430833384, -0.573791783, 0.477456590428, 0.000814343855],
                [
                    -0.665430833384,
                    -0.573791783,
                    0.477456590428,
                    0.000814343855,
                ],
                [0.665430833384, 0.477456590428, 0.573791783, 0.000814343855],
                [-0.665430833384, 0.477456590428, 0.573791783, 0.000814343855],
                [0.665430833384, -0.477456590428, 0.573791783, 0.000814343855],
                [
                    -0.665430833384,
                    -0.477456590428,
                    0.573791783,
                    0.000814343855,
                ],
                [0.202732358627, 0.03544122505, 0.978592617146, 0.00059804739],
                [
                    -0.202732358627,
                    0.03544122505,
                    0.978592617146,
                    0.00059804739,
                ],
                [
                    0.202732358627,
                    -0.03544122505,
                    0.978592617146,
                    0.00059804739,
                ],
                [
                    -0.202732358627,
                    -0.03544122505,
                    0.978592617146,
                    0.00059804739,
                ],
                [0.202732358627, 0.978592617146, 0.03544122505, 0.00059804739],
                [
                    -0.202732358627,
                    0.978592617146,
                    0.03544122505,
                    0.00059804739,
                ],
                [
                    0.202732358627,
                    -0.978592617146,
                    0.03544122505,
                    0.00059804739,
                ],
                [
                    -0.202732358627,
                    -0.978592617146,
                    0.03544122505,
                    0.00059804739,
                ],
                [0.03544122505, 0.202732358627, 0.978592617146, 0.00059804739],
                [
                    -0.03544122505,
                    0.202732358627,
                    0.978592617146,
                    0.00059804739,
                ],
                [
                    0.03544122505,
                    -0.202732358627,
                    0.978592617146,
                    0.00059804739,
                ],
                [
                    -0.03544122505,
                    -0.202732358627,
                    0.978592617146,
                    0.00059804739,
                ],
                [0.03544122505, 0.978592617146, 0.202732358627, 0.00059804739],
                [
                    -0.03544122505,
                    0.978592617146,
                    0.202732358627,
                    0.00059804739,
                ],
                [
                    0.03544122505,
                    -0.978592617146,
                    0.202732358627,
                    0.00059804739,
                ],
                [
                    -0.03544122505,
                    -0.978592617146,
                    0.202732358627,
                    0.00059804739,
                ],
                [0.978592617146, 0.202732358627, 0.03544122505, 0.00059804739],
                [
                    -0.978592617146,
                    0.202732358627,
                    0.03544122505,
                    0.00059804739,
                ],
                [
                    0.978592617146,
                    -0.202732358627,
                    0.03544122505,
                    0.00059804739,
                ],
                [
                    -0.978592617146,
                    -0.202732358627,
                    0.03544122505,
                    0.00059804739,
                ],
                [0.978592617146, 0.03544122505, 0.202732358627, 0.00059804739],
                [
                    -0.978592617146,
                    0.03544122505,
                    0.202732358627,
                    0.00059804739,
                ],
                [
                    0.978592617146,
                    -0.03544122505,
                    0.202732358627,
                    0.00059804739,
                ],
                [
                    -0.978592617146,
                    -0.03544122505,
                    0.202732358627,
                    0.00059804739,
                ],
                [
                    0.251694237519,
                    0.074183043886,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    -0.251694237519,
                    0.074183043886,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    0.251694237519,
                    -0.074183043886,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    -0.251694237519,
                    -0.074183043886,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    0.251694237519,
                    0.96495952599,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    -0.251694237519,
                    0.96495952599,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    0.251694237519,
                    -0.96495952599,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    -0.251694237519,
                    -0.96495952599,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    0.074183043886,
                    0.251694237519,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    -0.074183043886,
                    0.251694237519,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    0.074183043886,
                    -0.251694237519,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    -0.074183043886,
                    -0.251694237519,
                    0.96495952599,
                    0.000652590347,
                ],
                [
                    0.074183043886,
                    0.96495952599,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    -0.074183043886,
                    0.96495952599,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    0.074183043886,
                    -0.96495952599,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    -0.074183043886,
                    -0.96495952599,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    0.96495952599,
                    0.251694237519,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    -0.96495952599,
                    0.251694237519,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    0.96495952599,
                    -0.251694237519,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    -0.96495952599,
                    -0.251694237519,
                    0.074183043886,
                    0.000652590347,
                ],
                [
                    0.96495952599,
                    0.074183043886,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    -0.96495952599,
                    0.074183043886,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    0.96495952599,
                    -0.074183043886,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    -0.96495952599,
                    -0.074183043886,
                    0.251694237519,
                    0.000652590347,
                ],
                [
                    0.300022799526,
                    0.115050274573,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    -0.300022799526,
                    0.115050274573,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    0.300022799526,
                    -0.115050274573,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    -0.300022799526,
                    -0.115050274573,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    0.300022799526,
                    0.946968718641,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    -0.300022799526,
                    0.946968718641,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    0.300022799526,
                    -0.946968718641,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    -0.300022799526,
                    -0.946968718641,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    0.115050274573,
                    0.300022799526,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    -0.115050274573,
                    0.300022799526,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    0.115050274573,
                    -0.300022799526,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    -0.115050274573,
                    -0.300022799526,
                    0.946968718641,
                    0.000696526922,
                ],
                [
                    0.115050274573,
                    0.946968718641,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    -0.115050274573,
                    0.946968718641,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    0.115050274573,
                    -0.946968718641,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    -0.115050274573,
                    -0.946968718641,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    0.946968718641,
                    0.300022799526,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    -0.946968718641,
                    0.300022799526,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    0.946968718641,
                    -0.300022799526,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    -0.946968718641,
                    -0.300022799526,
                    0.115050274573,
                    0.000696526922,
                ],
                [
                    0.946968718641,
                    0.115050274573,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    -0.946968718641,
                    0.115050274573,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    0.946968718641,
                    -0.115050274573,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    -0.946968718641,
                    -0.115050274573,
                    0.300022799526,
                    0.000696526922,
                ],
                [
                    0.347480669105,
                    0.157196337121,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    -0.347480669105,
                    0.157196337121,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    0.347480669105,
                    -0.157196337121,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    -0.347480669105,
                    -0.157196337121,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    0.347480669105,
                    0.924416841146,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    -0.347480669105,
                    0.924416841146,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    0.347480669105,
                    -0.924416841146,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    -0.347480669105,
                    -0.924416841146,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    0.157196337121,
                    0.347480669105,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    -0.157196337121,
                    0.347480669105,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    0.157196337121,
                    -0.347480669105,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    -0.157196337121,
                    -0.347480669105,
                    0.924416841146,
                    0.000731319336,
                ],
                [
                    0.157196337121,
                    0.924416841146,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    -0.157196337121,
                    0.924416841146,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    0.157196337121,
                    -0.924416841146,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    -0.157196337121,
                    -0.924416841146,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    0.924416841146,
                    0.347480669105,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    -0.924416841146,
                    0.347480669105,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    0.924416841146,
                    -0.347480669105,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    -0.924416841146,
                    -0.347480669105,
                    0.157196337121,
                    0.000731319336,
                ],
                [
                    0.924416841146,
                    0.157196337121,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    -0.924416841146,
                    0.157196337121,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    0.924416841146,
                    -0.157196337121,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    -0.924416841146,
                    -0.157196337121,
                    0.347480669105,
                    0.000731319336,
                ],
                [
                    0.393810318036,
                    0.199963187725,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    -0.393810318036,
                    0.199963187725,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    0.393810318036,
                    -0.199963187725,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    -0.393810318036,
                    -0.199963187725,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    0.393810318036,
                    0.897177884794,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    -0.393810318036,
                    0.897177884794,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    0.393810318036,
                    -0.897177884794,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    -0.393810318036,
                    -0.897177884794,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    0.199963187725,
                    0.393810318036,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    -0.199963187725,
                    0.393810318036,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    0.199963187725,
                    -0.393810318036,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    -0.199963187725,
                    -0.393810318036,
                    0.897177884794,
                    0.000758348094,
                ],
                [
                    0.199963187725,
                    0.897177884794,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    -0.199963187725,
                    0.897177884794,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    0.199963187725,
                    -0.897177884794,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    -0.199963187725,
                    -0.897177884794,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    0.897177884794,
                    0.393810318036,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    -0.897177884794,
                    0.393810318036,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    0.897177884794,
                    -0.393810318036,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    -0.897177884794,
                    -0.393810318036,
                    0.199963187725,
                    0.000758348094,
                ],
                [
                    0.897177884794,
                    0.199963187725,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    -0.897177884794,
                    0.199963187725,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    0.897177884794,
                    -0.199963187725,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    -0.897177884794,
                    -0.199963187725,
                    0.393810318036,
                    0.000758348094,
                ],
                [
                    0.438751959046,
                    0.242807345785,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    -0.438751959046,
                    0.242807345785,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    0.438751959046,
                    -0.242807345785,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    -0.438751959046,
                    -0.242807345785,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    0.438751959046,
                    0.865182819563,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    -0.438751959046,
                    0.865182819563,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    0.438751959046,
                    -0.865182819563,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    -0.438751959046,
                    -0.865182819563,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    0.242807345785,
                    0.438751959046,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    -0.242807345785,
                    0.438751959046,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    0.242807345785,
                    -0.438751959046,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    -0.242807345785,
                    -0.438751959046,
                    0.865182819563,
                    0.00077880689,
                ],
                [
                    0.242807345785,
                    0.865182819563,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    -0.242807345785,
                    0.865182819563,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    0.242807345785,
                    -0.865182819563,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    -0.242807345785,
                    -0.865182819563,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    0.865182819563,
                    0.438751959046,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    -0.865182819563,
                    0.438751959046,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    0.865182819563,
                    -0.438751959046,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    -0.865182819563,
                    -0.438751959046,
                    0.242807345785,
                    0.00077880689,
                ],
                [
                    0.865182819563,
                    0.242807345785,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    -0.865182819563,
                    0.242807345785,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    0.865182819563,
                    -0.242807345785,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    -0.865182819563,
                    -0.242807345785,
                    0.438751959046,
                    0.00077880689,
                ],
                [
                    0.482050396008,
                    0.285257513291,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    -0.482050396008,
                    0.285257513291,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    0.482050396008,
                    -0.285257513291,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    -0.482050396008,
                    -0.285257513291,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    0.482050396008,
                    0.828405436257,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    -0.482050396008,
                    0.828405436257,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    0.482050396008,
                    -0.828405436257,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    -0.482050396008,
                    -0.828405436257,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    0.285257513291,
                    0.482050396008,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    -0.285257513291,
                    0.482050396008,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    0.285257513291,
                    -0.482050396008,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    -0.285257513291,
                    -0.482050396008,
                    0.828405436257,
                    0.000793720049,
                ],
                [
                    0.285257513291,
                    0.828405436257,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    -0.285257513291,
                    0.828405436257,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    0.285257513291,
                    -0.828405436257,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    -0.285257513291,
                    -0.828405436257,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    0.828405436257,
                    0.482050396008,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    -0.828405436257,
                    0.482050396008,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    0.828405436257,
                    -0.482050396008,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    -0.828405436257,
                    -0.482050396008,
                    0.285257513291,
                    0.000793720049,
                ],
                [
                    0.828405436257,
                    0.285257513291,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    -0.828405436257,
                    0.285257513291,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    0.828405436257,
                    -0.285257513291,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    -0.828405436257,
                    -0.285257513291,
                    0.482050396008,
                    0.000793720049,
                ],
                [
                    0.523457377848,
                    0.326888420867,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    -0.523457377848,
                    0.326888420867,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    0.523457377848,
                    -0.326888420867,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    -0.523457377848,
                    -0.326888420867,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    0.523457377848,
                    0.786852167742,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    -0.523457377848,
                    0.786852167742,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    0.523457377848,
                    -0.786852167742,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    -0.523457377848,
                    -0.786852167742,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    0.326888420867,
                    0.523457377848,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    -0.326888420867,
                    0.523457377848,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    0.326888420867,
                    -0.523457377848,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    -0.326888420867,
                    -0.523457377848,
                    0.786852167742,
                    0.00080398627,
                ],
                [
                    0.326888420867,
                    0.786852167742,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    -0.326888420867,
                    0.786852167742,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    0.326888420867,
                    -0.786852167742,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    -0.326888420867,
                    -0.786852167742,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    0.786852167742,
                    0.523457377848,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    -0.786852167742,
                    0.523457377848,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    0.786852167742,
                    -0.523457377848,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    -0.786852167742,
                    -0.523457377848,
                    0.326888420867,
                    0.00080398627,
                ],
                [
                    0.786852167742,
                    0.326888420867,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    -0.786852167742,
                    0.326888420867,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    0.786852167742,
                    -0.326888420867,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    -0.786852167742,
                    -0.326888420867,
                    0.523457377848,
                    0.00080398627,
                ],
                [
                    0.562731864724,
                    0.367303332168,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    -0.562731864724,
                    0.367303332168,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    0.562731864724,
                    -0.367303332168,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    -0.562731864724,
                    -0.367303332168,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    0.562731864724,
                    0.740554596639,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    -0.562731864724,
                    0.740554596639,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    0.562731864724,
                    -0.740554596639,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    -0.562731864724,
                    -0.740554596639,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    0.367303332168,
                    0.562731864724,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    -0.367303332168,
                    0.562731864724,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    0.367303332168,
                    -0.562731864724,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    -0.367303332168,
                    -0.562731864724,
                    0.740554596639,
                    0.00081042176,
                ],
                [
                    0.367303332168,
                    0.740554596639,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    -0.367303332168,
                    0.740554596639,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    0.367303332168,
                    -0.740554596639,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    -0.367303332168,
                    -0.740554596639,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    0.740554596639,
                    0.562731864724,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    -0.740554596639,
                    0.562731864724,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    0.740554596639,
                    -0.562731864724,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    -0.740554596639,
                    -0.562731864724,
                    0.367303332168,
                    0.00081042176,
                ],
                [
                    0.740554596639,
                    0.367303332168,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    -0.740554596639,
                    0.367303332168,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    0.740554596639,
                    -0.367303332168,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    -0.740554596639,
                    -0.367303332168,
                    0.562731864724,
                    0.00081042176,
                ],
                [
                    0.599639060716,
                    0.406121155183,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    -0.599639060716,
                    0.406121155183,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    0.599639060716,
                    -0.406121155183,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    -0.599639060716,
                    -0.406121155183,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    0.599639060716,
                    0.689564068218,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    -0.599639060716,
                    0.689564068218,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    0.599639060716,
                    -0.689564068218,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    -0.599639060716,
                    -0.689564068218,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    0.406121155183,
                    0.599639060716,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    -0.406121155183,
                    0.599639060716,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    0.406121155183,
                    -0.599639060716,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    -0.406121155183,
                    -0.599639060716,
                    0.689564068218,
                    0.000813795723,
                ],
                [
                    0.406121155183,
                    0.689564068218,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    -0.406121155183,
                    0.689564068218,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    0.406121155183,
                    -0.689564068218,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    -0.406121155183,
                    -0.689564068218,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    0.689564068218,
                    0.599639060716,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    -0.689564068218,
                    0.599639060716,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    0.689564068218,
                    -0.599639060716,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    -0.689564068218,
                    -0.599639060716,
                    0.406121155183,
                    0.000813795723,
                ],
                [
                    0.689564068218,
                    0.406121155183,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    -0.689564068218,
                    0.406121155183,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    0.689564068218,
                    -0.406121155183,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    -0.689564068218,
                    -0.406121155183,
                    0.599639060716,
                    0.000813795723,
                ],
                [
                    0.308478075379,
                    0.038601255231,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    -0.308478075379,
                    0.038601255231,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    0.308478075379,
                    -0.038601255231,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    -0.308478075379,
                    -0.038601255231,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    0.308478075379,
                    0.950447904993,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    -0.308478075379,
                    0.950447904993,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    0.308478075379,
                    -0.950447904993,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    -0.308478075379,
                    -0.950447904993,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    0.038601255231,
                    0.308478075379,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    -0.038601255231,
                    0.308478075379,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    0.038601255231,
                    -0.308478075379,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    -0.038601255231,
                    -0.308478075379,
                    0.950447904993,
                    0.00069085507,
                ],
                [
                    0.038601255231,
                    0.950447904993,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    -0.038601255231,
                    0.950447904993,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    0.038601255231,
                    -0.950447904993,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    -0.038601255231,
                    -0.950447904993,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    0.950447904993,
                    0.308478075379,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    -0.950447904993,
                    0.308478075379,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    0.950447904993,
                    -0.308478075379,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    -0.950447904993,
                    -0.308478075379,
                    0.038601255231,
                    0.00069085507,
                ],
                [
                    0.950447904993,
                    0.038601255231,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    -0.950447904993,
                    0.038601255231,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    0.950447904993,
                    -0.038601255231,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    -0.950447904993,
                    -0.038601255231,
                    0.308478075379,
                    0.00069085507,
                ],
                [
                    0.358998827592,
                    0.079289389871,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    -0.358998827592,
                    0.079289389871,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    0.358998827592,
                    -0.079289389871,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    -0.358998827592,
                    -0.079289389871,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    0.358998827592,
                    0.929963996315,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    -0.358998827592,
                    0.929963996315,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    0.358998827592,
                    -0.929963996315,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    -0.358998827592,
                    -0.929963996315,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    0.079289389871,
                    0.358998827592,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    -0.079289389871,
                    0.358998827592,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    0.079289389871,
                    -0.358998827592,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    -0.079289389871,
                    -0.358998827592,
                    0.929963996315,
                    0.000725992707,
                ],
                [
                    0.079289389871,
                    0.929963996315,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    -0.079289389871,
                    0.929963996315,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    0.079289389871,
                    -0.929963996315,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    -0.079289389871,
                    -0.929963996315,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    0.929963996315,
                    0.358998827592,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    -0.929963996315,
                    0.358998827592,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    0.929963996315,
                    -0.358998827592,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    -0.929963996315,
                    -0.358998827592,
                    0.079289389871,
                    0.000725992707,
                ],
                [
                    0.929963996315,
                    0.079289389871,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    -0.929963996315,
                    0.079289389871,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    0.929963996315,
                    -0.079289389871,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    -0.929963996315,
                    -0.079289389871,
                    0.358998827592,
                    0.000725992707,
                ],
                [
                    0.407862841588,
                    0.121261464303,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    -0.407862841588,
                    0.121261464303,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    0.407862841588,
                    -0.121261464303,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    -0.407862841588,
                    -0.121261464303,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    0.407862841588,
                    0.904955004255,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    -0.407862841588,
                    0.904955004255,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    0.407862841588,
                    -0.904955004255,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    -0.407862841588,
                    -0.904955004255,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    0.121261464303,
                    0.407862841588,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    -0.121261464303,
                    0.407862841588,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    0.121261464303,
                    -0.407862841588,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    -0.121261464303,
                    -0.407862841588,
                    0.904955004255,
                    0.000754037447,
                ],
                [
                    0.121261464303,
                    0.904955004255,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    -0.121261464303,
                    0.904955004255,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    0.121261464303,
                    -0.904955004255,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    -0.121261464303,
                    -0.904955004255,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    0.904955004255,
                    0.407862841588,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    -0.904955004255,
                    0.407862841588,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    0.904955004255,
                    -0.407862841588,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    -0.904955004255,
                    -0.407862841588,
                    0.121261464303,
                    0.000754037447,
                ],
                [
                    0.904955004255,
                    0.121261464303,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    -0.904955004255,
                    0.121261464303,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    0.904955004255,
                    -0.121261464303,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    -0.904955004255,
                    -0.121261464303,
                    0.407862841588,
                    0.000754037447,
                ],
                [
                    0.454928725889,
                    0.163877082738,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    -0.454928725889,
                    0.163877082738,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    0.454928725889,
                    -0.163877082738,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    -0.454928725889,
                    -0.163877082738,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    0.454928725889,
                    0.875319459463,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    -0.454928725889,
                    0.875319459463,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    0.454928725889,
                    -0.875319459463,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    -0.454928725889,
                    -0.875319459463,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    0.163877082738,
                    0.454928725889,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    -0.163877082738,
                    0.454928725889,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    0.163877082738,
                    -0.454928725889,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    -0.163877082738,
                    -0.454928725889,
                    0.875319459463,
                    0.000775721723,
                ],
                [
                    0.163877082738,
                    0.875319459463,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    -0.163877082738,
                    0.875319459463,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    0.163877082738,
                    -0.875319459463,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    -0.163877082738,
                    -0.875319459463,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    0.875319459463,
                    0.454928725889,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    -0.875319459463,
                    0.454928725889,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    0.875319459463,
                    -0.454928725889,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    -0.875319459463,
                    -0.454928725889,
                    0.163877082738,
                    0.000775721723,
                ],
                [
                    0.875319459463,
                    0.163877082738,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    -0.875319459463,
                    0.163877082738,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    0.875319459463,
                    -0.163877082738,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    -0.875319459463,
                    -0.163877082738,
                    0.454928725889,
                    0.000775721723,
                ],
                [
                    0.500027851296,
                    0.206596579826,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    -0.500027851296,
                    0.206596579826,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    0.500027851296,
                    -0.206596579826,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    -0.500027851296,
                    -0.206596579826,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    0.500027851296,
                    0.841005351429,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    -0.500027851296,
                    0.841005351429,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    0.500027851296,
                    -0.841005351429,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    -0.500027851296,
                    -0.841005351429,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    0.206596579826,
                    0.500027851296,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    -0.206596579826,
                    0.500027851296,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    0.206596579826,
                    -0.500027851296,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    -0.206596579826,
                    -0.500027851296,
                    0.841005351429,
                    0.000791813054,
                ],
                [
                    0.206596579826,
                    0.841005351429,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    -0.206596579826,
                    0.841005351429,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    0.206596579826,
                    -0.841005351429,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    -0.206596579826,
                    -0.841005351429,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    0.841005351429,
                    0.500027851296,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    -0.841005351429,
                    0.500027851296,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    0.841005351429,
                    -0.500027851296,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    -0.841005351429,
                    -0.500027851296,
                    0.206596579826,
                    0.000791813054,
                ],
                [
                    0.841005351429,
                    0.206596579826,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    -0.841005351429,
                    0.206596579826,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    0.841005351429,
                    -0.206596579826,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    -0.841005351429,
                    -0.206596579826,
                    0.500027851296,
                    0.000791813054,
                ],
                [
                    0.542978504493,
                    0.248943637885,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    -0.542978504493,
                    0.248943637885,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    0.542978504493,
                    -0.248943637885,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    -0.542978504493,
                    -0.248943637885,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    0.542978504493,
                    0.801998384547,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    -0.542978504493,
                    0.801998384547,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    0.542978504493,
                    -0.801998384547,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    -0.542978504493,
                    -0.801998384547,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    0.248943637885,
                    0.542978504493,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    -0.248943637885,
                    0.542978504493,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    0.248943637885,
                    -0.542978504493,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    -0.248943637885,
                    -0.542978504493,
                    0.801998384547,
                    0.000803057395,
                ],
                [
                    0.248943637885,
                    0.801998384547,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    -0.248943637885,
                    0.801998384547,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    0.248943637885,
                    -0.801998384547,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    -0.248943637885,
                    -0.801998384547,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    0.801998384547,
                    0.542978504493,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    -0.801998384547,
                    0.542978504493,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    0.801998384547,
                    -0.542978504493,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    -0.801998384547,
                    -0.542978504493,
                    0.248943637885,
                    0.000803057395,
                ],
                [
                    0.801998384547,
                    0.248943637885,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    -0.801998384547,
                    0.248943637885,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    0.801998384547,
                    -0.248943637885,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    -0.801998384547,
                    -0.248943637885,
                    0.542978504493,
                    0.000803057395,
                ],
                [
                    0.583593985049,
                    0.290481136895,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    -0.583593985049,
                    0.290481136895,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    0.583593985049,
                    -0.290481136895,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    -0.583593985049,
                    -0.290481136895,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    0.583593985049,
                    0.758313107972,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    -0.583593985049,
                    0.758313107972,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    0.583593985049,
                    -0.758313107972,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    -0.583593985049,
                    -0.758313107972,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    0.290481136895,
                    0.583593985049,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    -0.290481136895,
                    0.583593985049,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    0.290481136895,
                    -0.583593985049,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    -0.290481136895,
                    -0.583593985049,
                    0.758313107972,
                    0.000810173357,
                ],
                [
                    0.290481136895,
                    0.758313107972,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    -0.290481136895,
                    0.758313107972,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    0.290481136895,
                    -0.758313107972,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    -0.290481136895,
                    -0.758313107972,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    0.758313107972,
                    0.583593985049,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    -0.758313107972,
                    0.583593985049,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    0.758313107972,
                    -0.583593985049,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    -0.758313107972,
                    -0.583593985049,
                    0.290481136895,
                    0.000810173357,
                ],
                [
                    0.758313107972,
                    0.290481136895,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    -0.758313107972,
                    0.290481136895,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    0.758313107972,
                    -0.290481136895,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    -0.758313107972,
                    -0.290481136895,
                    0.583593985049,
                    0.000810173357,
                ],
                [
                    0.621687035344,
                    0.330794195767,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    -0.621687035344,
                    0.330794195767,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    0.621687035344,
                    -0.330794195767,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    -0.621687035344,
                    -0.330794195767,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    0.621687035344,
                    0.709986218269,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    -0.621687035344,
                    0.709986218269,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    0.621687035344,
                    -0.709986218269,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    -0.621687035344,
                    -0.709986218269,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    0.330794195767,
                    0.621687035344,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    -0.330794195767,
                    0.621687035344,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    0.330794195767,
                    -0.621687035344,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    -0.330794195767,
                    -0.621687035344,
                    0.709986218269,
                    0.000813864037,
                ],
                [
                    0.330794195767,
                    0.709986218269,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    -0.330794195767,
                    0.709986218269,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    0.330794195767,
                    -0.709986218269,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    -0.330794195767,
                    -0.709986218269,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    0.709986218269,
                    0.621687035344,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    -0.709986218269,
                    0.621687035344,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    0.709986218269,
                    -0.621687035344,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    -0.709986218269,
                    -0.621687035344,
                    0.330794195767,
                    0.000813864037,
                ],
                [
                    0.709986218269,
                    0.330794195767,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    -0.709986218269,
                    0.330794195767,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    0.709986218269,
                    -0.330794195767,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    -0.709986218269,
                    -0.330794195767,
                    0.621687035344,
                    0.000813864037,
                ],
                [
                    0.415110466271,
                    0.040648291461,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    -0.415110466271,
                    0.040648291461,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    0.415110466271,
                    -0.040648291461,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    -0.415110466271,
                    -0.040648291461,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    0.415110466271,
                    0.908862485304,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    -0.415110466271,
                    0.908862485304,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    0.415110466271,
                    -0.908862485304,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    -0.415110466271,
                    -0.908862485304,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    0.040648291461,
                    0.415110466271,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    -0.040648291461,
                    0.415110466271,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    0.040648291461,
                    -0.415110466271,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    -0.040648291461,
                    -0.415110466271,
                    0.908862485304,
                    0.000752024193,
                ],
                [
                    0.040648291461,
                    0.908862485304,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    -0.040648291461,
                    0.908862485304,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    0.040648291461,
                    -0.908862485304,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    -0.040648291461,
                    -0.908862485304,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    0.908862485304,
                    0.415110466271,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    -0.908862485304,
                    0.415110466271,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    0.908862485304,
                    -0.415110466271,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    -0.908862485304,
                    -0.415110466271,
                    0.040648291461,
                    0.000752024193,
                ],
                [
                    0.908862485304,
                    0.040648291461,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    -0.908862485304,
                    0.040648291461,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    0.908862485304,
                    -0.040648291461,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    -0.908862485304,
                    -0.040648291461,
                    0.415110466271,
                    0.000752024193,
                ],
                [
                    0.464980427501,
                    0.082584245473,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    -0.464980427501,
                    0.082584245473,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    0.464980427501,
                    -0.082584245473,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    -0.464980427501,
                    -0.082584245473,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    0.464980427501,
                    0.881460744696,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    -0.464980427501,
                    0.881460744696,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    0.464980427501,
                    -0.881460744696,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    -0.464980427501,
                    -0.881460744696,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    0.082584245473,
                    0.464980427501,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    -0.082584245473,
                    0.464980427501,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    0.082584245473,
                    -0.464980427501,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    -0.082584245473,
                    -0.464980427501,
                    0.881460744696,
                    0.000774193913,
                ],
                [
                    0.082584245473,
                    0.881460744696,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    -0.082584245473,
                    0.881460744696,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    0.082584245473,
                    -0.881460744696,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    -0.082584245473,
                    -0.881460744696,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    0.881460744696,
                    0.464980427501,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    -0.881460744696,
                    0.464980427501,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    0.881460744696,
                    -0.464980427501,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    -0.881460744696,
                    -0.464980427501,
                    0.082584245473,
                    0.000774193913,
                ],
                [
                    0.881460744696,
                    0.082584245473,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    -0.881460744696,
                    0.082584245473,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    0.881460744696,
                    -0.082584245473,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    -0.881460744696,
                    -0.082584245473,
                    0.464980427501,
                    0.000774193913,
                ],
                [
                    0.512469575701,
                    0.125184196203,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    -0.512469575701,
                    0.125184196203,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    0.512469575701,
                    -0.125184196203,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    -0.512469575701,
                    -0.125184196203,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    0.512469575701,
                    0.849531547973,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    -0.512469575701,
                    0.849531547973,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    0.512469575701,
                    -0.849531547973,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    -0.512469575701,
                    -0.849531547973,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    0.125184196203,
                    0.512469575701,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    -0.125184196203,
                    0.512469575701,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    0.125184196203,
                    -0.512469575701,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    -0.125184196203,
                    -0.512469575701,
                    0.849531547973,
                    0.000791057558,
                ],
                [
                    0.125184196203,
                    0.849531547973,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    -0.125184196203,
                    0.849531547973,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    0.125184196203,
                    -0.849531547973,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    -0.125184196203,
                    -0.849531547973,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    0.849531547973,
                    0.512469575701,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    -0.849531547973,
                    0.512469575701,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    0.849531547973,
                    -0.512469575701,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    -0.849531547973,
                    -0.512469575701,
                    0.125184196203,
                    0.000791057558,
                ],
                [
                    0.849531547973,
                    0.125184196203,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    -0.849531547973,
                    0.125184196203,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    0.849531547973,
                    -0.125184196203,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    -0.849531547973,
                    -0.125184196203,
                    0.512469575701,
                    0.000791057558,
                ],
                [
                    0.557471110061,
                    0.167910750598,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    -0.557471110061,
                    0.167910750598,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    0.557471110061,
                    -0.167910750598,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    -0.557471110061,
                    -0.167910750598,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    0.557471110061,
                    0.813038708354,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    -0.557471110061,
                    0.813038708354,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    0.557471110061,
                    -0.813038708354,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    -0.557471110061,
                    -0.813038708354,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    0.167910750598,
                    0.557471110061,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    -0.167910750598,
                    0.557471110061,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    0.167910750598,
                    -0.557471110061,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    -0.167910750598,
                    -0.557471110061,
                    0.813038708354,
                    0.000803072382,
                ],
                [
                    0.167910750598,
                    0.813038708354,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    -0.167910750598,
                    0.813038708354,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    0.167910750598,
                    -0.813038708354,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    -0.167910750598,
                    -0.813038708354,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    0.813038708354,
                    0.557471110061,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    -0.813038708354,
                    0.557471110061,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    0.813038708354,
                    -0.557471110061,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    -0.813038708354,
                    -0.557471110061,
                    0.167910750598,
                    0.000803072382,
                ],
                [
                    0.813038708354,
                    0.167910750598,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    -0.813038708354,
                    0.167910750598,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    0.813038708354,
                    -0.167910750598,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    -0.813038708354,
                    -0.167910750598,
                    0.557471110061,
                    0.000803072382,
                ],
                [
                    0.599859733329,
                    0.210280505736,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    -0.599859733329,
                    0.210280505736,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    0.599859733329,
                    -0.210280505736,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    -0.599859733329,
                    -0.210280505736,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    0.599859733329,
                    0.771978244019,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    -0.599859733329,
                    0.771978244019,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    0.599859733329,
                    -0.771978244019,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    -0.599859733329,
                    -0.771978244019,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    0.210280505736,
                    0.599859733329,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    -0.210280505736,
                    0.599859733329,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    0.210280505736,
                    -0.599859733329,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    -0.210280505736,
                    -0.599859733329,
                    0.771978244019,
                    0.000810767397,
                ],
                [
                    0.210280505736,
                    0.771978244019,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    -0.210280505736,
                    0.771978244019,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    0.210280505736,
                    -0.771978244019,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    -0.210280505736,
                    -0.771978244019,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    0.771978244019,
                    0.599859733329,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    -0.771978244019,
                    0.599859733329,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    0.771978244019,
                    -0.599859733329,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    -0.771978244019,
                    -0.599859733329,
                    0.210280505736,
                    0.000810767397,
                ],
                [
                    0.771978244019,
                    0.210280505736,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    -0.771978244019,
                    0.210280505736,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    0.771978244019,
                    -0.210280505736,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    -0.771978244019,
                    -0.210280505736,
                    0.599859733329,
                    0.000810767397,
                ],
                [
                    0.639500714852,
                    0.251841808777,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    -0.639500714852,
                    0.251841808777,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    0.639500714852,
                    -0.251841808777,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    -0.639500714852,
                    -0.251841808777,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    0.639500714852,
                    0.726370799975,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    -0.639500714852,
                    0.726370799975,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    0.639500714852,
                    -0.726370799975,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    -0.639500714852,
                    -0.726370799975,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    0.251841808777,
                    0.639500714852,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    -0.251841808777,
                    0.639500714852,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    0.251841808777,
                    -0.639500714852,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    -0.251841808777,
                    -0.639500714852,
                    0.726370799975,
                    0.000814715735,
                ],
                [
                    0.251841808777,
                    0.726370799975,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    -0.251841808777,
                    0.726370799975,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    0.251841808777,
                    -0.726370799975,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    -0.251841808777,
                    -0.726370799975,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    0.726370799975,
                    0.639500714852,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    -0.726370799975,
                    0.639500714852,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    0.726370799975,
                    -0.639500714852,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    -0.726370799975,
                    -0.639500714852,
                    0.251841808777,
                    0.000814715735,
                ],
                [
                    0.726370799975,
                    0.251841808777,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    -0.726370799975,
                    0.251841808777,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    0.726370799975,
                    -0.251841808777,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    -0.726370799975,
                    -0.251841808777,
                    0.639500714852,
                    0.000814715735,
                ],
                [
                    0.518845622475,
                    0.041943216761,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    -0.518845622475,
                    0.041943216761,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    0.518845622475,
                    -0.041943216761,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    -0.518845622475,
                    -0.041943216761,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    0.518845622475,
                    0.85383838436,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    -0.518845622475,
                    0.85383838436,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    0.518845622475,
                    -0.85383838436,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    -0.518845622475,
                    -0.85383838436,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    0.041943216761,
                    0.518845622475,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    -0.041943216761,
                    0.518845622475,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    0.041943216761,
                    -0.518845622475,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    -0.041943216761,
                    -0.518845622475,
                    0.85383838436,
                    0.000790925676,
                ],
                [
                    0.041943216761,
                    0.85383838436,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    -0.041943216761,
                    0.85383838436,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    0.041943216761,
                    -0.85383838436,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    -0.041943216761,
                    -0.85383838436,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    0.85383838436,
                    0.518845622475,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    -0.85383838436,
                    0.518845622475,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    0.85383838436,
                    -0.518845622475,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    -0.85383838436,
                    -0.518845622475,
                    0.041943216761,
                    0.000790925676,
                ],
                [
                    0.85383838436,
                    0.041943216761,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    -0.85383838436,
                    0.041943216761,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    0.85383838436,
                    -0.041943216761,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    -0.85383838436,
                    -0.041943216761,
                    0.518845622475,
                    0.000790925676,
                ],
                [
                    0.566419070794,
                    0.084576615519,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    -0.566419070794,
                    0.084576615519,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    0.566419070794,
                    -0.084576615519,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    -0.566419070794,
                    -0.084576615519,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    0.566419070794,
                    0.819765961935,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    -0.566419070794,
                    0.819765961935,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    0.566419070794,
                    -0.819765961935,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    -0.566419070794,
                    -0.819765961935,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    0.084576615519,
                    0.566419070794,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    -0.084576615519,
                    0.566419070794,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    0.084576615519,
                    -0.566419070794,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    -0.084576615519,
                    -0.566419070794,
                    0.819765961935,
                    0.000803529102,
                ],
                [
                    0.084576615519,
                    0.819765961935,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    -0.084576615519,
                    0.819765961935,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    0.084576615519,
                    -0.819765961935,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    -0.084576615519,
                    -0.819765961935,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    0.819765961935,
                    0.566419070794,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    -0.819765961935,
                    0.566419070794,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    0.819765961935,
                    -0.566419070794,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    -0.819765961935,
                    -0.566419070794,
                    0.084576615519,
                    0.000803529102,
                ],
                [
                    0.819765961935,
                    0.084576615519,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    -0.819765961935,
                    0.084576615519,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    0.819765961935,
                    -0.084576615519,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    -0.819765961935,
                    -0.084576615519,
                    0.566419070794,
                    0.000803529102,
                ],
                [
                    0.611046435328,
                    0.127365293252,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    -0.611046435328,
                    0.127365293252,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    0.611046435328,
                    -0.127365293252,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    -0.611046435328,
                    -0.127365293252,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    0.611046435328,
                    0.781281214383,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    -0.611046435328,
                    0.781281214383,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    0.611046435328,
                    -0.781281214383,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    -0.611046435328,
                    -0.781281214383,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    0.127365293252,
                    0.611046435328,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    -0.127365293252,
                    0.611046435328,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    0.127365293252,
                    -0.611046435328,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    -0.127365293252,
                    -0.611046435328,
                    0.781281214383,
                    0.00081180607,
                ],
                [
                    0.127365293252,
                    0.781281214383,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    -0.127365293252,
                    0.781281214383,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    0.127365293252,
                    -0.781281214383,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    -0.127365293252,
                    -0.781281214383,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    0.781281214383,
                    0.611046435328,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    -0.781281214383,
                    0.611046435328,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    0.781281214383,
                    -0.611046435328,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    -0.781281214383,
                    -0.611046435328,
                    0.127365293252,
                    0.00081180607,
                ],
                [
                    0.781281214383,
                    0.127365293252,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    -0.781281214383,
                    0.127365293252,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    0.781281214383,
                    -0.127365293252,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    -0.781281214383,
                    -0.127365293252,
                    0.611046435328,
                    0.00081180607,
                ],
                [
                    0.652643030205,
                    0.169817323908,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    -0.652643030205,
                    0.169817323908,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    0.652643030205,
                    -0.169817323908,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    -0.652643030205,
                    -0.169817323908,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    0.652643030205,
                    0.738389566303,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    -0.652643030205,
                    0.738389566303,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    0.652643030205,
                    -0.738389566303,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    -0.652643030205,
                    -0.738389566303,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    0.169817323908,
                    0.652643030205,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    -0.169817323908,
                    0.652643030205,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    0.169817323908,
                    -0.652643030205,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    -0.169817323908,
                    -0.652643030205,
                    0.738389566303,
                    0.000816113162,
                ],
                [
                    0.169817323908,
                    0.738389566303,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    -0.169817323908,
                    0.738389566303,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    0.169817323908,
                    -0.738389566303,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    -0.169817323908,
                    -0.738389566303,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    0.738389566303,
                    0.652643030205,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    -0.738389566303,
                    0.652643030205,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    0.738389566303,
                    -0.652643030205,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    -0.738389566303,
                    -0.652643030205,
                    0.169817323908,
                    0.000816113162,
                ],
                [
                    0.738389566303,
                    0.169817323908,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    -0.738389566303,
                    0.169817323908,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    0.738389566303,
                    -0.169817323908,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    -0.738389566303,
                    -0.169817323908,
                    0.652643030205,
                    0.000816113162,
                ],
                [
                    0.616755188038,
                    0.042663988515,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    -0.616755188038,
                    0.042663988515,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    0.616755188038,
                    -0.042663988515,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    -0.616755188038,
                    -0.042663988515,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    0.616755188038,
                    0.78599797844,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    -0.616755188038,
                    0.78599797844,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    0.616755188038,
                    -0.78599797844,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    -0.616755188038,
                    -0.78599797844,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    0.042663988515,
                    0.616755188038,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    -0.042663988515,
                    0.616755188038,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    0.042663988515,
                    -0.616755188038,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    -0.042663988515,
                    -0.616755188038,
                    0.78599797844,
                    0.000812603751,
                ],
                [
                    0.042663988515,
                    0.78599797844,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    -0.042663988515,
                    0.78599797844,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    0.042663988515,
                    -0.78599797844,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    -0.042663988515,
                    -0.78599797844,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    0.78599797844,
                    0.616755188038,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    -0.78599797844,
                    0.616755188038,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    0.78599797844,
                    -0.616755188038,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    -0.78599797844,
                    -0.616755188038,
                    0.042663988515,
                    0.000812603751,
                ],
                [
                    0.78599797844,
                    0.042663988515,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    -0.78599797844,
                    0.042663988515,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    0.78599797844,
                    -0.042663988515,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    -0.78599797844,
                    -0.042663988515,
                    0.616755188038,
                    0.000812603751,
                ],
                [
                    0.660719541836,
                    0.085519258142,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    -0.660719541836,
                    0.085519258142,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    0.660719541836,
                    -0.085519258142,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    -0.660719541836,
                    -0.085519258142,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    0.660719541836,
                    0.745745361047,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    -0.660719541836,
                    0.745745361047,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    0.660719541836,
                    -0.745745361047,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    -0.660719541836,
                    -0.745745361047,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    0.085519258142,
                    0.660719541836,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    -0.085519258142,
                    0.660719541836,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    0.085519258142,
                    -0.660719541836,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    -0.085519258142,
                    -0.660719541836,
                    0.745745361047,
                    0.000817438259,
                ],
                [
                    0.085519258142,
                    0.745745361047,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    -0.085519258142,
                    0.745745361047,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    0.085519258142,
                    -0.745745361047,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    -0.085519258142,
                    -0.745745361047,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    0.745745361047,
                    0.660719541836,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    -0.745745361047,
                    0.660719541836,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    0.745745361047,
                    -0.660719541836,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    -0.745745361047,
                    -0.660719541836,
                    0.085519258142,
                    0.000817438259,
                ],
                [
                    0.745745361047,
                    0.085519258142,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    -0.745745361047,
                    0.085519258142,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    0.745745361047,
                    -0.085519258142,
                    0.660719541836,
                    0.000817438259,
                ],
                [
                    -0.745745361047,
                    -0.085519258142,
                    0.660719541836,
                    0.000817438259,
                ],
            ]
        )
        return grid_array

    if grid == 11:
        grid_array = array(
            [
                [1.0, 0.0, 0.0, 9.735348e-06],
                [-1.0, 0.0, 0.0, 9.735348e-06],
                [0.0, 1.0, 0.0, 9.735348e-06],
                [0.0, -1.0, 0.0, 9.735348e-06],
                [0.0, 0.0, 1.0, 1.9470696e-05],
                [0.0, 0.707106781187, 0.707106781187, 0.000381516248],
                [0.0, -0.707106781187, 0.707106781187, 0.000381516248],
                [0.707106781187, 0.0, 0.707106781187, 0.000381516248],
                [-0.707106781187, 0.0, 0.707106781187, 0.000381516248],
                [0.707106781187, 0.707106781187, 0.0, 0.000190758124],
                [-0.707106781187, 0.707106781187, 0.0, 0.000190758124],
                [0.707106781187, -0.707106781187, 0.0, 0.000190758124],
                [-0.707106781187, -0.707106781187, 0.0, 0.000190758124],
                [0.57735026919, 0.57735026919, 0.57735026919, 0.000380211909],
                [-0.57735026919, 0.57735026919, 0.57735026919, 0.000380211909],
                [0.57735026919, -0.57735026919, 0.57735026919, 0.000380211909],
                [
                    -0.57735026919,
                    -0.57735026919,
                    0.57735026919,
                    0.000380211909,
                ],
                [
                    0.011823616624,
                    0.011823616624,
                    0.999860192317,
                    7.8528491e-05,
                ],
                [
                    -0.011823616624,
                    0.011823616624,
                    0.999860192317,
                    7.8528491e-05,
                ],
                [
                    0.011823616624,
                    -0.011823616624,
                    0.999860192317,
                    7.8528491e-05,
                ],
                [
                    -0.011823616624,
                    -0.011823616624,
                    0.999860192317,
                    7.8528491e-05,
                ],
                [
                    0.011823616624,
                    0.999860192317,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    -0.011823616624,
                    0.999860192317,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    0.011823616624,
                    -0.999860192317,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    -0.011823616624,
                    -0.999860192317,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    0.999860192317,
                    0.011823616624,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    -0.999860192317,
                    0.011823616624,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    0.999860192317,
                    -0.011823616624,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    -0.999860192317,
                    -0.011823616624,
                    0.011823616624,
                    7.8528491e-05,
                ],
                [
                    0.030621450091,
                    0.030621450091,
                    0.999061886766,
                    0.000133358109,
                ],
                [
                    -0.030621450091,
                    0.030621450091,
                    0.999061886766,
                    0.000133358109,
                ],
                [
                    0.030621450091,
                    -0.030621450091,
                    0.999061886766,
                    0.000133358109,
                ],
                [
                    -0.030621450091,
                    -0.030621450091,
                    0.999061886766,
                    0.000133358109,
                ],
                [
                    0.030621450091,
                    0.999061886766,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    -0.030621450091,
                    0.999061886766,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    0.030621450091,
                    -0.999061886766,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    -0.030621450091,
                    -0.999061886766,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    0.999061886766,
                    0.030621450091,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    -0.999061886766,
                    0.030621450091,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    0.999061886766,
                    -0.030621450091,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    -0.999061886766,
                    -0.030621450091,
                    0.030621450091,
                    0.000133358109,
                ],
                [
                    0.053297940368,
                    0.053297940368,
                    0.997155283346,
                    0.000177377826,
                ],
                [
                    -0.053297940368,
                    0.053297940368,
                    0.997155283346,
                    0.000177377826,
                ],
                [
                    0.053297940368,
                    -0.053297940368,
                    0.997155283346,
                    0.000177377826,
                ],
                [
                    -0.053297940368,
                    -0.053297940368,
                    0.997155283346,
                    0.000177377826,
                ],
                [
                    0.053297940368,
                    0.997155283346,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    -0.053297940368,
                    0.997155283346,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    0.053297940368,
                    -0.997155283346,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    -0.053297940368,
                    -0.997155283346,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    0.997155283346,
                    0.053297940368,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    -0.997155283346,
                    0.053297940368,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    0.997155283346,
                    -0.053297940368,
                    0.053297940368,
                    0.000177377826,
                ],
                [
                    -0.997155283346,
                    -0.053297940368,
                    0.053297940368,
                    0.000177377826,
                ],
                [0.078481655329, 0.078481655329, 0.993821543112, 0.0002132612],
                [
                    -0.078481655329,
                    0.078481655329,
                    0.993821543112,
                    0.0002132612,
                ],
                [
                    0.078481655329,
                    -0.078481655329,
                    0.993821543112,
                    0.0002132612,
                ],
                [
                    -0.078481655329,
                    -0.078481655329,
                    0.993821543112,
                    0.0002132612,
                ],
                [0.078481655329, 0.993821543112, 0.078481655329, 0.0002132612],
                [
                    -0.078481655329,
                    0.993821543112,
                    0.078481655329,
                    0.0002132612,
                ],
                [
                    0.078481655329,
                    -0.993821543112,
                    0.078481655329,
                    0.0002132612,
                ],
                [
                    -0.078481655329,
                    -0.993821543112,
                    0.078481655329,
                    0.0002132612,
                ],
                [0.993821543112, 0.078481655329, 0.078481655329, 0.0002132612],
                [
                    -0.993821543112,
                    0.078481655329,
                    0.078481655329,
                    0.0002132612,
                ],
                [
                    0.993821543112,
                    -0.078481655329,
                    0.078481655329,
                    0.0002132612,
                ],
                [
                    -0.993821543112,
                    -0.078481655329,
                    0.078481655329,
                    0.0002132612,
                ],
                [
                    0.105403815764,
                    0.105403815764,
                    0.988827624637,
                    0.000242901349,
                ],
                [
                    -0.105403815764,
                    0.105403815764,
                    0.988827624637,
                    0.000242901349,
                ],
                [
                    0.105403815764,
                    -0.105403815764,
                    0.988827624637,
                    0.000242901349,
                ],
                [
                    -0.105403815764,
                    -0.105403815764,
                    0.988827624637,
                    0.000242901349,
                ],
                [
                    0.105403815764,
                    0.988827624637,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    -0.105403815764,
                    0.988827624637,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    0.105403815764,
                    -0.988827624637,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    -0.105403815764,
                    -0.988827624637,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    0.988827624637,
                    0.105403815764,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    -0.988827624637,
                    0.105403815764,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    0.988827624637,
                    -0.105403815764,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    -0.988827624637,
                    -0.105403815764,
                    0.105403815764,
                    0.000242901349,
                ],
                [
                    0.133557779777,
                    0.133557779777,
                    0.982000325317,
                    0.000267610936,
                ],
                [
                    -0.133557779777,
                    0.133557779777,
                    0.982000325317,
                    0.000267610936,
                ],
                [
                    0.133557779777,
                    -0.133557779777,
                    0.982000325317,
                    0.000267610936,
                ],
                [
                    -0.133557779777,
                    -0.133557779777,
                    0.982000325317,
                    0.000267610936,
                ],
                [
                    0.133557779777,
                    0.982000325317,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    -0.133557779777,
                    0.982000325317,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    0.133557779777,
                    -0.982000325317,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    -0.133557779777,
                    -0.982000325317,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    0.982000325317,
                    0.133557779777,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    -0.982000325317,
                    0.133557779777,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    0.982000325317,
                    -0.133557779777,
                    0.133557779777,
                    0.000267610936,
                ],
                [
                    -0.982000325317,
                    -0.133557779777,
                    0.133557779777,
                    0.000267610936,
                ],
                [0.16257699555, 0.16257699555, 0.973209864847, 0.000288335405],
                [
                    -0.16257699555,
                    0.16257699555,
                    0.973209864847,
                    0.000288335405,
                ],
                [
                    0.16257699555,
                    -0.16257699555,
                    0.973209864847,
                    0.000288335405,
                ],
                [
                    -0.16257699555,
                    -0.16257699555,
                    0.973209864847,
                    0.000288335405,
                ],
                [0.16257699555, 0.973209864847, 0.16257699555, 0.000288335405],
                [
                    -0.16257699555,
                    0.973209864847,
                    0.16257699555,
                    0.000288335405,
                ],
                [
                    0.16257699555,
                    -0.973209864847,
                    0.16257699555,
                    0.000288335405,
                ],
                [
                    -0.16257699555,
                    -0.973209864847,
                    0.16257699555,
                    0.000288335405,
                ],
                [0.973209864847, 0.16257699555, 0.16257699555, 0.000288335405],
                [
                    -0.973209864847,
                    0.16257699555,
                    0.16257699555,
                    0.000288335405,
                ],
                [
                    0.973209864847,
                    -0.16257699555,
                    0.16257699555,
                    0.000288335405,
                ],
                [
                    -0.973209864847,
                    -0.16257699555,
                    0.16257699555,
                    0.000288335405,
                ],
                [
                    0.192178719341,
                    0.192178719341,
                    0.962358914161,
                    0.00030577604,
                ],
                [
                    -0.192178719341,
                    0.192178719341,
                    0.962358914161,
                    0.00030577604,
                ],
                [
                    0.192178719341,
                    -0.192178719341,
                    0.962358914161,
                    0.00030577604,
                ],
                [
                    -0.192178719341,
                    -0.192178719341,
                    0.962358914161,
                    0.00030577604,
                ],
                [
                    0.192178719341,
                    0.962358914161,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    -0.192178719341,
                    0.962358914161,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    0.192178719341,
                    -0.962358914161,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    -0.192178719341,
                    -0.962358914161,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    0.962358914161,
                    0.192178719341,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    -0.962358914161,
                    0.192178719341,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    0.962358914161,
                    -0.192178719341,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    -0.962358914161,
                    -0.192178719341,
                    0.192178719341,
                    0.00030577604,
                ],
                [
                    0.222134053469,
                    0.222134053469,
                    0.949375017882,
                    0.000320466125,
                ],
                [
                    -0.222134053469,
                    0.222134053469,
                    0.949375017882,
                    0.000320466125,
                ],
                [
                    0.222134053469,
                    -0.222134053469,
                    0.949375017882,
                    0.000320466125,
                ],
                [
                    -0.222134053469,
                    -0.222134053469,
                    0.949375017882,
                    0.000320466125,
                ],
                [
                    0.222134053469,
                    0.949375017882,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    -0.222134053469,
                    0.949375017882,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    0.222134053469,
                    -0.949375017882,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    -0.222134053469,
                    -0.949375017882,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    0.949375017882,
                    0.222134053469,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    -0.949375017882,
                    0.222134053469,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    0.949375017882,
                    -0.222134053469,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    -0.949375017882,
                    -0.222134053469,
                    0.222134053469,
                    0.000320466125,
                ],
                [
                    0.252250491279,
                    0.252250491279,
                    0.934205212627,
                    0.000332820531,
                ],
                [
                    -0.252250491279,
                    0.252250491279,
                    0.934205212627,
                    0.000332820531,
                ],
                [
                    0.252250491279,
                    -0.252250491279,
                    0.934205212627,
                    0.000332820531,
                ],
                [
                    -0.252250491279,
                    -0.252250491279,
                    0.934205212627,
                    0.000332820531,
                ],
                [
                    0.252250491279,
                    0.934205212627,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    -0.252250491279,
                    0.934205212627,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    0.252250491279,
                    -0.934205212627,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    -0.252250491279,
                    -0.934205212627,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    0.934205212627,
                    0.252250491279,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    -0.934205212627,
                    0.252250491279,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    0.934205212627,
                    -0.252250491279,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    -0.934205212627,
                    -0.252250491279,
                    0.252250491279,
                    0.000332820531,
                ],
                [
                    0.282361086068,
                    0.282361086068,
                    0.916812104059,
                    0.000343169171,
                ],
                [
                    -0.282361086068,
                    0.282361086068,
                    0.916812104059,
                    0.000343169171,
                ],
                [
                    0.282361086068,
                    -0.282361086068,
                    0.916812104059,
                    0.000343169171,
                ],
                [
                    -0.282361086068,
                    -0.282361086068,
                    0.916812104059,
                    0.000343169171,
                ],
                [
                    0.282361086068,
                    0.916812104059,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    -0.282361086068,
                    0.916812104059,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    0.282361086068,
                    -0.916812104059,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    -0.282361086068,
                    -0.916812104059,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    0.916812104059,
                    0.282361086068,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    -0.916812104059,
                    0.282361086068,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    0.916812104059,
                    -0.282361086068,
                    0.282361086068,
                    0.000343169171,
                ],
                [
                    -0.916812104059,
                    -0.282361086068,
                    0.282361086068,
                    0.000343169171,
                ],
                [0.312317396627, 0.312317396627, 0.897170935513, 0.0003517802],
                [
                    -0.312317396627,
                    0.312317396627,
                    0.897170935513,
                    0.0003517802,
                ],
                [
                    0.312317396627,
                    -0.312317396627,
                    0.897170935513,
                    0.0003517802,
                ],
                [
                    -0.312317396627,
                    -0.312317396627,
                    0.897170935513,
                    0.0003517802,
                ],
                [0.312317396627, 0.897170935513, 0.312317396627, 0.0003517802],
                [
                    -0.312317396627,
                    0.897170935513,
                    0.312317396627,
                    0.0003517802,
                ],
                [
                    0.312317396627,
                    -0.897170935513,
                    0.312317396627,
                    0.0003517802,
                ],
                [
                    -0.312317396627,
                    -0.897170935513,
                    0.312317396627,
                    0.0003517802,
                ],
                [0.897170935513, 0.312317396627, 0.312317396627, 0.0003517802],
                [
                    -0.897170935513,
                    0.312317396627,
                    0.312317396627,
                    0.0003517802,
                ],
                [
                    0.897170935513,
                    -0.312317396627,
                    0.312317396627,
                    0.0003517802,
                ],
                [
                    -0.897170935513,
                    -0.312317396627,
                    0.312317396627,
                    0.0003517802,
                ],
                [
                    0.341984703695,
                    0.341984703695,
                    0.875267344802,
                    0.000358876497,
                ],
                [
                    -0.341984703695,
                    0.341984703695,
                    0.875267344802,
                    0.000358876497,
                ],
                [
                    0.341984703695,
                    -0.341984703695,
                    0.875267344802,
                    0.000358876497,
                ],
                [
                    -0.341984703695,
                    -0.341984703695,
                    0.875267344802,
                    0.000358876497,
                ],
                [
                    0.341984703695,
                    0.875267344802,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    -0.341984703695,
                    0.875267344802,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    0.341984703695,
                    -0.875267344802,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    -0.341984703695,
                    -0.875267344802,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    0.875267344802,
                    0.341984703695,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    -0.875267344802,
                    0.341984703695,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    0.875267344802,
                    -0.341984703695,
                    0.341984703695,
                    0.000358876497,
                ],
                [
                    -0.875267344802,
                    -0.341984703695,
                    0.341984703695,
                    0.000358876497,
                ],
                [0.3712386457, 0.3712386457, 0.851095609128, 0.000364647621],
                [-0.3712386457, 0.3712386457, 0.851095609128, 0.000364647621],
                [0.3712386457, -0.3712386457, 0.851095609128, 0.000364647621],
                [-0.3712386457, -0.3712386457, 0.851095609128, 0.000364647621],
                [0.3712386457, 0.851095609128, 0.3712386457, 0.000364647621],
                [-0.3712386457, 0.851095609128, 0.3712386457, 0.000364647621],
                [0.3712386457, -0.851095609128, 0.3712386457, 0.000364647621],
                [-0.3712386457, -0.851095609128, 0.3712386457, 0.000364647621],
                [0.851095609128, 0.3712386457, 0.3712386457, 0.000364647621],
                [-0.851095609128, 0.3712386457, 0.3712386457, 0.000364647621],
                [0.851095609128, -0.3712386457, 0.3712386457, 0.000364647621],
                [-0.851095609128, -0.3712386457, 0.3712386457, 0.000364647621],
                [
                    0.399962764988,
                    0.399962764988,
                    0.824657245919,
                    0.000369258651,
                ],
                [
                    -0.399962764988,
                    0.399962764988,
                    0.824657245919,
                    0.000369258651,
                ],
                [
                    0.399962764988,
                    -0.399962764988,
                    0.824657245919,
                    0.000369258651,
                ],
                [
                    -0.399962764988,
                    -0.399962764988,
                    0.824657245919,
                    0.000369258651,
                ],
                [
                    0.399962764988,
                    0.824657245919,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    -0.399962764988,
                    0.824657245919,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    0.399962764988,
                    -0.824657245919,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    -0.399962764988,
                    -0.824657245919,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    0.824657245919,
                    0.399962764988,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    -0.824657245919,
                    0.399962764988,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    0.824657245919,
                    -0.399962764988,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    -0.824657245919,
                    -0.399962764988,
                    0.399962764988,
                    0.000369258651,
                ],
                [
                    0.428046645865,
                    0.428046645865,
                    0.795959884622,
                    0.000372856816,
                ],
                [
                    -0.428046645865,
                    0.428046645865,
                    0.795959884622,
                    0.000372856816,
                ],
                [
                    0.428046645865,
                    -0.428046645865,
                    0.795959884622,
                    0.000372856816,
                ],
                [
                    -0.428046645865,
                    -0.428046645865,
                    0.795959884622,
                    0.000372856816,
                ],
                [
                    0.428046645865,
                    0.795959884622,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    -0.428046645865,
                    0.795959884622,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    0.428046645865,
                    -0.795959884622,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    -0.428046645865,
                    -0.795959884622,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    0.795959884622,
                    0.428046645865,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    -0.795959884622,
                    0.428046645865,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    0.795959884622,
                    -0.428046645865,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    -0.795959884622,
                    -0.428046645865,
                    0.428046645865,
                    0.000372856816,
                ],
                [
                    0.455384436019,
                    0.455384436019,
                    0.765016359867,
                    0.000375576539,
                ],
                [
                    -0.455384436019,
                    0.455384436019,
                    0.765016359867,
                    0.000375576539,
                ],
                [
                    0.455384436019,
                    -0.455384436019,
                    0.765016359867,
                    0.000375576539,
                ],
                [
                    -0.455384436019,
                    -0.455384436019,
                    0.765016359867,
                    0.000375576539,
                ],
                [
                    0.455384436019,
                    0.765016359867,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    -0.455384436019,
                    0.765016359867,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    0.455384436019,
                    -0.765016359867,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    -0.455384436019,
                    -0.765016359867,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    0.765016359867,
                    0.455384436019,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    -0.765016359867,
                    0.455384436019,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    0.765016359867,
                    -0.455384436019,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    -0.765016359867,
                    -0.455384436019,
                    0.455384436019,
                    0.000375576539,
                ],
                [
                    0.481873609444,
                    0.481873609444,
                    0.731844005949,
                    0.000377543264,
                ],
                [
                    -0.481873609444,
                    0.481873609444,
                    0.731844005949,
                    0.000377543264,
                ],
                [
                    0.481873609444,
                    -0.481873609444,
                    0.731844005949,
                    0.000377543264,
                ],
                [
                    -0.481873609444,
                    -0.481873609444,
                    0.731844005949,
                    0.000377543264,
                ],
                [
                    0.481873609444,
                    0.731844005949,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    -0.481873609444,
                    0.731844005949,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    0.481873609444,
                    -0.731844005949,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    -0.481873609444,
                    -0.731844005949,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    0.731844005949,
                    0.481873609444,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    -0.731844005949,
                    0.481873609444,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    0.731844005949,
                    -0.481873609444,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    -0.731844005949,
                    -0.481873609444,
                    0.481873609444,
                    0.000377543264,
                ],
                [
                    0.507413870926,
                    0.507413870926,
                    0.696464160732,
                    0.000378876328,
                ],
                [
                    -0.507413870926,
                    0.507413870926,
                    0.696464160732,
                    0.000378876328,
                ],
                [
                    0.507413870926,
                    -0.507413870926,
                    0.696464160732,
                    0.000378876328,
                ],
                [
                    -0.507413870926,
                    -0.507413870926,
                    0.696464160732,
                    0.000378876328,
                ],
                [
                    0.507413870926,
                    0.696464160732,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    -0.507413870926,
                    0.696464160732,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    0.507413870926,
                    -0.696464160732,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    -0.507413870926,
                    -0.696464160732,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    0.696464160732,
                    0.507413870926,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    -0.696464160732,
                    0.507413870926,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    0.696464160732,
                    -0.507413870926,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    -0.696464160732,
                    -0.507413870926,
                    0.507413870926,
                    0.000378876328,
                ],
                [
                    0.531906130457,
                    0.531906130457,
                    0.658901917408,
                    0.00037969098,
                ],
                [
                    -0.531906130457,
                    0.531906130457,
                    0.658901917408,
                    0.00037969098,
                ],
                [
                    0.531906130457,
                    -0.531906130457,
                    0.658901917408,
                    0.00037969098,
                ],
                [
                    -0.531906130457,
                    -0.531906130457,
                    0.658901917408,
                    0.00037969098,
                ],
                [
                    0.531906130457,
                    0.658901917408,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    -0.531906130457,
                    0.658901917408,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    0.531906130457,
                    -0.658901917408,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    -0.531906130457,
                    -0.658901917408,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    0.658901917408,
                    0.531906130457,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    -0.658901917408,
                    0.531906130457,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    0.658901917408,
                    -0.531906130457,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    -0.658901917408,
                    -0.531906130457,
                    0.531906130457,
                    0.00037969098,
                ],
                [
                    0.555251497868,
                    0.555251497868,
                    0.619186198353,
                    0.000380099586,
                ],
                [
                    -0.555251497868,
                    0.555251497868,
                    0.619186198353,
                    0.000380099586,
                ],
                [
                    0.555251497868,
                    -0.555251497868,
                    0.619186198353,
                    0.000380099586,
                ],
                [
                    -0.555251497868,
                    -0.555251497868,
                    0.619186198353,
                    0.000380099586,
                ],
                [
                    0.555251497868,
                    0.619186198353,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    -0.555251497868,
                    0.619186198353,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    0.555251497868,
                    -0.619186198353,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    -0.555251497868,
                    -0.619186198353,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    0.619186198353,
                    0.555251497868,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    -0.619186198353,
                    0.555251497868,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    0.619186198353,
                    -0.555251497868,
                    0.555251497868,
                    0.000380099586,
                ],
                [
                    -0.619186198353,
                    -0.555251497868,
                    0.555251497868,
                    0.000380099586,
                ],
                [0.598100902525, 0.598100902525, 0.533432864378, 0.0003801343],
                [
                    -0.598100902525,
                    0.598100902525,
                    0.533432864378,
                    0.0003801343,
                ],
                [
                    0.598100902525,
                    -0.598100902525,
                    0.533432864378,
                    0.0003801343,
                ],
                [
                    -0.598100902525,
                    -0.598100902525,
                    0.533432864378,
                    0.0003801343,
                ],
                [0.598100902525, 0.533432864378, 0.598100902525, 0.0003801343],
                [
                    -0.598100902525,
                    0.533432864378,
                    0.598100902525,
                    0.0003801343,
                ],
                [
                    0.598100902525,
                    -0.533432864378,
                    0.598100902525,
                    0.0003801343,
                ],
                [
                    -0.598100902525,
                    -0.533432864378,
                    0.598100902525,
                    0.0003801343,
                ],
                [0.533432864378, 0.598100902525, 0.598100902525, 0.0003801343],
                [
                    -0.533432864378,
                    0.598100902525,
                    0.598100902525,
                    0.0003801343,
                ],
                [
                    0.533432864378,
                    -0.598100902525,
                    0.598100902525,
                    0.0003801343,
                ],
                [
                    -0.533432864378,
                    -0.598100902525,
                    0.598100902525,
                    0.0003801343,
                ],
                [
                    0.617399019223,
                    0.617399019223,
                    0.487480155622,
                    0.000379967511,
                ],
                [
                    -0.617399019223,
                    0.617399019223,
                    0.487480155622,
                    0.000379967511,
                ],
                [
                    0.617399019223,
                    -0.617399019223,
                    0.487480155622,
                    0.000379967511,
                ],
                [
                    -0.617399019223,
                    -0.617399019223,
                    0.487480155622,
                    0.000379967511,
                ],
                [
                    0.617399019223,
                    0.487480155622,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    -0.617399019223,
                    0.487480155622,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    0.617399019223,
                    -0.487480155622,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    -0.617399019223,
                    -0.487480155622,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    0.487480155622,
                    0.617399019223,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    -0.487480155622,
                    0.617399019223,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    0.487480155622,
                    -0.617399019223,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    -0.487480155622,
                    -0.617399019223,
                    0.617399019223,
                    0.000379967511,
                ],
                [
                    0.635136523941,
                    0.635136523941,
                    0.439548850427,
                    0.000379802823,
                ],
                [
                    -0.635136523941,
                    0.635136523941,
                    0.439548850427,
                    0.000379802823,
                ],
                [
                    0.635136523941,
                    -0.635136523941,
                    0.439548850427,
                    0.000379802823,
                ],
                [
                    -0.635136523941,
                    -0.635136523941,
                    0.439548850427,
                    0.000379802823,
                ],
                [
                    0.635136523941,
                    0.439548850427,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    -0.635136523941,
                    0.439548850427,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    0.635136523941,
                    -0.439548850427,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    -0.635136523941,
                    -0.439548850427,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    0.439548850427,
                    0.635136523941,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    -0.439548850427,
                    0.635136523941,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    0.439548850427,
                    -0.635136523941,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    -0.439548850427,
                    -0.635136523941,
                    0.635136523941,
                    0.000379802823,
                ],
                [
                    0.651201022823,
                    0.651201022823,
                    0.389710733428,
                    0.000379716252,
                ],
                [
                    -0.651201022823,
                    0.651201022823,
                    0.389710733428,
                    0.000379716252,
                ],
                [
                    0.651201022823,
                    -0.651201022823,
                    0.389710733428,
                    0.000379716252,
                ],
                [
                    -0.651201022823,
                    -0.651201022823,
                    0.389710733428,
                    0.000379716252,
                ],
                [
                    0.651201022823,
                    0.389710733428,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    -0.651201022823,
                    0.389710733428,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    0.651201022823,
                    -0.389710733428,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    -0.651201022823,
                    -0.389710733428,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    0.389710733428,
                    0.651201022823,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    -0.389710733428,
                    0.651201022823,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    0.389710733428,
                    -0.651201022823,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    -0.389710733428,
                    -0.651201022823,
                    0.651201022823,
                    0.000379716252,
                ],
                [
                    0.665475836395,
                    0.665475836395,
                    0.338058903668,
                    0.000379760951,
                ],
                [
                    -0.665475836395,
                    0.665475836395,
                    0.338058903668,
                    0.000379760951,
                ],
                [
                    0.665475836395,
                    -0.665475836395,
                    0.338058903668,
                    0.000379760951,
                ],
                [
                    -0.665475836395,
                    -0.665475836395,
                    0.338058903668,
                    0.000379760951,
                ],
                [
                    0.665475836395,
                    0.338058903668,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    -0.665475836395,
                    0.338058903668,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    0.665475836395,
                    -0.338058903668,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    -0.665475836395,
                    -0.338058903668,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    0.338058903668,
                    0.665475836395,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    -0.338058903668,
                    0.665475836395,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    0.338058903668,
                    -0.665475836395,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    -0.338058903668,
                    -0.665475836395,
                    0.665475836395,
                    0.000379760951,
                ],
                [
                    0.677841041485,
                    0.677841041485,
                    0.28471572657,
                    0.000379958722,
                ],
                [
                    -0.677841041485,
                    0.677841041485,
                    0.28471572657,
                    0.000379958722,
                ],
                [
                    0.677841041485,
                    -0.677841041485,
                    0.28471572657,
                    0.000379958722,
                ],
                [
                    -0.677841041485,
                    -0.677841041485,
                    0.28471572657,
                    0.000379958722,
                ],
                [
                    0.677841041485,
                    0.28471572657,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    -0.677841041485,
                    0.28471572657,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    0.677841041485,
                    -0.28471572657,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    -0.677841041485,
                    -0.28471572657,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    0.28471572657,
                    0.677841041485,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    -0.28471572657,
                    0.677841041485,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    0.28471572657,
                    -0.677841041485,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    -0.28471572657,
                    -0.677841041485,
                    0.677841041485,
                    0.000379958722,
                ],
                [
                    0.688176088748,
                    0.688176088748,
                    0.229841993008,
                    0.000380292911,
                ],
                [
                    -0.688176088748,
                    0.688176088748,
                    0.229841993008,
                    0.000380292911,
                ],
                [
                    0.688176088748,
                    -0.688176088748,
                    0.229841993008,
                    0.000380292911,
                ],
                [
                    -0.688176088748,
                    -0.688176088748,
                    0.229841993008,
                    0.000380292911,
                ],
                [
                    0.688176088748,
                    0.229841993008,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    -0.688176088748,
                    0.229841993008,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    0.688176088748,
                    -0.229841993008,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    -0.688176088748,
                    -0.229841993008,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    0.229841993008,
                    0.688176088748,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    -0.229841993008,
                    0.688176088748,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    0.229841993008,
                    -0.688176088748,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    -0.229841993008,
                    -0.688176088748,
                    0.688176088748,
                    0.000380292911,
                ],
                [
                    0.696364526709,
                    0.696364526709,
                    0.173645880692,
                    0.000380706649,
                ],
                [
                    -0.696364526709,
                    0.696364526709,
                    0.173645880692,
                    0.000380706649,
                ],
                [
                    0.696364526709,
                    -0.696364526709,
                    0.173645880692,
                    0.000380706649,
                ],
                [
                    -0.696364526709,
                    -0.696364526709,
                    0.173645880692,
                    0.000380706649,
                ],
                [
                    0.696364526709,
                    0.173645880692,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    -0.696364526709,
                    0.173645880692,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    0.696364526709,
                    -0.173645880692,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    -0.696364526709,
                    -0.173645880692,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    0.173645880692,
                    0.696364526709,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    -0.173645880692,
                    0.696364526709,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    0.173645880692,
                    -0.696364526709,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    -0.173645880692,
                    -0.696364526709,
                    0.696364526709,
                    0.000380706649,
                ],
                [
                    0.702301061715,
                    0.702301061715,
                    0.116389163701,
                    0.000381111232,
                ],
                [
                    -0.702301061715,
                    0.702301061715,
                    0.116389163701,
                    0.000381111232,
                ],
                [
                    0.702301061715,
                    -0.702301061715,
                    0.116389163701,
                    0.000381111232,
                ],
                [
                    -0.702301061715,
                    -0.702301061715,
                    0.116389163701,
                    0.000381111232,
                ],
                [
                    0.702301061715,
                    0.116389163701,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    -0.702301061715,
                    0.116389163701,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    0.702301061715,
                    -0.116389163701,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    -0.702301061715,
                    -0.116389163701,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    0.116389163701,
                    0.702301061715,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    -0.116389163701,
                    0.702301061715,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    0.116389163701,
                    -0.702301061715,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    -0.116389163701,
                    -0.702301061715,
                    0.702301061715,
                    0.000381111232,
                ],
                [
                    0.705900463663,
                    0.705900463663,
                    0.058387248617,
                    0.000381407431,
                ],
                [
                    -0.705900463663,
                    0.705900463663,
                    0.058387248617,
                    0.000381407431,
                ],
                [
                    0.705900463663,
                    -0.705900463663,
                    0.058387248617,
                    0.000381407431,
                ],
                [
                    -0.705900463663,
                    -0.705900463663,
                    0.058387248617,
                    0.000381407431,
                ],
                [
                    0.705900463663,
                    0.058387248617,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    -0.705900463663,
                    0.058387248617,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    0.705900463663,
                    -0.058387248617,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    -0.705900463663,
                    -0.058387248617,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    0.058387248617,
                    0.705900463663,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    -0.058387248617,
                    0.705900463663,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    0.058387248617,
                    -0.705900463663,
                    0.705900463663,
                    0.000381407431,
                ],
                [
                    -0.058387248617,
                    -0.705900463663,
                    0.705900463663,
                    0.000381407431,
                ],
                [0.035524703125, 0.999368798526, 0.0, 5.9929978e-05],
                [-0.035524703125, 0.999368798526, 0.0, 5.9929978e-05],
                [0.035524703125, -0.999368798526, 0.0, 5.9929978e-05],
                [-0.035524703125, -0.999368798526, 0.0, 5.9929978e-05],
                [0.999368798526, 0.035524703125, 0.0, 5.9929978e-05],
                [-0.999368798526, 0.035524703125, 0.0, 5.9929978e-05],
                [0.999368798526, -0.035524703125, 0.0, 5.9929978e-05],
                [-0.999368798526, -0.035524703125, 0.0, 5.9929978e-05],
                [0.035524703125, 0.0, 0.999368798526, 0.000119859957],
                [-0.035524703125, 0.0, 0.999368798526, 0.000119859957],
                [0.999368798526, 0.0, 0.035524703125, 0.000119859957],
                [-0.999368798526, 0.0, 0.035524703125, 0.000119859957],
                [0.0, 0.035524703125, 0.999368798526, 0.000119859957],
                [0.0, -0.035524703125, 0.999368798526, 0.000119859957],
                [0.0, 0.999368798526, 0.035524703125, 0.000119859957],
                [0.0, -0.999368798526, 0.035524703125, 0.000119859957],
                [0.091511766208, 0.995803995094, 0.0, 9.7490594e-05],
                [-0.091511766208, 0.995803995094, 0.0, 9.7490594e-05],
                [0.091511766208, -0.995803995094, 0.0, 9.7490594e-05],
                [-0.091511766208, -0.995803995094, 0.0, 9.7490594e-05],
                [0.995803995094, 0.091511766208, 0.0, 9.7490594e-05],
                [-0.995803995094, 0.091511766208, 0.0, 9.7490594e-05],
                [0.995803995094, -0.091511766208, 0.0, 9.7490594e-05],
                [-0.995803995094, -0.091511766208, 0.0, 9.7490594e-05],
                [0.091511766208, 0.0, 0.995803995094, 0.000194981188],
                [-0.091511766208, 0.0, 0.995803995094, 0.000194981188],
                [0.995803995094, 0.0, 0.091511766208, 0.000194981188],
                [-0.995803995094, 0.0, 0.091511766208, 0.000194981188],
                [0.0, 0.091511766208, 0.995803995094, 0.000194981188],
                [0.0, -0.091511766208, 0.995803995094, 0.000194981188],
                [0.0, 0.995803995094, 0.091511766208, 0.000194981188],
                [0.0, -0.995803995094, 0.091511766208, 0.000194981188],
                [0.156619793007, 0.987658969705, 0.0, 0.00012416808],
                [-0.156619793007, 0.987658969705, 0.0, 0.00012416808],
                [0.156619793007, -0.987658969705, 0.0, 0.00012416808],
                [-0.156619793007, -0.987658969705, 0.0, 0.00012416808],
                [0.987658969705, 0.156619793007, 0.0, 0.00012416808],
                [-0.987658969705, 0.156619793007, 0.0, 0.00012416808],
                [0.987658969705, -0.156619793007, 0.0, 0.00012416808],
                [-0.987658969705, -0.156619793007, 0.0, 0.00012416808],
                [0.156619793007, 0.0, 0.987658969705, 0.000248336161],
                [-0.156619793007, 0.0, 0.987658969705, 0.000248336161],
                [0.987658969705, 0.0, 0.156619793007, 0.000248336161],
                [-0.987658969705, 0.0, 0.156619793007, 0.000248336161],
                [0.0, 0.156619793007, 0.987658969705, 0.000248336161],
                [0.0, -0.156619793007, 0.987658969705, 0.000248336161],
                [0.0, 0.987658969705, 0.156619793007, 0.000248336161],
                [0.0, -0.987658969705, 0.156619793007, 0.000248336161],
                [0.226546759927, 0.974000290332, 0.0, 0.000143762615],
                [-0.226546759927, 0.974000290332, 0.0, 0.000143762615],
                [0.226546759927, -0.974000290332, 0.0, 0.000143762615],
                [-0.226546759927, -0.974000290332, 0.0, 0.000143762615],
                [0.974000290332, 0.226546759927, 0.0, 0.000143762615],
                [-0.974000290332, 0.226546759927, 0.0, 0.000143762615],
                [0.974000290332, -0.226546759927, 0.0, 0.000143762615],
                [-0.974000290332, -0.226546759927, 0.0, 0.000143762615],
                [0.226546759927, 0.0, 0.974000290332, 0.000287525231],
                [-0.226546759927, 0.0, 0.974000290332, 0.000287525231],
                [0.974000290332, 0.0, 0.226546759927, 0.000287525231],
                [-0.974000290332, 0.0, 0.226546759927, 0.000287525231],
                [0.0, 0.226546759927, 0.974000290332, 0.000287525231],
                [0.0, -0.226546759927, 0.974000290332, 0.000287525231],
                [0.0, 0.974000290332, 0.226546759927, 0.000287525231],
                [0.0, -0.974000290332, 0.226546759927, 0.000287525231],
                [0.298824231858, 0.954308167446, 0.0, 0.000158420005],
                [-0.298824231858, 0.954308167446, 0.0, 0.000158420005],
                [0.298824231858, -0.954308167446, 0.0, 0.000158420005],
                [-0.298824231858, -0.954308167446, 0.0, 0.000158420005],
                [0.954308167446, 0.298824231858, 0.0, 0.000158420005],
                [-0.954308167446, 0.298824231858, 0.0, 0.000158420005],
                [0.954308167446, -0.298824231858, 0.0, 0.000158420005],
                [-0.954308167446, -0.298824231858, 0.0, 0.000158420005],
                [0.298824231858, 0.0, 0.954308167446, 0.000316840011],
                [-0.298824231858, 0.0, 0.954308167446, 0.000316840011],
                [0.954308167446, 0.0, 0.298824231858, 0.000316840011],
                [-0.954308167446, 0.0, 0.298824231858, 0.000316840011],
                [0.0, 0.298824231858, 0.954308167446, 0.000316840011],
                [0.0, -0.298824231858, 0.954308167446, 0.000316840011],
                [0.0, 0.954308167446, 0.298824231858, 0.000316840011],
                [0.0, -0.954308167446, 0.298824231858, 0.000316840011],
                [0.37174824197, 0.928333584759, 0.0, 0.000169443655],
                [-0.37174824197, 0.928333584759, 0.0, 0.000169443655],
                [0.37174824197, -0.928333584759, 0.0, 0.000169443655],
                [-0.37174824197, -0.928333584759, 0.0, 0.000169443655],
                [0.928333584759, 0.37174824197, 0.0, 0.000169443655],
                [-0.928333584759, 0.37174824197, 0.0, 0.000169443655],
                [0.928333584759, -0.37174824197, 0.0, 0.000169443655],
                [-0.928333584759, -0.37174824197, 0.0, 0.000169443655],
                [0.37174824197, 0.0, 0.928333584759, 0.00033888731],
                [-0.37174824197, 0.0, 0.928333584759, 0.00033888731],
                [0.928333584759, 0.0, 0.37174824197, 0.00033888731],
                [-0.928333584759, 0.0, 0.37174824197, 0.00033888731],
                [0.0, 0.37174824197, 0.928333584759, 0.00033888731],
                [0.0, -0.37174824197, 0.928333584759, 0.00033888731],
                [0.0, 0.928333584759, 0.37174824197, 0.00033888731],
                [0.0, -0.928333584759, 0.37174824197, 0.00033888731],
                [0.444009449176, 0.896022102988, 0.0, 0.000177661701],
                [-0.444009449176, 0.896022102988, 0.0, 0.000177661701],
                [0.444009449176, -0.896022102988, 0.0, 0.000177661701],
                [-0.444009449176, -0.896022102988, 0.0, 0.000177661701],
                [0.896022102988, 0.444009449176, 0.0, 0.000177661701],
                [-0.896022102988, 0.444009449176, 0.0, 0.000177661701],
                [0.896022102988, -0.444009449176, 0.0, 0.000177661701],
                [-0.896022102988, -0.444009449176, 0.0, 0.000177661701],
                [0.444009449176, 0.0, 0.896022102988, 0.000355323403],
                [-0.444009449176, 0.0, 0.896022102988, 0.000355323403],
                [0.896022102988, 0.0, 0.444009449176, 0.000355323403],
                [-0.896022102988, 0.0, 0.444009449176, 0.000355323403],
                [0.0, 0.444009449176, 0.896022102988, 0.000355323403],
                [0.0, -0.444009449176, 0.896022102988, 0.000355323403],
                [0.0, 0.896022102988, 0.444009449176, 0.000355323403],
                [0.0, -0.896022102988, 0.444009449176, 0.000355323403],
                [0.514533709676, 0.857470152021, 0.0, 0.000183613243],
                [-0.514533709676, 0.857470152021, 0.0, 0.000183613243],
                [0.514533709676, -0.857470152021, 0.0, 0.000183613243],
                [-0.514533709676, -0.857470152021, 0.0, 0.000183613243],
                [0.857470152021, 0.514533709676, 0.0, 0.000183613243],
                [-0.857470152021, 0.514533709676, 0.0, 0.000183613243],
                [0.857470152021, -0.514533709676, 0.0, 0.000183613243],
                [-0.857470152021, -0.514533709676, 0.0, 0.000183613243],
                [0.514533709676, 0.0, 0.857470152021, 0.000367226487],
                [-0.514533709676, 0.0, 0.857470152021, 0.000367226487],
                [0.857470152021, 0.0, 0.514533709676, 0.000367226487],
                [-0.857470152021, 0.0, 0.514533709676, 0.000367226487],
                [0.0, 0.514533709676, 0.857470152021, 0.000367226487],
                [0.0, -0.514533709676, 0.857470152021, 0.000367226487],
                [0.0, 0.857470152021, 0.514533709676, 0.000367226487],
                [0.0, -0.857470152021, 0.514533709676, 0.000367226487],
                [0.582405367286, 0.812898510367, 0.0, 0.000187649473],
                [-0.582405367286, 0.812898510367, 0.0, 0.000187649473],
                [0.582405367286, -0.812898510367, 0.0, 0.000187649473],
                [-0.582405367286, -0.812898510367, 0.0, 0.000187649473],
                [0.812898510367, 0.582405367286, 0.0, 0.000187649473],
                [-0.812898510367, 0.582405367286, 0.0, 0.000187649473],
                [0.812898510367, -0.582405367286, 0.0, 0.000187649473],
                [-0.812898510367, -0.582405367286, 0.0, 0.000187649473],
                [0.582405367286, 0.0, 0.812898510367, 0.000375298945],
                [-0.582405367286, 0.0, 0.812898510367, 0.000375298945],
                [0.812898510367, 0.0, 0.582405367286, 0.000375298945],
                [-0.812898510367, 0.0, 0.582405367286, 0.000375298945],
                [0.0, 0.582405367286, 0.812898510367, 0.000375298945],
                [0.0, -0.582405367286, 0.812898510367, 0.000375298945],
                [0.0, 0.812898510367, 0.582405367286, 0.000375298945],
                [0.0, -0.812898510367, 0.582405367286, 0.000375298945],
                [0.646828396104, 0.762635578762, 0.0, 0.000189990654],
                [-0.646828396104, 0.762635578762, 0.0, 0.000189990654],
                [0.646828396104, -0.762635578762, 0.0, 0.000189990654],
                [-0.646828396104, -0.762635578762, 0.0, 0.000189990654],
                [0.762635578762, 0.646828396104, 0.0, 0.000189990654],
                [-0.762635578762, 0.646828396104, 0.0, 0.000189990654],
                [0.762635578762, -0.646828396104, 0.0, 0.000189990654],
                [-0.762635578762, -0.646828396104, 0.0, 0.000189990654],
                [0.646828396104, 0.0, 0.762635578762, 0.000379981307],
                [-0.646828396104, 0.0, 0.762635578762, 0.000379981307],
                [0.762635578762, 0.0, 0.646828396104, 0.000379981307],
                [-0.762635578762, 0.0, 0.646828396104, 0.000379981307],
                [0.0, 0.646828396104, 0.762635578762, 0.000379981307],
                [0.0, -0.646828396104, 0.762635578762, 0.000379981307],
                [0.0, 0.762635578762, 0.646828396104, 0.000379981307],
                [0.0, -0.762635578762, 0.646828396104, 0.000379981307],
                [
                    0.060959642591,
                    0.017878282753,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    -0.060959642591,
                    0.017878282753,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    0.060959642591,
                    -0.017878282753,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    -0.060959642591,
                    -0.017878282753,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    0.060959642591,
                    0.997980104502,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    -0.060959642591,
                    0.997980104502,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    0.060959642591,
                    -0.997980104502,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    -0.060959642591,
                    -0.997980104502,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    0.017878282753,
                    0.060959642591,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    -0.017878282753,
                    0.060959642591,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    0.017878282753,
                    -0.060959642591,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    -0.017878282753,
                    -0.060959642591,
                    0.997980104502,
                    0.000162865056,
                ],
                [
                    0.017878282753,
                    0.997980104502,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    -0.017878282753,
                    0.997980104502,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    0.017878282753,
                    -0.997980104502,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    -0.017878282753,
                    -0.997980104502,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    0.997980104502,
                    0.060959642591,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    -0.997980104502,
                    0.060959642591,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    0.997980104502,
                    -0.060959642591,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    -0.997980104502,
                    -0.060959642591,
                    0.017878282753,
                    0.000162865056,
                ],
                [
                    0.997980104502,
                    0.017878282753,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    -0.997980104502,
                    0.017878282753,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    0.997980104502,
                    -0.017878282753,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    -0.997980104502,
                    -0.017878282753,
                    0.060959642591,
                    0.000162865056,
                ],
                [
                    0.08811962271,
                    0.039538887408,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    -0.08811962271,
                    0.039538887408,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    0.08811962271,
                    -0.039538887408,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    -0.08811962271,
                    -0.039538887408,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    0.08811962271,
                    0.995324875845,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    -0.08811962271,
                    0.995324875845,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    0.08811962271,
                    -0.995324875845,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    -0.08811962271,
                    -0.995324875845,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    0.039538887408,
                    0.08811962271,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    -0.039538887408,
                    0.08811962271,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    0.039538887408,
                    -0.08811962271,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    -0.039538887408,
                    -0.08811962271,
                    0.995324875845,
                    0.000199977198,
                ],
                [
                    0.039538887408,
                    0.995324875845,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    -0.039538887408,
                    0.995324875845,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    0.039538887408,
                    -0.995324875845,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    -0.039538887408,
                    -0.995324875845,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    0.995324875845,
                    0.08811962271,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    -0.995324875845,
                    0.08811962271,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    0.995324875845,
                    -0.08811962271,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    -0.995324875845,
                    -0.08811962271,
                    0.039538887408,
                    0.000199977198,
                ],
                [
                    0.995324875845,
                    0.039538887408,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    -0.995324875845,
                    0.039538887408,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    0.995324875845,
                    -0.039538887408,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    -0.995324875845,
                    -0.039538887408,
                    0.08811962271,
                    0.000199977198,
                ],
                [
                    0.116593672243,
                    0.063781217977,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    -0.116593672243,
                    0.063781217977,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    0.116593672243,
                    -0.063781217977,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    -0.116593672243,
                    -0.063781217977,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    0.116593672243,
                    0.991129593861,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    -0.116593672243,
                    0.991129593861,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    0.116593672243,
                    -0.991129593861,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    -0.116593672243,
                    -0.991129593861,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    0.063781217977,
                    0.116593672243,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    -0.063781217977,
                    0.116593672243,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    0.063781217977,
                    -0.116593672243,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    -0.063781217977,
                    -0.116593672243,
                    0.991129593861,
                    0.000231239881,
                ],
                [
                    0.063781217977,
                    0.991129593861,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    -0.063781217977,
                    0.991129593861,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    0.063781217977,
                    -0.991129593861,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    -0.063781217977,
                    -0.991129593861,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    0.991129593861,
                    0.116593672243,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    -0.991129593861,
                    0.116593672243,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    0.991129593861,
                    -0.116593672243,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    -0.991129593861,
                    -0.116593672243,
                    0.063781217977,
                    0.000231239881,
                ],
                [
                    0.991129593861,
                    0.063781217977,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    -0.991129593861,
                    0.063781217977,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    0.991129593861,
                    -0.063781217977,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    -0.991129593861,
                    -0.063781217977,
                    0.116593672243,
                    0.000231239881,
                ],
                [
                    0.146023285703,
                    0.089858908137,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    -0.146023285703,
                    0.089858908137,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    0.146023285703,
                    -0.089858908137,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    -0.146023285703,
                    -0.089858908137,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    0.146023285703,
                    0.985191644636,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    -0.146023285703,
                    0.985191644636,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    0.146023285703,
                    -0.985191644636,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    -0.146023285703,
                    -0.985191644636,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    0.089858908137,
                    0.146023285703,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    -0.089858908137,
                    0.146023285703,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    0.089858908137,
                    -0.146023285703,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    -0.089858908137,
                    -0.146023285703,
                    0.985191644636,
                    0.000257526419,
                ],
                [
                    0.089858908137,
                    0.985191644636,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    -0.089858908137,
                    0.985191644636,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    0.089858908137,
                    -0.985191644636,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    -0.089858908137,
                    -0.985191644636,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    0.985191644636,
                    0.146023285703,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    -0.985191644636,
                    0.146023285703,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    0.985191644636,
                    -0.146023285703,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    -0.985191644636,
                    -0.146023285703,
                    0.089858908137,
                    0.000257526419,
                ],
                [
                    0.985191644636,
                    0.089858908137,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    -0.985191644636,
                    0.089858908137,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    0.985191644636,
                    -0.089858908137,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    -0.985191644636,
                    -0.089858908137,
                    0.146023285703,
                    0.000257526419,
                ],
                [
                    0.176119711018,
                    0.117260651058,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    -0.176119711018,
                    0.117260651058,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    0.176119711018,
                    -0.117260651058,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    -0.176119711018,
                    -0.117260651058,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    0.176119711018,
                    0.977359599689,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    -0.176119711018,
                    0.977359599689,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    0.176119711018,
                    -0.977359599689,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    -0.176119711018,
                    -0.977359599689,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    0.117260651058,
                    0.176119711018,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    -0.117260651058,
                    0.176119711018,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    0.117260651058,
                    -0.176119711018,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    -0.117260651058,
                    -0.176119711018,
                    0.977359599689,
                    0.000279675729,
                ],
                [
                    0.117260651058,
                    0.977359599689,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    -0.117260651058,
                    0.977359599689,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    0.117260651058,
                    -0.977359599689,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    -0.117260651058,
                    -0.977359599689,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    0.977359599689,
                    0.176119711018,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    -0.977359599689,
                    0.176119711018,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    0.977359599689,
                    -0.176119711018,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    -0.977359599689,
                    -0.176119711018,
                    0.117260651058,
                    0.000279675729,
                ],
                [
                    0.977359599689,
                    0.117260651058,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    -0.977359599689,
                    0.117260651058,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    0.977359599689,
                    -0.117260651058,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    -0.977359599689,
                    -0.117260651058,
                    0.176119711018,
                    0.000279675729,
                ],
                [
                    0.206647119046,
                    0.145610287697,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    -0.206647119046,
                    0.145610287697,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    0.206647119046,
                    -0.145610287697,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    -0.206647119046,
                    -0.145610287697,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    0.206647119046,
                    0.967519825278,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    -0.206647119046,
                    0.967519825278,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    0.206647119046,
                    -0.967519825278,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    -0.206647119046,
                    -0.967519825278,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    0.145610287697,
                    0.206647119046,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    -0.145610287697,
                    0.206647119046,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    0.145610287697,
                    -0.206647119046,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    -0.145610287697,
                    -0.206647119046,
                    0.967519825278,
                    0.000298375294,
                ],
                [
                    0.145610287697,
                    0.967519825278,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    -0.145610287697,
                    0.967519825278,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    0.145610287697,
                    -0.967519825278,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    -0.145610287697,
                    -0.967519825278,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    0.967519825278,
                    0.206647119046,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    -0.967519825278,
                    0.206647119046,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    0.967519825278,
                    -0.206647119046,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    -0.967519825278,
                    -0.206647119046,
                    0.145610287697,
                    0.000298375294,
                ],
                [
                    0.967519825278,
                    0.145610287697,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    -0.967519825278,
                    0.145610287697,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    0.967519825278,
                    -0.145610287697,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    -0.967519825278,
                    -0.145610287697,
                    0.206647119046,
                    0.000298375294,
                ],
                [
                    0.237407602633,
                    0.174615382301,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    -0.237407602633,
                    0.174615382301,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    0.237407602633,
                    -0.174615382301,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    -0.237407602633,
                    -0.174615382301,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    0.237407602633,
                    0.955587305523,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    -0.237407602633,
                    0.955587305523,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    0.237407602633,
                    -0.955587305523,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    -0.237407602633,
                    -0.955587305523,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    0.174615382301,
                    0.237407602633,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    -0.174615382301,
                    0.237407602633,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    0.174615382301,
                    -0.237407602633,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    -0.174615382301,
                    -0.237407602633,
                    0.955587305523,
                    0.000314171136,
                ],
                [
                    0.174615382301,
                    0.955587305523,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    -0.174615382301,
                    0.955587305523,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    0.174615382301,
                    -0.955587305523,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    -0.174615382301,
                    -0.955587305523,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    0.955587305523,
                    0.237407602633,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    -0.955587305523,
                    0.237407602633,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    0.955587305523,
                    -0.237407602633,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    -0.955587305523,
                    -0.237407602633,
                    0.174615382301,
                    0.000314171136,
                ],
                [
                    0.955587305523,
                    0.174615382301,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    -0.955587305523,
                    0.174615382301,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    0.955587305523,
                    -0.174615382301,
                    0.237407602633,
                    0.000314171136,
                ],
                [
                    -0.955587305523,
                    -0.174615382301,
                    0.237407602633,
                    0.000314171136,
                ],
                [0.268230547434, 0.20403830703, 0.941499199515, 0.00032749679],
                [
                    -0.268230547434,
                    0.20403830703,
                    0.941499199515,
                    0.00032749679,
                ],
                [
                    0.268230547434,
                    -0.20403830703,
                    0.941499199515,
                    0.00032749679,
                ],
                [
                    -0.268230547434,
                    -0.20403830703,
                    0.941499199515,
                    0.00032749679,
                ],
                [0.268230547434, 0.941499199515, 0.20403830703, 0.00032749679],
                [
                    -0.268230547434,
                    0.941499199515,
                    0.20403830703,
                    0.00032749679,
                ],
                [
                    0.268230547434,
                    -0.941499199515,
                    0.20403830703,
                    0.00032749679,
                ],
                [
                    -0.268230547434,
                    -0.941499199515,
                    0.20403830703,
                    0.00032749679,
                ],
                [0.20403830703, 0.268230547434, 0.941499199515, 0.00032749679],
                [
                    -0.20403830703,
                    0.268230547434,
                    0.941499199515,
                    0.00032749679,
                ],
                [
                    0.20403830703,
                    -0.268230547434,
                    0.941499199515,
                    0.00032749679,
                ],
                [
                    -0.20403830703,
                    -0.268230547434,
                    0.941499199515,
                    0.00032749679,
                ],
                [0.20403830703, 0.941499199515, 0.268230547434, 0.00032749679],
                [
                    -0.20403830703,
                    0.941499199515,
                    0.268230547434,
                    0.00032749679,
                ],
                [
                    0.20403830703,
                    -0.941499199515,
                    0.268230547434,
                    0.00032749679,
                ],
                [
                    -0.20403830703,
                    -0.941499199515,
                    0.268230547434,
                    0.00032749679,
                ],
                [0.941499199515, 0.268230547434, 0.20403830703, 0.00032749679],
                [
                    -0.941499199515,
                    0.268230547434,
                    0.20403830703,
                    0.00032749679,
                ],
                [
                    0.941499199515,
                    -0.268230547434,
                    0.20403830703,
                    0.00032749679,
                ],
                [
                    -0.941499199515,
                    -0.268230547434,
                    0.20403830703,
                    0.00032749679,
                ],
                [0.941499199515, 0.20403830703, 0.268230547434, 0.00032749679],
                [
                    -0.941499199515,
                    0.20403830703,
                    0.268230547434,
                    0.00032749679,
                ],
                [
                    0.941499199515,
                    -0.20403830703,
                    0.268230547434,
                    0.00032749679,
                ],
                [
                    -0.941499199515,
                    -0.20403830703,
                    0.268230547434,
                    0.00032749679,
                ],
                [0.298965331214, 0.2336788634, 0.92521020289, 0.000338700113],
                [-0.298965331214, 0.2336788634, 0.92521020289, 0.000338700113],
                [0.298965331214, -0.2336788634, 0.92521020289, 0.000338700113],
                [
                    -0.298965331214,
                    -0.2336788634,
                    0.92521020289,
                    0.000338700113,
                ],
                [0.298965331214, 0.92521020289, 0.2336788634, 0.000338700113],
                [-0.298965331214, 0.92521020289, 0.2336788634, 0.000338700113],
                [0.298965331214, -0.92521020289, 0.2336788634, 0.000338700113],
                [
                    -0.298965331214,
                    -0.92521020289,
                    0.2336788634,
                    0.000338700113,
                ],
                [0.2336788634, 0.298965331214, 0.92521020289, 0.000338700113],
                [-0.2336788634, 0.298965331214, 0.92521020289, 0.000338700113],
                [0.2336788634, -0.298965331214, 0.92521020289, 0.000338700113],
                [
                    -0.2336788634,
                    -0.298965331214,
                    0.92521020289,
                    0.000338700113,
                ],
                [0.2336788634, 0.92521020289, 0.298965331214, 0.000338700113],
                [-0.2336788634, 0.92521020289, 0.298965331214, 0.000338700113],
                [0.2336788634, -0.92521020289, 0.298965331214, 0.000338700113],
                [
                    -0.2336788634,
                    -0.92521020289,
                    0.298965331214,
                    0.000338700113,
                ],
                [0.92521020289, 0.298965331214, 0.2336788634, 0.000338700113],
                [-0.92521020289, 0.298965331214, 0.2336788634, 0.000338700113],
                [0.92521020289, -0.298965331214, 0.2336788634, 0.000338700113],
                [
                    -0.92521020289,
                    -0.298965331214,
                    0.2336788634,
                    0.000338700113,
                ],
                [0.92521020289, 0.2336788634, 0.298965331214, 0.000338700113],
                [-0.92521020289, 0.2336788634, 0.298965331214, 0.000338700113],
                [0.92521020289, -0.2336788634, 0.298965331214, 0.000338700113],
                [
                    -0.92521020289,
                    -0.2336788634,
                    0.298965331214,
                    0.000338700113,
                ],
                [
                    0.329476275277,
                    0.263363275265,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    -0.329476275277,
                    0.263363275265,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    0.329476275277,
                    -0.263363275265,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    -0.329476275277,
                    -0.263363275265,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    0.329476275277,
                    0.906689124933,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    -0.329476275277,
                    0.906689124933,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    0.329476275277,
                    -0.906689124933,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    -0.329476275277,
                    -0.906689124933,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    0.263363275265,
                    0.329476275277,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    -0.263363275265,
                    0.329476275277,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    0.263363275265,
                    -0.329476275277,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    -0.263363275265,
                    -0.329476275277,
                    0.906689124933,
                    0.000348064554,
                ],
                [
                    0.263363275265,
                    0.906689124933,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    -0.263363275265,
                    0.906689124933,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    0.263363275265,
                    -0.906689124933,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    -0.263363275265,
                    -0.906689124933,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    0.906689124933,
                    0.329476275277,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    -0.906689124933,
                    0.329476275277,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    0.906689124933,
                    -0.329476275277,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    -0.906689124933,
                    -0.329476275277,
                    0.263363275265,
                    0.000348064554,
                ],
                [
                    0.906689124933,
                    0.263363275265,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    -0.906689124933,
                    0.263363275265,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    0.906689124933,
                    -0.263363275265,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    -0.906689124933,
                    -0.263363275265,
                    0.329476275277,
                    0.000348064554,
                ],
                [
                    0.359639088728,
                    0.292936909805,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    -0.359639088728,
                    0.292936909805,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    0.359639088728,
                    -0.292936909805,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    -0.359639088728,
                    -0.292936909805,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    0.359639088728,
                    0.885916301201,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    -0.359639088728,
                    0.885916301201,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    0.359639088728,
                    -0.885916301201,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    -0.359639088728,
                    -0.885916301201,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    0.292936909805,
                    0.359639088728,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    -0.292936909805,
                    0.359639088728,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    0.292936909805,
                    -0.359639088728,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    -0.292936909805,
                    -0.359639088728,
                    0.885916301201,
                    0.000355825327,
                ],
                [
                    0.292936909805,
                    0.885916301201,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    -0.292936909805,
                    0.885916301201,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    0.292936909805,
                    -0.885916301201,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    -0.292936909805,
                    -0.885916301201,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    0.885916301201,
                    0.359639088728,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    -0.885916301201,
                    0.359639088728,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    0.885916301201,
                    -0.359639088728,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    -0.885916301201,
                    -0.359639088728,
                    0.292936909805,
                    0.000355825327,
                ],
                [
                    0.885916301201,
                    0.292936909805,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    -0.885916301201,
                    0.292936909805,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    0.885916301201,
                    -0.292936909805,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    -0.885916301201,
                    -0.292936909805,
                    0.359639088728,
                    0.000355825327,
                ],
                [
                    0.38933830464,
                    0.322259278528,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    -0.38933830464,
                    0.322259278528,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    0.38933830464,
                    -0.322259278528,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    -0.38933830464,
                    -0.322259278528,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    0.38933830464,
                    0.862881592076,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    -0.38933830464,
                    0.862881592076,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    0.38933830464,
                    -0.862881592076,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    -0.38933830464,
                    -0.862881592076,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    0.322259278528,
                    0.38933830464,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    -0.322259278528,
                    0.38933830464,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    0.322259278528,
                    -0.38933830464,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    -0.322259278528,
                    -0.38933830464,
                    0.862881592076,
                    0.000362181622,
                ],
                [
                    0.322259278528,
                    0.862881592076,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    -0.322259278528,
                    0.862881592076,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    0.322259278528,
                    -0.862881592076,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    -0.322259278528,
                    -0.862881592076,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    0.862881592076,
                    0.38933830464,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    -0.862881592076,
                    0.38933830464,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    0.862881592076,
                    -0.38933830464,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    -0.862881592076,
                    -0.38933830464,
                    0.322259278528,
                    0.000362181622,
                ],
                [
                    0.862881592076,
                    0.322259278528,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    -0.862881592076,
                    0.322259278528,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    0.862881592076,
                    -0.322259278528,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    -0.862881592076,
                    -0.322259278528,
                    0.38933830464,
                    0.000362181622,
                ],
                [
                    0.418465378936,
                    0.35120047912,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    -0.418465378936,
                    0.35120047912,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    0.418465378936,
                    -0.35120047912,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    -0.418465378936,
                    -0.35120047912,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    0.418465378936,
                    0.837582801936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    -0.418465378936,
                    0.837582801936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    0.418465378936,
                    -0.837582801936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    -0.418465378936,
                    -0.837582801936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    0.35120047912,
                    0.418465378936,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    -0.35120047912,
                    0.418465378936,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    0.35120047912,
                    -0.418465378936,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    -0.35120047912,
                    -0.418465378936,
                    0.837582801936,
                    0.000367305827,
                ],
                [
                    0.35120047912,
                    0.837582801936,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    -0.35120047912,
                    0.837582801936,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    0.35120047912,
                    -0.837582801936,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    -0.35120047912,
                    -0.837582801936,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    0.837582801936,
                    0.418465378936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    -0.837582801936,
                    0.418465378936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    0.837582801936,
                    -0.418465378936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    -0.837582801936,
                    -0.418465378936,
                    0.35120047912,
                    0.000367305827,
                ],
                [
                    0.837582801936,
                    0.35120047912,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    -0.837582801936,
                    0.35120047912,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    0.837582801936,
                    -0.35120047912,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    -0.837582801936,
                    -0.35120047912,
                    0.418465378936,
                    0.000367305827,
                ],
                [
                    0.446917231908,
                    0.379638567768,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    -0.446917231908,
                    0.379638567768,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    0.446917231908,
                    -0.379638567768,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    -0.446917231908,
                    -0.379638567768,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    0.446917231908,
                    0.81002441055,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    -0.446917231908,
                    0.81002441055,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    0.446917231908,
                    -0.81002441055,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    -0.446917231908,
                    -0.81002441055,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    0.379638567768,
                    0.446917231908,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    -0.379638567768,
                    0.446917231908,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    0.379638567768,
                    -0.446917231908,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    -0.379638567768,
                    -0.446917231908,
                    0.81002441055,
                    0.000371350568,
                ],
                [
                    0.379638567768,
                    0.81002441055,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    -0.379638567768,
                    0.81002441055,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    0.379638567768,
                    -0.81002441055,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    -0.379638567768,
                    -0.81002441055,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    0.81002441055,
                    0.446917231908,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    -0.81002441055,
                    0.446917231908,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    0.81002441055,
                    -0.446917231908,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    -0.81002441055,
                    -0.446917231908,
                    0.379638567768,
                    0.000371350568,
                ],
                [
                    0.81002441055,
                    0.379638567768,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    -0.81002441055,
                    0.379638567768,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    0.81002441055,
                    -0.379638567768,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    -0.81002441055,
                    -0.379638567768,
                    0.446917231908,
                    0.000371350568,
                ],
                [
                    0.474595081328,
                    0.407457537826,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    -0.474595081328,
                    0.407457537826,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    0.474595081328,
                    -0.407457537826,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    -0.474595081328,
                    -0.407457537826,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    0.474595081328,
                    0.780216549202,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    -0.474595081328,
                    0.780216549202,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    0.474595081328,
                    -0.780216549202,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    -0.474595081328,
                    -0.780216549202,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    0.407457537826,
                    0.474595081328,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    -0.407457537826,
                    0.474595081328,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    0.407457537826,
                    -0.474595081328,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    -0.407457537826,
                    -0.474595081328,
                    0.780216549202,
                    0.000374454113,
                ],
                [
                    0.407457537826,
                    0.780216549202,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    -0.407457537826,
                    0.780216549202,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    0.407457537826,
                    -0.780216549202,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    -0.407457537826,
                    -0.780216549202,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    0.780216549202,
                    0.474595081328,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    -0.780216549202,
                    0.474595081328,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    0.780216549202,
                    -0.474595081328,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    -0.780216549202,
                    -0.474595081328,
                    0.407457537826,
                    0.000374454113,
                ],
                [
                    0.780216549202,
                    0.407457537826,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    -0.780216549202,
                    0.407457537826,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    0.780216549202,
                    -0.407457537826,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    -0.780216549202,
                    -0.407457537826,
                    0.474595081328,
                    0.000374454113,
                ],
                [
                    0.501403460141,
                    0.434545690603,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    -0.501403460141,
                    0.434545690603,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    0.501403460141,
                    -0.434545690603,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    -0.501403460141,
                    -0.434545690603,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    0.501403460141,
                    0.748174186227,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    -0.501403460141,
                    0.748174186227,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    0.501403460141,
                    -0.748174186227,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    -0.501403460141,
                    -0.748174186227,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    0.434545690603,
                    0.501403460141,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    -0.434545690603,
                    0.501403460141,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    0.434545690603,
                    -0.501403460141,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    -0.434545690603,
                    -0.501403460141,
                    0.748174186227,
                    0.000376744529,
                ],
                [
                    0.434545690603,
                    0.748174186227,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    -0.434545690603,
                    0.748174186227,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    0.434545690603,
                    -0.748174186227,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    -0.434545690603,
                    -0.748174186227,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    0.748174186227,
                    0.501403460141,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    -0.748174186227,
                    0.501403460141,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    0.748174186227,
                    -0.501403460141,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    -0.748174186227,
                    -0.501403460141,
                    0.434545690603,
                    0.000376744529,
                ],
                [
                    0.748174186227,
                    0.434545690603,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    -0.748174186227,
                    0.434545690603,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    0.748174186227,
                    -0.434545690603,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    -0.748174186227,
                    -0.434545690603,
                    0.501403460141,
                    0.000376744529,
                ],
                [
                    0.527249340455,
                    0.460794251521,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    -0.527249340455,
                    0.460794251521,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    0.527249340455,
                    -0.460794251521,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    -0.527249340455,
                    -0.460794251521,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    0.527249340455,
                    0.713916515256,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    -0.527249340455,
                    0.713916515256,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    0.527249340455,
                    -0.713916515256,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    -0.527249340455,
                    -0.713916515256,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    0.460794251521,
                    0.527249340455,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    -0.460794251521,
                    0.527249340455,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    0.460794251521,
                    -0.527249340455,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    -0.460794251521,
                    -0.527249340455,
                    0.713916515256,
                    0.000378342865,
                ],
                [
                    0.460794251521,
                    0.713916515256,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    -0.460794251521,
                    0.713916515256,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    0.460794251521,
                    -0.713916515256,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    -0.460794251521,
                    -0.713916515256,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    0.713916515256,
                    0.527249340455,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    -0.713916515256,
                    0.527249340455,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    0.713916515256,
                    -0.527249340455,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    -0.713916515256,
                    -0.527249340455,
                    0.460794251521,
                    0.000378342865,
                ],
                [
                    0.713916515256,
                    0.460794251521,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    -0.713916515256,
                    0.460794251521,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    0.713916515256,
                    -0.460794251521,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    -0.713916515256,
                    -0.460794251521,
                    0.527249340455,
                    0.000378342865,
                ],
                [
                    0.552041305185,
                    0.486096128418,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    -0.552041305185,
                    0.486096128418,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    0.552041305185,
                    -0.486096128418,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    -0.552041305185,
                    -0.486096128418,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    0.552041305185,
                    0.677466568405,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    -0.552041305185,
                    0.677466568405,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    0.552041305185,
                    -0.677466568405,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    -0.552041305185,
                    -0.677466568405,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    0.486096128418,
                    0.552041305185,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    -0.486096128418,
                    0.552041305185,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    0.486096128418,
                    -0.552041305185,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    -0.486096128418,
                    -0.552041305185,
                    0.677466568405,
                    0.000379365496,
                ],
                [
                    0.486096128418,
                    0.677466568405,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    -0.486096128418,
                    0.677466568405,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    0.486096128418,
                    -0.677466568405,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    -0.486096128418,
                    -0.677466568405,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    0.677466568405,
                    0.552041305185,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    -0.677466568405,
                    0.552041305185,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    0.677466568405,
                    -0.552041305185,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    -0.677466568405,
                    -0.552041305185,
                    0.486096128418,
                    0.000379365496,
                ],
                [
                    0.677466568405,
                    0.486096128418,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    -0.677466568405,
                    0.486096128418,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    0.677466568405,
                    -0.486096128418,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    -0.677466568405,
                    -0.486096128418,
                    0.552041305185,
                    0.000379365496,
                ],
                [
                    0.57568872375,
                    0.510344739534,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    -0.57568872375,
                    0.510344739534,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    0.57568872375,
                    -0.510344739534,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    -0.57568872375,
                    -0.510344739534,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    0.57568872375,
                    0.638851109552,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    -0.57568872375,
                    0.638851109552,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    0.57568872375,
                    -0.638851109552,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    -0.57568872375,
                    -0.638851109552,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    0.510344739534,
                    0.57568872375,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    -0.510344739534,
                    0.57568872375,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    0.510344739534,
                    -0.57568872375,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    -0.510344739534,
                    -0.57568872375,
                    0.638851109552,
                    0.000379925683,
                ],
                [
                    0.510344739534,
                    0.638851109552,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    -0.510344739534,
                    0.638851109552,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    0.510344739534,
                    -0.638851109552,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    -0.510344739534,
                    -0.638851109552,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    0.638851109552,
                    0.57568872375,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    -0.638851109552,
                    0.57568872375,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    0.638851109552,
                    -0.57568872375,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    -0.638851109552,
                    -0.57568872375,
                    0.510344739534,
                    0.000379925683,
                ],
                [
                    0.638851109552,
                    0.510344739534,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    -0.638851109552,
                    0.510344739534,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    0.638851109552,
                    -0.510344739534,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    -0.638851109552,
                    -0.510344739534,
                    0.57568872375,
                    0.000379925683,
                ],
                [
                    0.122503943059,
                    0.021364559227,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    -0.122503943059,
                    0.021364559227,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    0.122503943059,
                    -0.021364559227,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    -0.122503943059,
                    -0.021364559227,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    0.122503943059,
                    0.992238045806,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    -0.122503943059,
                    0.992238045806,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    0.122503943059,
                    -0.992238045806,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    -0.122503943059,
                    -0.992238045806,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    0.021364559227,
                    0.122503943059,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    -0.021364559227,
                    0.122503943059,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    0.021364559227,
                    -0.122503943059,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    -0.021364559227,
                    -0.122503943059,
                    0.992238045806,
                    0.000224660366,
                ],
                [
                    0.021364559227,
                    0.992238045806,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    -0.021364559227,
                    0.992238045806,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    0.021364559227,
                    -0.992238045806,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    -0.021364559227,
                    -0.992238045806,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    0.992238045806,
                    0.122503943059,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    -0.992238045806,
                    0.122503943059,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    0.992238045806,
                    -0.122503943059,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    -0.992238045806,
                    -0.122503943059,
                    0.021364559227,
                    0.000224660366,
                ],
                [
                    0.992238045806,
                    0.021364559227,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    -0.992238045806,
                    0.021364559227,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    0.992238045806,
                    -0.021364559227,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    -0.992238045806,
                    -0.021364559227,
                    0.122503943059,
                    0.000224660366,
                ],
                [
                    0.153911321732,
                    0.045209261661,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    -0.153911321732,
                    0.045209261661,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    0.153911321732,
                    -0.045209261661,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    -0.153911321732,
                    -0.045209261661,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    0.153911321732,
                    0.987049860799,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    -0.153911321732,
                    0.987049860799,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    0.153911321732,
                    -0.987049860799,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    -0.153911321732,
                    -0.987049860799,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    0.045209261661,
                    0.153911321732,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    -0.045209261661,
                    0.153911321732,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    0.045209261661,
                    -0.153911321732,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    -0.045209261661,
                    -0.153911321732,
                    0.987049860799,
                    0.000250739765,
                ],
                [
                    0.045209261661,
                    0.987049860799,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    -0.045209261661,
                    0.987049860799,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    0.045209261661,
                    -0.987049860799,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    -0.045209261661,
                    -0.987049860799,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    0.987049860799,
                    0.153911321732,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    -0.987049860799,
                    0.153911321732,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    0.987049860799,
                    -0.153911321732,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    -0.987049860799,
                    -0.153911321732,
                    0.045209261661,
                    0.000250739765,
                ],
                [
                    0.987049860799,
                    0.045209261661,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    -0.987049860799,
                    0.045209261661,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    0.987049860799,
                    -0.045209261661,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    -0.987049860799,
                    -0.045209261661,
                    0.153911321732,
                    0.000250739765,
                ],
                [
                    0.185621309864,
                    0.070864681779,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    -0.185621309864,
                    0.070864681779,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    0.185621309864,
                    -0.070864681779,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    -0.185621309864,
                    -0.070864681779,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    0.185621309864,
                    0.980062715443,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    -0.185621309864,
                    0.980062715443,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    0.185621309864,
                    -0.980062715443,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    -0.185621309864,
                    -0.980062715443,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    0.070864681779,
                    0.185621309864,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    -0.070864681779,
                    0.185621309864,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    0.070864681779,
                    -0.185621309864,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    -0.070864681779,
                    -0.185621309864,
                    0.980062715443,
                    0.000273253224,
                ],
                [
                    0.070864681779,
                    0.980062715443,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    -0.070864681779,
                    0.980062715443,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    0.070864681779,
                    -0.980062715443,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    -0.070864681779,
                    -0.980062715443,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    0.980062715443,
                    0.185621309864,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    -0.980062715443,
                    0.185621309864,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    0.980062715443,
                    -0.185621309864,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    -0.980062715443,
                    -0.185621309864,
                    0.070864681779,
                    0.000273253224,
                ],
                [
                    0.980062715443,
                    0.070864681779,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    -0.980062715443,
                    0.070864681779,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    0.980062715443,
                    -0.070864681779,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    -0.980062715443,
                    -0.070864681779,
                    0.185621309864,
                    0.000273253224,
                ],
                [
                    0.217499872804,
                    0.097852394888,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    -0.217499872804,
                    0.097852394888,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    0.217499872804,
                    -0.097852394888,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    -0.217499872804,
                    -0.097852394888,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    0.217499872804,
                    0.971142993665,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    -0.217499872804,
                    0.971142993665,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    0.217499872804,
                    -0.971142993665,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    -0.217499872804,
                    -0.971142993665,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    0.097852394888,
                    0.217499872804,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    -0.097852394888,
                    0.217499872804,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    0.097852394888,
                    -0.217499872804,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    -0.097852394888,
                    -0.217499872804,
                    0.971142993665,
                    0.000292547371,
                ],
                [
                    0.097852394888,
                    0.971142993665,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    -0.097852394888,
                    0.971142993665,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    0.097852394888,
                    -0.971142993665,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    -0.097852394888,
                    -0.971142993665,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    0.971142993665,
                    0.217499872804,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    -0.971142993665,
                    0.217499872804,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    0.971142993665,
                    -0.217499872804,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    -0.971142993665,
                    -0.217499872804,
                    0.097852394888,
                    0.000292547371,
                ],
                [
                    0.971142993665,
                    0.097852394888,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    -0.971142993665,
                    0.097852394888,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    0.971142993665,
                    -0.097852394888,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    -0.971142993665,
                    -0.097852394888,
                    0.217499872804,
                    0.000292547371,
                ],
                [
                    0.249412833694,
                    0.125810639627,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    -0.249412833694,
                    0.125810639627,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    0.249412833694,
                    -0.125810639627,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    -0.249412833694,
                    -0.125810639627,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    0.249412833694,
                    0.96019004439,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    -0.249412833694,
                    0.96019004439,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    0.249412833694,
                    -0.96019004439,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    -0.249412833694,
                    -0.96019004439,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    0.125810639627,
                    0.249412833694,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    -0.125810639627,
                    0.249412833694,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    0.125810639627,
                    -0.249412833694,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    -0.125810639627,
                    -0.249412833694,
                    0.96019004439,
                    0.000309015293,
                ],
                [
                    0.125810639627,
                    0.96019004439,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    -0.125810639627,
                    0.96019004439,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    0.125810639627,
                    -0.96019004439,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    -0.125810639627,
                    -0.96019004439,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    0.96019004439,
                    0.249412833694,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    -0.96019004439,
                    0.249412833694,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    0.96019004439,
                    -0.249412833694,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    -0.96019004439,
                    -0.249412833694,
                    0.125810639627,
                    0.000309015293,
                ],
                [
                    0.96019004439,
                    0.125810639627,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    -0.96019004439,
                    0.125810639627,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    0.96019004439,
                    -0.125810639627,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    -0.96019004439,
                    -0.125810639627,
                    0.249412833694,
                    0.000309015293,
                ],
                [
                    0.281232156214,
                    0.154452912505,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    -0.281232156214,
                    0.154452912505,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    0.281232156214,
                    -0.154452912505,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    -0.281232156214,
                    -0.154452912505,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    0.281232156214,
                    0.947128698821,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    -0.281232156214,
                    0.947128698821,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    0.281232156214,
                    -0.947128698821,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    -0.281232156214,
                    -0.947128698821,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    0.154452912505,
                    0.281232156214,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    -0.154452912505,
                    0.281232156214,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    0.154452912505,
                    -0.281232156214,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    -0.154452912505,
                    -0.281232156214,
                    0.947128698821,
                    0.000323019256,
                ],
                [
                    0.154452912505,
                    0.947128698821,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    -0.154452912505,
                    0.947128698821,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    0.154452912505,
                    -0.947128698821,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    -0.154452912505,
                    -0.947128698821,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    0.947128698821,
                    0.281232156214,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    -0.947128698821,
                    0.281232156214,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    0.947128698821,
                    -0.281232156214,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    -0.947128698821,
                    -0.281232156214,
                    0.154452912505,
                    0.000323019256,
                ],
                [
                    0.947128698821,
                    0.154452912505,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    -0.947128698821,
                    0.154452912505,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    0.947128698821,
                    -0.154452912505,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    -0.947128698821,
                    -0.154452912505,
                    0.281232156214,
                    0.000323019256,
                ],
                [
                    0.312837227646,
                    0.18354335122,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    -0.312837227646,
                    0.18354335122,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    0.312837227646,
                    -0.18354335122,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    -0.312837227646,
                    -0.18354335122,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    0.312837227646,
                    0.931903807923,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    -0.312837227646,
                    0.931903807923,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    0.312837227646,
                    -0.931903807923,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    -0.312837227646,
                    -0.931903807923,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    0.18354335122,
                    0.312837227646,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    -0.18354335122,
                    0.312837227646,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    0.18354335122,
                    -0.312837227646,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    -0.18354335122,
                    -0.312837227646,
                    0.931903807923,
                    0.000334873328,
                ],
                [
                    0.18354335122,
                    0.931903807923,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    -0.18354335122,
                    0.931903807923,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    0.18354335122,
                    -0.931903807923,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    -0.18354335122,
                    -0.931903807923,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    0.931903807923,
                    0.312837227646,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    -0.931903807923,
                    0.312837227646,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    0.931903807923,
                    -0.312837227646,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    -0.931903807923,
                    -0.312837227646,
                    0.18354335122,
                    0.000334873328,
                ],
                [
                    0.931903807923,
                    0.18354335122,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    -0.931903807923,
                    0.18354335122,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    0.931903807923,
                    -0.18354335122,
                    0.312837227646,
                    0.000334873328,
                ],
                [
                    -0.931903807923,
                    -0.18354335122,
                    0.312837227646,
                    0.000334873328,
                ],
                [0.344114516018, 0.212881325862, 0.914476211263, 0.000344845],
                [-0.344114516018, 0.212881325862, 0.914476211263, 0.000344845],
                [0.344114516018, -0.212881325862, 0.914476211263, 0.000344845],
                [
                    -0.344114516018,
                    -0.212881325862,
                    0.914476211263,
                    0.000344845,
                ],
                [0.344114516018, 0.914476211263, 0.212881325862, 0.000344845],
                [-0.344114516018, 0.914476211263, 0.212881325862, 0.000344845],
                [0.344114516018, -0.914476211263, 0.212881325862, 0.000344845],
                [
                    -0.344114516018,
                    -0.914476211263,
                    0.212881325862,
                    0.000344845,
                ],
                [0.212881325862, 0.344114516018, 0.914476211263, 0.000344845],
                [-0.212881325862, 0.344114516018, 0.914476211263, 0.000344845],
                [0.212881325862, -0.344114516018, 0.914476211263, 0.000344845],
                [
                    -0.212881325862,
                    -0.344114516018,
                    0.914476211263,
                    0.000344845,
                ],
                [0.212881325862, 0.914476211263, 0.344114516018, 0.000344845],
                [-0.212881325862, 0.914476211263, 0.344114516018, 0.000344845],
                [0.212881325862, -0.914476211263, 0.344114516018, 0.000344845],
                [
                    -0.212881325862,
                    -0.914476211263,
                    0.344114516018,
                    0.000344845,
                ],
                [0.914476211263, 0.344114516018, 0.212881325862, 0.000344845],
                [-0.914476211263, 0.344114516018, 0.212881325862, 0.000344845],
                [0.914476211263, -0.344114516018, 0.212881325862, 0.000344845],
                [
                    -0.914476211263,
                    -0.344114516018,
                    0.212881325862,
                    0.000344845,
                ],
                [0.914476211263, 0.212881325862, 0.344114516018, 0.000344845],
                [-0.914476211263, 0.212881325862, 0.344114516018, 0.000344845],
                [0.914476211263, -0.212881325862, 0.344114516018, 0.000344845],
                [
                    -0.914476211263,
                    -0.212881325862,
                    0.344114516018,
                    0.000344845,
                ],
                [
                    0.374956771485,
                    0.242291373488,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    -0.374956771485,
                    0.242291373488,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    0.374956771485,
                    -0.242291373488,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    -0.374956771485,
                    -0.242291373488,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    0.374956771485,
                    0.894819708014,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    -0.374956771485,
                    0.894819708014,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    0.374956771485,
                    -0.894819708014,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    -0.374956771485,
                    -0.894819708014,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    0.242291373488,
                    0.374956771485,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    -0.242291373488,
                    0.374956771485,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    0.242291373488,
                    -0.374956771485,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    -0.242291373488,
                    -0.374956771485,
                    0.894819708014,
                    0.000353162165,
                ],
                [
                    0.242291373488,
                    0.894819708014,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    -0.242291373488,
                    0.894819708014,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    0.242291373488,
                    -0.894819708014,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    -0.242291373488,
                    -0.894819708014,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    0.894819708014,
                    0.374956771485,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    -0.894819708014,
                    0.374956771485,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    0.894819708014,
                    -0.374956771485,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    -0.894819708014,
                    -0.374956771485,
                    0.242291373488,
                    0.000353162165,
                ],
                [
                    0.894819708014,
                    0.242291373488,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    -0.894819708014,
                    0.242291373488,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    0.894819708014,
                    -0.242291373488,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    -0.894819708014,
                    -0.242291373488,
                    0.374956771485,
                    0.000353162165,
                ],
                [
                    0.405262173202,
                    0.271616374839,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    -0.405262173202,
                    0.271616374839,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    0.405262173202,
                    -0.271616374839,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    -0.405262173202,
                    -0.271616374839,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    0.405262173202,
                    0.872918733841,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    -0.405262173202,
                    0.872918733841,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    0.405262173202,
                    -0.872918733841,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    -0.405262173202,
                    -0.872918733841,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    0.271616374839,
                    0.405262173202,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    -0.271616374839,
                    0.405262173202,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    0.271616374839,
                    -0.405262173202,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    -0.271616374839,
                    -0.405262173202,
                    0.872918733841,
                    0.000360020825,
                ],
                [
                    0.271616374839,
                    0.872918733841,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    -0.271616374839,
                    0.872918733841,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    0.271616374839,
                    -0.872918733841,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    -0.271616374839,
                    -0.872918733841,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    0.872918733841,
                    0.405262173202,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    -0.872918733841,
                    0.405262173202,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    0.872918733841,
                    -0.405262173202,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    -0.872918733841,
                    -0.405262173202,
                    0.271616374839,
                    0.000360020825,
                ],
                [
                    0.872918733841,
                    0.271616374839,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    -0.872918733841,
                    0.271616374839,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    0.872918733841,
                    -0.271616374839,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    -0.872918733841,
                    -0.271616374839,
                    0.405262173202,
                    0.000360020825,
                ],
                [
                    0.434933545352,
                    0.300712767124,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    -0.434933545352,
                    0.300712767124,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    0.434933545352,
                    -0.300712767124,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    -0.434933545352,
                    -0.300712767124,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    0.434933545352,
                    0.848766541998,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    -0.434933545352,
                    0.848766541998,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    0.434933545352,
                    -0.848766541998,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    -0.434933545352,
                    -0.848766541998,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    0.300712767124,
                    0.434933545352,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    -0.300712767124,
                    0.434933545352,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    0.300712767124,
                    -0.434933545352,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    -0.300712767124,
                    -0.434933545352,
                    0.848766541998,
                    0.000365592087,
                ],
                [
                    0.300712767124,
                    0.848766541998,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    -0.300712767124,
                    0.848766541998,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    0.300712767124,
                    -0.848766541998,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    -0.300712767124,
                    -0.848766541998,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    0.848766541998,
                    0.434933545352,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    -0.848766541998,
                    0.434933545352,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    0.848766541998,
                    -0.434933545352,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    -0.848766541998,
                    -0.434933545352,
                    0.300712767124,
                    0.000365592087,
                ],
                [
                    0.848766541998,
                    0.300712767124,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    -0.848766541998,
                    0.300712767124,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    0.848766541998,
                    -0.300712767124,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    -0.848766541998,
                    -0.300712767124,
                    0.434933545352,
                    0.000365592087,
                ],
                [
                    0.463877664152,
                    0.329447067722,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    -0.463877664152,
                    0.329447067722,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    0.463877664152,
                    -0.329447067722,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    -0.463877664152,
                    -0.329447067722,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    0.463877664152,
                    0.822363753013,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    -0.463877664152,
                    0.822363753013,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    0.463877664152,
                    -0.822363753013,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    -0.463877664152,
                    -0.822363753013,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    0.329447067722,
                    0.463877664152,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    -0.329447067722,
                    0.463877664152,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    0.329447067722,
                    -0.463877664152,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    -0.329447067722,
                    -0.463877664152,
                    0.822363753013,
                    0.00037002806,
                ],
                [
                    0.329447067722,
                    0.822363753013,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    -0.329447067722,
                    0.822363753013,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    0.329447067722,
                    -0.822363753013,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    -0.329447067722,
                    -0.822363753013,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    0.822363753013,
                    0.463877664152,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    -0.822363753013,
                    0.463877664152,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    0.822363753013,
                    -0.463877664152,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    -0.822363753013,
                    -0.463877664152,
                    0.329447067722,
                    0.00037002806,
                ],
                [
                    0.822363753013,
                    0.329447067722,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    -0.822363753013,
                    0.329447067722,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    0.822363753013,
                    -0.329447067722,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    -0.822363753013,
                    -0.329447067722,
                    0.463877664152,
                    0.00037002806,
                ],
                [
                    0.492004641046,
                    0.35769325437,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    -0.492004641046,
                    0.35769325437,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    0.492004641046,
                    -0.35769325437,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    -0.492004641046,
                    -0.35769325437,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    0.492004641046,
                    0.793717184498,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    -0.492004641046,
                    0.793717184498,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    0.492004641046,
                    -0.793717184498,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    -0.492004641046,
                    -0.793717184498,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    0.35769325437,
                    0.492004641046,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    -0.35769325437,
                    0.492004641046,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    0.35769325437,
                    -0.492004641046,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    -0.35769325437,
                    -0.492004641046,
                    0.793717184498,
                    0.000373466701,
                ],
                [
                    0.35769325437,
                    0.793717184498,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    -0.35769325437,
                    0.793717184498,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    0.35769325437,
                    -0.793717184498,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    -0.35769325437,
                    -0.793717184498,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    0.793717184498,
                    0.492004641046,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    -0.793717184498,
                    0.492004641046,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    0.793717184498,
                    -0.492004641046,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    -0.793717184498,
                    -0.492004641046,
                    0.35769325437,
                    0.000373466701,
                ],
                [
                    0.793717184498,
                    0.35769325437,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    -0.793717184498,
                    0.35769325437,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    0.793717184498,
                    -0.35769325437,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    -0.793717184498,
                    -0.35769325437,
                    0.492004641046,
                    0.000373466701,
                ],
                [
                    0.519227355486,
                    0.385330705976,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    -0.519227355486,
                    0.385330705976,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    0.519227355486,
                    -0.385330705976,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    -0.519227355486,
                    -0.385330705976,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    0.519227355486,
                    0.762838908517,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    -0.519227355486,
                    0.762838908517,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    0.519227355486,
                    -0.762838908517,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    -0.519227355486,
                    -0.762838908517,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    0.385330705976,
                    0.519227355486,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    -0.385330705976,
                    0.519227355486,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    0.385330705976,
                    -0.519227355486,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    -0.385330705976,
                    -0.519227355486,
                    0.762838908517,
                    0.000376035738,
                ],
                [
                    0.385330705976,
                    0.762838908517,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    -0.385330705976,
                    0.762838908517,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    0.385330705976,
                    -0.762838908517,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    -0.385330705976,
                    -0.762838908517,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    0.762838908517,
                    0.519227355486,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    -0.762838908517,
                    0.519227355486,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    0.762838908517,
                    -0.519227355486,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    -0.762838908517,
                    -0.519227355486,
                    0.385330705976,
                    0.000376035738,
                ],
                [
                    0.762838908517,
                    0.385330705976,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    -0.762838908517,
                    0.385330705976,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    0.762838908517,
                    -0.385330705976,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    -0.762838908517,
                    -0.385330705976,
                    0.519227355486,
                    0.000376035738,
                ],
                [
                    0.545460908114,
                    0.412242504445,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    -0.545460908114,
                    0.412242504445,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    0.545460908114,
                    -0.412242504445,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    -0.545460908114,
                    -0.412242504445,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    0.545460908114,
                    0.729745514031,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    -0.545460908114,
                    0.729745514031,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    0.545460908114,
                    -0.729745514031,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    -0.545460908114,
                    -0.729745514031,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    0.412242504445,
                    0.545460908114,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    -0.412242504445,
                    0.545460908114,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    0.412242504445,
                    -0.545460908114,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    -0.412242504445,
                    -0.545460908114,
                    0.729745514031,
                    0.000377855785,
                ],
                [
                    0.412242504445,
                    0.729745514031,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    -0.412242504445,
                    0.729745514031,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    0.412242504445,
                    -0.729745514031,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    -0.412242504445,
                    -0.729745514031,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    0.729745514031,
                    0.545460908114,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    -0.729745514031,
                    0.545460908114,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    0.729745514031,
                    -0.545460908114,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    -0.729745514031,
                    -0.545460908114,
                    0.412242504445,
                    0.000377855785,
                ],
                [
                    0.729745514031,
                    0.412242504445,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    -0.729745514031,
                    0.412242504445,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    0.729745514031,
                    -0.412242504445,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    -0.729745514031,
                    -0.412242504445,
                    0.545460908114,
                    0.000377855785,
                ],
                [
                    0.570622066142,
                    0.438313958778,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    -0.570622066142,
                    0.438313958778,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    0.570622066142,
                    -0.438313958778,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    -0.570622066142,
                    -0.438313958778,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    0.570622066142,
                    0.694457580542,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    -0.570622066142,
                    0.694457580542,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    0.570622066142,
                    -0.694457580542,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    -0.570622066142,
                    -0.694457580542,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    0.438313958778,
                    0.570622066142,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    -0.438313958778,
                    0.570622066142,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    0.438313958778,
                    -0.570622066142,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    -0.438313958778,
                    -0.570622066142,
                    0.694457580542,
                    0.000379042767,
                ],
                [
                    0.438313958778,
                    0.694457580542,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    -0.438313958778,
                    0.694457580542,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    0.438313958778,
                    -0.694457580542,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    -0.438313958778,
                    -0.694457580542,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    0.694457580542,
                    0.570622066142,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    -0.694457580542,
                    0.570622066142,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    0.694457580542,
                    -0.570622066142,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    -0.694457580542,
                    -0.570622066142,
                    0.438313958778,
                    0.000379042767,
                ],
                [
                    0.694457580542,
                    0.438313958778,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    -0.694457580542,
                    0.438313958778,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    0.694457580542,
                    -0.438313958778,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    -0.694457580542,
                    -0.438313958778,
                    0.570622066142,
                    0.000379042767,
                ],
                [
                    0.594628675518,
                    0.46343125363,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    -0.594628675518,
                    0.46343125363,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    0.594628675518,
                    -0.46343125363,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    -0.594628675518,
                    -0.46343125363,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    0.594628675518,
                    0.656999399855,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    -0.594628675518,
                    0.656999399855,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    0.594628675518,
                    -0.656999399855,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    -0.594628675518,
                    -0.656999399855,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    0.46343125363,
                    0.594628675518,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    -0.46343125363,
                    0.594628675518,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    0.46343125363,
                    -0.594628675518,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    -0.46343125363,
                    -0.594628675518,
                    0.656999399855,
                    0.000379709655,
                ],
                [
                    0.46343125363,
                    0.656999399855,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    -0.46343125363,
                    0.656999399855,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    0.46343125363,
                    -0.656999399855,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    -0.46343125363,
                    -0.656999399855,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    0.656999399855,
                    0.594628675518,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    -0.656999399855,
                    0.594628675518,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    0.656999399855,
                    -0.594628675518,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    -0.656999399855,
                    -0.594628675518,
                    0.46343125363,
                    0.000379709655,
                ],
                [
                    0.656999399855,
                    0.46343125363,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    -0.656999399855,
                    0.46343125363,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    0.656999399855,
                    -0.46343125363,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    -0.656999399855,
                    -0.46343125363,
                    0.594628675518,
                    0.000379709655,
                ],
                [
                    0.190537079092,
                    0.023713115378,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    -0.190537079092,
                    0.023713115378,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    0.190537079092,
                    -0.023713115378,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    -0.190537079092,
                    -0.023713115378,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    0.190537079092,
                    0.981393554926,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    -0.190537079092,
                    0.981393554926,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    0.190537079092,
                    -0.981393554926,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    -0.190537079092,
                    -0.981393554926,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    0.023713115378,
                    0.190537079092,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    -0.023713115378,
                    0.190537079092,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    0.023713115378,
                    -0.190537079092,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    -0.023713115378,
                    -0.190537079092,
                    0.981393554926,
                    0.000269821187,
                ],
                [
                    0.023713115378,
                    0.981393554926,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    -0.023713115378,
                    0.981393554926,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    0.023713115378,
                    -0.981393554926,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    -0.023713115378,
                    -0.981393554926,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    0.981393554926,
                    0.190537079092,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    -0.981393554926,
                    0.190537079092,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    0.981393554926,
                    -0.190537079092,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    -0.981393554926,
                    -0.190537079092,
                    0.023713115378,
                    0.000269821187,
                ],
                [
                    0.981393554926,
                    0.023713115378,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    -0.981393554926,
                    0.023713115378,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    0.981393554926,
                    -0.023713115378,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    -0.981393554926,
                    -0.023713115378,
                    0.190537079092,
                    0.000269821187,
                ],
                [
                    0.224251871775,
                    0.049178780593,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    -0.224251871775,
                    0.049178780593,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    0.224251871775,
                    -0.049178780593,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    -0.224251871775,
                    -0.049178780593,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    0.224251871775,
                    0.973289548667,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    -0.224251871775,
                    0.973289548667,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    0.224251871775,
                    -0.973289548667,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    -0.224251871775,
                    -0.973289548667,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    0.049178780593,
                    0.224251871775,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    -0.049178780593,
                    0.224251871775,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    0.049178780593,
                    -0.224251871775,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    -0.049178780593,
                    -0.224251871775,
                    0.973289548667,
                    0.000288812014,
                ],
                [
                    0.049178780593,
                    0.973289548667,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    -0.049178780593,
                    0.973289548667,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    0.049178780593,
                    -0.973289548667,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    -0.049178780593,
                    -0.973289548667,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    0.973289548667,
                    0.224251871775,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    -0.973289548667,
                    0.224251871775,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    0.973289548667,
                    -0.224251871775,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    -0.973289548667,
                    -0.224251871775,
                    0.049178780593,
                    0.000288812014,
                ],
                [
                    0.973289548667,
                    0.049178780593,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    -0.973289548667,
                    0.049178780593,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    0.973289548667,
                    -0.049178780593,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    -0.973289548667,
                    -0.049178780593,
                    0.224251871775,
                    0.000288812014,
                ],
                [
                    0.257719080803,
                    0.075954989605,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    -0.257719080803,
                    0.075954989605,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    0.257719080803,
                    -0.075954989605,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    -0.257719080803,
                    -0.075954989605,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    0.257719080803,
                    0.963229834953,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    -0.257719080803,
                    0.963229834953,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    0.257719080803,
                    -0.963229834953,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    -0.257719080803,
                    -0.963229834953,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    0.075954989605,
                    0.257719080803,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    -0.075954989605,
                    0.257719080803,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    0.075954989605,
                    -0.257719080803,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    -0.075954989605,
                    -0.257719080803,
                    0.963229834953,
                    0.000305359478,
                ],
                [
                    0.075954989605,
                    0.963229834953,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    -0.075954989605,
                    0.963229834953,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    0.075954989605,
                    -0.963229834953,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    -0.075954989605,
                    -0.963229834953,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    0.963229834953,
                    0.257719080803,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    -0.963229834953,
                    0.257719080803,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    0.963229834953,
                    -0.257719080803,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    -0.963229834953,
                    -0.257719080803,
                    0.075954989605,
                    0.000305359478,
                ],
                [
                    0.963229834953,
                    0.075954989605,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    -0.963229834953,
                    0.075954989605,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    0.963229834953,
                    -0.075954989605,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    -0.963229834953,
                    -0.075954989605,
                    0.257719080803,
                    0.000305359478,
                ],
                [
                    0.290872453493,
                    0.103699108319,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    -0.290872453493,
                    0.103699108319,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    0.290872453493,
                    -0.103699108319,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    -0.290872453493,
                    -0.103699108319,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    0.290872453493,
                    0.951125496837,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    -0.290872453493,
                    0.951125496837,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    0.290872453493,
                    -0.951125496837,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    -0.290872453493,
                    -0.951125496837,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    0.103699108319,
                    0.290872453493,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    -0.103699108319,
                    0.290872453493,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    0.103699108319,
                    -0.290872453493,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    -0.103699108319,
                    -0.290872453493,
                    0.951125496837,
                    0.000319641754,
                ],
                [
                    0.103699108319,
                    0.951125496837,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    -0.103699108319,
                    0.951125496837,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    0.103699108319,
                    -0.951125496837,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    -0.103699108319,
                    -0.951125496837,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    0.951125496837,
                    0.290872453493,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    -0.951125496837,
                    0.290872453493,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    0.951125496837,
                    -0.290872453493,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    -0.951125496837,
                    -0.290872453493,
                    0.103699108319,
                    0.000319641754,
                ],
                [
                    0.951125496837,
                    0.103699108319,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    -0.951125496837,
                    0.103699108319,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    0.951125496837,
                    -0.103699108319,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    -0.951125496837,
                    -0.103699108319,
                    0.290872453493,
                    0.000319641754,
                ],
                [
                    0.323635402006,
                    0.132134858445,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    -0.323635402006,
                    0.132134858445,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    0.323635402006,
                    -0.132134858445,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    -0.323635402006,
                    -0.132134858445,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    0.323635402006,
                    0.936910084134,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    -0.323635402006,
                    0.936910084134,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    0.323635402006,
                    -0.936910084134,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    -0.323635402006,
                    -0.936910084134,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    0.132134858445,
                    0.323635402006,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    -0.132134858445,
                    0.323635402006,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    0.132134858445,
                    -0.323635402006,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    -0.132134858445,
                    -0.323635402006,
                    0.936910084134,
                    0.000331870874,
                ],
                [
                    0.132134858445,
                    0.936910084134,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    -0.132134858445,
                    0.936910084134,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    0.132134858445,
                    -0.936910084134,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    -0.132134858445,
                    -0.936910084134,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    0.936910084134,
                    0.323635402006,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    -0.936910084134,
                    0.323635402006,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    0.936910084134,
                    -0.323635402006,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    -0.936910084134,
                    -0.323635402006,
                    0.132134858445,
                    0.000331870874,
                ],
                [
                    0.936910084134,
                    0.132134858445,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    -0.936910084134,
                    0.132134858445,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    0.936910084134,
                    -0.132134858445,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    -0.936910084134,
                    -0.132134858445,
                    0.323635402006,
                    0.000331870874,
                ],
                [
                    0.35592673593,
                    0.161031657131,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    -0.35592673593,
                    0.161031657131,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    0.35592673593,
                    -0.161031657131,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    -0.35592673593,
                    -0.161031657131,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    0.35592673593,
                    0.920535150905,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    -0.35592673593,
                    0.920535150905,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    0.35592673593,
                    -0.920535150905,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    -0.35592673593,
                    -0.920535150905,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    0.161031657131,
                    0.35592673593,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    -0.161031657131,
                    0.35592673593,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    0.161031657131,
                    -0.35592673593,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    -0.161031657131,
                    -0.35592673593,
                    0.920535150905,
                    0.000342255982,
                ],
                [
                    0.161031657131,
                    0.920535150905,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    -0.161031657131,
                    0.920535150905,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    0.161031657131,
                    -0.920535150905,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    -0.161031657131,
                    -0.920535150905,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    0.920535150905,
                    0.35592673593,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    -0.920535150905,
                    0.35592673593,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    0.920535150905,
                    -0.35592673593,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    -0.920535150905,
                    -0.35592673593,
                    0.161031657131,
                    0.000342255982,
                ],
                [
                    0.920535150905,
                    0.161031657131,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    -0.920535150905,
                    0.161031657131,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    0.920535150905,
                    -0.161031657131,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    -0.920535150905,
                    -0.161031657131,
                    0.35592673593,
                    0.000342255982,
                ],
                [
                    0.387663712368,
                    0.19019120804,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    -0.387663712368,
                    0.19019120804,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    0.387663712368,
                    -0.19019120804,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    -0.387663712368,
                    -0.19019120804,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    0.387663712368,
                    0.901966823391,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    -0.387663712368,
                    0.901966823391,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    0.387663712368,
                    -0.901966823391,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    -0.387663712368,
                    -0.901966823391,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    0.19019120804,
                    0.387663712368,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    -0.19019120804,
                    0.387663712368,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    0.19019120804,
                    -0.387663712368,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    -0.19019120804,
                    -0.387663712368,
                    0.901966823391,
                    0.000350990545,
                ],
                [
                    0.19019120804,
                    0.901966823391,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    -0.19019120804,
                    0.901966823391,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    0.19019120804,
                    -0.901966823391,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    -0.19019120804,
                    -0.901966823391,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    0.901966823391,
                    0.387663712368,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    -0.901966823391,
                    0.387663712368,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    0.901966823391,
                    -0.387663712368,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    -0.901966823391,
                    -0.387663712368,
                    0.19019120804,
                    0.000350990545,
                ],
                [
                    0.901966823391,
                    0.19019120804,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    -0.901966823391,
                    0.19019120804,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    0.901966823391,
                    -0.19019120804,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    -0.901966823391,
                    -0.19019120804,
                    0.387663712368,
                    0.000350990545,
                ],
                [
                    0.418763670522,
                    0.219438495014,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    -0.418763670522,
                    0.219438495014,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    0.418763670522,
                    -0.219438495014,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    -0.418763670522,
                    -0.219438495014,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    0.418763670522,
                    0.881183145071,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    -0.418763670522,
                    0.881183145071,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    0.418763670522,
                    -0.881183145071,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    -0.418763670522,
                    -0.881183145071,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    0.219438495014,
                    0.418763670522,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    -0.219438495014,
                    0.418763670522,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    0.219438495014,
                    -0.418763670522,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    -0.219438495014,
                    -0.418763670522,
                    0.881183145071,
                    0.00035824957,
                ],
                [
                    0.219438495014,
                    0.881183145071,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    -0.219438495014,
                    0.881183145071,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    0.219438495014,
                    -0.881183145071,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    -0.219438495014,
                    -0.881183145071,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    0.881183145071,
                    0.418763670522,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    -0.881183145071,
                    0.418763670522,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    0.881183145071,
                    -0.418763670522,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    -0.881183145071,
                    -0.418763670522,
                    0.219438495014,
                    0.00035824957,
                ],
                [
                    0.881183145071,
                    0.219438495014,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    -0.881183145071,
                    0.219438495014,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    0.881183145071,
                    -0.219438495014,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    -0.881183145071,
                    -0.219438495014,
                    0.418763670522,
                    0.00035824957,
                ],
                [
                    0.449144901988,
                    0.248615533476,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    -0.449144901988,
                    0.248615533476,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    0.449144901988,
                    -0.248615533476,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    -0.449144901988,
                    -0.248615533476,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    0.449144901988,
                    0.858171995309,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    -0.449144901988,
                    0.858171995309,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    0.449144901988,
                    -0.858171995309,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    -0.449144901988,
                    -0.858171995309,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    0.248615533476,
                    0.449144901988,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    -0.248615533476,
                    0.449144901988,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    0.248615533476,
                    -0.449144901988,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    -0.248615533476,
                    -0.449144901988,
                    0.858171995309,
                    0.00036419086,
                ],
                [
                    0.248615533476,
                    0.858171995309,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    -0.248615533476,
                    0.858171995309,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    0.248615533476,
                    -0.858171995309,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    -0.248615533476,
                    -0.858171995309,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    0.858171995309,
                    0.449144901988,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    -0.858171995309,
                    0.449144901988,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    0.858171995309,
                    -0.449144901988,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    -0.858171995309,
                    -0.449144901988,
                    0.248615533476,
                    0.00036419086,
                ],
                [
                    0.858171995309,
                    0.248615533476,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    -0.858171995309,
                    0.248615533476,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    0.858171995309,
                    -0.248615533476,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    -0.858171995309,
                    -0.248615533476,
                    0.449144901988,
                    0.00036419086,
                ],
                [
                    0.478727093243,
                    0.277576893181,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    -0.478727093243,
                    0.277576893181,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    0.478727093243,
                    -0.277576893181,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    -0.478727093243,
                    -0.277576893181,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    0.478727093243,
                    0.832929431925,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    -0.478727093243,
                    0.832929431925,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    0.478727093243,
                    -0.832929431925,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    -0.478727093243,
                    -0.832929431925,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    0.277576893181,
                    0.478727093243,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    -0.277576893181,
                    0.478727093243,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    0.277576893181,
                    -0.478727093243,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    -0.277576893181,
                    -0.478727093243,
                    0.832929431925,
                    0.000368957705,
                ],
                [
                    0.277576893181,
                    0.832929431925,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    -0.277576893181,
                    0.832929431925,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    0.277576893181,
                    -0.832929431925,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    -0.277576893181,
                    -0.832929431925,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    0.832929431925,
                    0.478727093243,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    -0.832929431925,
                    0.478727093243,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    0.832929431925,
                    -0.478727093243,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    -0.832929431925,
                    -0.478727093243,
                    0.277576893181,
                    0.000368957705,
                ],
                [
                    0.832929431925,
                    0.277576893181,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    -0.832929431925,
                    0.277576893181,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    0.832929431925,
                    -0.277576893181,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    -0.832929431925,
                    -0.277576893181,
                    0.478727093243,
                    0.000368957705,
                ],
                [
                    0.507431515306,
                    0.306186378659,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    -0.507431515306,
                    0.306186378659,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    0.507431515306,
                    -0.306186378659,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    -0.507431515306,
                    -0.306186378659,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    0.507431515306,
                    0.805458353236,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    -0.507431515306,
                    0.805458353236,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    0.507431515306,
                    -0.805458353236,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    -0.507431515306,
                    -0.805458353236,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    0.306186378659,
                    0.507431515306,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    -0.306186378659,
                    0.507431515306,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    0.306186378659,
                    -0.507431515306,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    -0.306186378659,
                    -0.507431515306,
                    0.805458353236,
                    0.000372681896,
                ],
                [
                    0.306186378659,
                    0.805458353236,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    -0.306186378659,
                    0.805458353236,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    0.306186378659,
                    -0.805458353236,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    -0.306186378659,
                    -0.805458353236,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    0.805458353236,
                    0.507431515306,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    -0.805458353236,
                    0.507431515306,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    0.805458353236,
                    -0.507431515306,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    -0.805458353236,
                    -0.507431515306,
                    0.306186378659,
                    0.000372681896,
                ],
                [
                    0.805458353236,
                    0.306186378659,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    -0.805458353236,
                    0.306186378659,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    0.805458353236,
                    -0.306186378659,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    -0.805458353236,
                    -0.306186378659,
                    0.507431515306,
                    0.000372681896,
                ],
                [
                    0.535181050774,
                    0.334314471815,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    -0.535181050774,
                    0.334314471815,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    0.535181050774,
                    -0.334314471815,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    -0.535181050774,
                    -0.334314471815,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    0.535181050774,
                    0.775767411553,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    -0.535181050774,
                    0.775767411553,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    0.535181050774,
                    -0.775767411553,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    -0.535181050774,
                    -0.775767411553,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    0.334314471815,
                    0.535181050774,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    -0.334314471815,
                    0.535181050774,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    0.334314471815,
                    -0.535181050774,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    -0.334314471815,
                    -0.535181050774,
                    0.775767411553,
                    0.000375486602,
                ],
                [
                    0.334314471815,
                    0.775767411553,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    -0.334314471815,
                    0.775767411553,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    0.334314471815,
                    -0.775767411553,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    -0.334314471815,
                    -0.775767411553,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    0.775767411553,
                    0.535181050774,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    -0.775767411553,
                    0.535181050774,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    0.775767411553,
                    -0.535181050774,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    -0.775767411553,
                    -0.535181050774,
                    0.334314471815,
                    0.000375486602,
                ],
                [
                    0.775767411553,
                    0.334314471815,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    -0.775767411553,
                    0.334314471815,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    0.775767411553,
                    -0.334314471815,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    -0.775767411553,
                    -0.334314471815,
                    0.535181050774,
                    0.000375486602,
                ],
                [
                    0.561900102598,
                    0.361836272903,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    -0.561900102598,
                    0.361836272903,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    0.561900102598,
                    -0.361836272903,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    -0.561900102598,
                    -0.361836272903,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    0.561900102598,
                    0.743870140759,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    -0.561900102598,
                    0.743870140759,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    0.561900102598,
                    -0.743870140759,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    -0.561900102598,
                    -0.743870140759,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    0.361836272903,
                    0.561900102598,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    -0.361836272903,
                    0.561900102598,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    0.361836272903,
                    -0.561900102598,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    -0.361836272903,
                    -0.561900102598,
                    0.743870140759,
                    0.000377488909,
                ],
                [
                    0.361836272903,
                    0.743870140759,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    -0.361836272903,
                    0.743870140759,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    0.361836272903,
                    -0.743870140759,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    -0.361836272903,
                    -0.743870140759,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    0.743870140759,
                    0.561900102598,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    -0.743870140759,
                    0.561900102598,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    0.743870140759,
                    -0.561900102598,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    -0.743870140759,
                    -0.561900102598,
                    0.361836272903,
                    0.000377488909,
                ],
                [
                    0.743870140759,
                    0.361836272903,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    -0.743870140759,
                    0.361836272903,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    0.743870140759,
                    -0.361836272903,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    -0.743870140759,
                    -0.361836272903,
                    0.561900102598,
                    0.000377488909,
                ],
                [
                    0.587514403527,
                    0.388629758362,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    -0.587514403527,
                    0.388629758362,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    0.587514403527,
                    -0.388629758362,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    -0.587514403527,
                    -0.388629758362,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    0.587514403527,
                    0.709784288755,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    -0.587514403527,
                    0.709784288755,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    0.587514403527,
                    -0.709784288755,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    -0.587514403527,
                    -0.709784288755,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    0.388629758362,
                    0.587514403527,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    -0.388629758362,
                    0.587514403527,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    0.388629758362,
                    -0.587514403527,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    -0.388629758362,
                    -0.587514403527,
                    0.709784288755,
                    0.000378801966,
                ],
                [
                    0.388629758362,
                    0.709784288755,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    -0.388629758362,
                    0.709784288755,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    0.388629758362,
                    -0.709784288755,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    -0.388629758362,
                    -0.709784288755,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    0.709784288755,
                    0.587514403527,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    -0.709784288755,
                    0.587514403527,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    0.709784288755,
                    -0.587514403527,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    -0.709784288755,
                    -0.587514403527,
                    0.388629758362,
                    0.000378801966,
                ],
                [
                    0.709784288755,
                    0.388629758362,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    -0.709784288755,
                    0.388629758362,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    0.709784288755,
                    -0.388629758362,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    -0.709784288755,
                    -0.388629758362,
                    0.587514403527,
                    0.000378801966,
                ],
                [
                    0.611950730873,
                    0.414574227779,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    -0.611950730873,
                    0.414574227779,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    0.611950730873,
                    -0.414574227779,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    -0.611950730873,
                    -0.414574227779,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    0.611950730873,
                    0.673531374655,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    -0.611950730873,
                    0.673531374655,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    0.611950730873,
                    -0.673531374655,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    -0.611950730873,
                    -0.673531374655,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    0.414574227779,
                    0.611950730873,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    -0.414574227779,
                    0.611950730873,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    0.414574227779,
                    -0.611950730873,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    -0.414574227779,
                    -0.611950730873,
                    0.673531374655,
                    0.000379536669,
                ],
                [
                    0.414574227779,
                    0.673531374655,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    -0.414574227779,
                    0.673531374655,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    0.414574227779,
                    -0.673531374655,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    -0.414574227779,
                    -0.673531374655,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    0.673531374655,
                    0.611950730873,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    -0.673531374655,
                    0.611950730873,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    0.673531374655,
                    -0.611950730873,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    -0.673531374655,
                    -0.611950730873,
                    0.414574227779,
                    0.000379536669,
                ],
                [
                    0.673531374655,
                    0.414574227779,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    -0.673531374655,
                    0.414574227779,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    0.673531374655,
                    -0.414574227779,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    -0.673531374655,
                    -0.414574227779,
                    0.611950730873,
                    0.000379536669,
                ],
                [
                    0.261973387012,
                    0.025400471864,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    -0.261973387012,
                    0.025400471864,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    0.261973387012,
                    -0.025400471864,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    -0.261973387012,
                    -0.025400471864,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    0.261973387012,
                    0.964740773745,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    -0.261973387012,
                    0.964740773745,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    0.261973387012,
                    -0.964740773745,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    -0.261973387012,
                    -0.964740773745,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    0.025400471864,
                    0.261973387012,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    -0.025400471864,
                    0.261973387012,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    0.025400471864,
                    -0.261973387012,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    -0.025400471864,
                    -0.261973387012,
                    0.964740773745,
                    0.000303465407,
                ],
                [
                    0.025400471864,
                    0.964740773745,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    -0.025400471864,
                    0.964740773745,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    0.025400471864,
                    -0.964740773745,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    -0.025400471864,
                    -0.964740773745,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    0.964740773745,
                    0.261973387012,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    -0.964740773745,
                    0.261973387012,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    0.964740773745,
                    -0.261973387012,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    -0.964740773745,
                    -0.261973387012,
                    0.025400471864,
                    0.000303465407,
                ],
                [
                    0.964740773745,
                    0.025400471864,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    -0.964740773745,
                    0.025400471864,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    0.964740773745,
                    -0.025400471864,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    -0.964740773745,
                    -0.025400471864,
                    0.261973387012,
                    0.000303465407,
                ],
                [
                    0.296814974324,
                    0.052081070185,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    -0.296814974324,
                    0.052081070185,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    0.296814974324,
                    -0.052081070185,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    -0.296814974324,
                    -0.052081070185,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    0.296814974324,
                    0.95351372992,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    -0.296814974324,
                    0.95351372992,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    0.296814974324,
                    -0.95351372992,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    -0.296814974324,
                    -0.95351372992,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    0.052081070185,
                    0.296814974324,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    -0.052081070185,
                    0.296814974324,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    0.052081070185,
                    -0.296814974324,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    -0.052081070185,
                    -0.296814974324,
                    0.95351372992,
                    0.000317548111,
                ],
                [
                    0.052081070185,
                    0.95351372992,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    -0.052081070185,
                    0.95351372992,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    0.052081070185,
                    -0.95351372992,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    -0.052081070185,
                    -0.95351372992,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    0.95351372992,
                    0.296814974324,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    -0.95351372992,
                    0.296814974324,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    0.95351372992,
                    -0.296814974324,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    -0.95351372992,
                    -0.296814974324,
                    0.052081070185,
                    0.000317548111,
                ],
                [
                    0.95351372992,
                    0.052081070185,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    -0.95351372992,
                    0.052081070185,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    0.95351372992,
                    -0.052081070185,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    -0.95351372992,
                    -0.052081070185,
                    0.296814974324,
                    0.000317548111,
                ],
                [
                    0.331045150486,
                    0.079718284709,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    -0.331045150486,
                    0.079718284709,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    0.331045150486,
                    -0.079718284709,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    -0.331045150486,
                    -0.079718284709,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    0.331045150486,
                    0.940241513348,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    -0.331045150486,
                    0.940241513348,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    0.331045150486,
                    -0.940241513348,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    -0.331045150486,
                    -0.940241513348,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    0.079718284709,
                    0.331045150486,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    -0.079718284709,
                    0.331045150486,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    0.079718284709,
                    -0.331045150486,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    -0.079718284709,
                    -0.331045150486,
                    0.940241513348,
                    0.000329818676,
                ],
                [
                    0.079718284709,
                    0.940241513348,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    -0.079718284709,
                    0.940241513348,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    0.079718284709,
                    -0.940241513348,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    -0.079718284709,
                    -0.940241513348,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    0.940241513348,
                    0.331045150486,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    -0.940241513348,
                    0.331045150486,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    0.940241513348,
                    -0.331045150486,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    -0.940241513348,
                    -0.331045150486,
                    0.079718284709,
                    0.000329818676,
                ],
                [
                    0.940241513348,
                    0.079718284709,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    -0.940241513348,
                    0.079718284709,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    0.940241513348,
                    -0.079718284709,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    -0.940241513348,
                    -0.079718284709,
                    0.331045150486,
                    0.000329818676,
                ],
                [
                    0.364621556738,
                    0.108046599918,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    -0.364621556738,
                    0.108046599918,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    0.364621556738,
                    -0.108046599918,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    -0.364621556738,
                    -0.108046599918,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    0.364621556738,
                    0.924865964672,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    -0.364621556738,
                    0.924865964672,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    0.364621556738,
                    -0.924865964672,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    -0.364621556738,
                    -0.924865964672,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    0.108046599918,
                    0.364621556738,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    -0.108046599918,
                    0.364621556738,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    0.108046599918,
                    -0.364621556738,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    -0.108046599918,
                    -0.364621556738,
                    0.924865964672,
                    0.000340383043,
                ],
                [
                    0.108046599918,
                    0.924865964672,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    -0.108046599918,
                    0.924865964672,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    0.108046599918,
                    -0.924865964672,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    -0.108046599918,
                    -0.924865964672,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    0.924865964672,
                    0.364621556738,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    -0.924865964672,
                    0.364621556738,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    0.924865964672,
                    -0.364621556738,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    -0.924865964672,
                    -0.364621556738,
                    0.108046599918,
                    0.000340383043,
                ],
                [
                    0.924865964672,
                    0.108046599918,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    -0.924865964672,
                    0.108046599918,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    0.924865964672,
                    -0.108046599918,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    -0.924865964672,
                    -0.108046599918,
                    0.364621556738,
                    0.000340383043,
                ],
                [
                    0.397491678528,
                    0.136841384937,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    -0.397491678528,
                    0.136841384937,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    0.397491678528,
                    -0.136841384937,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    -0.397491678528,
                    -0.136841384937,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    0.397491678528,
                    0.907344918358,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    -0.397491678528,
                    0.907344918358,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    0.397491678528,
                    -0.907344918358,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    -0.397491678528,
                    -0.907344918358,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    0.136841384937,
                    0.397491678528,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    -0.136841384937,
                    0.397491678528,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    0.136841384937,
                    -0.397491678528,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    -0.136841384937,
                    -0.397491678528,
                    0.907344918358,
                    0.000349369551,
                ],
                [
                    0.136841384937,
                    0.907344918358,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    -0.136841384937,
                    0.907344918358,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    0.136841384937,
                    -0.907344918358,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    -0.136841384937,
                    -0.907344918358,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    0.907344918358,
                    0.397491678528,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    -0.907344918358,
                    0.397491678528,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    0.907344918358,
                    -0.397491678528,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    -0.907344918358,
                    -0.397491678528,
                    0.136841384937,
                    0.000349369551,
                ],
                [
                    0.907344918358,
                    0.136841384937,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    -0.907344918358,
                    0.136841384937,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    0.907344918358,
                    -0.136841384937,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    -0.907344918358,
                    -0.136841384937,
                    0.397491678528,
                    0.000349369551,
                ],
                [
                    0.429596740377,
                    0.165907318476,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    -0.429596740377,
                    0.165907318476,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    0.429596740377,
                    -0.165907318476,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    -0.429596740377,
                    -0.165907318476,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    0.429596740377,
                    0.887649369027,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    -0.429596740377,
                    0.887649369027,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    0.429596740377,
                    -0.887649369027,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    -0.429596740377,
                    -0.887649369027,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    0.165907318476,
                    0.429596740377,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    -0.165907318476,
                    0.429596740377,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    0.165907318476,
                    -0.429596740377,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    -0.165907318476,
                    -0.429596740377,
                    0.887649369027,
                    0.000356911102,
                ],
                [
                    0.165907318476,
                    0.887649369027,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    -0.165907318476,
                    0.887649369027,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    0.165907318476,
                    -0.887649369027,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    -0.165907318476,
                    -0.887649369027,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    0.887649369027,
                    0.429596740377,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    -0.887649369027,
                    0.429596740377,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    0.887649369027,
                    -0.429596740377,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    -0.887649369027,
                    -0.429596740377,
                    0.165907318476,
                    0.000356911102,
                ],
                [
                    0.887649369027,
                    0.165907318476,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    -0.887649369027,
                    0.165907318476,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    0.887649369027,
                    -0.165907318476,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    -0.887649369027,
                    -0.165907318476,
                    0.429596740377,
                    0.000356911102,
                ],
                [
                    0.460874285447,
                    0.195070373045,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    -0.460874285447,
                    0.195070373045,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    0.460874285447,
                    -0.195070373045,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    -0.460874285447,
                    -0.195070373045,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    0.460874285447,
                    0.865761192578,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    -0.460874285447,
                    0.865761192578,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    0.460874285447,
                    -0.865761192578,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    -0.460874285447,
                    -0.865761192578,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    0.195070373045,
                    0.460874285447,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    -0.195070373045,
                    0.460874285447,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    0.195070373045,
                    -0.460874285447,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    -0.195070373045,
                    -0.460874285447,
                    0.865761192578,
                    0.000363137512,
                ],
                [
                    0.195070373045,
                    0.865761192578,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    -0.195070373045,
                    0.865761192578,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    0.195070373045,
                    -0.865761192578,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    -0.195070373045,
                    -0.865761192578,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    0.865761192578,
                    0.460874285447,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    -0.865761192578,
                    0.460874285447,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    0.865761192578,
                    -0.460874285447,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    -0.865761192578,
                    -0.460874285447,
                    0.195070373045,
                    0.000363137512,
                ],
                [
                    0.865761192578,
                    0.195070373045,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    -0.865761192578,
                    0.195070373045,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    0.865761192578,
                    -0.195070373045,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    -0.865761192578,
                    -0.195070373045,
                    0.460874285447,
                    0.000363137512,
                ],
                [
                    0.491259885895,
                    0.224172114438,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    -0.491259885895,
                    0.224172114438,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    0.491259885895,
                    -0.224172114438,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    -0.491259885895,
                    -0.224172114438,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    0.491259885895,
                    0.841671306164,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    -0.491259885895,
                    0.841671306164,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    0.491259885895,
                    -0.841671306164,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    -0.491259885895,
                    -0.841671306164,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    0.224172114438,
                    0.491259885895,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    -0.224172114438,
                    0.491259885895,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    0.224172114438,
                    -0.491259885895,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    -0.224172114438,
                    -0.491259885895,
                    0.841671306164,
                    0.000368172874,
                ],
                [
                    0.224172114438,
                    0.841671306164,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    -0.224172114438,
                    0.841671306164,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    0.224172114438,
                    -0.841671306164,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    -0.224172114438,
                    -0.841671306164,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    0.841671306164,
                    0.491259885895,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    -0.841671306164,
                    0.491259885895,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    0.841671306164,
                    -0.491259885895,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    -0.841671306164,
                    -0.491259885895,
                    0.224172114438,
                    0.000368172874,
                ],
                [
                    0.841671306164,
                    0.224172114438,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    -0.841671306164,
                    0.224172114438,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    0.841671306164,
                    -0.224172114438,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    -0.841671306164,
                    -0.224172114438,
                    0.491259885895,
                    0.000368172874,
                ],
                [
                    0.520688275895,
                    0.253065525541,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    -0.520688275895,
                    0.253065525541,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    0.520688275895,
                    -0.253065525541,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    -0.520688275895,
                    -0.253065525541,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    0.520688275895,
                    0.815378169397,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    -0.520688275895,
                    0.815378169397,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    0.520688275895,
                    -0.815378169397,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    -0.520688275895,
                    -0.815378169397,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    0.253065525541,
                    0.520688275895,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    -0.253065525541,
                    0.520688275895,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    0.253065525541,
                    -0.520688275895,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    -0.253065525541,
                    -0.520688275895,
                    0.815378169397,
                    0.000372135357,
                ],
                [
                    0.253065525541,
                    0.815378169397,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    -0.253065525541,
                    0.815378169397,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    0.253065525541,
                    -0.815378169397,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    -0.253065525541,
                    -0.815378169397,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    0.815378169397,
                    0.520688275895,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    -0.815378169397,
                    0.520688275895,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    0.815378169397,
                    -0.520688275895,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    -0.815378169397,
                    -0.520688275895,
                    0.253065525541,
                    0.000372135357,
                ],
                [
                    0.815378169397,
                    0.253065525541,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    -0.815378169397,
                    0.253065525541,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    0.815378169397,
                    -0.253065525541,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    -0.815378169397,
                    -0.253065525541,
                    0.520688275895,
                    0.000372135357,
                ],
                [
                    0.549094091402,
                    0.281611840973,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    -0.549094091402,
                    0.281611840973,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    0.549094091402,
                    -0.281611840973,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    -0.549094091402,
                    -0.281611840973,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    0.549094091402,
                    0.786886554601,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    -0.549094091402,
                    0.786886554601,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    0.549094091402,
                    -0.786886554601,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    -0.549094091402,
                    -0.786886554601,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    0.281611840973,
                    0.549094091402,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    -0.281611840973,
                    0.549094091402,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    0.281611840973,
                    -0.549094091402,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    -0.281611840973,
                    -0.549094091402,
                    0.786886554601,
                    0.000375138117,
                ],
                [
                    0.281611840973,
                    0.786886554601,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    -0.281611840973,
                    0.786886554601,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    0.281611840973,
                    -0.786886554601,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    -0.281611840973,
                    -0.786886554601,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    0.786886554601,
                    0.549094091402,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    -0.786886554601,
                    0.549094091402,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    0.786886554601,
                    -0.549094091402,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    -0.786886554601,
                    -0.549094091402,
                    0.281611840973,
                    0.000375138117,
                ],
                [
                    0.786886554601,
                    0.281611840973,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    -0.786886554601,
                    0.281611840973,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    0.786886554601,
                    -0.281611840973,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    -0.786886554601,
                    -0.281611840973,
                    0.549094091402,
                    0.000375138117,
                ],
                [
                    0.576412330203,
                    0.309678050459,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    -0.576412330203,
                    0.309678050459,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    0.576412330203,
                    -0.309678050459,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    -0.576412330203,
                    -0.309678050459,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    0.576412330203,
                    0.75620653968,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    -0.576412330203,
                    0.75620653968,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    0.576412330203,
                    -0.75620653968,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    -0.576412330203,
                    -0.75620653968,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    0.309678050459,
                    0.576412330203,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    -0.309678050459,
                    0.576412330203,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    0.309678050459,
                    -0.576412330203,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    -0.309678050459,
                    -0.576412330203,
                    0.75620653968,
                    0.000377290647,
                ],
                [
                    0.309678050459,
                    0.75620653968,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    -0.309678050459,
                    0.75620653968,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    0.309678050459,
                    -0.75620653968,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    -0.309678050459,
                    -0.75620653968,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    0.75620653968,
                    0.576412330203,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    -0.75620653968,
                    0.576412330203,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    0.75620653968,
                    -0.576412330203,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    -0.75620653968,
                    -0.576412330203,
                    0.309678050459,
                    0.000377290647,
                ],
                [
                    0.75620653968,
                    0.309678050459,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    -0.75620653968,
                    0.309678050459,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    0.75620653968,
                    -0.309678050459,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    -0.75620653968,
                    -0.309678050459,
                    0.576412330203,
                    0.000377290647,
                ],
                [
                    0.602578600421,
                    0.337134836639,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    -0.602578600421,
                    0.337134836639,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    0.602578600421,
                    -0.337134836639,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    -0.602578600421,
                    -0.337134836639,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    0.602578600421,
                    0.723352702517,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    -0.602578600421,
                    0.723352702517,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    0.602578600421,
                    -0.723352702517,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    -0.602578600421,
                    -0.723352702517,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    0.337134836639,
                    0.602578600421,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    -0.337134836639,
                    0.602578600421,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    0.337134836639,
                    -0.602578600421,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    -0.337134836639,
                    -0.602578600421,
                    0.723352702517,
                    0.000378700225,
                ],
                [
                    0.337134836639,
                    0.723352702517,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    -0.337134836639,
                    0.723352702517,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    0.337134836639,
                    -0.723352702517,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    -0.337134836639,
                    -0.723352702517,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    0.723352702517,
                    0.602578600421,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    -0.723352702517,
                    0.602578600421,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    0.723352702517,
                    -0.602578600421,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    -0.723352702517,
                    -0.602578600421,
                    0.337134836639,
                    0.000378700225,
                ],
                [
                    0.723352702517,
                    0.337134836639,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    -0.723352702517,
                    0.337134836639,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    0.723352702517,
                    -0.337134836639,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    -0.723352702517,
                    -0.337134836639,
                    0.602578600421,
                    0.000378700225,
                ],
                [
                    0.627529196479,
                    0.363854782769,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    -0.627529196479,
                    0.363854782769,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    0.627529196479,
                    -0.363854782769,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    -0.627529196479,
                    -0.363854782769,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    0.627529196479,
                    0.688343522249,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    -0.627529196479,
                    0.688343522249,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    0.627529196479,
                    -0.688343522249,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    -0.627529196479,
                    -0.688343522249,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    0.363854782769,
                    0.627529196479,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    -0.363854782769,
                    0.627529196479,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    0.363854782769,
                    -0.627529196479,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    -0.363854782769,
                    -0.627529196479,
                    0.688343522249,
                    0.000379473237,
                ],
                [
                    0.363854782769,
                    0.688343522249,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    -0.363854782769,
                    0.688343522249,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    0.363854782769,
                    -0.688343522249,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    -0.363854782769,
                    -0.688343522249,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    0.688343522249,
                    0.627529196479,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    -0.688343522249,
                    0.627529196479,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    0.688343522249,
                    -0.627529196479,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    -0.688343522249,
                    -0.627529196479,
                    0.363854782769,
                    0.000379473237,
                ],
                [
                    0.688343522249,
                    0.363854782769,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    -0.688343522249,
                    0.363854782769,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    0.688343522249,
                    -0.363854782769,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    -0.688343522249,
                    -0.363854782769,
                    0.627529196479,
                    0.000379473237,
                ],
                [
                    0.334818947986,
                    0.026648419355,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    -0.334818947986,
                    0.026648419355,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    0.334818947986,
                    -0.026648419355,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    -0.334818947986,
                    -0.026648419355,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    0.334818947986,
                    0.941905586466,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    -0.334818947986,
                    0.941905586466,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    0.334818947986,
                    -0.941905586466,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    -0.334818947986,
                    -0.941905586466,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    0.026648419355,
                    0.334818947986,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    -0.026648419355,
                    0.334818947986,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    0.026648419355,
                    -0.334818947986,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    -0.026648419355,
                    -0.334818947986,
                    0.941905586466,
                    0.000328781763,
                ],
                [
                    0.026648419355,
                    0.941905586466,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    -0.026648419355,
                    0.941905586466,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    0.026648419355,
                    -0.941905586466,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    -0.026648419355,
                    -0.941905586466,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    0.941905586466,
                    0.334818947986,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    -0.941905586466,
                    0.334818947986,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    0.941905586466,
                    -0.334818947986,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    -0.941905586466,
                    -0.334818947986,
                    0.026648419355,
                    0.000328781763,
                ],
                [
                    0.941905586466,
                    0.026648419355,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    -0.941905586466,
                    0.026648419355,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    0.941905586466,
                    -0.026648419355,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    -0.941905586466,
                    -0.026648419355,
                    0.334818947986,
                    0.000328781763,
                ],
                [
                    0.369951554586,
                    0.054240000668,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    -0.369951554586,
                    0.054240000668,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    0.369951554586,
                    -0.054240000668,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    -0.369951554586,
                    -0.054240000668,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    0.369951554586,
                    0.927466371135,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    -0.369951554586,
                    0.927466371135,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    0.369951554586,
                    -0.927466371135,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    -0.369951554586,
                    -0.927466371135,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    0.054240000668,
                    0.369951554586,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    -0.054240000668,
                    0.369951554586,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    0.054240000668,
                    -0.369951554586,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    -0.054240000668,
                    -0.369951554586,
                    0.927466371135,
                    0.00033926007,
                ],
                [
                    0.054240000668,
                    0.927466371135,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    -0.054240000668,
                    0.927466371135,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    0.054240000668,
                    -0.927466371135,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    -0.054240000668,
                    -0.927466371135,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    0.927466371135,
                    0.369951554586,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    -0.927466371135,
                    0.369951554586,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    0.927466371135,
                    -0.369951554586,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    -0.927466371135,
                    -0.369951554586,
                    0.054240000668,
                    0.00033926007,
                ],
                [
                    0.927466371135,
                    0.054240000668,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    -0.927466371135,
                    0.054240000668,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    0.927466371135,
                    -0.054240000668,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    -0.927466371135,
                    -0.054240000668,
                    0.369951554586,
                    0.00033926007,
                ],
                [
                    0.404200307147,
                    0.082519927154,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    -0.404200307147,
                    0.082519927154,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    0.404200307147,
                    -0.082519927154,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    -0.404200307147,
                    -0.082519927154,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    0.404200307147,
                    0.910940488355,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    -0.404200307147,
                    0.910940488355,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    0.404200307147,
                    -0.910940488355,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    -0.404200307147,
                    -0.910940488355,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    0.082519927154,
                    0.404200307147,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    -0.082519927154,
                    0.404200307147,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    0.082519927154,
                    -0.404200307147,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    -0.082519927154,
                    -0.404200307147,
                    0.910940488355,
                    0.000348310621,
                ],
                [
                    0.082519927154,
                    0.910940488355,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    -0.082519927154,
                    0.910940488355,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    0.082519927154,
                    -0.910940488355,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    -0.082519927154,
                    -0.910940488355,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    0.910940488355,
                    0.404200307147,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    -0.910940488355,
                    0.404200307147,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    0.910940488355,
                    -0.404200307147,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    -0.910940488355,
                    -0.404200307147,
                    0.082519927154,
                    0.000348310621,
                ],
                [
                    0.910940488355,
                    0.082519927154,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    -0.910940488355,
                    0.082519927154,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    0.910940488355,
                    -0.082519927154,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    -0.910940488355,
                    -0.082519927154,
                    0.404200307147,
                    0.000348310621,
                ],
                [
                    0.437532010018,
                    0.111269518248,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    -0.437532010018,
                    0.111269518248,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    0.437532010018,
                    -0.111269518248,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    -0.437532010018,
                    -0.111269518248,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    0.437532010018,
                    0.892291899839,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    -0.437532010018,
                    0.892291899839,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    0.437532010018,
                    -0.892291899839,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    -0.437532010018,
                    -0.892291899839,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    0.111269518248,
                    0.437532010018,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    -0.111269518248,
                    0.437532010018,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    0.111269518248,
                    -0.437532010018,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    -0.111269518248,
                    -0.437532010018,
                    0.892291899839,
                    0.000356003056,
                ],
                [
                    0.111269518248,
                    0.892291899839,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    -0.111269518248,
                    0.892291899839,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    0.111269518248,
                    -0.892291899839,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    -0.111269518248,
                    -0.892291899839,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    0.892291899839,
                    0.437532010018,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    -0.892291899839,
                    0.437532010018,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    0.892291899839,
                    -0.437532010018,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    -0.892291899839,
                    -0.437532010018,
                    0.111269518248,
                    0.000356003056,
                ],
                [
                    0.892291899839,
                    0.111269518248,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    -0.892291899839,
                    0.111269518248,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    0.892291899839,
                    -0.111269518248,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    -0.892291899839,
                    -0.111269518248,
                    0.437532010018,
                    0.000356003056,
                ],
                [
                    0.469905449034,
                    0.140296411647,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    -0.469905449034,
                    0.140296411647,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    0.469905449034,
                    -0.140296411647,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    -0.469905449034,
                    -0.140296411647,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    0.469905449034,
                    0.871496291356,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    -0.469905449034,
                    0.871496291356,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    0.469905449034,
                    -0.871496291356,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    -0.469905449034,
                    -0.871496291356,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    0.140296411647,
                    0.469905449034,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    -0.140296411647,
                    0.469905449034,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    0.140296411647,
                    -0.469905449034,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    -0.140296411647,
                    -0.469905449034,
                    0.871496291356,
                    0.000362423357,
                ],
                [
                    0.140296411647,
                    0.871496291356,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    -0.140296411647,
                    0.871496291356,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    0.140296411647,
                    -0.871496291356,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    -0.140296411647,
                    -0.871496291356,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    0.871496291356,
                    0.469905449034,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    -0.871496291356,
                    0.469905449034,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    0.871496291356,
                    -0.469905449034,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    -0.871496291356,
                    -0.469905449034,
                    0.140296411647,
                    0.000362423357,
                ],
                [
                    0.871496291356,
                    0.140296411647,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    -0.871496291356,
                    0.140296411647,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    0.871496291356,
                    -0.140296411647,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    -0.871496291356,
                    -0.140296411647,
                    0.469905449034,
                    0.000362423357,
                ],
                [
                    0.501273987943,
                    0.169427511758,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    -0.501273987943,
                    0.169427511758,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    0.501273987943,
                    -0.169427511758,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    -0.501273987943,
                    -0.169427511758,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    0.501273987943,
                    0.848539160717,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    -0.501273987943,
                    0.848539160717,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    0.501273987943,
                    -0.848539160717,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    -0.501273987943,
                    -0.848539160717,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    0.169427511758,
                    0.501273987943,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    -0.169427511758,
                    0.501273987943,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    0.169427511758,
                    -0.501273987943,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    -0.169427511758,
                    -0.501273987943,
                    0.848539160717,
                    0.000367664632,
                ],
                [
                    0.169427511758,
                    0.848539160717,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    -0.169427511758,
                    0.848539160717,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    0.169427511758,
                    -0.848539160717,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    -0.169427511758,
                    -0.848539160717,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    0.848539160717,
                    0.501273987943,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    -0.848539160717,
                    0.501273987943,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    0.848539160717,
                    -0.501273987943,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    -0.848539160717,
                    -0.501273987943,
                    0.169427511758,
                    0.000367664632,
                ],
                [
                    0.848539160717,
                    0.169427511758,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    -0.848539160717,
                    0.169427511758,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    0.848539160717,
                    -0.169427511758,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    -0.848539160717,
                    -0.169427511758,
                    0.501273987943,
                    0.000367664632,
                ],
                [
                    0.531587488375,
                    0.198503823531,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    -0.531587488375,
                    0.198503823531,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    0.531587488375,
                    -0.198503823531,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    -0.531587488375,
                    -0.198503823531,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    0.531587488375,
                    0.823414217904,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    -0.531587488375,
                    0.823414217904,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    0.531587488375,
                    -0.823414217904,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    -0.531587488375,
                    -0.823414217904,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    0.198503823531,
                    0.531587488375,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    -0.198503823531,
                    0.531587488375,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    0.198503823531,
                    -0.531587488375,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    -0.198503823531,
                    -0.531587488375,
                    0.823414217904,
                    0.000371822624,
                ],
                [
                    0.198503823531,
                    0.823414217904,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    -0.198503823531,
                    0.823414217904,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    0.198503823531,
                    -0.823414217904,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    -0.198503823531,
                    -0.823414217904,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    0.823414217904,
                    0.531587488375,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    -0.823414217904,
                    0.531587488375,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    0.823414217904,
                    -0.531587488375,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    -0.823414217904,
                    -0.531587488375,
                    0.198503823531,
                    0.000371822624,
                ],
                [
                    0.823414217904,
                    0.198503823531,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    -0.823414217904,
                    0.198503823531,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    0.823414217904,
                    -0.198503823531,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    -0.823414217904,
                    -0.198503823531,
                    0.531587488375,
                    0.000371822624,
                ],
                [
                    0.560793710962,
                    0.227376566002,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    -0.560793710962,
                    0.227376566002,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    0.560793710962,
                    -0.227376566002,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    -0.560793710962,
                    -0.227376566002,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    0.560793710962,
                    0.796122045278,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    -0.560793710962,
                    0.796122045278,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    0.560793710962,
                    -0.796122045278,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    -0.560793710962,
                    -0.796122045278,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    0.227376566002,
                    0.560793710962,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    -0.227376566002,
                    0.560793710962,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    0.227376566002,
                    -0.560793710962,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    -0.227376566002,
                    -0.560793710962,
                    0.796122045278,
                    0.000374993844,
                ],
                [
                    0.227376566002,
                    0.796122045278,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    -0.227376566002,
                    0.796122045278,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    0.227376566002,
                    -0.796122045278,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    -0.227376566002,
                    -0.796122045278,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    0.796122045278,
                    0.560793710962,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    -0.796122045278,
                    0.560793710962,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    0.796122045278,
                    -0.560793710962,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    -0.796122045278,
                    -0.560793710962,
                    0.227376566002,
                    0.000374993844,
                ],
                [
                    0.796122045278,
                    0.227376566002,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    -0.796122045278,
                    0.227376566002,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    0.796122045278,
                    -0.227376566002,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    -0.796122045278,
                    -0.227376566002,
                    0.560793710962,
                    0.000374993844,
                ],
                [
                    0.58883932235,
                    0.255904149285,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    -0.58883932235,
                    0.255904149285,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    0.58883932235,
                    -0.255904149285,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    -0.58883932235,
                    -0.255904149285,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    0.58883932235,
                    0.766668976047,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    -0.58883932235,
                    0.766668976047,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    0.58883932235,
                    -0.766668976047,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    -0.58883932235,
                    -0.766668976047,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    0.255904149285,
                    0.58883932235,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    -0.255904149285,
                    0.58883932235,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    0.255904149285,
                    -0.58883932235,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    -0.255904149285,
                    -0.58883932235,
                    0.766668976047,
                    0.000377275123,
                ],
                [
                    0.255904149285,
                    0.766668976047,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    -0.255904149285,
                    0.766668976047,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    0.255904149285,
                    -0.766668976047,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    -0.255904149285,
                    -0.766668976047,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    0.766668976047,
                    0.58883932235,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    -0.766668976047,
                    0.58883932235,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    0.766668976047,
                    -0.58883932235,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    -0.766668976047,
                    -0.58883932235,
                    0.255904149285,
                    0.000377275123,
                ],
                [
                    0.766668976047,
                    0.255904149285,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    -0.766668976047,
                    0.255904149285,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    0.766668976047,
                    -0.255904149285,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    -0.766668976047,
                    -0.255904149285,
                    0.58883932235,
                    0.000377275123,
                ],
                [
                    0.615670597916,
                    0.283949725198,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    -0.615670597916,
                    0.283949725198,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    0.615670597916,
                    -0.283949725198,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    -0.615670597916,
                    -0.283949725198,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    0.615670597916,
                    0.735066166016,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    -0.615670597916,
                    0.735066166016,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    0.615670597916,
                    -0.735066166016,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    -0.615670597916,
                    -0.735066166016,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    0.283949725198,
                    0.615670597916,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    -0.283949725198,
                    0.615670597916,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    0.283949725198,
                    -0.615670597916,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    -0.283949725198,
                    -0.615670597916,
                    0.735066166016,
                    0.000378763915,
                ],
                [
                    0.283949725198,
                    0.735066166016,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    -0.283949725198,
                    0.735066166016,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    0.283949725198,
                    -0.735066166016,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    -0.283949725198,
                    -0.735066166016,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    0.735066166016,
                    0.615670597916,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    -0.735066166016,
                    0.615670597916,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    0.735066166016,
                    -0.615670597916,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    -0.735066166016,
                    -0.615670597916,
                    0.283949725198,
                    0.000378763915,
                ],
                [
                    0.735066166016,
                    0.283949725198,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    -0.735066166016,
                    0.283949725198,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    0.735066166016,
                    -0.283949725198,
                    0.615670597916,
                    0.000378763915,
                ],
                [
                    -0.735066166016,
                    -0.283949725198,
                    0.615670597916,
                    0.000378763915,
                ],
                [0.641233880908, 0.31137910605, 0.701328854598, 0.00037955895],
                [
                    -0.641233880908,
                    0.31137910605,
                    0.701328854598,
                    0.00037955895,
                ],
                [
                    0.641233880908,
                    -0.31137910605,
                    0.701328854598,
                    0.00037955895,
                ],
                [
                    -0.641233880908,
                    -0.31137910605,
                    0.701328854598,
                    0.00037955895,
                ],
                [0.641233880908, 0.701328854598, 0.31137910605, 0.00037955895],
                [
                    -0.641233880908,
                    0.701328854598,
                    0.31137910605,
                    0.00037955895,
                ],
                [
                    0.641233880908,
                    -0.701328854598,
                    0.31137910605,
                    0.00037955895,
                ],
                [
                    -0.641233880908,
                    -0.701328854598,
                    0.31137910605,
                    0.00037955895,
                ],
                [0.31137910605, 0.641233880908, 0.701328854598, 0.00037955895],
                [
                    -0.31137910605,
                    0.641233880908,
                    0.701328854598,
                    0.00037955895,
                ],
                [
                    0.31137910605,
                    -0.641233880908,
                    0.701328854598,
                    0.00037955895,
                ],
                [
                    -0.31137910605,
                    -0.641233880908,
                    0.701328854598,
                    0.00037955895,
                ],
                [0.31137910605, 0.701328854598, 0.641233880908, 0.00037955895],
                [
                    -0.31137910605,
                    0.701328854598,
                    0.641233880908,
                    0.00037955895,
                ],
                [
                    0.31137910605,
                    -0.701328854598,
                    0.641233880908,
                    0.00037955895,
                ],
                [
                    -0.31137910605,
                    -0.701328854598,
                    0.641233880908,
                    0.00037955895,
                ],
                [0.701328854598, 0.641233880908, 0.31137910605, 0.00037955895],
                [
                    -0.701328854598,
                    0.641233880908,
                    0.31137910605,
                    0.00037955895,
                ],
                [
                    0.701328854598,
                    -0.641233880908,
                    0.31137910605,
                    0.00037955895,
                ],
                [
                    -0.701328854598,
                    -0.641233880908,
                    0.31137910605,
                    0.00037955895,
                ],
                [0.701328854598, 0.31137910605, 0.641233880908, 0.00037955895],
                [
                    -0.701328854598,
                    0.31137910605,
                    0.641233880908,
                    0.00037955895,
                ],
                [
                    0.701328854598,
                    -0.31137910605,
                    0.641233880908,
                    0.00037955895,
                ],
                [
                    -0.701328854598,
                    -0.31137910605,
                    0.641233880908,
                    0.00037955895,
                ],
                [
                    0.407605125926,
                    0.027577922909,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    -0.407605125926,
                    0.027577922909,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    0.407605125926,
                    -0.027577922909,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    -0.407605125926,
                    -0.027577922909,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    0.407605125926,
                    0.912741759474,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    -0.407605125926,
                    0.912741759474,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    0.407605125926,
                    -0.912741759474,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    -0.407605125926,
                    -0.912741759474,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    0.027577922909,
                    0.407605125926,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    -0.027577922909,
                    0.407605125926,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    0.027577922909,
                    -0.407605125926,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    -0.027577922909,
                    -0.407605125926,
                    0.912741759474,
                    0.000347792785,
                ],
                [
                    0.027577922909,
                    0.912741759474,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    -0.027577922909,
                    0.912741759474,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    0.027577922909,
                    -0.912741759474,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    -0.027577922909,
                    -0.912741759474,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    0.912741759474,
                    0.407605125926,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    -0.912741759474,
                    0.407605125926,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    0.912741759474,
                    -0.407605125926,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    -0.912741759474,
                    -0.407605125926,
                    0.027577922909,
                    0.000347792785,
                ],
                [
                    0.912741759474,
                    0.027577922909,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    -0.912741759474,
                    0.027577922909,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    0.912741759474,
                    -0.027577922909,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    -0.912741759474,
                    -0.027577922909,
                    0.407605125926,
                    0.000347792785,
                ],
                [
                    0.442378812579,
                    0.05584136835,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    -0.442378812579,
                    0.05584136835,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    0.442378812579,
                    -0.05584136835,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    -0.442378812579,
                    -0.05584136835,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    0.442378812579,
                    0.895088111731,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    -0.442378812579,
                    0.895088111731,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    0.442378812579,
                    -0.895088111731,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    -0.442378812579,
                    -0.895088111731,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    0.05584136835,
                    0.442378812579,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    -0.05584136835,
                    0.442378812579,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    0.05584136835,
                    -0.442378812579,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    -0.05584136835,
                    -0.442378812579,
                    0.895088111731,
                    0.000355488472,
                ],
                [
                    0.05584136835,
                    0.895088111731,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    -0.05584136835,
                    0.895088111731,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    0.05584136835,
                    -0.895088111731,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    -0.05584136835,
                    -0.895088111731,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    0.895088111731,
                    0.442378812579,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    -0.895088111731,
                    0.442378812579,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    0.895088111731,
                    -0.442378812579,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    -0.895088111731,
                    -0.442378812579,
                    0.05584136835,
                    0.000355488472,
                ],
                [
                    0.895088111731,
                    0.05584136835,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    -0.895088111731,
                    0.05584136835,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    0.895088111731,
                    -0.05584136835,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    -0.895088111731,
                    -0.05584136835,
                    0.442378812579,
                    0.000355488472,
                ],
                [
                    0.476048091733,
                    0.084577720877,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    -0.476048091733,
                    0.084577720877,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    0.476048091733,
                    -0.084577720877,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    -0.476048091733,
                    -0.084577720877,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    0.476048091733,
                    0.875342689173,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    -0.476048091733,
                    0.875342689173,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    0.476048091733,
                    -0.875342689173,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    -0.476048091733,
                    -0.875342689173,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    0.084577720877,
                    0.476048091733,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    -0.084577720877,
                    0.476048091733,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    0.084577720877,
                    -0.476048091733,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    -0.084577720877,
                    -0.476048091733,
                    0.875342689173,
                    0.000362002163,
                ],
                [
                    0.084577720877,
                    0.875342689173,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    -0.084577720877,
                    0.875342689173,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    0.084577720877,
                    -0.875342689173,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    -0.084577720877,
                    -0.875342689173,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    0.875342689173,
                    0.476048091733,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    -0.875342689173,
                    0.476048091733,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    0.875342689173,
                    -0.476048091733,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    -0.875342689173,
                    -0.476048091733,
                    0.084577720877,
                    0.000362002163,
                ],
                [
                    0.875342689173,
                    0.084577720877,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    -0.875342689173,
                    0.084577720877,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    0.875342689173,
                    -0.084577720877,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    -0.875342689173,
                    -0.084577720877,
                    0.476048091733,
                    0.000362002163,
                ],
                [
                    0.508583872595,
                    0.113597584636,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    -0.508583872595,
                    0.113597584636,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    0.508583872595,
                    -0.113597584636,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    -0.508583872595,
                    -0.113597584636,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    0.508583872595,
                    0.853485813181,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    -0.508583872595,
                    0.853485813181,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    0.508583872595,
                    -0.853485813181,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    -0.508583872595,
                    -0.853485813181,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    0.113597584636,
                    0.508583872595,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    -0.113597584636,
                    0.508583872595,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    0.113597584636,
                    -0.508583872595,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    -0.113597584636,
                    -0.508583872595,
                    0.853485813181,
                    0.000367384064,
                ],
                [
                    0.113597584636,
                    0.853485813181,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    -0.113597584636,
                    0.853485813181,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    0.113597584636,
                    -0.853485813181,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    -0.113597584636,
                    -0.853485813181,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    0.853485813181,
                    0.508583872595,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    -0.853485813181,
                    0.508583872595,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    0.853485813181,
                    -0.508583872595,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    -0.853485813181,
                    -0.508583872595,
                    0.113597584636,
                    0.000367384064,
                ],
                [
                    0.853485813181,
                    0.113597584636,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    -0.853485813181,
                    0.113597584636,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    0.853485813181,
                    -0.113597584636,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    -0.853485813181,
                    -0.113597584636,
                    0.508583872595,
                    0.000367384064,
                ],
                [
                    0.539951363739,
                    0.142728690477,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    -0.539951363739,
                    0.142728690477,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    0.539951363739,
                    -0.142728690477,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    -0.539951363739,
                    -0.142728690477,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    0.539951363739,
                    0.829506507335,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    -0.539951363739,
                    0.829506507335,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    0.539951363739,
                    -0.829506507335,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    -0.539951363739,
                    -0.829506507335,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    0.142728690477,
                    0.539951363739,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    -0.142728690477,
                    0.539951363739,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    0.142728690477,
                    -0.539951363739,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    -0.142728690477,
                    -0.539951363739,
                    0.829506507335,
                    0.000371697895,
                ],
                [
                    0.142728690477,
                    0.829506507335,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    -0.142728690477,
                    0.829506507335,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    0.142728690477,
                    -0.829506507335,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    -0.142728690477,
                    -0.829506507335,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    0.829506507335,
                    0.539951363739,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    -0.829506507335,
                    0.539951363739,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    0.829506507335,
                    -0.539951363739,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    -0.829506507335,
                    -0.539951363739,
                    0.142728690477,
                    0.000371697895,
                ],
                [
                    0.829506507335,
                    0.142728690477,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    -0.829506507335,
                    0.142728690477,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    0.829506507335,
                    -0.142728690477,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    -0.829506507335,
                    -0.142728690477,
                    0.539951363739,
                    0.000371697895,
                ],
                [
                    0.570111843364,
                    0.171811274006,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    -0.570111843364,
                    0.171811274006,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    0.570111843364,
                    -0.171811274006,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    -0.570111843364,
                    -0.171811274006,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    0.570111843364,
                    0.803401127819,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    -0.570111843364,
                    0.803401127819,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    0.570111843364,
                    -0.803401127819,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    -0.570111843364,
                    -0.803401127819,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    0.171811274006,
                    0.570111843364,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    -0.171811274006,
                    0.570111843364,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    0.171811274006,
                    -0.570111843364,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    -0.171811274006,
                    -0.570111843364,
                    0.803401127819,
                    0.000375015868,
                ],
                [
                    0.171811274006,
                    0.803401127819,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    -0.171811274006,
                    0.803401127819,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    0.171811274006,
                    -0.803401127819,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    -0.171811274006,
                    -0.803401127819,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    0.803401127819,
                    0.570111843364,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    -0.803401127819,
                    0.570111843364,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    0.803401127819,
                    -0.570111843364,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    -0.803401127819,
                    -0.570111843364,
                    0.171811274006,
                    0.000375015868,
                ],
                [
                    0.803401127819,
                    0.171811274006,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    -0.803401127819,
                    0.171811274006,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    0.803401127819,
                    -0.171811274006,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    -0.803401127819,
                    -0.171811274006,
                    0.570111843364,
                    0.000375015868,
                ],
                [
                    0.599024053061,
                    0.200694485599,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    -0.599024053061,
                    0.200694485599,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    0.599024053061,
                    -0.200694485599,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    -0.599024053061,
                    -0.200694485599,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    0.599024053061,
                    0.775172179135,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    -0.599024053061,
                    0.775172179135,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    0.599024053061,
                    -0.775172179135,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    -0.599024053061,
                    -0.775172179135,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    0.200694485599,
                    0.599024053061,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    -0.200694485599,
                    0.599024053061,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    0.200694485599,
                    -0.599024053061,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    -0.200694485599,
                    -0.599024053061,
                    0.775172179135,
                    0.000377416048,
                ],
                [
                    0.200694485599,
                    0.775172179135,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    -0.200694485599,
                    0.775172179135,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    0.200694485599,
                    -0.775172179135,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    -0.200694485599,
                    -0.775172179135,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    0.775172179135,
                    0.599024053061,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    -0.775172179135,
                    0.599024053061,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    0.775172179135,
                    -0.599024053061,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    -0.775172179135,
                    -0.599024053061,
                    0.200694485599,
                    0.000377416048,
                ],
                [
                    0.775172179135,
                    0.200694485599,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    -0.775172179135,
                    0.200694485599,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    0.775172179135,
                    -0.200694485599,
                    0.599024053061,
                    0.000377416048,
                ],
                [
                    -0.775172179135,
                    -0.200694485599,
                    0.599024053061,
                    0.000377416048,
                ],
                [0.626645268514, 0.22923350906, 0.744827299294, 0.00037898115],
                [
                    -0.626645268514,
                    0.22923350906,
                    0.744827299294,
                    0.00037898115,
                ],
                [
                    0.626645268514,
                    -0.22923350906,
                    0.744827299294,
                    0.00037898115,
                ],
                [
                    -0.626645268514,
                    -0.22923350906,
                    0.744827299294,
                    0.00037898115,
                ],
                [0.626645268514, 0.744827299294, 0.22923350906, 0.00037898115],
                [
                    -0.626645268514,
                    0.744827299294,
                    0.22923350906,
                    0.00037898115,
                ],
                [
                    0.626645268514,
                    -0.744827299294,
                    0.22923350906,
                    0.00037898115,
                ],
                [
                    -0.626645268514,
                    -0.744827299294,
                    0.22923350906,
                    0.00037898115,
                ],
                [0.22923350906, 0.626645268514, 0.744827299294, 0.00037898115],
                [
                    -0.22923350906,
                    0.626645268514,
                    0.744827299294,
                    0.00037898115,
                ],
                [
                    0.22923350906,
                    -0.626645268514,
                    0.744827299294,
                    0.00037898115,
                ],
                [
                    -0.22923350906,
                    -0.626645268514,
                    0.744827299294,
                    0.00037898115,
                ],
                [0.22923350906, 0.744827299294, 0.626645268514, 0.00037898115],
                [
                    -0.22923350906,
                    0.744827299294,
                    0.626645268514,
                    0.00037898115,
                ],
                [
                    0.22923350906,
                    -0.744827299294,
                    0.626645268514,
                    0.00037898115,
                ],
                [
                    -0.22923350906,
                    -0.744827299294,
                    0.626645268514,
                    0.00037898115,
                ],
                [0.744827299294, 0.626645268514, 0.22923350906, 0.00037898115],
                [
                    -0.744827299294,
                    0.626645268514,
                    0.22923350906,
                    0.00037898115,
                ],
                [
                    0.744827299294,
                    -0.626645268514,
                    0.22923350906,
                    0.00037898115,
                ],
                [
                    -0.744827299294,
                    -0.626645268514,
                    0.22923350906,
                    0.00037898115,
                ],
                [0.744827299294, 0.22923350906, 0.626645268514, 0.00037898115],
                [
                    -0.744827299294,
                    0.22923350906,
                    0.626645268514,
                    0.00037898115,
                ],
                [
                    0.744827299294,
                    -0.22923350906,
                    0.626645268514,
                    0.00037898115,
                ],
                [
                    -0.744827299294,
                    -0.22923350906,
                    0.626645268514,
                    0.00037898115,
                ],
                [
                    0.652932097142,
                    0.257287151235,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    -0.652932097142,
                    0.257287151235,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    0.652932097142,
                    -0.257287151235,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    -0.652932097142,
                    -0.257287151235,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    0.652932097142,
                    0.712378409507,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    -0.652932097142,
                    0.712378409507,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    0.652932097142,
                    -0.712378409507,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    -0.652932097142,
                    -0.712378409507,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    0.257287151235,
                    0.652932097142,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    -0.257287151235,
                    0.652932097142,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    0.257287151235,
                    -0.652932097142,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    -0.257287151235,
                    -0.652932097142,
                    0.712378409507,
                    0.000379798212,
                ],
                [
                    0.257287151235,
                    0.712378409507,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    -0.257287151235,
                    0.712378409507,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    0.257287151235,
                    -0.712378409507,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    -0.257287151235,
                    -0.712378409507,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    0.712378409507,
                    0.652932097142,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    -0.712378409507,
                    0.652932097142,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    0.712378409507,
                    -0.652932097142,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    -0.712378409507,
                    -0.652932097142,
                    0.257287151235,
                    0.000379798212,
                ],
                [
                    0.712378409507,
                    0.257287151235,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    -0.712378409507,
                    0.257287151235,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    0.712378409507,
                    -0.257287151235,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    -0.712378409507,
                    -0.257287151235,
                    0.652932097142,
                    0.000379798212,
                ],
                [
                    0.479158383461,
                    0.028260941977,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    -0.479158383461,
                    0.028260941977,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    0.479158383461,
                    -0.028260941977,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    -0.479158383461,
                    -0.028260941977,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    0.479158383461,
                    0.877273368294,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    -0.479158383461,
                    0.877273368294,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    0.479158383461,
                    -0.877273368294,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    -0.479158383461,
                    -0.877273368294,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    0.028260941977,
                    0.479158383461,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    -0.028260941977,
                    0.479158383461,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    0.028260941977,
                    -0.479158383461,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    -0.028260941977,
                    -0.479158383461,
                    0.877273368294,
                    0.000361813003,
                ],
                [
                    0.028260941977,
                    0.877273368294,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    -0.028260941977,
                    0.877273368294,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    0.028260941977,
                    -0.877273368294,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    -0.028260941977,
                    -0.877273368294,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    0.877273368294,
                    0.479158383461,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    -0.877273368294,
                    0.479158383461,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    0.877273368294,
                    -0.479158383461,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    -0.877273368294,
                    -0.479158383461,
                    0.028260941977,
                    0.000361813003,
                ],
                [
                    0.877273368294,
                    0.028260941977,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    -0.877273368294,
                    0.028260941977,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    0.877273368294,
                    -0.028260941977,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    -0.877273368294,
                    -0.028260941977,
                    0.479158383461,
                    0.000361813003,
                ],
                [
                    0.51303739528,
                    0.056998713597,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    -0.51303739528,
                    0.056998713597,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    0.51303739528,
                    -0.056998713597,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    -0.51303739528,
                    -0.056998713597,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    0.51303739528,
                    0.856471702798,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    -0.51303739528,
                    0.856471702798,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    0.51303739528,
                    -0.856471702798,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    -0.51303739528,
                    -0.856471702798,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    0.056998713597,
                    0.51303739528,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    -0.056998713597,
                    0.51303739528,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    0.056998713597,
                    -0.51303739528,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    -0.056998713597,
                    -0.51303739528,
                    0.856471702798,
                    0.000367259424,
                ],
                [
                    0.056998713597,
                    0.856471702798,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    -0.056998713597,
                    0.856471702798,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    0.056998713597,
                    -0.856471702798,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    -0.056998713597,
                    -0.856471702798,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    0.856471702798,
                    0.51303739528,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    -0.856471702798,
                    0.51303739528,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    0.856471702798,
                    -0.51303739528,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    -0.856471702798,
                    -0.51303739528,
                    0.056998713597,
                    0.000367259424,
                ],
                [
                    0.856471702798,
                    0.056998713597,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    -0.856471702798,
                    0.056998713597,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    0.856471702798,
                    -0.056998713597,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    -0.856471702798,
                    -0.056998713597,
                    0.51303739528,
                    0.000367259424,
                ],
                [
                    0.545625242963,
                    0.086027125286,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    -0.545625242963,
                    0.086027125286,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    0.545625242963,
                    -0.086027125286,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    -0.545625242963,
                    -0.086027125286,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    0.545625242963,
                    0.833602080106,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    -0.545625242963,
                    0.833602080106,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    0.545625242963,
                    -0.833602080106,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    -0.545625242963,
                    -0.833602080106,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    0.086027125286,
                    0.545625242963,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    -0.086027125286,
                    0.545625242963,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    0.086027125286,
                    -0.545625242963,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    -0.086027125286,
                    -0.545625242963,
                    0.833602080106,
                    0.000371685383,
                ],
                [
                    0.086027125286,
                    0.833602080106,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    -0.086027125286,
                    0.833602080106,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    0.086027125286,
                    -0.833602080106,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    -0.086027125286,
                    -0.833602080106,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    0.833602080106,
                    0.545625242963,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    -0.833602080106,
                    0.545625242963,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    0.833602080106,
                    -0.545625242963,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    -0.833602080106,
                    -0.545625242963,
                    0.086027125286,
                    0.000371685383,
                ],
                [
                    0.833602080106,
                    0.086027125286,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    -0.833602080106,
                    0.086027125286,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    0.833602080106,
                    -0.086027125286,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    -0.833602080106,
                    -0.086027125286,
                    0.545625242963,
                    0.000371685383,
                ],
                [
                    0.576895632968,
                    0.115174813722,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    -0.576895632968,
                    0.115174813722,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    0.576895632968,
                    -0.115174813722,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    -0.576895632968,
                    -0.115174813722,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    0.576895632968,
                    0.808657029244,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    -0.576895632968,
                    0.808657029244,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    0.576895632968,
                    -0.808657029244,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    -0.576895632968,
                    -0.808657029244,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    0.115174813722,
                    0.576895632968,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    -0.115174813722,
                    0.576895632968,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    0.115174813722,
                    -0.576895632968,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    -0.115174813722,
                    -0.576895632968,
                    0.808657029244,
                    0.00037513082,
                ],
                [
                    0.115174813722,
                    0.808657029244,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    -0.115174813722,
                    0.808657029244,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    0.115174813722,
                    -0.808657029244,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    -0.115174813722,
                    -0.808657029244,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    0.808657029244,
                    0.576895632968,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    -0.808657029244,
                    0.576895632968,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    0.808657029244,
                    -0.576895632968,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    -0.808657029244,
                    -0.576895632968,
                    0.115174813722,
                    0.00037513082,
                ],
                [
                    0.808657029244,
                    0.115174813722,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    -0.808657029244,
                    0.115174813722,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    0.808657029244,
                    -0.115174813722,
                    0.576895632968,
                    0.00037513082,
                ],
                [
                    -0.808657029244,
                    -0.115174813722,
                    0.576895632968,
                    0.00037513082,
                ],
                [0.60681869447, 0.144281165414, 0.781635476004, 0.00037764815],
                [
                    -0.60681869447,
                    0.144281165414,
                    0.781635476004,
                    0.00037764815,
                ],
                [
                    0.60681869447,
                    -0.144281165414,
                    0.781635476004,
                    0.00037764815,
                ],
                [
                    -0.60681869447,
                    -0.144281165414,
                    0.781635476004,
                    0.00037764815,
                ],
                [0.60681869447, 0.781635476004, 0.144281165414, 0.00037764815],
                [
                    -0.60681869447,
                    0.781635476004,
                    0.144281165414,
                    0.00037764815,
                ],
                [
                    0.60681869447,
                    -0.781635476004,
                    0.144281165414,
                    0.00037764815,
                ],
                [
                    -0.60681869447,
                    -0.781635476004,
                    0.144281165414,
                    0.00037764815,
                ],
                [0.144281165414, 0.60681869447, 0.781635476004, 0.00037764815],
                [
                    -0.144281165414,
                    0.60681869447,
                    0.781635476004,
                    0.00037764815,
                ],
                [
                    0.144281165414,
                    -0.60681869447,
                    0.781635476004,
                    0.00037764815,
                ],
                [
                    -0.144281165414,
                    -0.60681869447,
                    0.781635476004,
                    0.00037764815,
                ],
                [0.144281165414, 0.781635476004, 0.60681869447, 0.00037764815],
                [
                    -0.144281165414,
                    0.781635476004,
                    0.60681869447,
                    0.00037764815,
                ],
                [
                    0.144281165414,
                    -0.781635476004,
                    0.60681869447,
                    0.00037764815,
                ],
                [
                    -0.144281165414,
                    -0.781635476004,
                    0.60681869447,
                    0.00037764815,
                ],
                [0.781635476004, 0.60681869447, 0.144281165414, 0.00037764815],
                [
                    -0.781635476004,
                    0.60681869447,
                    0.144281165414,
                    0.00037764815,
                ],
                [
                    0.781635476004,
                    -0.60681869447,
                    0.144281165414,
                    0.00037764815,
                ],
                [
                    -0.781635476004,
                    -0.60681869447,
                    0.144281165414,
                    0.00037764815,
                ],
                [0.781635476004, 0.144281165414, 0.60681869447, 0.00037764815],
                [
                    -0.781635476004,
                    0.144281165414,
                    0.60681869447,
                    0.00037764815,
                ],
                [
                    0.781635476004,
                    -0.144281165414,
                    0.60681869447,
                    0.00037764815,
                ],
                [
                    -0.781635476004,
                    -0.144281165414,
                    0.60681869447,
                    0.00037764815,
                ],
                [
                    0.635362224802,
                    0.173193032166,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    -0.635362224802,
                    0.173193032166,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    0.635362224802,
                    -0.173193032166,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    -0.635362224802,
                    -0.173193032166,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    0.635362224802,
                    0.752541704428,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    -0.635362224802,
                    0.752541704428,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    0.635362224802,
                    -0.752541704428,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    -0.635362224802,
                    -0.752541704428,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    0.173193032166,
                    0.635362224802,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    -0.173193032166,
                    0.635362224802,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    0.173193032166,
                    -0.635362224802,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    -0.173193032166,
                    -0.635362224802,
                    0.752541704428,
                    0.000379299477,
                ],
                [
                    0.173193032166,
                    0.752541704428,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    -0.173193032166,
                    0.752541704428,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    0.173193032166,
                    -0.752541704428,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    -0.173193032166,
                    -0.752541704428,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    0.752541704428,
                    0.635362224802,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    -0.752541704428,
                    0.635362224802,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    0.752541704428,
                    -0.635362224802,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    -0.752541704428,
                    -0.635362224802,
                    0.173193032166,
                    0.000379299477,
                ],
                [
                    0.752541704428,
                    0.173193032166,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    -0.752541704428,
                    0.173193032166,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    0.752541704428,
                    -0.173193032166,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    -0.752541704428,
                    -0.173193032166,
                    0.635362224802,
                    0.000379299477,
                ],
                [
                    0.662492703573,
                    0.201761995876,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    -0.662492703573,
                    0.201761995876,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    0.662492703573,
                    -0.201761995876,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    -0.662492703573,
                    -0.201761995876,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    0.662492703573,
                    0.72138444309,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    -0.662492703573,
                    0.72138444309,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    0.662492703573,
                    -0.72138444309,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    -0.662492703573,
                    -0.72138444309,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    0.201761995876,
                    0.662492703573,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    -0.201761995876,
                    0.662492703573,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    0.201761995876,
                    -0.662492703573,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    -0.201761995876,
                    -0.662492703573,
                    0.72138444309,
                    0.000380155106,
                ],
                [
                    0.201761995876,
                    0.72138444309,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    -0.201761995876,
                    0.72138444309,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    0.201761995876,
                    -0.72138444309,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    -0.201761995876,
                    -0.72138444309,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    0.72138444309,
                    0.662492703573,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    -0.72138444309,
                    0.662492703573,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    0.72138444309,
                    -0.662492703573,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    -0.72138444309,
                    -0.662492703573,
                    0.201761995876,
                    0.000380155106,
                ],
                [
                    0.72138444309,
                    0.201761995876,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    -0.72138444309,
                    0.201761995876,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    0.72138444309,
                    -0.201761995876,
                    0.662492703573,
                    0.000380155106,
                ],
                [
                    -0.72138444309,
                    -0.201761995876,
                    0.662492703573,
                    0.000380155106,
                ],
                [0.548493350803, 0.028742197559, 0.8356607746, 0.000371705008],
                [
                    -0.548493350803,
                    0.028742197559,
                    0.8356607746,
                    0.000371705008,
                ],
                [
                    0.548493350803,
                    -0.028742197559,
                    0.8356607746,
                    0.000371705008,
                ],
                [
                    -0.548493350803,
                    -0.028742197559,
                    0.8356607746,
                    0.000371705008,
                ],
                [0.548493350803, 0.8356607746, 0.028742197559, 0.000371705008],
                [
                    -0.548493350803,
                    0.8356607746,
                    0.028742197559,
                    0.000371705008,
                ],
                [
                    0.548493350803,
                    -0.8356607746,
                    0.028742197559,
                    0.000371705008,
                ],
                [
                    -0.548493350803,
                    -0.8356607746,
                    0.028742197559,
                    0.000371705008,
                ],
                [0.028742197559, 0.548493350803, 0.8356607746, 0.000371705008],
                [
                    -0.028742197559,
                    0.548493350803,
                    0.8356607746,
                    0.000371705008,
                ],
                [
                    0.028742197559,
                    -0.548493350803,
                    0.8356607746,
                    0.000371705008,
                ],
                [
                    -0.028742197559,
                    -0.548493350803,
                    0.8356607746,
                    0.000371705008,
                ],
                [0.028742197559, 0.8356607746, 0.548493350803, 0.000371705008],
                [
                    -0.028742197559,
                    0.8356607746,
                    0.548493350803,
                    0.000371705008,
                ],
                [
                    0.028742197559,
                    -0.8356607746,
                    0.548493350803,
                    0.000371705008,
                ],
                [
                    -0.028742197559,
                    -0.8356607746,
                    0.548493350803,
                    0.000371705008,
                ],
                [0.8356607746, 0.548493350803, 0.028742197559, 0.000371705008],
                [
                    -0.8356607746,
                    0.548493350803,
                    0.028742197559,
                    0.000371705008,
                ],
                [
                    0.8356607746,
                    -0.548493350803,
                    0.028742197559,
                    0.000371705008,
                ],
                [
                    -0.8356607746,
                    -0.548493350803,
                    0.028742197559,
                    0.000371705008,
                ],
                [0.8356607746, 0.028742197559, 0.548493350803, 0.000371705008],
                [
                    -0.8356607746,
                    0.028742197559,
                    0.548493350803,
                    0.000371705008,
                ],
                [
                    0.8356607746,
                    -0.028742197559,
                    0.548493350803,
                    0.000371705008,
                ],
                [
                    -0.8356607746,
                    -0.028742197559,
                    0.548493350803,
                    0.000371705008,
                ],
                [
                    0.581020768214,
                    0.057783121237,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    -0.581020768214,
                    0.057783121237,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    0.581020768214,
                    -0.057783121237,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    -0.581020768214,
                    -0.057783121237,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    0.581020768214,
                    0.811834944927,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    -0.581020768214,
                    0.811834944927,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    0.581020768214,
                    -0.811834944927,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    -0.581020768214,
                    -0.811834944927,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    0.057783121237,
                    0.581020768214,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    -0.057783121237,
                    0.581020768214,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    0.057783121237,
                    -0.581020768214,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    -0.057783121237,
                    -0.581020768214,
                    0.811834944927,
                    0.000375249738,
                ],
                [
                    0.057783121237,
                    0.811834944927,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    -0.057783121237,
                    0.811834944927,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    0.057783121237,
                    -0.811834944927,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    -0.057783121237,
                    -0.811834944927,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    0.811834944927,
                    0.581020768214,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    -0.811834944927,
                    0.581020768214,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    0.811834944927,
                    -0.581020768214,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    -0.811834944927,
                    -0.581020768214,
                    0.057783121237,
                    0.000375249738,
                ],
                [
                    0.811834944927,
                    0.057783121237,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    -0.811834944927,
                    0.057783121237,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    0.811834944927,
                    -0.057783121237,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    -0.811834944927,
                    -0.057783121237,
                    0.581020768214,
                    0.000375249738,
                ],
                [
                    0.612095519718,
                    0.086952623714,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    -0.612095519718,
                    0.086952623714,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    0.612095519718,
                    -0.086952623714,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    -0.612095519718,
                    -0.086952623714,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    0.612095519718,
                    0.785988750537,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    -0.612095519718,
                    0.785988750537,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    0.612095519718,
                    -0.785988750537,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    -0.612095519718,
                    -0.785988750537,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    0.086952623714,
                    0.612095519718,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    -0.086952623714,
                    0.612095519718,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    0.086952623714,
                    -0.612095519718,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    -0.086952623714,
                    -0.612095519718,
                    0.785988750537,
                    0.000377880888,
                ],
                [
                    0.086952623714,
                    0.785988750537,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    -0.086952623714,
                    0.785988750537,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    0.086952623714,
                    -0.785988750537,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    -0.086952623714,
                    -0.785988750537,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    0.785988750537,
                    0.612095519718,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    -0.785988750537,
                    0.612095519718,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    0.785988750537,
                    -0.612095519718,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    -0.785988750537,
                    -0.612095519718,
                    0.086952623714,
                    0.000377880888,
                ],
                [
                    0.785988750537,
                    0.086952623714,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    -0.785988750537,
                    0.086952623714,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    0.785988750537,
                    -0.086952623714,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    -0.785988750537,
                    -0.086952623714,
                    0.612095519718,
                    0.000377880888,
                ],
                [
                    0.641694428429,
                    0.116089376706,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    -0.641694428429,
                    0.116089376706,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    0.641694428429,
                    -0.116089376706,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    -0.641694428429,
                    -0.116089376706,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    0.641694428429,
                    0.758123681953,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    -0.641694428429,
                    0.758123681953,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    0.641694428429,
                    -0.758123681953,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    -0.641694428429,
                    -0.758123681953,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    0.116089376706,
                    0.641694428429,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    -0.116089376706,
                    0.641694428429,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    0.116089376706,
                    -0.641694428429,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    -0.116089376706,
                    -0.641694428429,
                    0.758123681953,
                    0.000379633708,
                ],
                [
                    0.116089376706,
                    0.758123681953,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    -0.116089376706,
                    0.758123681953,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    0.116089376706,
                    -0.758123681953,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    -0.116089376706,
                    -0.758123681953,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    0.758123681953,
                    0.641694428429,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    -0.758123681953,
                    0.641694428429,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    0.758123681953,
                    -0.641694428429,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    -0.758123681953,
                    -0.641694428429,
                    0.116089376706,
                    0.000379633708,
                ],
                [
                    0.758123681953,
                    0.116089376706,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    -0.758123681953,
                    0.116089376706,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    0.758123681953,
                    -0.116089376706,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    -0.758123681953,
                    -0.116089376706,
                    0.641694428429,
                    0.000379633708,
                ],
                [
                    0.669792639173,
                    0.145037882674,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    -0.669792639173,
                    0.145037882674,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    0.669792639173,
                    -0.145037882674,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    -0.669792639173,
                    -0.145037882674,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    0.669792639173,
                    0.728245723021,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    -0.669792639173,
                    0.728245723021,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    0.669792639173,
                    -0.728245723021,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    -0.669792639173,
                    -0.728245723021,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    0.145037882674,
                    0.669792639173,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    -0.145037882674,
                    0.669792639173,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    0.145037882674,
                    -0.669792639173,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    -0.145037882674,
                    -0.669792639173,
                    0.728245723021,
                    0.000380555988,
                ],
                [
                    0.145037882674,
                    0.728245723021,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    -0.145037882674,
                    0.728245723021,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    0.145037882674,
                    -0.728245723021,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    -0.145037882674,
                    -0.728245723021,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    0.728245723021,
                    0.669792639173,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    -0.728245723021,
                    0.669792639173,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    0.728245723021,
                    -0.669792639173,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    -0.728245723021,
                    -0.669792639173,
                    0.145037882674,
                    0.000380555988,
                ],
                [
                    0.728245723021,
                    0.145037882674,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    -0.728245723021,
                    0.145037882674,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    0.728245723021,
                    -0.145037882674,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    -0.728245723021,
                    -0.145037882674,
                    0.669792639173,
                    0.000380555988,
                ],
                [
                    0.614759439059,
                    0.029049576223,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    -0.614759439059,
                    0.029049576223,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    0.614759439059,
                    -0.029049576223,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    -0.614759439059,
                    -0.029049576223,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    0.614759439059,
                    0.788179519024,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    -0.614759439059,
                    0.788179519024,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    0.614759439059,
                    -0.788179519024,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    -0.614759439059,
                    -0.788179519024,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    0.029049576223,
                    0.614759439059,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    -0.029049576223,
                    0.614759439059,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    0.029049576223,
                    -0.614759439059,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    -0.029049576223,
                    -0.614759439059,
                    0.788179519024,
                    0.000378025128,
                ],
                [
                    0.029049576223,
                    0.788179519024,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    -0.029049576223,
                    0.788179519024,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    0.029049576223,
                    -0.788179519024,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    -0.029049576223,
                    -0.788179519024,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    0.788179519024,
                    0.614759439059,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    -0.788179519024,
                    0.614759439059,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    0.788179519024,
                    -0.614759439059,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    -0.788179519024,
                    -0.614759439059,
                    0.029049576223,
                    0.000378025128,
                ],
                [
                    0.788179519024,
                    0.029049576223,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    -0.788179519024,
                    0.029049576223,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    0.788179519024,
                    -0.029049576223,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    -0.788179519024,
                    -0.029049576223,
                    0.614759439059,
                    0.000378025128,
                ],
                [
                    0.645539002636,
                    0.058238091526,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    -0.645539002636,
                    0.058238091526,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    0.645539002636,
                    -0.058238091526,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    -0.645539002636,
                    -0.058238091526,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    0.645539002636,
                    0.761503592094,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    -0.645539002636,
                    0.761503592094,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    0.645539002636,
                    -0.761503592094,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    -0.645539002636,
                    -0.761503592094,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    0.058238091526,
                    0.645539002636,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    -0.058238091526,
                    0.645539002636,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    0.058238091526,
                    -0.645539002636,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    -0.058238091526,
                    -0.645539002636,
                    0.761503592094,
                    0.000379886928,
                ],
                [
                    0.058238091526,
                    0.761503592094,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    -0.058238091526,
                    0.761503592094,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    0.058238091526,
                    -0.761503592094,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    -0.058238091526,
                    -0.761503592094,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    0.761503592094,
                    0.645539002636,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    -0.761503592094,
                    0.645539002636,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    0.761503592094,
                    -0.645539002636,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    -0.761503592094,
                    -0.645539002636,
                    0.058238091526,
                    0.000379886928,
                ],
                [
                    0.761503592094,
                    0.058238091526,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    -0.761503592094,
                    0.058238091526,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    0.761503592094,
                    -0.058238091526,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    -0.761503592094,
                    -0.058238091526,
                    0.645539002636,
                    0.000379886928,
                ],
                [
                    0.674725858837,
                    0.087403848999,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    -0.674725858837,
                    0.087403848999,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    0.674725858837,
                    -0.087403848999,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    -0.674725858837,
                    -0.087403848999,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    0.674725858837,
                    0.73287487513,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    -0.674725858837,
                    0.73287487513,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    0.674725858837,
                    -0.73287487513,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    -0.674725858837,
                    -0.73287487513,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    0.087403848999,
                    0.674725858837,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    -0.087403848999,
                    0.674725858837,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    0.087403848999,
                    -0.674725858837,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    -0.087403848999,
                    -0.674725858837,
                    0.73287487513,
                    0.000380904171,
                ],
                [
                    0.087403848999,
                    0.73287487513,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    -0.087403848999,
                    0.73287487513,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    0.087403848999,
                    -0.73287487513,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    -0.087403848999,
                    -0.73287487513,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    0.73287487513,
                    0.674725858837,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    -0.73287487513,
                    0.674725858837,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    0.73287487513,
                    -0.674725858837,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    -0.73287487513,
                    -0.674725858837,
                    0.087403848999,
                    0.000380904171,
                ],
                [
                    0.73287487513,
                    0.087403848999,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    -0.73287487513,
                    0.087403848999,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    0.73287487513,
                    -0.087403848999,
                    0.674725858837,
                    0.000380904171,
                ],
                [
                    -0.73287487513,
                    -0.087403848999,
                    0.674725858837,
                    0.000380904171,
                ],
                [0.67721357504, 0.029199461358, 0.735206886011, 0.0003811069],
                [-0.67721357504, 0.029199461358, 0.735206886011, 0.0003811069],
                [0.67721357504, -0.029199461358, 0.735206886011, 0.0003811069],
                [
                    -0.67721357504,
                    -0.029199461358,
                    0.735206886011,
                    0.0003811069,
                ],
                [0.67721357504, 0.735206886011, 0.029199461358, 0.0003811069],
                [-0.67721357504, 0.735206886011, 0.029199461358, 0.0003811069],
                [0.67721357504, -0.735206886011, 0.029199461358, 0.0003811069],
                [
                    -0.67721357504,
                    -0.735206886011,
                    0.029199461358,
                    0.0003811069,
                ],
                [0.029199461358, 0.67721357504, 0.735206886011, 0.0003811069],
                [-0.029199461358, 0.67721357504, 0.735206886011, 0.0003811069],
                [0.029199461358, -0.67721357504, 0.735206886011, 0.0003811069],
                [
                    -0.029199461358,
                    -0.67721357504,
                    0.735206886011,
                    0.0003811069,
                ],
                [0.029199461358, 0.735206886011, 0.67721357504, 0.0003811069],
                [-0.029199461358, 0.735206886011, 0.67721357504, 0.0003811069],
                [0.029199461358, -0.735206886011, 0.67721357504, 0.0003811069],
                [
                    -0.029199461358,
                    -0.735206886011,
                    0.67721357504,
                    0.0003811069,
                ],
                [0.735206886011, 0.67721357504, 0.029199461358, 0.0003811069],
                [-0.735206886011, 0.67721357504, 0.029199461358, 0.0003811069],
                [0.735206886011, -0.67721357504, 0.029199461358, 0.0003811069],
                [
                    -0.735206886011,
                    -0.67721357504,
                    0.029199461358,
                    0.0003811069,
                ],
                [0.735206886011, 0.029199461358, 0.67721357504, 0.0003811069],
                [-0.735206886011, 0.029199461358, 0.67721357504, 0.0003811069],
                [0.735206886011, -0.029199461358, 0.67721357504, 0.0003811069],
                [
                    -0.735206886011,
                    -0.029199461358,
                    0.67721357504,
                    0.0003811069,
                ],
            ]
        )
        return grid_array

    raise ValueError(f"Grid number has to lay between 0-11")
